@@ -1,13 +1,12 @@
 #include <c10/util/Exception.h>
-#include <c10/util/Unicode.h>
 #include <ATen/DynamicLibrary.h>
-#include <ATen/Utils.h>
 
 #ifndef _WIN32
 #include <dlfcn.h>
 #include <libgen.h>
 #else
 #include <c10/util/win32-headers.h>
+#include <c10/util/Unicode.h>
 #endif
 
 namespace at {
@@ -20,19 +19,20 @@ namespace at {
 
 static void* checkDL(void* x) {
   if (!x) {
-    AT_ERROR("Error in dlopen or dlsym: ", dlerror());
+    TORCH_CHECK_WITH(DynamicLibraryError, false, "Error in dlopen or dlsym: ", dlerror());
   }
 
   return x;
 }
-DynamicLibrary::DynamicLibrary(const char* name, const char* alt_name) {
-  // NOLINTNEXTLINE(hicpp-signed-bitwise)
-  handle = dlopen(name, RTLD_LOCAL | RTLD_NOW);
+DynamicLibrary::DynamicLibrary(const char* name, const char* alt_name, bool leak_handle_): leak_handle(leak_handle_), handle(dlopen(name, RTLD_LOCAL | RTLD_NOW)) {
   if (!handle) {
     if (alt_name) {
-      handle = checkDL(dlopen(alt_name, RTLD_LOCAL | RTLD_NOW));
+      handle = dlopen(alt_name, RTLD_LOCAL | RTLD_NOW);
+      if (!handle) {
+        TORCH_CHECK_WITH(DynamicLibraryError, false, "Error in dlopen for library ", name, "and ", alt_name);
+      }
     } else {
-        AT_ERROR("Error in dlopen or dlsym: ", dlerror());
+      TORCH_CHECK_WITH(DynamicLibraryError, false, "Error in dlopen: ", dlerror());
     }
   }
 }
@@ -43,8 +43,9 @@ void* DynamicLibrary::sym(const char* name) {
 }
 
 DynamicLibrary::~DynamicLibrary() {
-  if (!handle)
+  if (!handle || leak_handle) {
     return;
+  }
   dlclose(handle);
 }
 
@@ -52,7 +53,7 @@ DynamicLibrary::~DynamicLibrary() {
 
 // Windows
 
-DynamicLibrary::DynamicLibrary(const char* name, const char* alt_name) {
+DynamicLibrary::DynamicLibrary(const char* name, const char* alt_name, bool leak_handle_): leak_handle(leak_handle_) {
   // NOLINTNEXTLINE(hicpp-signed-bitwise)
   HMODULE theModule;
   bool reload = true;
@@ -80,7 +81,7 @@ DynamicLibrary::DynamicLibrary(const char* name, const char* alt_name) {
     FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                   NULL, dw, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                   buf, (sizeof(buf) / sizeof(char)), NULL);
-    AT_ERROR("error in LoadLibrary for ", name, ". WinError ", dw, ": ", buf);
+    TORCH_CHECK_WITH(DynamicLibraryError, false, "error in LoadLibrary for ", name, ". WinError ", dw, ": ", buf);
   }
 }
 
@@ -88,13 +89,13 @@ void* DynamicLibrary::sym(const char* name) {
   AT_ASSERT(handle);
   FARPROC procAddress = GetProcAddress((HMODULE)handle, name);
   if (!procAddress) {
-    AT_ERROR("error in GetProcAddress");
+    TORCH_CHECK_WITH(DynamicLibraryError, false, "error in GetProcAddress");
   }
   return (void*)procAddress;
 }
 
 DynamicLibrary::~DynamicLibrary() {
-  if (!handle) {
+  if (!handle || leak_handle) {
     return;
   }
   FreeLibrary((HMODULE)handle);
