@@ -1,7 +1,5 @@
 #include <torch/library.h>
 
-#include <ATen/core/boxing/KernelFunction.h>
-
 using torch::CppFunction;
 
 TORCH_LIBRARY_IMPL(_, Named, m) {
@@ -9,17 +7,15 @@ TORCH_LIBRARY_IMPL(_, Named, m) {
 }
 
 TORCH_LIBRARY_IMPL(aten, Named, m) {
-  m.impl("_bmm", CppFunction::makeFallthrough());
-  m.impl("_bmm.out", CppFunction::makeFallthrough());
   m.impl("_cdist_forward", CppFunction::makeFallthrough());
   m.impl("_fused_dropout", CppFunction::makeFallthrough());
+  m.impl("native_dropout", CppFunction::makeFallthrough());
   m.impl("_local_scalar_dense", CppFunction::makeFallthrough());
   m.impl("_sparse_log_softmax.Dimname", CppFunction::makeFallthrough());
   m.impl("_sparse_log_softmax.int", CppFunction::makeFallthrough());
   m.impl("_sparse_softmax.Dimname", CppFunction::makeFallthrough());
   m.impl("_sparse_softmax.int", CppFunction::makeFallthrough());
-  m.impl("_std", CppFunction::makeFallthrough());
-  m.impl("_var", CppFunction::makeFallthrough());
+  m.impl("_to_copy", CppFunction::makeFallthrough());
   m.impl("abs", CppFunction::makeFallthrough());
   m.impl("abs.out", CppFunction::makeFallthrough());
   m.impl("abs_", CppFunction::makeFallthrough());
@@ -100,17 +96,25 @@ TORCH_LIBRARY_IMPL(aten, Named, m) {
   m.impl("ceil_", CppFunction::makeFallthrough());
   m.impl("chunk", CppFunction::makeFallthrough());
   m.impl("clamp", CppFunction::makeFallthrough());
+  m.impl("clamp.Tensor", CppFunction::makeFallthrough());
   m.impl("clamp.out", CppFunction::makeFallthrough());
+  m.impl("clamp.Tensor_out", CppFunction::makeFallthrough());
   m.impl("clamp_", CppFunction::makeFallthrough());
+  m.impl("clamp_.Tensor", CppFunction::makeFallthrough());
   m.impl("clamp_max", CppFunction::makeFallthrough());
+  m.impl("clamp_max.Tensor", CppFunction::makeFallthrough());
   m.impl("clamp_max.out", CppFunction::makeFallthrough());
+  m.impl("clamp_max.Tensor_out", CppFunction::makeFallthrough());
   m.impl("clamp_max_", CppFunction::makeFallthrough());
+  m.impl("clamp_max_.Tensor", CppFunction::makeFallthrough());
   m.impl("clamp_min", CppFunction::makeFallthrough());
+  m.impl("clamp_min.Tensor", CppFunction::makeFallthrough());
   m.impl("clamp_min.out", CppFunction::makeFallthrough());
+  m.impl("clamp_min.Tensor_out", CppFunction::makeFallthrough());
   m.impl("clamp_min_", CppFunction::makeFallthrough());
+  m.impl("clamp_min_.Tensor", CppFunction::makeFallthrough());
   m.impl("clone", CppFunction::makeFallthrough());
   m.impl("conj", CppFunction::makeFallthrough());
-  m.impl("conj.out", CppFunction::makeFallthrough());
   m.impl("contiguous", CppFunction::makeFallthrough());
   m.impl("copy_", CppFunction::makeFallthrough());
   m.impl("cos", CppFunction::makeFallthrough());
@@ -228,6 +232,7 @@ TORCH_LIBRARY_IMPL(aten, Named, m) {
   m.impl("is_coalesced", CppFunction::makeFallthrough());
   m.impl("is_complex", CppFunction::makeFallthrough());
   m.impl("is_floating_point", CppFunction::makeFallthrough());
+  m.impl("is_inference", CppFunction::makeFallthrough());
   m.impl("is_nonzero", CppFunction::makeFallthrough());
   m.impl("is_pinned", CppFunction::makeFallthrough());
   m.impl("is_same_size", CppFunction::makeFallthrough());
@@ -438,9 +443,15 @@ TORCH_LIBRARY_IMPL(aten, Named, m) {
   m.impl("std.names_dim", CppFunction::makeFallthrough());
   m.impl("std.names_out", CppFunction::makeFallthrough());
   m.impl("std.out", CppFunction::makeFallthrough());
+  m.impl("std.correction", CppFunction::makeFallthrough());
+  m.impl("std.correction_out", CppFunction::makeFallthrough());
+  m.impl("std.correction_names", CppFunction::makeFallthrough());
+  m.impl("std.correction_names_out", CppFunction::makeFallthrough());
   m.impl("std_mean", CppFunction::makeFallthrough());
   m.impl("std_mean.dim", CppFunction::makeFallthrough());
   m.impl("std_mean.names_dim", CppFunction::makeFallthrough());
+  m.impl("std_mean.correction", CppFunction::makeFallthrough());
+  m.impl("std_mean.correction_names", CppFunction::makeFallthrough());
   m.impl("stride.Dimname", CppFunction::makeFallthrough());
   m.impl("stride.int", CppFunction::makeFallthrough());
   m.impl("sub.Scalar", CppFunction::makeFallthrough());
@@ -493,9 +504,15 @@ TORCH_LIBRARY_IMPL(aten, Named, m) {
   m.impl("var.names_dim", CppFunction::makeFallthrough());
   m.impl("var.names_out", CppFunction::makeFallthrough());
   m.impl("var.out", CppFunction::makeFallthrough());
+  m.impl("var.correction", CppFunction::makeFallthrough());
+  m.impl("var.correction_out", CppFunction::makeFallthrough());
+  m.impl("var.correction_names", CppFunction::makeFallthrough());
+  m.impl("var.correction_names_out", CppFunction::makeFallthrough());
   m.impl("var_mean", CppFunction::makeFallthrough());
   m.impl("var_mean.dim", CppFunction::makeFallthrough());
   m.impl("var_mean.names_dim", CppFunction::makeFallthrough());
+  m.impl("var_mean.correction", CppFunction::makeFallthrough());
+  m.impl("var_mean.correction_names", CppFunction::makeFallthrough());
   m.impl("zero_", CppFunction::makeFallthrough());
   m.impl("zeros_like", CppFunction::makeFallthrough());
 
@@ -511,4 +528,5 @@ TORCH_LIBRARY_IMPL(aten, Named, m) {
   m.impl("requires_grad_", CppFunction::makeFallthrough());
   m.impl("retain_grad", CppFunction::makeFallthrough());
   m.impl("_fw_primal", CppFunction::makeFallthrough());
+  m.impl("_make_dual", CppFunction::makeFallthrough());
 }
