@@ -1,11 +1,11 @@
 #include <ATen/core/Range.h>
 
-#include <iostream>
+#include <ostream>
 
 namespace at {
 
 std::ostream& operator<<(std::ostream& out, const Range& range) {
-  out << "Range[" << range.begin << ", " << range.end << "]";
+  out << "Range[" << range.begin << ", " << range.end << ']';
   return out;
 }
 
