@@ -1,11 +1,13 @@
+#include <ATen/NumericUtils.h>
 #include <c10/macros/Macros.h>
 #include <c10/util/Half.h>
 #include <c10/util/BFloat16.h>
 #include <c10/util/MathConstants.h>
-#include <ATen/NumericUtils.h>
-#include <limits>
+#include <cmath>
 #include <cstdint>
 #include <cassert>
+#include <limits>
+#include <type_traits>
 
 namespace at {
 
@@ -54,14 +56,14 @@ C10_HOST_DEVICE inline T uniform_int_full_range(V val) {
  * in this overloaded version
  */
 template <typename T, typename V>
-C10_HOST_DEVICE inline typename std::enable_if<!(std::is_floating_point<T>::value), T>::type uniform_int(V val) {
-  if (std::is_same<T, bool>::value) {
+C10_HOST_DEVICE inline std::enable_if_t<!std::is_floating_point_v<T>, T>uniform_int(V val) {
+  if constexpr (std::is_same_v<T, bool>) {
     return static_cast<bool>(val & 1);
-  } else if (std::is_same<T, int64_t>::value) {
+  } else if constexpr (std::is_same_v<T, int64_t>) {
     return static_cast<T>(val % (static_cast<uint64_t>(std::numeric_limits<T>::max()) + 1));
-  } else if (std::is_same<T, at::Half>::value || std::is_same<T, at::BFloat16>::value) {
+  } else if constexpr (std::is_same_v<T, at::Half> || std::is_same_v<T, at::BFloat16>) {
     return static_cast<T>(val % static_cast<uint64_t>((1ULL << std::numeric_limits<T>::digits) + 1));
-  } else if (std::is_integral<T>::value) {
+  } else if constexpr (std::is_integral_v<T>) {
     return static_cast<T>(val % (static_cast<uint64_t>(std::numeric_limits<T>::max()) + 1));
   } else {
     assert(false);
@@ -74,7 +76,7 @@ C10_HOST_DEVICE inline typename std::enable_if<!(std::is_floating_point<T>::valu
  * added to fix compiler warnings reported in GitHub issue 46391. T is either float or double in this version.
  */
 template<typename T, typename V>
-C10_HOST_DEVICE inline typename std::enable_if<std::is_floating_point<T>::value, T>::type uniform_int(V val) {
+C10_HOST_DEVICE inline std::enable_if_t<std::is_floating_point_v<T>, T>uniform_int(V val) {
   return static_cast<T>(val % static_cast<uint64_t>((1ULL << std::numeric_limits<T>::digits) + 1));
 }
 
@@ -102,34 +104,54 @@ C10_HOST_DEVICE inline T normal(T val, T mean, T std) {
 template <typename T>
 C10_HOST_DEVICE inline T cauchy(T val, T median, T sigma) {
   // https://en.wikipedia.org/wiki/Cauchy_distribution#Cumulative_distribution_function
+  // __tanf overflows and returns `inf/-inf` when (val > 1 - eps) or (val < 0 + eps),
+  // thus we clip those values.
+  constexpr T eps = std::numeric_limits<T>::epsilon();
+  constexpr T one_minus_eps = 1 - eps;
+  constexpr T zero_plus_eps = 0 + eps;
+  val = (val > one_minus_eps ? one_minus_eps : val);
+  val = (val < zero_plus_eps ? zero_plus_eps : val);
   return median + sigma * at::tan(c10::pi<T> * (val - static_cast<T>(0.5)));
 }
 
+template <>
+C10_HOST_DEVICE inline double cauchy(double val, double median, double sigma) {
+  // https://en.wikipedia.org/wiki/Cauchy_distribution#Cumulative_distribution_function
+  return median + sigma * at::tan(c10::pi<double> * (val - 0.5));
+}
+
 /**
  * Transforms uniformly distributed `val` between 0.0 and 1.0 to
- * exponentialy distributed with `lambda` parameter of the distribution.
+ * exponentially distributed with `lambda` parameter of the distribution.
  */
 template <typename T>
-C10_HOST_DEVICE __ubsan_ignore_float_divide_by_zero__ inline T exponential(T val, T lambda) {
+C10_HOST_DEVICE inline T exponential(T val, T lambda) {
   // https://en.wikipedia.org/wiki/Exponential_distribution#Generating_exponential_variates
   // Different implementations for CUDA and CPU to preserve original logic
   // TODO: must be investigated and unified!!!
   // https://github.com/pytorch/pytorch/issues/38662
 #if defined(__CUDACC__) || defined(__HIPCC__)
-  return static_cast<T>(-1.0) / lambda * at::log(val);
+      // BEFORE TOUCHING THIS CODE READ: https://github.com/pytorch/pytorch/issues/16706
+      // curand_uniform has (0,1] bounds. log(1) is 0 and exponential excludes 0.
+      // we need log to be not 0, and not underflow when converted to half
+      // fast __logf approximation can underflow, so set log to -epsilon/2 for 1 or close to 1 args
+  auto log = val >= static_cast<T>(1.) - std::numeric_limits<T>::epsilon() / 2
+      ? -std::numeric_limits<T>::epsilon() / 2
+      : at::log(val);
+  return static_cast<T>(-1.0) / lambda * log;
 #else
-  return static_cast<T>(-1.0) / lambda * at::log(static_cast<T>(1.0) - val);
+  return static_cast<T>(-1.0) / lambda * at::log1p(-val);
 #endif
 }
 
 /**
  * Transforms uniformly distributed `val` between 0.0 and 1.0 to
- * geometricaly distributed with success probability `p`.
+ * geometrically distributed with success probability `p`.
  */
 template <typename T>
 C10_HOST_DEVICE inline T geometric(T val, T p) {
   // https://en.wikipedia.org/wiki/Geometric_distribution#Related_distributions
-  return static_cast<T>(::ceil(at::log(val) / at::log(static_cast<T>(1.0) - p)));
+  return static_cast<T>(::ceil(at::log(val) / at::log1p(-p)));
 }
 
 /**
