@@ -1,16 +1,11 @@
+// aten_interned_strings.h includes the names of all operators
+#undef TORCH_ASSERT_ONLY_METHOD_OPERATORS
+
 #include <ATen/core/interned_strings.h>
-#include <cstdint>
 #include <cstring>
-#include <iostream>
 #include <mutex>
-#include <sstream>
 #include <string>
-#include <unordered_map>
-#include <vector>
-#include <c10/util/Exception.h>
 #include <ATen/core/interned_strings_class.h>
-#include <c10/util/Exception.h>
-#include <c10/util/Optional.h>
 
 namespace c10 {
 
@@ -53,6 +48,7 @@ Symbol InternedStrings::ns(Symbol sym) {
 #define DEFINE_CASE(ns, s) \
   case static_cast<unique_t>(ns::s): \
     return namespaces::ns;
+    // NOLINTNEXTLINE(bugprone-branch-clone)
     FORALL_NS_SYMBOLS(DEFINE_CASE)
 #undef DEFINE_CASE
     default: {
@@ -69,11 +65,7 @@ Symbol InternedStrings::_symbol(const std::string& s) {
     return it->second;
 
   auto pos = s.find("::");
-  if (pos == std::string::npos) {
-    std::stringstream ss;
-    ss << "all symbols must have a namespace, <namespace>::<string>, but found: " << s;
-    throw std::runtime_error(ss.str());
-  }
+  TORCH_CHECK(pos != std::string::npos, "all symbols must have a namespace, <namespace>::<string>, but found: ", s);
   Symbol ns = _symbol("namespaces::" + s.substr(0, pos));
 
   Symbol sym(sym_to_info_.size());
@@ -122,14 +114,20 @@ std::string Symbol::domainString() const {
 }
 
 Symbol Symbol::fromDomainAndUnqualString(const std::string & d, const std::string & s) {
-  if (d.compare(0, domain_prefix().size(), domain_prefix()) != 0) {
-    std::ostringstream ss;
-    ss << "Symbol: domain string is expected to be prefixed with '"
-       << domain_prefix() << "', e.g. 'org.pytorch.aten'";
-    throw std::runtime_error(ss.str());
-  }
+  TORCH_CHECK(d.compare(0, domain_prefix().size(), domain_prefix()) == 0, "Symbol: domain string is expected to be prefixed with '", domain_prefix(), "', e.g. 'org.pytorch.aten'");
   std::string qualString = d.substr(domain_prefix().size()) + "::" + s;
   return fromQualString(qualString);
 }
 
+bool Symbol::is_attr() const { return ns() == namespaces::attr; }
+bool Symbol::is_aten() const { return ns() == namespaces::aten; }
+bool Symbol::is_cuda() const { return ns() == namespaces::cuda; }
+bool Symbol::is_prim() const { return ns() == namespaces::prim; }
+bool Symbol::is_prims() const { return ns() == namespaces::prims; }
+bool Symbol::is_nvprims() const { return ns() == namespaces::nvprims; }
+bool Symbol::is_onnx() const { return ns() == namespaces::onnx; }
+bool Symbol::is_user() const { return ns() == namespaces::user; }
+bool Symbol::is_caffe2() const { return ns() == namespaces::_caffe2; }
+bool Symbol::is_dimname() const { return ns() == namespaces::dimname; }
+
 } // namespace c10
