@@ -1,4 +1,3 @@
-#include <ATen/ATen.h>
 #include <ATen/autocast_mode.h>
 #include <torch/library.h>
 
@@ -9,8 +8,8 @@
 #include <ATen/native/cudnn/RNNUtils.h>
 #endif
 
-namespace at {
-namespace autocast {
+
+namespace at::autocast {
 
 /********************************************************************************
 Autocast wrapper for CuDNN RNNs (the weight reflattening needs special attention)
@@ -18,13 +17,13 @@ Autocast wrapper for CuDNN RNNs (the weight reflattening needs special attention
 
 // To be registered for the "_cudnn_rnn(...)" schema.
 // _cudnn_rnn is autograd-exposed (test_autocast_cudnn_rnn in test_cuda.py includes a test to confirm)
-std::tuple<Tensor,Tensor,Tensor,Tensor,Tensor>
+static std::tuple<Tensor,Tensor,Tensor,Tensor,Tensor>
 _cudnn_rnn_cast_reflatten(const Tensor & input,
                           TensorList weight,
                           int64_t weight_stride0,
-                          const c10::optional<Tensor>& weight_buf_opt,
+                          const std::optional<Tensor>& weight_buf_opt,
                           const Tensor& hx,
-                          const c10::optional<Tensor>& cx,
+                          const std::optional<Tensor>& cx,
                           int64_t mode,
                           int64_t hidden_size,
                           int64_t proj_size,
@@ -34,7 +33,7 @@ _cudnn_rnn_cast_reflatten(const Tensor & input,
                           bool train,
                           bool bidirectional,
                           IntArrayRef batch_sizes,
-                          const c10::optional<Tensor>& dropout_state) {
+                          const std::optional<Tensor>& dropout_state) {
 #if AT_CUDNN_ENABLED()
   c10::impl::ExcludeDispatchKeyGuard no_autocast(DispatchKey::Autocast);
 
@@ -113,7 +112,7 @@ _cudnn_rnn_cast_reflatten(const Tensor & input,
       batch_sizes,
       dropout_state);
 #else // AT_CUDNN_ENABLED()
-  AT_ERROR("autocast::_cudnn_rnn_cast_reflatten: ATen not compiled with cuDNN support");
+  TORCH_CHECK(false, "autocast::_cudnn_rnn_cast_reflatten: ATen not compiled with cuDNN support");
   return {Tensor{}, Tensor{}, Tensor{}, Tensor{}, Tensor{}}; // never reached, placates the compiler
 #endif // AT_CUDNN_ENABLED()
 }
@@ -125,5 +124,4 @@ TORCH_LIBRARY_IMPL(aten, Autocast, m) {
 }
 } // anonymous namespace
 
-} // namespace autocast
-} // namespace at
+} // namespace at::autocast
