@@ -2,6 +2,7 @@
 
 #include <ATen/mps/MPSProfiler.h>
 #include <c10/util/Exception.h>
+#include <c10/util/env.h>
 #include <fmt/format.h>
 
 // these need to be literal strings when passed to os_signpost*()
@@ -38,9 +39,9 @@ std::string BaseInfo::buildTensorString(const Tensor& tensor, bool includeBuffer
     // see comments for INCLUDE_BUFFER_ID
     if (includeBufferId && deviceType == at::kMPS) {
       id<MTLBuffer> buffer = __builtin_bit_cast(id<MTLBuffer>, tensor.storage().data());
-      tensorStr << "(buf#" << (getIMPSAllocator()->getBufferId(buffer)) << ":" << buffer.retainCount << ")";
+      tensorStr << "(buf#" << (getIMPSAllocator()->getBufferId(buffer)) << ':' << buffer.retainCount << ')';
     }
-    tensorStr << ":" << tensor.scalar_type() << tensor.sizes();
+    tensorStr << ':' << tensor.scalar_type() << tensor.sizes();
     return tensorStr.str();
   } else {
     return "undefined";
@@ -91,11 +92,11 @@ std::string CopyInfo::buildTensorString(const void* buffer, const OptionalTensor
 
 MPSProfiler::MPSProfiler() : m_os_log_events(nullptr), m_os_log_intervals(nullptr) {
   // see enum LogOptions for the description.
-  static const char* log_options_str = getenv(kEVLogProfileInfoStr);
-  m_log_options = log_options_str ? strtol(log_options_str, nullptr, 0) : 0;
+  static const auto log_options_str = c10::utils::get_env(kEVLogProfileInfoStr);
+  m_log_options = log_options_str ? strtol(log_options_str->c_str(), nullptr, 0) : 0;
   // see enums profilerOptions and SignpostTypes for the description.
-  static const char* trace_signpost_str = getenv(kEVTraceSignpostsStr);
-  uint32_t trace_signposts = trace_signpost_str ? strtol(trace_signpost_str, nullptr, 0) : 0;
+  static const auto trace_signpost_str = c10::utils::get_env(kEVTraceSignpostsStr);
+  uint32_t trace_signposts = trace_signpost_str ? strtol(trace_signpost_str->c_str(), nullptr, 0) : 0;
 
   TORCH_CHECK(m_log_options <= LogOptions::LOG_COUNT,
               "invalid log options ",
@@ -779,8 +780,8 @@ void MPSProfiler::handleIntSignal(int signal) {
 }
 
 // used to capture sigint signal to log profiling stats
-struct sigaction MPSProfiler::currentSigint {};
-struct sigaction MPSProfiler::previousSigint {};
+struct sigaction MPSProfiler::currentSigint{};
+struct sigaction MPSProfiler::previousSigint{};
 
 bool MPSProfiler::isCapturing() const {
   return [captureManager isCapturing];
