@@ -1,37 +1,50 @@
-#include <ATen/ATen.h>
+#define TORCH_ASSERT_ONLY_METHOD_OPERATORS
+#include <ATen/core/Tensor.h>
 #include <ATen/AccumulateType.h>
 #include <ATen/Dispatch.h>
 #include <ATen/TensorUtils.h>
 #include <ATen/native/LossMulti.h>
+#include <c10/util/irange.h>
 
-namespace at {
-namespace native {
+#ifndef AT_PER_OPERATOR_HEADERS
+#include <ATen/Functions.h>
+#include <ATen/NativeFunctions.h>
+#else
+#include <ATen/ops/empty.h>
+#include <ATen/ops/multilabel_margin_loss_backward_native.h>
+#include <ATen/ops/multilabel_margin_loss_forward.h>
+#include <ATen/ops/multilabel_margin_loss_forward_native.h>
+#include <ATen/ops/multilabel_margin_loss_native.h>
+#include <ATen/ops/zeros_like.h>
+#endif
+
+namespace at::native {
 
 namespace {
 
 template <typename scalar_t>
 inline scalar_t multilabel_margin_loss_forward_inner_sum_cpu(
-    scalar_t* input_data,
-    int64_t* target_data,
+    const scalar_t* input_data,
+    const int64_t* target_data,
     scalar_t* is_target_data,
     int64_t dim) {
   using accscalar_t = at::acc_type<scalar_t, false>;
   accscalar_t sum = 0;
-  for (int64_t ddt = 0; ddt < dim; ddt++) {
+  for (const auto ddt : c10::irange(dim)) {
     int64_t target_idx = target_data[ddt];
     if (target_idx < 0) {
       break;
     }
     is_target_data[target_idx] = 1;
   }
-  for (int64_t dt = 0; dt < dim; dt++) {
+  for (const auto dt : c10::irange(dim)) {
     int64_t target_idx = target_data[dt];
     if (target_idx < 0) {
       break;
     }
 
     scalar_t input_target = input_data[target_idx];
-    for (int64_t d = 0; d < dim; d++) {
+    for (const auto d : c10::irange(dim)) {
       if (!is_target_data[d]) {
         scalar_t z = 1 - input_target + input_data[d];
         if (z > 0) {
@@ -40,12 +53,12 @@ inline scalar_t multilabel_margin_loss_forward_inner_sum_cpu(
       }
     }
   }
-  
+
   return sum;
 }
 
 template <typename scalar_t>
-static void multilabel_margin_loss_forward_out_frame(
+void multilabel_margin_loss_forward_out_frame(
     const Tensor& input_contiguous,
     const Tensor& target_contiguous,
     Tensor& output,
@@ -54,8 +67,8 @@ static void multilabel_margin_loss_forward_out_frame(
     int64_t nframe,
     int64_t dim) {
   using accscalar_t = at::acc_type<scalar_t, false>;
-  scalar_t* input_data = input_contiguous.data_ptr<scalar_t>();
-  int64_t* target_data = target_contiguous.data_ptr<int64_t>();
+  const scalar_t* input_data = input_contiguous.const_data_ptr<scalar_t>();
+  const int64_t* target_data = target_contiguous.const_data_ptr<int64_t>();
   scalar_t* is_target_data = is_target.data_ptr<scalar_t>();
 
   if (reduction != Reduction::None || output.dim() == 0) {
@@ -63,7 +76,7 @@ static void multilabel_margin_loss_forward_out_frame(
 
     accscalar_t sum = 0;
 
-    for (int64_t t = 0; t < nframe; t++) {
+    for ([[maybe_unused]] const auto t : c10::irange(nframe)) {
       sum += multilabel_margin_loss_forward_inner_sum_cpu(
           input_data, target_data, is_target_data, dim);
 
@@ -81,7 +94,7 @@ static void multilabel_margin_loss_forward_out_frame(
   } else {
     auto output_acc = output.accessor<scalar_t, 1>();
 
-    for (int64_t t = 0; t < nframe; t++) {
+    for (const auto t : c10::irange(nframe)) {
       scalar_t sum = multilabel_margin_loss_forward_inner_sum_cpu(
           input_data, target_data, is_target_data, dim);
 
@@ -95,25 +108,19 @@ static void multilabel_margin_loss_forward_out_frame(
   }
 }
 
-static void multilabel_margin_loss_forward_out_cpu_template(
+void multilabel_margin_loss_forward_out_cpu_template(
     const Tensor& input,
     const Tensor& target,
     Tensor& output,
     Tensor& is_target,
     int64_t reduction) {
+#ifndef STRIP_ERROR_MESSAGES
   auto target_arg = TensorArg(target, "target", 2);
-  int64_t nframe, dim;
-  const int64_t ndims = input.dim();  
-  if (ndims <= 1) {
-    nframe = 1;
-    dim = ndims == 0 ? 1 : input.size(0);
-  }
-  else {
-    nframe = input.size(0);
-    dim = input.size(1);
-  }
-  multilabel_margin_loss_shape_check(nframe, dim, ndims, target_arg, input, target);
-  
+#endif
+  int64_t nframe = 0, dim = 0;
+  const int64_t ndims = input.dim();
+  multilabel_margin_loss_shape_check(nframe, dim, ndims, input, target);
+
   // special case target.dim() <= 1: produce scalar output for scalar inputs
   // even if reduction == Reduction::None
   if (reduction != Reduction::None || target.dim() <= 1) {
@@ -146,7 +153,7 @@ static void multilabel_margin_loss_forward_out_cpu_template(
 }
 
 template <typename scalar_t>
-static void multilabel_margin_loss_backward_out_frame(
+void multilabel_margin_loss_backward_out_frame(
     Tensor& grad_input,
     const Tensor& grad_output,
     const Tensor& input_contiguous,
@@ -155,30 +162,32 @@ static void multilabel_margin_loss_backward_out_frame(
     const Tensor& is_target_contiguous,
     int64_t nframe,
     int64_t dim) {
-  CheckedFrom c = "multilabel_margin_loss_backward_out_frame";
+#ifndef STRIP_ERROR_MESSAGES
   auto is_target_arg = TensorArg(is_target_contiguous, "is_target", 5);
+#endif
 
   TORCH_CHECK(
       is_target_contiguous.min().item<scalar_t>() >= 0, is_target_arg, " is out of range");
   TORCH_CHECK(
       is_target_contiguous.max().item<scalar_t>() <= 1, is_target_arg, " is out of range");
 
-  scalar_t* input_data = input_contiguous.data_ptr<scalar_t>();
-  int64_t* target_data = target_contiguous.data_ptr<int64_t>();
-  scalar_t* is_target_data = is_target_contiguous.data_ptr<scalar_t>();
+  const scalar_t* input_data = input_contiguous.const_data_ptr<scalar_t>();
+  const int64_t* target_data = target_contiguous.const_data_ptr<int64_t>();
+  const scalar_t* is_target_data = is_target_contiguous.const_data_ptr<scalar_t>();
   scalar_t g = static_cast<scalar_t>(
+      // NOLINTNEXTLINE(cppcoreguidelines-narrowing-conversions,bugprone-narrowing-conversions)
       reduction == Reduction::Mean ? 1. / (nframe * dim) : 1. / dim);
 
-  scalar_t* grad_input_row_data = grad_input.data_ptr<scalar_t>();
-  for (int64_t t = 0; t < nframe; t++) {
-    for (int64_t dt = 0; dt < dim; dt++) {
+  scalar_t* grad_input_row_data = grad_input.mutable_data_ptr<scalar_t>();
+  for ([[maybe_unused]] const auto t : c10::irange(nframe)) {
+    for (const auto dt : c10::irange(dim)) {
       int64_t target_idx = target_data[dt];
       if (target_idx < 0) {
         break;
       }
 
       scalar_t input_target = input_data[target_idx];
-      for (int64_t d = 0; d < dim; d++) {
+      for (const auto d : c10::irange(dim)) {
         if (!is_target_data[d]) {
           scalar_t z = 1 - input_target + input_data[d];
           if (z > 0) {
@@ -194,46 +203,46 @@ static void multilabel_margin_loss_backward_out_frame(
     grad_input_row_data += dim;
   }
 
-  scalar_t* grad_input_data = grad_input.data_ptr<scalar_t>();
+  scalar_t* grad_input_data = grad_input.mutable_data_ptr<scalar_t>();
   if (reduction != Reduction::None || grad_output.dim() == 0) {
     assert(
         reduction != Reduction::None || grad_output.dim() > 0 || nframe == 1);
-    const auto d = *grad_output.data_ptr<scalar_t>();
+    const auto d = *grad_output.const_data_ptr<scalar_t>();
     for (int64_t t = 0; t < nframe * dim; t++) {
       grad_input_data[t] *= d;
     }
   } else {
     check_dim_size(grad_output, 1, 0, nframe);
-    auto grad_output_acc = grad_output.accessor<scalar_t, 1>();
-    for (int64_t t = 0; t < nframe; t++) {
-      for (int64_t d = 0; d < dim; d++) {
+    auto grad_output_acc = grad_output.accessor<const scalar_t, 1>();
+    for (const auto t : c10::irange(nframe)) {
+      for (const auto d : c10::irange(dim)) {
         grad_input_data[t * dim + d] *= grad_output_acc[t];
       }
     }
   }
 }
 
-static void multilabel_margin_loss_backward_out_cpu_template(
+void multilabel_margin_loss_backward_out_cpu_template(
     Tensor& grad_input,
     const Tensor& grad_output,
     const Tensor& input,
     const Tensor& target,
     int64_t reduction,
     const Tensor& is_target) {
-  int64_t nframe, dim;
+  int64_t nframe = 0, dim = 0;
   CheckedFrom c = "multilabel_margin_loss_backward_cpu_template";
   auto target_arg = TensorArg(target, "target", 3);
   auto is_target_arg = TensorArg(is_target, "is_target", 5);
   const int64_t ndims = input.dim();
 
-  multilabel_margin_loss_shape_check(nframe, dim, ndims, target_arg, input, target);
+  multilabel_margin_loss_shape_check(nframe, dim, ndims, input, target);
   checkSameSize(c, target_arg, is_target_arg);
-  
+
   grad_input.resize_as_(input);
   if (grad_input.numel() == 0) {
     return;
   }
-  
+
   TORCH_CHECK(grad_input.is_contiguous(), "grad_input must be contiguous");
   grad_input.zero_();
 
@@ -262,12 +271,11 @@ static void multilabel_margin_loss_backward_out_cpu_template(
 
 } // namespace
 
-std::tuple<Tensor&, Tensor&> multilabel_margin_loss_forward_out_cpu(
-    Tensor& output,
-    Tensor& is_target,
-    const Tensor& self,
+std::tuple<Tensor&, Tensor&> multilabel_margin_loss_forward_out_cpu(const Tensor& self,
     const Tensor& target,
-    int64_t reduction) {
+    int64_t reduction,
+    Tensor& output,
+    Tensor& is_target) {
   multilabel_margin_loss_forward_out_cpu_template(
       self, target, output, is_target, reduction);
   return std::tuple<Tensor&, Tensor&>(output, is_target);
@@ -279,18 +287,17 @@ std::tuple<Tensor, Tensor> multilabel_margin_loss_forward_cpu(
     int64_t reduction) {
   auto output = at::empty({0}, self.options());
   auto is_target = at::empty({0}, self.options());
-  multilabel_margin_loss_forward_out_cpu(
-      output, is_target, self, target, reduction);
+  at::native::multilabel_margin_loss_forward_out_cpu(
+      self, target, reduction, output, is_target);
   return std::make_tuple(output, is_target);
 }
 
-Tensor& multilabel_margin_loss_backward_cpu_out(
-    Tensor& grad_input,
-    const Tensor& grad_output,
+Tensor& multilabel_margin_loss_backward_cpu_out(const Tensor& grad_output,
     const Tensor& self,
     const Tensor& target,
     int64_t reduction,
-    const Tensor& is_target) {
+    const Tensor& is_target,
+    Tensor& grad_input) {
   multilabel_margin_loss_backward_out_cpu_template(
       grad_input, grad_output, self, target, reduction, is_target);
   return grad_input;
@@ -303,12 +310,12 @@ Tensor multilabel_margin_loss_backward_cpu(
     int64_t reduction,
     const Tensor& is_target) {
   auto grad_input = at::zeros_like(self, LEGACY_CONTIGUOUS_MEMORY_FORMAT);
-  multilabel_margin_loss_backward_cpu_out(
-      grad_input, grad_output, self, target, reduction, is_target);
+  at::native::multilabel_margin_loss_backward_cpu_out(
+      grad_output, self, target, reduction, is_target, grad_input);
   return grad_input;
 }
 
-Tensor & multilabel_margin_loss_out(Tensor & output, const Tensor & self, const Tensor & target, int64_t reduction) {
+Tensor & multilabel_margin_loss_out(const Tensor & self, const Tensor & target, int64_t reduction, Tensor & output) {
   Tensor is_target = at::empty({0}, self.options());
   return std::get<0>(at::multilabel_margin_loss_forward_out(output, is_target, self, target, reduction));
 }
@@ -317,5 +324,4 @@ Tensor multilabel_margin_loss(const Tensor & self, const Tensor & target, int64_
   return std::get<0>(at::multilabel_margin_loss_forward(self, target, reduction));
 }
 
-} // namespace native
-} // namespace at
+} // namespace at::native
