@@ -6,6 +6,7 @@
 #if AT_MKLDNN_ENABLED()
 
 #include <ideep.hpp>
+#include <dnnl.hpp>
 
 namespace at::native {
 
@@ -41,26 +42,26 @@ using IDeepTensorWrapperPtr = c10::intrusive_ptr<IDeepTensorWrapper>;
 using MKLDNNTensorImpl = OpaqueTensorImpl<IDeepTensorWrapperPtr>;
 using MKLDNNTensor = Tensor;
 
-ideep::tensor::data_type get_mkldnn_dtype(ScalarType type) {
+dnnl::memory::data_type get_mkldnn_dtype(ScalarType type) {
   switch (type) {
     case ScalarType::Float:
-      return ideep::tensor::data_type::f32;
+      return dnnl::memory::data_type::f32;
     case ScalarType::QInt32:
-      return ideep::tensor::data_type::s32;
+      return dnnl::memory::data_type::s32;
     case ScalarType::QInt8:
     case ScalarType::Char:
-      return ideep::tensor::data_type::s8;
+      return dnnl::memory::data_type::s8;
     case ScalarType::QUInt8:
     case ScalarType::Byte:
-      return ideep::tensor::data_type::u8;
+      return dnnl::memory::data_type::u8;
     case ScalarType::BFloat16:
-      return ideep::tensor::data_type::bf16;
+      return dnnl::memory::data_type::bf16;
     case ScalarType::Half:
-      return ideep::tensor::data_type::f16;
+      return dnnl::memory::data_type::f16;
     case ScalarType::Float8_e4m3fn:
-      return ideep::tensor::data_type::f8_e4m3;
+      return dnnl::memory::data_type::f8_e4m3;
     case ScalarType::Float8_e5m2:
-      return ideep::tensor::data_type::f8_e5m2;
+      return dnnl::memory::data_type::f8_e5m2;
     default:
       TORCH_CHECK(false, "get_mkldnn_dtype: unsupported data type");
   }
@@ -82,11 +83,11 @@ at::Tensor mkldnn_tensor_from_data_ptr(
   std::vector<uint8_t> vector_serialized_md{
       opaque_metadata, opaque_metadata + opaque_metadata_size};
   ideep::tensor::desc deserialized_ideep_desc;
-#if IDEEP_PREREQ(3, 4, 1, 2)
+#if DNNL_PREREQ(3, 4, 1)
   // groups is needed for grouped conv
   deserialized_ideep_desc = ideep::tensor::desc(vector_serialized_md);
 #else
-  TORCH_CHECK(false, "Unexpected IDeep version to do weight deserialization.");
+  TORCH_CHECK(false, "Unexpected oneDNN version to do weight deserialization.");
 #endif
 
   auto a = ideep::tensor(deserialized_ideep_desc, data_ptr);
