@@ -175,7 +175,8 @@ sycl::event deconvolution(
 
   // create primitive desc
   dnnl::memory::dims _stride = stride.vec();
-  dnnl::memory::dims _padding = padding.vec();
+  dnnl::memory::dims _padding_l = padding.vec();
+  dnnl::memory::dims _padding_r = padding_r(padding, dst_padding);
   dnnl::memory::dims _dilation = deconv_compatible_dilation(dilation);
 
   // construct primitive attr
@@ -200,8 +201,8 @@ sycl::event deconvolution(
       dst_md,
       _stride,
       _dilation,
-      _padding,
-      _padding,
+      _padding_l,
+      _padding_r,
       pattr);
 
   dnnl::memory src_m, weight_m, dst_m, bia_m;
@@ -244,6 +245,7 @@ sycl::event deconvolution_backward_data(
     const at::Tensor& weight,
     IntArrayRef stride,
     IntArrayRef padding,
+    IntArrayRef dst_padding,
     IntArrayRef dilation,
     int64_t groups,
     bool bias_defined,
@@ -273,7 +275,8 @@ sycl::event deconvolution_backward_data(
 #endif
 
   dnnl::memory::dims _stride = stride.vec();
-  dnnl::memory::dims _padding = padding.vec();
+  dnnl::memory::dims _padding_l = padding.vec();
+  dnnl::memory::dims _padding_r = padding_r(padding, dst_padding);
   dnnl::memory::dims _dilation = deconv_compatible_dilation(dilation);
   auto deconv_fwd_pd = dnnl::deconvolution_forward::primitive_desc(
       engine,
@@ -285,8 +288,8 @@ sycl::event deconvolution_backward_data(
       dst_md,
       _stride,
       _dilation,
-      _padding,
-      _padding,
+      _padding_l,
+      _padding_r,
       pattr);
 
   // create bwd primitive desc
@@ -299,8 +302,8 @@ sycl::event deconvolution_backward_data(
           dst_md,
           _stride,
           _dilation,
-          _padding,
-          _padding,
+          _padding_l,
+          _padding_r,
           deconv_fwd_pd,
           pattr);
 
@@ -342,6 +345,7 @@ sycl::event deconvolution_backward_weights(
     const at::Tensor& src,
     IntArrayRef stride,
     IntArrayRef padding,
+    IntArrayRef dst_padding,
     IntArrayRef dilation,
     int64_t groups,
     const std::vector<sycl::event>& deps) {
@@ -361,7 +365,8 @@ sycl::event deconvolution_backward_weights(
 
   // create fwd primitive desc hint
   dnnl::memory::dims _stride = stride.vec();
-  dnnl::memory::dims _padding = padding.vec();
+  dnnl::memory::dims _padding_l = padding.vec();
+  dnnl::memory::dims _padding_r = padding_r(padding, dst_padding);
   dnnl::memory::dims _dilation = deconv_compatible_dilation(dilation);
   dnnl::primitive_attr pattr;
 
@@ -381,8 +386,8 @@ sycl::event deconvolution_backward_weights(
       dst_md,
       _stride,
       _dilation,
-      _padding,
-      _padding,
+      _padding_l,
+      _padding_r,
       pattr);
 
   auto deconv_bwd_w_pd = dnnl::deconvolution_backward_weights::primitive_desc(
@@ -394,8 +399,8 @@ sycl::event deconvolution_backward_weights(
       dst_md,
       _stride,
       _dilation,
-      _padding,
-      _padding,
+      _padding_l,
+      _padding_r,
       deconv_fwd_pd,
       pattr);
 
