@@ -519,6 +519,10 @@ at::Tensor _qconv_prepack_onednn(
       dilation.size() == (decltype(dilation.size()))kSpatialDim,
       "dilation should contain ", kSpatialDim, " elements for ",
       kSpatialDim, "D convolution.");
+  TORCH_CHECK(
+      weight.scalar_type() == at::kChar || weight.scalar_type() == at::kFloat8_e4m3fn,
+      "Weight should have dtype int8 or fp8_e4m3fn but got ", weight.scalar_type());
+  bool is_fp8 = weight.scalar_type() == at::kFloat8_e4m3fn;
 
   bool is_1d = (1 == kSpatialDim);
   auto x_dims = input_shape.has_value()?input_shape.value().vec():ideep::dims();
@@ -535,6 +539,12 @@ at::Tensor _qconv_prepack_onednn(
     dilation = quant_utils::MakeArgForConv1d(dilation, 1);
     kSpatialDim += 1;
   }
+  if (is_fp8) {
+    // The current version of oneDNN does not support fp8 conv
+    // TODO(weiwen) Remove this when oneDNN supports fp8 conv
+    // FP8 convolution is not supported by oneDNN until v3.9
+    return weight;
+  }
   auto w_dims = weight.sizes().vec();
   auto strides = stride.vec();
   auto padding_l = padding.vec();
@@ -581,11 +591,13 @@ at::Tensor _qconv_prepack_onednn(
   ideep::dims dims_iohw, dims_giohw;
   ideep::tag w_tag = ideep::tag::any;
   const bool with_groups = groups > 1;
+  auto w_dnnl_dtype = at::native::get_mkldnn_dtype(weight.scalar_type());
+  auto x_dnnl_dtype = is_fp8 ? dnnl::memory::data_type::f8_e4m3 : dnnl::memory::data_type::u8;
   w_desc = ideep::convolution_forward::expected_weights_desc(
-      w_dims, dnnl::memory::data_type::s8,
+      w_dims, w_dnnl_dtype,
       strides, padding_l, padding_r, dilates, groups,
       dnnl::algorithm::convolution_direct, dnnl::prop_kind::forward_inference,
-      dnnl::memory::data_type::u8, x_dims, op_attr, /*is_channels_last=*/true);
+      x_dnnl_dtype, x_dims, op_attr, /*is_channels_last=*/true);
 
   // Note: Weight in Conv1D will unsqueeze into Conv2D in previous step
   weight_copy = weight.clone(c10::MemoryFormat::Contiguous);
@@ -598,7 +610,7 @@ at::Tensor _qconv_prepack_onednn(
   ideep::dims wei_dims = with_groups ? ideep::utils::group_dims(w_desc.get_dims(), groups)
                                   : w_desc.get_dims();
   ideep::tensor wgt = ideep::tensor(
-      ideep::tensor::desc({wei_dims, dnnl::memory::data_type::s8, w_tag}, groups),
+      ideep::tensor::desc({wei_dims, w_dnnl_dtype, w_tag}, groups),
       weight_copy.data_ptr());
 
   wgt.set_scale(weights_scales); // Scales are needed for feed_from().
@@ -634,7 +646,7 @@ class QConvPackWeightInt8 final {
     torch::List<int64_t> output_padding;
     output_padding.reserve(kSpatialDim);
     for ([[maybe_unused]] const auto idx : c10::irange(kSpatialDim)) {
-      output_padding.push_back((int64_t)0);
+      output_padding.push_back(0);
     }
     return _run(weight, bias, stride, padding, output_padding, dilation, groups,
                 /*transpose=*/false);
