@@ -471,9 +471,8 @@ CREATE_UNARY_UFUNC(tanh)
 CREATE_UNARY_UFUNC(trunc)
 CREATE_UNARY_UFUNC(conj_physical)
 
-C10_DIAGNOSTIC_PUSH_AND_IGNORED_IF_DEFINED("-Wunused-function")
-static CREATE_UNARY_UFUNC(relu)
-C10_DIAGNOSTIC_POP()
+CREATE_UNARY_UFUNC_FUNCTIONAL(relu)
+CREATE_UNARY_UFUNC_INPLACE(relu)
 
 // With addition of `round.decimals` overload, using CREATE_UNARY_UFUNC leads
 // to unresolved overload.
@@ -531,7 +530,7 @@ static void addmm_out_sparse_csr_native_cpu(
     const Tensor& dense,
     const Tensor& r,
     Scalar alpha,
-    Scalar beta) {
+    const Scalar& beta) {
   auto dim_i = sparse.size(0);
   auto dim_k = dense.size(1);
 
@@ -1330,18 +1329,18 @@ Tensor reduce_sparse_csr_cpu_template(const Tensor& sparse, IntArrayRef dims_to_
 
 template <typename scalar_t>
 struct ReductionAddOp {
-  inline scalar_t operator()(const scalar_t& a, const scalar_t& b) const {
+  scalar_t operator()(const scalar_t& a, const scalar_t& b) const {
     return a + b;
   }
-  inline scalar_t identity() const { return 0; }
+  scalar_t identity() const { return 0; }
 };
 
 template <typename scalar_t>
 struct ReductionMulOp {
-  inline scalar_t operator()(const scalar_t& a, const scalar_t& b) const {
+  scalar_t operator()(const scalar_t& a, const scalar_t& b) const {
     return a * b;
   }
-  inline scalar_t identity() const { return 1; }
+  scalar_t identity() const { return 1; }
 };
 
 }  // namespace
