@@ -614,8 +614,8 @@ at::Tensor preprocess_mask(
 // This causes the kernel to maybe alias query, key, value
 // So instead we pad the head_dimensions to be a multiple of 8 in the composite
 // region
-template <int alignment_size, bool slice>
-at::Tensor pad_last_dim(const at::Tensor& attn_bias) {
+template <bool slice>
+at::Tensor pad_last_dim(const at::Tensor& attn_bias, int alignment_size) {
   auto last_dim_size = attn_bias.sym_size(-1);
   if (last_dim_size % alignment_size == 0) {
     return attn_bias;
@@ -743,11 +743,13 @@ Tensor scaled_dot_product_attention(
       return std::get<0>(out_lse_softmax);
     }
     case SDPBackend::flash_attention: {
-      if(query_device_type == DeviceType::CUDA){
+      if(query_device_type == DeviceType::CUDA ||
+         query_device_type == DeviceType::XPU) {
         c10::SymInt og_size = query_.sym_size(-1);
-        Tensor query_padded = pad_last_dim<8, false>(query_);
-        Tensor key_padded = pad_last_dim<8, false>(key);
-        Tensor value_padded = pad_last_dim<8, false>(value);
+        int alignment_size = (query_device_type == DeviceType::XPU) ? 64 : 8;
+        Tensor query_padded = pad_last_dim<false>(query_, alignment_size);
+        Tensor key_padded = pad_last_dim<false>(key, alignment_size);
+        Tensor value_padded = pad_last_dim<false>(value, alignment_size);
         // We need to calculate the scale based off the OG head dim size
         auto og_scale = sdp::calculate_scale(query_, scale);
         auto out_lse_softmax = at::_scaled_dot_product_flash_attention(
@@ -774,6 +776,18 @@ Tensor scaled_dot_product_attention(
     }
     case SDPBackend::math: {
 #ifdef USE_MPS
+      TORCH_CHECK_NOT_IMPLEMENTED(
+        c10::isFloatingType(query_.scalar_type()),
+        "scaled_dot_product_attention for MPS does not support dtype ",
+        query_.scalar_type());
+      TORCH_CHECK_NOT_IMPLEMENTED(
+        c10::isFloatingType(key.scalar_type()),
+        "scaled_dot_product_attention for MPS does not support dtype ",
+        key.scalar_type());
+      TORCH_CHECK_NOT_IMPLEMENTED(
+        c10::isFloatingType(value.scalar_type()),
+        "scaled_dot_product_attention for MPS does not support dtype ",
+        value.scalar_type());
       const auto any_nested = query_.is_nested() || key.is_nested() || value.is_nested();
       const bool any_inputs_require_grad = query_.requires_grad() || key.requires_grad() || value.requires_grad();
       const auto all_contiguous = query_.is_contiguous_or_false() && key.is_contiguous_or_false() && value.is_contiguous_or_false();
