@@ -10,13 +10,13 @@ import pandas as pd
 
 flaky_models = {
     "yolov3",
-    "gluon_inception_v3",
     "detectron2_maskrcnn_r_101_c4",
-    "timm_efficientnet",  # see https://github.com/pytorch/pytorch/issues/148699
     "XGLMForCausalLM",  # discovered in https://github.com/pytorch/pytorch/pull/128148
     "moondream",  # discovered in https://github.com/pytorch/pytorch/pull/159291
     # discovered in https://github.com/pytorch/pytorch/issues/161419. Its not flaky but really hard to repro, so skipping it
     "mobilenetv3_large_100",
+    # https://github.com/pytorch/pytorch/issues/163670
+    "vision_maskrcnn",
 }
 
 
@@ -34,44 +34,20 @@ def check_accuracy(actual_csv, expected_csv, expected_filename):
     if "rocm" in expected_filename:
         flaky_models.update(
             {
-                "alexnet",
-                "cait_m36_384",
-                "demucs",
-                "densenet121",
-                "detectron2_fcos_r_50_fpn",
-                "doctr_det_predictor",
-                "doctr_reco_predictor",
-                "hf_BigBird",
-                "hf_Longformer",
-                "hf_Reformer",
-                "hf_Roberta_base",
-                "hf_T5",
-                "hf_T5_base",
-                "levit_128",
-                "llava",
-                "microbench_unbacked_tolist_sum",
+                "Background_Matting",
                 "mnasnet1_0",
-                "mobilenet_v2",
-                "pytorch_CycleGAN_and_pix2pix",
-                "pytorch_stargan",
+                "llava",
+                "repvgg_a2",
                 "resnet152",
                 "resnet18",
                 "resnet50",
-                "resnext50_32x4d",
-                "sam",
-                "sam_fast",
-                "shufflenet_v2_x1_0",
-                "squeezenet1_1",
-                "stable_diffusion_text_encoder",
                 "stable_diffusion_unet",
-                "timm_efficientdet",
-                "timm_efficientnet",
-                "timm_nfnet",
-                "timm_regnet",
-                "timm_resnest",
-                "timm_vovnet",
                 "torchrec_dlrm",
+                "shufflenet_v2_x1_0",
                 "vgg16",
+                "BERT_pytorch",
+                # LLM
+                "google/gemma-2-2b",
             }
         )
 
