@@ -7,6 +7,7 @@ import os
 import re
 import subprocess
 import sys
+import types
 import warnings
 
 
@@ -58,7 +59,6 @@ imports = [
     "BigBirdConfig",
     "BlenderbotForConditionalGeneration",
     "BlenderbotModel",
-    "BlenderbotSmallForConditionalGeneration",
     "BlenderbotSmallModel",
     "CLIPModel",
     "CLIPVisionModel",
@@ -72,7 +72,6 @@ imports = [
     "MarianForCausalLM",
     "MarianModel",
     "MarianMTModel",
-    "PegasusForConditionalGeneration",
     "PegasusModel",
     "ReformerConfig",
     "ViTForImageClassification",
@@ -125,7 +124,13 @@ with open(MODELS_FILENAME) as fh:
             continue
         batch_size = int(batch_size)
         BATCH_SIZE_KNOWN_MODELS[model_name] = batch_size
-assert len(BATCH_SIZE_KNOWN_MODELS)
+assert BATCH_SIZE_KNOWN_MODELS
+
+
+try:
+    from .huggingface_llm_models import HF_LLM_MODELS
+except ImportError:
+    from huggingface_llm_models import HF_LLM_MODELS
 
 
 def get_module_cls_by_model_name(model_cls_name):
@@ -160,7 +165,7 @@ def get_sequence_length(model_cls, model_name):
             "Bert",
             "Roberta",
         )
-    ) or model_name in ("DistillGPT2", "GoogleFnet", "YituTechConvBert", "CamemBert"):
+    ) or model_name in ("DistillGPT2", "GoogleFnet", "YituTechConvBert"):
         seq_length = 512
     elif model_name in ("TrOCRForCausalLM"):
         seq_length = 256
@@ -215,9 +220,7 @@ def generate_inputs_for_model(
         BlenderbotModel,
         BlenderbotSmallModel,
         BlenderbotForConditionalGeneration,
-        BlenderbotSmallForConditionalGeneration,
         PegasusModel,
-        PegasusForConditionalGeneration,
         MarianModel,
         MarianMTModel,
     ]:
@@ -326,10 +329,6 @@ EXTRA_MODELS = {
         AutoConfig.from_pretrained("YituTech/conv-bert-base"),
         AutoModelForMaskedLM,
     ),
-    "CamemBert": (
-        AutoConfig.from_pretrained("camembert-base"),
-        AutoModelForMaskedLM,
-    ),
 }
 
 
@@ -368,8 +367,6 @@ class HuggingfaceRunner(BenchmarkRunner):
 
     def use_larger_multiplier_for_smaller_tensor(self, name):
         return name in [
-            "ElectraForQuestionAnswering",
-            "MegatronBertForQuestionAnswering",
             "GPT2ForSequenceClassification",
         ]
 
@@ -418,11 +415,8 @@ class HuggingfaceRunner(BenchmarkRunner):
         use_eval_mode = self.args.use_eval_mode
         dtype = torch.float32
         reset_rng_state()
-        model_cls, config = self._get_model_cls_and_config(model_name)
-        model = self._download_model(model_name)
-        model = model.to(device, dtype=dtype)
-        if self.args.enable_activation_checkpointing:
-            model.gradient_checkpointing_enable()
+
+        # Get batch size
         if model_name in BATCH_SIZE_KNOWN_MODELS:
             batch_size_default = BATCH_SIZE_KNOWN_MODELS[model_name]
         elif batch_size is None:
@@ -440,14 +434,46 @@ class HuggingfaceRunner(BenchmarkRunner):
                     f"Running smaller batch size={batch_size} for {model_name}, orig batch_size={batch_size_default}"  # noqa: G004
                 )
 
-        example_inputs = generate_inputs_for_model(
-            model_cls, model, model_name, batch_size, device, include_loss_args=True
-        )
+        # Get model and example inputs
+        if model_name in HF_LLM_MODELS:
+            benchmark_cls = HF_LLM_MODELS[model_name]
+            model, example_inputs = benchmark_cls.get_model_and_inputs(
+                model_name, device
+            )
 
-        # So we can check for correct gradients without eliminating the dropout computation
-        for attr in dir(config):
-            if "drop" in attr and isinstance(getattr(config, attr), float):
-                setattr(config, attr, 1e-30)
+            # Set this flag so that when we test for speedup, we use
+            # model.generate instead of using model.forward
+            self.hf_llm = True
+
+            def generate(self, _, example_inputs, collect_outputs=True):
+                return model.generate(**example_inputs)
+
+            self.generate = types.MethodType(generate, self)
+
+        else:
+            self.hf_llm = False
+
+            model_cls, config = self._get_model_cls_and_config(model_name)
+            model = self._download_model(model_name)
+            model = model.to(device, dtype=dtype)
+
+            example_inputs = generate_inputs_for_model(
+                model_cls, model, model_name, batch_size, device, include_loss_args=True
+            )
+
+            # So we can check for correct gradients without eliminating the dropout computation
+            for attr in dir(config):
+                if "drop" in attr and isinstance(getattr(config, attr), float):
+                    setattr(config, attr, 1e-30)
+
+            # Turning off kv cache for torchbench models. This is not the right
+            # thing to do, but the pt2 dashboard is outdated. Real transformers
+            # benchmarks will be added soon using a different infra.
+            if hasattr(model, "config") and hasattr(model.config, "use_cache"):
+                model.config.use_cache = False
+
+        if self.args.enable_activation_checkpointing:
+            model.gradient_checkpointing_enable()
 
         if (
             is_training
@@ -460,12 +486,6 @@ class HuggingfaceRunner(BenchmarkRunner):
         else:
             model.eval()
 
-        # Turning off kv cache for torchbench models. This is not the right
-        # thing to do, but the pt2 dashboard is outdated. Real transformers
-        # benchmarks will be added soon using a different infra.
-        if hasattr(model, "config") and hasattr(model.config, "use_cache"):
-            model.config.use_cache = False
-
         self.validate_model(model, example_inputs)
         return device, model_name, model, example_inputs, batch_size
 
@@ -530,7 +550,8 @@ class HuggingfaceRunner(BenchmarkRunner):
 
     def forward_pass(self, mod, inputs, collect_outputs=True):
         with self.autocast(**self.autocast_arg):
-            return mod(**inputs)
+            res = mod(**inputs)
+        return res.logits if self.hf_llm else res
 
     def forward_and_backward_pass(self, mod, inputs, collect_outputs=True):
         cloned_inputs = clone_inputs(inputs)
