@@ -52,6 +52,20 @@ class BatchedBinaryOpBenchmark(op_bench.TorchBenchmarkBase):
     def forward(self, batch1, batch2):
         return self.op_func(batch1, batch2)
 
+    def get_memory_traffic_bytes(self):
+        """Override for bmm: (B, M, N) @ (B, N, K) -> (B, M, K)
+        Memory traffic: read(B*M*N + B*N*K) + write(B*M*K)
+        """
+        batch1 = self.inputs["batch1"]
+        batch2 = self.inputs["batch2"]
+        B, M, N = batch1.shape
+        B_check, N_check, K = batch2.shape
+        assert B == B_check and N == N_check, "Batch dimensions must match for bmm"
+
+        bytes_per_element = batch1.element_size()
+        total_elements = B * (M * N + N * K + M * K)
+        return total_elements * bytes_per_element
+
 
 op_bench.generate_pt_tests_from_op_list(
     batched_binary_ops,
@@ -90,6 +104,25 @@ class BatchedTernaryOpBenchmark(op_bench.TorchBenchmarkBase):
     def forward(self, input_, batch1, batch2):
         return self.op_func(input_, batch1, batch2)
 
+    def get_memory_traffic_bytes(self):
+        """Override for baddbmm: input + (batch1 @ batch2) -> (B, M, K)
+        Memory traffic: read(B*M*K + B*M*N + B*N*K) + write(B*M*K)
+        """
+        input_ = self.inputs["input_"]
+        batch1 = self.inputs["batch1"]
+        batch2 = self.inputs["batch2"]
+        B, M, K = input_.shape
+        B_check1, M_check, N = batch1.shape
+        B_check2, N_check, K_check = batch2.shape
+        assert B == B_check1 == B_check2, "Batch dimensions must match"
+        assert M == M_check and K == K_check and N == N_check, (
+            "Matrix dimensions must match"
+        )
+
+        bytes_per_element = input_.element_size()
+        total_elements = B * (M * K + M * N + N * K + M * K)
+        return total_elements * bytes_per_element
+
 
 op_bench.generate_pt_tests_from_op_list(
     batched_ternary_ops,
