@@ -2,7 +2,6 @@
 
 #include <c10/cuda/CUDADeviceAssertionHost.h>
 #include <c10/util/Exception.h>
-#include <cuda_runtime.h>
 
 #include <string>
 
@@ -10,9 +9,9 @@ namespace c10::cuda {
 
 void c10_cuda_check_implementation(
     const int32_t err,
-    const char* /*filename*/,
-    const char* /*function_name*/,
-    const int /*line_number*/,
+    const char* filename,
+    const char* function_name,
+    const uint32_t line_number,
     const bool include_device_assertions) {
   const auto cuda_error = static_cast<cudaError_t>(err);
   const auto cuda_kernel_failure = include_device_assertions
@@ -28,7 +27,9 @@ void c10_cuda_check_implementation(
   std::string check_message;
 #ifndef STRIP_ERROR_MESSAGES
   check_message.append("CUDA error: ");
-  check_message.append(cudaGetErrorString(cuda_error));
+  const char* error_string = cudaGetErrorString(cuda_error);
+  check_message.append(error_string);
+  check_message.append(c10::cuda::get_cuda_error_help(cuda_error));
   check_message.append(c10::cuda::get_cuda_check_suffix());
   check_message.append("\n");
   if (include_device_assertions) {
@@ -39,7 +40,7 @@ void c10_cuda_check_implementation(
   }
 #endif
   throw c10::AcceleratorError(
-      {__func__, __FILE__, int32_t(__LINE__)}, err, check_message);
+      {function_name, filename, line_number}, err, check_message);
 }
 
 } // namespace c10::cuda
