@@ -435,7 +435,7 @@ TEST(DispatchKeySet, TestFunctionalityDispatchKeyToString) {
     if (i > 0) {
       ASSERT_TRUE(res.find("Unknown") == std::string::npos)
           << i << " (before is " << toString(static_cast<DispatchKey>(i - 1))
-          << ")";
+          << ')';
     } else {
       ASSERT_TRUE(res.find("Unknown") == std::string::npos) << i;
     }
@@ -443,3 +443,13 @@ TEST(DispatchKeySet, TestFunctionalityDispatchKeyToString) {
     seen_strings.insert(res);
   }
 }
+
+TEST(DispatchKeySet, TestGetRuntimeDispatchKeySet) {
+  // Check if getRuntimeDispatchKeySet and runtimeDispatchKeySetHas agree.
+  for (auto dk1 : DispatchKeySet(DispatchKeySet::FULL)) {
+    auto dks = getRuntimeDispatchKeySet(dk1);
+    for (auto dk2 : DispatchKeySet(DispatchKeySet::FULL)) {
+      ASSERT_EQ(dks.has(dk2), runtimeDispatchKeySetHas(dk1, dk2));
+    }
+  }
+}
