@@ -27,6 +27,10 @@
 #include "caffe2/serialize/versions.h"
 #include "miniz.h"
 
+#ifdef _WIN32
+#include <Windows.h>
+#endif // _WIN32
+
 namespace caffe2 {
 namespace serialize {
 constexpr std::string_view kDebugPklSuffix(".debug_pkl");
@@ -711,21 +715,35 @@ void PyTorchStreamWriter::setup(const string& file_name) {
   if (archive_name_.size() == 0) {
     CAFFE_THROW("invalid file name: ", file_name);
   }
-  if (!writer_func_) {
-    file_stream_.open(
-        file_name,
-        std::ofstream::out | std::ofstream::trunc | std::ofstream::binary);
-    valid("opening archive ", file_name.c_str());
 
-    const std::string dir_name = parentdir(file_name);
-    if (!dir_name.empty()) {
-      struct stat st;
-      bool dir_exists =
-          (stat(dir_name.c_str(), &st) == 0 && (st.st_mode & S_IFDIR));
-      TORCH_CHECK(
-          dir_exists, "Parent directory ", dir_name, " does not exist.");
+  const std::string dir_name = parentdir(file_name);
+  if (!dir_name.empty()) {
+    struct stat st;
+    bool dir_exists =
+        (stat(dir_name.c_str(), &st) == 0 && (st.st_mode & S_IFDIR));
+    TORCH_CHECK(
+        dir_exists, "Parent directory ", dir_name, " does not exist.");
+  }
+  TORCH_CHECK(file_stream_, "File ", file_name, " cannot be opened.");
+
+  if (!writer_func_) {
+    valid("opening archive ", file_name.c_str());
+    try {
+      file_stream_.exceptions(std::ios_base::failbit | std::ios_base::badbit);
+      file_stream_.open(
+          file_name,
+          std::ofstream::out | std::ofstream::trunc | std::ofstream::binary
+        );
+    } catch (const std::ios_base::failure&) {
+#ifdef _WIN32
+      // Windows have verbose error code, we prefer to use it than std errno.
+      uint32_t error_code = GetLastError();
+      CAFFE_THROW("open file failed with error code: ", error_code);
+#else // !_WIN32
+      CAFFE_THROW("open file failed with strerror: ", strerror(errno));
+#endif // _WIN32
     }
-    TORCH_CHECK(file_stream_, "File ", file_name, " cannot be opened.");
+
     writer_func_ = [this](const void* buf, size_t nbytes) -> size_t {
       if (!buf) {
         // See [Note: write_record_metadata]
@@ -755,8 +773,20 @@ void PyTorchStreamWriter::writeRecord(
     bool compress) {
   AT_ASSERT(!finalized_);
   AT_ASSERT(!archive_name_plus_slash_.empty());
-  TORCH_INTERNAL_ASSERT(
-      files_written_.count(name) == 0, "Tried to serialize file twice: ", name);
+  if (files_written_.count(name) > 0) {
+    // Allow multiple writes for triton binaries
+    bool is_triton_extension =
+        c10::ends_with(name, ".so") ||
+        c10::ends_with(name, ".cubin") ||
+        c10::ends_with(name, ".hsaco");
+
+    if (is_triton_extension) {
+      LOG(WARNING) << "File '" << name << "' is being serialized multiple times";
+      return;
+    }
+
+    TORCH_INTERNAL_ASSERT(false, "Tried to serialize file twice: ", name);
+  }
   if (name == kSerializationIdRecordName && serialization_id_.empty()) {
     // In case of copying records from another file, skip writing a different
     // serialization_id than the one computed in this writer.
