@@ -23,7 +23,10 @@ from torch.distributed.checkpoint import (
 )
 from torch.distributed.checkpoint._extension import ZStandard
 from torch.distributed.checkpoint.default_planner import DefaultSavePlanner
-from torch.testing._internal.common_distributed import requires_nccl, skip_if_lt_x_gpu
+from torch.testing._internal.common_distributed import (
+    requires_accelerator_dist_backend,
+    skip_if_lt_x_gpu,
+)
 from torch.testing._internal.common_utils import (
     instantiate_parametrized_tests,
     parametrize,
@@ -45,6 +48,9 @@ from torch.testing._internal.distributed.checkpoint_utils import (
 )
 
 
+device_type = acc.type if (acc := torch.accelerator.current_accelerator()) else "cpu"
+
+
 if TEST_WITH_DEV_DBG_ASAN:
     print(
         "Skip dev-asan as torch + multiprocessing spawn have known issues",
@@ -166,7 +172,7 @@ class TestDistributedStateDictSaveLoadWithSharedTensor(ShardedTensorTestBase):
 
     @with_comms(init_rpc=False)
     @skip_if_lt_x_gpu(2)
-    @requires_nccl()
+    @requires_accelerator_dist_backend()
     @parametrize("extensions", [None, [Rot13Example()], [ZStandard()]])
     def test_read_write_shard_tensor(self, extensions) -> None:
         paths = [tempfile.mkdtemp()]
@@ -178,8 +184,8 @@ class TestDistributedStateDictSaveLoadWithSharedTensor(ShardedTensorTestBase):
         spec = ChunkShardingSpec(
             dim=0,
             placements=[
-                "rank:0/cuda:0",
-                "rank:1/cuda:1",
+                f"rank:0/{device_type}:0",
+                f"rank:1/{device_type}:1",
             ],
         )
 
@@ -228,14 +234,16 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
 
     def load_tensor(self, tensor: ShardedTensor) -> torch.Tensor:
         res = (
-            torch.zeros(tensor.shape, device="cuda:0") if dist.get_rank() == 0 else None
+            torch.zeros(tensor.shape, device=f"{device_type}:0")
+            if dist.get_rank() == 0
+            else None
         )
         tensor.gather(out=res)
         return res
 
     @with_comms(init_rpc=False)
     @skip_if_lt_x_gpu(2)
-    @requires_nccl()
+    @requires_accelerator_dist_backend()
     def test_load_with_different_shard_plan(self) -> None:
         path = self.get_file_path()
 
@@ -247,18 +255,18 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
             ChunkShardingSpec(
                 dim=0,
                 placements=[
-                    "rank:0/cuda:0",
-                    "rank:1/cuda:1",
+                    f"rank:0/{device_type}:0",
+                    f"rank:1/{device_type}:1",
                 ],
             ),
             # pyre-fixme [28]: Unexpected keyword argument `dim` to call `dist._sharding_spec.api.ChunkShardingSpec.__init__`.
             ChunkShardingSpec(
                 dim=0,
                 placements=[
-                    "rank:0/cuda:0",
-                    "rank:1/cuda:1",
-                    "rank:1/cuda:1",
-                    "rank:0/cuda:0",
+                    f"rank:0/{device_type}:0",
+                    f"rank:1/{device_type}:1",
+                    f"rank:1/{device_type}:1",
+                    f"rank:0/{device_type}:0",
                 ],
             ),
             # This requires the tensors to be [10, 20]
@@ -267,27 +275,27 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
                     ShardMetadata(
                         shard_offsets=[0, 0],
                         shard_sizes=[2, 20],
-                        placement="rank:0/cuda:0",
+                        placement=f"rank:0/{device_type}:0",
                     ),
                     ShardMetadata(
                         shard_offsets=[2, 0],
                         shard_sizes=[1, 20],
-                        placement="rank:1/cuda:1",
+                        placement=f"rank:1/{device_type}:1",
                     ),
                     ShardMetadata(
                         shard_offsets=[3, 0],
                         shard_sizes=[3, 20],
-                        placement="rank:0/cuda:0",
+                        placement=f"rank:0/{device_type}:0",
                     ),
                     ShardMetadata(
                         shard_offsets=[6, 0],
                         shard_sizes=[3, 20],
-                        placement="rank:1/cuda:1",
+                        placement=f"rank:1/{device_type}:1",
                     ),
                     ShardMetadata(
                         shard_offsets=[9, 0],
                         shard_sizes=[1, 20],
-                        placement="rank:0/cuda:0",
+                        placement=f"rank:0/{device_type}:0",
                     ),
                 ]
             ),
@@ -297,12 +305,12 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
                     ShardMetadata(
                         shard_offsets=[0, 0],
                         shard_sizes=[8, 20],
-                        placement="rank:1/cuda:1",
+                        placement=f"rank:1/{device_type}:1",
                     ),
                     ShardMetadata(
                         shard_offsets=[8, 0],
                         shard_sizes=[2, 20],
-                        placement="rank:0/cuda:0",
+                        placement=f"rank:0/{device_type}:0",
                     ),
                 ]
             ),
@@ -350,7 +358,7 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
 
     @with_comms(init_rpc=False)
     @skip_if_lt_x_gpu(2)
-    @requires_nccl()
+    @requires_accelerator_dist_backend()
     def test_load_rowwise_to_colwise(self) -> None:
         path = self.get_file_path()
         self.assertEqual(self.world_size, dist.get_world_size())
@@ -359,8 +367,8 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
         src_spec = ChunkShardingSpec(
             dim=0,
             placements=[
-                "rank:0/cuda:0",
-                "rank:1/cuda:1",
+                f"rank:0/{device_type}:0",
+                f"rank:1/{device_type}:1",
             ],
         )
 
@@ -368,8 +376,8 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
         dst_spec = ChunkShardingSpec(
             dim=1,
             placements=[
-                "rank:0/cuda:0",
-                "rank:1/cuda:1",
+                f"rank:0/{device_type}:0",
+                f"rank:1/{device_type}:1",
             ],
         )
 
@@ -377,14 +385,14 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
             shutil.rmtree(path, ignore_errors=True)
             os.makedirs(path)
 
-        model_to_save = MyShardedModel3(src_spec).cuda(dist.get_rank())
+        model_to_save = MyShardedModel3(src_spec).to(dist.get_rank())
         model_to_save._register_state_dict_hook(state_dict_hook)
         state_dict_to_save = model_to_save.state_dict()
 
         fs_writer = FileSystemWriter(path=path)
         save_state_dict(state_dict=state_dict_to_save, storage_writer=fs_writer)
 
-        model_to_load = MyShardedModel3(dst_spec).cuda(dist.get_rank())
+        model_to_load = MyShardedModel3(dst_spec).to(dist.get_rank())
         model_to_load._register_state_dict_hook(state_dict_hook)
         state_dict_to_load_to = model_to_load.state_dict()
 
@@ -401,7 +409,7 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
 
     @with_comms(init_rpc=False)
     @skip_if_lt_x_gpu(2)
-    @requires_nccl()
+    @requires_accelerator_dist_backend()
     def test_save_load_bytes(self) -> None:
         path = self.get_file_path()
 
@@ -420,7 +428,7 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
 
     @with_comms(init_rpc=False)
     @skip_if_lt_x_gpu(2)
-    @requires_nccl()
+    @requires_accelerator_dist_backend()
     def test_switch_between_sharded_tensor_to_tensor(self) -> None:
         path = self.get_file_path()
         tensor_size = 32
@@ -429,17 +437,17 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
             ChunkShardingSpec(
                 dim=0,
                 placements=[
-                    "rank:0/cuda:0",
-                    "rank:1/cuda:1",
+                    f"rank:0/{device_type}:0",
+                    f"rank:1/{device_type}:1",
                 ],
             ),
             ChunkShardingSpec(
                 dim=0,
                 placements=[
-                    "rank:0/cuda:0",
-                    "rank:1/cuda:1",
-                    "rank:1/cuda:1",
-                    "rank:0/cuda:0",
+                    f"rank:0/{device_type}:0",
+                    f"rank:1/{device_type}:1",
+                    f"rank:1/{device_type}:1",
+                    f"rank:0/{device_type}:0",
                 ],
             ),
             EnumerableShardingSpec(
@@ -447,12 +455,12 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
                     ShardMetadata(
                         shard_offsets=[0],
                         shard_sizes=[8],
-                        placement="rank:1/cuda:1",
+                        placement=f"rank:1/{device_type}:1",
                     ),
                     ShardMetadata(
                         shard_offsets=[8],
                         shard_sizes=[tensor_size - 8],
-                        placement="rank:0/cuda:0",
+                        placement=f"rank:0/{device_type}:0",
                     ),
                 ]
             ),
@@ -461,12 +469,12 @@ class TestDistributedReshardOnLoad(ShardedTensorTestBase):
                     ShardMetadata(
                         shard_offsets=[0],
                         shard_sizes=[10],
-                        placement="rank:0/cuda:0",
+                        placement=f"rank:0/{device_type}:0",
                     ),
                     ShardMetadata(
                         shard_offsets=[10],
                         shard_sizes=[tensor_size - 10],
-                        placement="rank:1/cuda:1",
+                        placement=f"rank:1/{device_type}:1",
                     ),
                 ]
             ),
@@ -512,15 +520,15 @@ class TestDistributedStateDictSaveLoadWithCaching(ShardedTensorTestBase):
 
     @with_comms(init_rpc=False)
     @skip_if_lt_x_gpu(2)
-    @requires_nccl()
+    @requires_accelerator_dist_backend()
     @with_temp_dir
     def test_read_write_shard_tensor(self) -> None:
         # pyre-fixme [28]: Unexpected keyword argument `dim` to call `dist._sharding_spec.api.ChunkShardingSpec.__init__`.
         spec = ChunkShardingSpec(
             dim=0,
             placements=[
-                "rank:0/cuda:0",
-                "rank:1/cuda:1",
+                f"rank:0/{device_type}:0",
+                f"rank:1/{device_type}:1",
             ],
         )
 
