@@ -2,8 +2,9 @@
 
 import shutil
 import tempfile
+from collections.abc import Callable
 from functools import wraps
-from typing import Any, Callable, Optional
+from typing import Any, Optional
 
 import torch
 import torch.distributed as dist
