@@ -1,20 +1,39 @@
 # Owner(s): ["oncall: distributed"]
 
 import dataclasses
+import os
+import tempfile
+import unittest
+from datetime import timedelta
 
 import torch
 import torch.distributed as dist
+from torch.distributed._shard.sharded_tensor import (
+    init_from_local_shards,
+    Shard as ShardedTensorShard,
+    ShardedTensor,
+    ShardMetadata,
+)
 from torch.distributed._tensor import DTensor
-from torch.distributed._tensor.placement_types import Shard
+from torch.distributed._tensor.placement_types import Replicate, Shard
 from torch.distributed.checkpoint._state_dict_stager import StateDictStager
-from torch.testing._internal.common_distributed import requires_nccl, skip_if_lt_x_gpu
-from torch.testing._internal.common_utils import requires_cuda, run_tests, TestCase
+from torch.distributed.checkpoint.staging import _ReplicationStager
+from torch.distributed.tensor import DeviceMesh, distribute_tensor
+from torch.testing._internal.common_distributed import (
+    HAS_ACCELERATOR,
+    requires_accelerator_dist_backend,
+    skip_if_lt_x_gpu,
+)
+from torch.testing._internal.common_utils import run_tests, TestCase
 from torch.testing._internal.distributed._tensor.common_dtensor import (
     DTensorTestBase,
     with_comms,
 )
 
 
+device_type = acc.type if (acc := torch.accelerator.current_accelerator()) else "cpu"
+
+
 def create_cpu_state_dict(state_dict):
     cpu_state_dict = {}
     for key, value in state_dict.items():
@@ -22,16 +41,16 @@ def create_cpu_state_dict(state_dict):
     return cpu_state_dict
 
 
-def compare_state_dicts(cuda_state_dict, cpu_state_dict, rtol=1e-5, atol=1e-8):
+def compare_state_dicts(gpu_state_dict, cpu_state_dict, rtol=1e-5, atol=1e-8):
     """
-    Compare if two state dictionaries (one on CUDA, one on CPU) are otherwise the same.
+    Compare if two state dictionaries (one on GPU, one on CPU) are otherwise the same.
 
     This function checks if the tensors in both state dictionaries have the same values,
     shapes, dtypes, etc., ignoring the device difference. It also checks if tensors that
     share storage in one state dict also share storage in the other.
 
     Args:
-        cuda_state_dict: The state dictionary with tensors on CUDA
+        gpu_state_dict: The state dictionary with tensors on GPU
         cpu_state_dict: The state dictionary with tensors on CPU
         rtol: Relative tolerance for comparing tensor values
         atol: Absolute tolerance for comparing tensor values
@@ -41,65 +60,65 @@ def compare_state_dicts(cuda_state_dict, cpu_state_dict, rtol=1e-5, atol=1e-8):
         str: Error message if the state dictionaries are not equivalent, empty string otherwise
     """
     # Track storage data pointers to check storage sharing
-    cuda_storage_ptrs = {}
+    gpu_storage_ptrs = {}
     cpu_storage_ptrs = {}
 
-    def compare_objects(cuda_obj, cpu_obj, path=""):
+    def compare_objects(gpu_obj, cpu_obj, path=""):
         # If objects are tensors, compare them
-        if isinstance(cuda_obj, torch.Tensor) and isinstance(cpu_obj, torch.Tensor):
+        if isinstance(gpu_obj, torch.Tensor) and isinstance(cpu_obj, torch.Tensor):
             # Check if devices are as expected
-            if cuda_obj.device.type != "cuda":
+            if gpu_obj.device.type != device_type:
                 return (
                     False,
-                    f"Expected CUDA tensor, got {cuda_obj.device.type} tensor at {path}",
+                    f"Expected accelerator tensor, got {gpu_obj.device.type} tensor at {path}",
                 )
             if cpu_obj.device.type != "cpu":
                 return (
                     False,
                     f"Expected CPU tensor, got {cpu_obj.device.type} tensor at {path}",
                 )
-            if cuda_obj.storage_offset() != cpu_obj.storage_offset():
+            if gpu_obj.storage_offset() != cpu_obj.storage_offset():
                 return (
                     False,
-                    f"Storage offset mismatch at {path}: {cuda_obj.storage_offset()} vs {cpu_obj.storage_offset()}",
+                    f"Storage offset mismatch at {path}: {gpu_obj.storage_offset()} vs {cpu_obj.storage_offset()}",
                 )
 
-            if not torch.equal(cuda_obj.cpu(), cpu_obj):
+            if not torch.equal(gpu_obj.cpu(), cpu_obj):
                 return (
                     False,
                     f"Tensors are not same at {path}",
                 )
 
             # Track storage sharing
-            cuda_storage_ptr = cuda_obj.storage().data_ptr()
+            gpu_storage_ptr = gpu_obj.storage().data_ptr()
             cpu_storage_ptr = cpu_obj.storage().data_ptr()
 
-            if cuda_storage_ptr in cuda_storage_ptrs:
-                # This CUDA tensor shares storage with another tensor
+            if gpu_storage_ptr in gpu_storage_ptrs:
+                # This GPU tensor shares storage with another tensor
                 # Check if the corresponding CPU tensors also share storage
-                if cpu_storage_ptr != cuda_storage_ptrs[cuda_storage_ptr]:
+                if cpu_storage_ptr != gpu_storage_ptrs[gpu_storage_ptr]:
                     return (
                         False,
-                        f"Storage sharing mismatch: CUDA tensors share storage but CPU tensors don't at {path}",
+                        f"Storage sharing mismatch: GPU tensors share storage but CPU tensors don't at {path}",
                     )
             else:
                 # First time seeing this storage
-                cuda_storage_ptrs[cuda_storage_ptr] = cpu_storage_ptr
-                cpu_storage_ptrs[cpu_storage_ptr] = cuda_storage_ptr
+                gpu_storage_ptrs[gpu_storage_ptr] = cpu_storage_ptr
+                cpu_storage_ptrs[cpu_storage_ptr] = gpu_storage_ptr
 
             return True, ""
 
         # If objects are dictionaries, compare them recursively
-        elif isinstance(cuda_obj, dict) and isinstance(cpu_obj, dict):
-            if cuda_obj.keys() != cpu_obj.keys():
+        elif isinstance(gpu_obj, dict) and isinstance(cpu_obj, dict):
+            if gpu_obj.keys() != cpu_obj.keys():
                 return (
                     False,
-                    f"Dictionary keys mismatch at {path}: {cuda_obj.keys()} vs {cpu_obj.keys()}",
+                    f"Dictionary keys mismatch at {path}: {gpu_obj.keys()} vs {cpu_obj.keys()}",
                 )
 
-            for key in cuda_obj:
+            for key in gpu_obj:
                 result, error = compare_objects(
-                    cuda_obj[key], cpu_obj[key], f"{path}.{key}" if path else key
+                    gpu_obj[key], cpu_obj[key], f"{path}.{key}" if path else key
                 )
                 if not result:
                     return False, error
@@ -107,37 +126,37 @@ def compare_state_dicts(cuda_state_dict, cpu_state_dict, rtol=1e-5, atol=1e-8):
             return True, ""
 
         # If objects are lists, tuples, or sets, compare them recursively
-        elif isinstance(cuda_obj, (list, tuple, set)) and isinstance(
+        elif isinstance(gpu_obj, (list, tuple, set)) and isinstance(
             cpu_obj, (list, tuple, set)
         ):
-            if len(cuda_obj) != len(cpu_obj):
+            if len(gpu_obj) != len(cpu_obj):
                 return (
                     False,
-                    f"Collection length mismatch at {path}: {len(cuda_obj)} vs {len(cpu_obj)}",
+                    f"Collection length mismatch at {path}: {len(gpu_obj)} vs {len(cpu_obj)}",
                 )
-            if type(cuda_obj) != type(cpu_obj):
+            if type(gpu_obj) is not type(cpu_obj):
                 return (
                     False,
-                    f"Collection type mismatch at {path}: {type(cuda_obj)} vs {type(cpu_obj)}",
+                    f"Collection type mismatch at {path}: {type(gpu_obj)} vs {type(cpu_obj)}",
                 )
 
-            for i, (cuda_item, cpu_item) in enumerate(zip(cuda_obj, cpu_obj)):
-                result, error = compare_objects(cuda_item, cpu_item, f"{path}[{i}]")
+            for i, (gpu_item, cpu_item) in enumerate(zip(gpu_obj, cpu_obj)):
+                result, error = compare_objects(gpu_item, cpu_item, f"{path}[{i}]")
                 if not result:
                     return False, error
 
             return True, ""
 
         # If objects are custom classes, compare their attributes
-        elif hasattr(cuda_obj, "__dict__") and hasattr(cpu_obj, "__dict__"):
-            if type(cuda_obj) != type(cpu_obj):
+        elif hasattr(gpu_obj, "__dict__") and hasattr(cpu_obj, "__dict__"):
+            if type(gpu_obj) is not type(cpu_obj):
                 return (
                     False,
-                    f"Object type mismatch at {path}: {type(cuda_obj)} vs {type(cpu_obj)}",
+                    f"Object type mismatch at {path}: {type(gpu_obj)} vs {type(cpu_obj)}",
                 )
 
             result, error = compare_objects(
-                cuda_obj.__dict__, cpu_obj.__dict__, f"{path}.__dict__"
+                gpu_obj.__dict__, cpu_obj.__dict__, f"{path}.__dict__"
             )
             if not result:
                 return False, error
@@ -146,18 +165,18 @@ def compare_state_dicts(cuda_state_dict, cpu_state_dict, rtol=1e-5, atol=1e-8):
 
         # For other types, use direct equality comparison
         else:
-            if type(cuda_obj) != type(cpu_obj):
+            if type(gpu_obj) is not type(cpu_obj):
                 return (
                     False,
-                    f"Type mismatch at {path}: {type(cuda_obj)} vs {type(cpu_obj)}",
+                    f"Type mismatch at {path}: {type(gpu_obj)} vs {type(cpu_obj)}",
                 )
-            if cuda_obj != cpu_obj:
-                return False, f"Value mismatch at {path}: {cuda_obj} vs {cpu_obj}"
+            if gpu_obj != cpu_obj:
+                return False, f"Value mismatch at {path}: {gpu_obj} vs {cpu_obj}"
 
             return True, ""
 
     # Start the recursive comparison
-    result, error = compare_objects(cuda_state_dict, cpu_state_dict)
+    result, error = compare_objects(gpu_state_dict, cpu_state_dict)
     return result, error
 
 
@@ -187,7 +206,7 @@ class FrozenDataClass:
 
 
 class TestStateDictStager(TestCase):
-    @requires_cuda
+    @unittest.skipIf(not HAS_ACCELERATOR, "No accelerator")
     def test_views(self):
         test_configs = [
             (False, False),  # pin_memory=False, share_memory=False,
@@ -197,9 +216,9 @@ class TestStateDictStager(TestCase):
         ]
         for pin_memory, share_memory in test_configs:
             with self.subTest(pin_memory=pin_memory, share_memory=share_memory):
-                tensor1 = torch.randn(4, 4).cuda()
+                tensor1 = torch.randn(4, 4).to(device_type)
                 tensor2 = tensor1.view(16)
-                tensor3 = torch.randn(4, 4).cuda()
+                tensor3 = torch.randn(4, 4).to(device_type)
                 state_dict = {
                     "tensor1": tensor1,
                     "tensor2": tensor2,
@@ -242,7 +261,7 @@ class TestStateDictStager(TestCase):
                 assert num_bytes == expected_bytes, (
                     f"Expected {expected_bytes} bytes, got {num_bytes}"
                 )
-                # Verify that the CPU state dict is equivalent to the original CUDA state dict
+                # Verify that the CPU state dict is equivalent to the original GPU state dict
                 result, error = compare_state_dicts(state_dict, cpu_state_dict)
                 assert result, f"State dicts are not equivalent: {error}"
 
@@ -262,7 +281,7 @@ class TestStateDictStager(TestCase):
                     == recursive["type"].tensor1.storage().data_ptr()
                 )
 
-    @requires_cuda
+    @unittest.skipIf(not HAS_ACCELERATOR, "No accelerator")
     def test_caching(self):
         """
         Test that the StateDictStager correctly caches and reuses storages.
@@ -276,9 +295,9 @@ class TestStateDictStager(TestCase):
         for pin_memory, share_memory in test_configs:
             with self.subTest(pin_memory=pin_memory, share_memory=share_memory):
                 # Create test tensors and state dict
-                tensor1 = torch.randn(4, 4).cuda()
+                tensor1 = torch.randn(4, 4).to(device_type)
                 tensor2 = tensor1.view(16)
-                tensor3 = torch.randn(4, 4).cuda()
+                tensor3 = torch.randn(4, 4).to(device_type)
                 state_dict = {
                     "tensor1": tensor1,
                     "tensor2": tensor2,
@@ -354,14 +373,14 @@ class TestStateDictStager(TestCase):
                     "Updated values should be reflected in the cached state dict"
                 )
 
-    @requires_cuda
+    @unittest.skipIf(not HAS_ACCELERATOR, "No accelerator")
     def test_tensor_attrs(self):
         """
         Test that tensor attributes are preserved during stage with StateDictStager.
         """
-        tensor1 = torch.randn(4, 4).cuda()
+        tensor1 = torch.randn(4, 4).to(device_type)
         tensor2 = tensor1.view(16)
-        tensor3 = torch.randn(4, 4).cuda()
+        tensor3 = torch.randn(4, 4).to(device_type)
 
         # Add custom attributes to tensors
         tensor1.a = 42
@@ -400,18 +419,22 @@ class TestStateDictStager(TestCase):
             "Tensor attribute 'c' has incorrect value"
         )
 
-    @requires_cuda
+    @unittest.skipIf(not HAS_ACCELERATOR, "No accelerator")
     def test_different_dtypes(self):
         """
         Test that StateDictStager works correctly with tensors of different data types.
         """
         # Create tensors with different dtypes
         tensors = {
-            "float32": torch.randn(4, 4, dtype=torch.float32).cuda(),
-            "float64": torch.randn(4, 4, dtype=torch.float64).cuda(),
-            "int32": torch.randint(-100, 100, (4, 4), dtype=torch.int32).cuda(),
-            "int64": torch.randint(-100, 100, (4, 4), dtype=torch.int64).cuda(),
-            "bool": torch.randint(0, 2, (4, 4), dtype=torch.bool).cuda(),
+            "float32": torch.randn(4, 4, dtype=torch.float32).to(device_type),
+            "float64": torch.randn(4, 4, dtype=torch.float64).to(device_type),
+            "int32": torch.randint(-100, 100, (4, 4), dtype=torch.int32).to(
+                device_type
+            ),
+            "int64": torch.randint(-100, 100, (4, 4), dtype=torch.int64).to(
+                device_type
+            ),
+            "bool": torch.randint(0, 2, (4, 4), dtype=torch.bool).to(device_type),
         }
 
         # Create a state dict with these tensors
@@ -436,7 +459,7 @@ class TestStateDictStager(TestCase):
                 f"Tensor {dtype_name} has incorrect values",
             )
 
-    @requires_cuda
+    @unittest.skipIf(not HAS_ACCELERATOR, "No accelerator")
     def test_empty_tensors(self):
         """
         Test that StateDictStager works correctly with empty tensors.
@@ -451,15 +474,17 @@ class TestStateDictStager(TestCase):
             with self.subTest(pin_memory=pin_memory, share_memory=share_memory):
                 # Create empty tensors with different shapes
                 tensors = {
-                    "empty_0d": torch.tensor([], dtype=torch.float32).cuda(),
-                    "empty_1d": torch.tensor([], dtype=torch.float32).reshape(0).cuda(),
+                    "empty_0d": torch.tensor([], dtype=torch.float32).to(device_type),
+                    "empty_1d": torch.tensor([], dtype=torch.float32)
+                    .reshape(0)
+                    .to(device_type),
                     "empty_2d": torch.tensor([], dtype=torch.float32)
                     .reshape(0, 0)
-                    .cuda(),
+                    .to(device_type),
                     "empty_3d": torch.tensor([], dtype=torch.float32)
                     .reshape(0, 0, 0)
-                    .cuda(),
-                    "zero_dim": torch.tensor(0.0).cuda(),  # scalar tensor
+                    .to(device_type),
+                    "zero_dim": torch.tensor(0.0).to(device_type),  # scalar tensor
                 }
 
                 # Create a state dict with these tensors
@@ -489,13 +514,13 @@ class TestStateDictStager(TestCase):
                         f"Tensor {tensor_name} has incorrect dtype",
                     )
 
-    @requires_cuda
+    @unittest.skipIf(not HAS_ACCELERATOR, "No accelerator")
     def test_complex_storage_sharing(self):
         """
         Test that StateDictStager correctly handles complex storage sharing scenarios.
         """
         # Create a base tensor
-        base_tensor = torch.randn(10, 10).cuda()
+        base_tensor = torch.randn(10, 10).to(device_type)
 
         # Create various views and slices that share storage
         view1 = base_tensor.view(100)
@@ -571,13 +596,13 @@ class TestStateDictStager(TestCase):
             "slice3 should reflect changes to base",
         )
 
-    @requires_cuda
+    @unittest.skipIf(not HAS_ACCELERATOR, "No accelerator")
     def test_dataclasses(self):
         # Create tensors
-        tensor1 = torch.randn(4, 4).cuda()
-        tensor2 = torch.randn(8, 8).cuda()
-        tensor3 = torch.randn(2, 6).cuda()
-        tensor4 = torch.randn(3, 5).cuda()
+        tensor1 = torch.randn(4, 4).to(device_type)
+        tensor2 = torch.randn(8, 8).to(device_type)
+        tensor3 = torch.randn(2, 6).to(device_type)
+        tensor4 = torch.randn(3, 5).to(device_type)
 
         # Create dataclass instances
         nested = NestedTensorStruct(tensor=tensor3)
@@ -684,14 +709,14 @@ class TestStateDictStager(TestCase):
             "CPU tensor should have the same values as the original tensor",
         )
 
-    @requires_cuda
+    @unittest.skipIf(not HAS_ACCELERATOR, "No accelerator")
     def test_tensor_pinned_and_shared(self):
         """
         Test that verifies tensors are actually pinned and shared using tensor.is_pinned() and tensor.is_shared() methods.
         """
         # Create test tensors
-        tensor1 = torch.randn(4, 4).cuda()
-        tensor2 = torch.randn(8, 8).cuda()
+        tensor1 = torch.randn(4, 4).to(device_type)
+        tensor2 = torch.randn(8, 8).to(device_type)
 
         # Create a state dict with these tensors
         state_dict = {
@@ -786,15 +811,17 @@ class TestStateDictStager(TestCase):
 
 class TestDTensorStateDictStager(DTensorTestBase):
     @with_comms
-    @requires_nccl()
+    @requires_accelerator_dist_backend()
     @skip_if_lt_x_gpu(2)
     def test_dtensor(self):
         """
         Test that StateDictStager works correctly with DTensors.
         """
         # Create a DTensor
-        device_mesh = dist.DeviceMesh("cuda", list(range(dist.get_world_size())))
-        tensor = torch.randn(3, 3, device="cuda")
+        device_mesh = dist.DeviceMesh(
+            self.device_type, list(range(dist.get_world_size()))
+        )
+        tensor = torch.randn(3, 3, device=self.device_type)
         dtensor = DTensor.from_local(tensor, device_mesh, [Shard(0)])
 
         dtensor = dtensor + 1
@@ -815,6 +842,525 @@ class TestDTensorStateDictStager(DTensorTestBase):
             )
         )
         self.assertEqual(cpu_state_dict["dtensor"]._spec, dtensor._spec)
+        self.assertEqual(cpu_state_dict["dtensor"].size(), dtensor.size())
+
+
+class TestReplicationStager(DTensorTestBase):
+    """
+    Test suite for _ReplicationStager functionality.
+    Tests replication of state_dict across training ranks using CPU tensors only.
+    """
+
+    @property
+    def backend(self) -> str:
+        return "cpu:gloo,cuda:nccl"
+
+    def _create_simple_state_dict(self, rank: int) -> dict:
+        """
+        Create a simple state_dict with CPU tensors, deterministically unique per rank.
+
+        Args:
+            rank: The rank number to create unique tensors for
+
+        Returns:
+            dict: A state dictionary with CPU tensors
+        """
+        # Create unique tensors for each rank
+        torch.manual_seed(42 + rank)  # Different seed per rank
+
+        return {
+            "layer1.weight": torch.randn(64, 128, device="cpu"),
+            "layer1.bias": torch.randn(64, device="cpu"),
+            "layer2.weight": torch.randn(32, 64, device="cpu"),
+            "layer2.bias": torch.randn(32, device="cpu"),
+            "nested": {
+                "param": torch.randn(16, 16, device="cpu"),
+                "buffer": torch.randn(8, device="cpu"),
+            },
+            "scalar": torch.tensor(float(rank), device="cpu"),
+        }
+
+    def _verify_simple_state_dict_replication(
+        self, replicated_dict: dict, rank: int, partner_rank: int
+    ):
+        """
+        Verify that replication worked correctly.
+
+        Args:
+            replicated_dict: The replicated state_dict received from partner
+            rank: Current rank
+            partner_rank: Partner rank we should have received from
+        """
+        # Create expected state_dict (what partner rank would have created)
+        expected_dict = self._create_simple_state_dict(partner_rank)
+
+        def compare_tensors(actual, expected, path=""):
+            if isinstance(actual, dict) and isinstance(expected, dict):
+                self.assertEqual(
+                    actual.keys(), expected.keys(), f"Keys mismatch at {path}"
+                )
+                for key in actual:
+                    compare_tensors(
+                        actual[key], expected[key], f"{path}.{key}" if path else key
+                    )
+            elif isinstance(actual, torch.Tensor) and isinstance(
+                expected, torch.Tensor
+            ):
+                self.assertEqual(
+                    actual.device.type, "cpu", f"Tensor at {path} should be on CPU"
+                )
+                self.assertEqual(
+                    actual.shape, expected.shape, f"Shape mismatch at {path}"
+                )
+                self.assertEqual(
+                    actual.dtype, expected.dtype, f"Dtype mismatch at {path}"
+                )
+                self.assertTrue(
+                    torch.equal(actual, expected), f"Values mismatch at {path}"
+                )
+            else:
+                self.assertEqual(actual, expected, f"Value mismatch at {path}")
+
+        compare_tensors(replicated_dict, expected_dict)
+
+    def _create_dtensor_state_dict(self, rank: int, device_mesh: DeviceMesh) -> dict:
+        """
+        Create state_dict with DTensor and regular tensors for deterministic testing
+        due to DTensor Shard, Replicate placements.
+
+        Args:
+            rank: Current rank
+            device_mesh: DeviceMesh for DTensor creation
+
+        Returns:
+            dict: State dictionary with DTensors
+        """
+        # Create a large global tensor with deterministic values
+        # Each position contains a unique value that encodes both position and rank info
+        global_size = 128
+        global_tensor = torch.arange(0, global_size * 16, dtype=torch.float32).reshape(
+            global_size, 16
+        )
+
+        # Create DTensor with Shard(0) - each rank gets different rows
+        sharded_dtensor = distribute_tensor(global_tensor, device_mesh, [Shard(0)])
+
+        # Create DTensor with Replicate() - all ranks have the same data
+        replicated_global = torch.full(
+            (8, 8), float(global_size * 100), dtype=torch.float32, device="cpu"
+        )
+        replicated_dtensor = distribute_tensor(
+            replicated_global, device_mesh, [Replicate()]
+        )
+
+        return {
+            "sharded_param": sharded_dtensor,
+            "replicated_param": replicated_dtensor,
+            "rank_scalar": torch.tensor(float(rank), device="cpu"),
+        }
+
+    def _verify_dtensor_replication(
+        self, replicated_dict: dict, rank: int, partner_rank: int
+    ):
+        """
+        Verify DTensor replication accuracy by checking local shards and global reconstruction.
+
+        Args:
+            replicated_dict: Replicated state_dict received from partner
+            rank: Current rank
+            partner_rank: Partner rank we should have received from
+        """
+        # Verify sharded DTensor
+        if "sharded_param" in replicated_dict:
+            replicated_sharded = replicated_dict["sharded_param"]
+            self.assertIsInstance(replicated_sharded, DTensor, "Should receive DTensor")
+
+            # Get local shard from replicated DTensor
+            replicated_local = replicated_sharded.to_local()
+
+            # Create expected local shard (what partner rank would have)
+            expected_global = torch.arange(0, 128 * 16, dtype=torch.float32).reshape(
+                128, 16
+            )
+
+            # Calculate expected shard for this rank's position
+            world_size = dist.get_world_size()
+            shard_size = 128 // world_size
+            start_idx = partner_rank * shard_size
+            end_idx = (partner_rank + 1) * shard_size
+            expected_local = expected_global[start_idx:end_idx]
+
+            self.assertTrue(
+                torch.equal(replicated_local, expected_local),
+                "Sharded DTensor value mismatch",
+            )
+
+            # Verify DTensor metadata is preserved
+            self.assertEqual(
+                replicated_sharded._spec.placements[0].__class__.__name__,
+                "Shard",
+                "DTensor should maintain Shard placement",
+            )
+
+        # Verify replicated DTensor
+        if "replicated_param" in replicated_dict:
+            replicated_replicated = replicated_dict["replicated_param"]
+            self.assertIsInstance(
+                replicated_replicated, DTensor, "Should receive DTensor"
+            )
+
+            # Get local data from replicated DTensor
+            replicated_local = replicated_replicated.to_local()
+
+            # Expected value should be global_size * 100
+            expected_value = float(128 * 100)
+            expected_tensor = torch.full(
+                (8, 8), expected_value, dtype=torch.float32, device="cpu"
+            )
+
+            self.assertTrue(
+                torch.equal(replicated_local, expected_tensor),
+                "Replicated DTensor value mismatch",
+            )
+
+            # Verify DTensor metadata is preserved
+            self.assertEqual(
+                replicated_replicated._spec.placements[0].__class__.__name__,
+                "Replicate",
+                "DTensor should maintain Replicate placement",
+            )
+
+        # Verify regular tensors
+        if "rank_scalar" in replicated_dict:
+            self.assertEqual(
+                replicated_dict["rank_scalar"].item(),
+                float(partner_rank),
+                f"Rank scalar should be {partner_rank}, got {replicated_dict['rank_scalar'].item()}",
+            )
+
+    def _create_sharded_tensor_state_dict(self, rank: int, world_size: int) -> dict:
+        """
+        Create state_dict with ShardedTensor for deterministic testing.
+
+        Args:
+            rank: Current rank
+            world_size: Total world size
+
+        Returns:
+            dict: State dictionary with ShardedTensor
+        """
+        # Create deterministic local shard for this rank
+        global_size = 64
+        shard_size = global_size // world_size
+        start_idx = rank * shard_size
+        end_idx = (rank + 1) * shard_size
+
+        # Create local tensor with deterministic values
+        local_tensor = torch.arange(
+            start_idx * 8, end_idx * 8, dtype=torch.float32, device="cpu"
+        ).reshape(shard_size, 8)
+
+        # Create ShardedTensor using init_from_local_shards
+        sharded_tensor = init_from_local_shards(
+            [
+                ShardedTensorShard(
+                    tensor=local_tensor,
+                    metadata=ShardMetadata(
+                        shard_offsets=[start_idx, 0],
+                        shard_sizes=[shard_size, 8],
+                        placement=f"rank:{rank}/cpu",
+                    ),
+                )
+            ],
+            global_size,
+            8,
+        )
+
+        return {
+            "sharded_tensor": sharded_tensor,
+            "rank_scalar": torch.tensor(float(rank), device="cpu"),
+        }
+
+    def _verify_sharded_tensor_replication(
+        self, replicated_dict: dict, rank: int, partner_rank: int
+    ):
+        """
+        Verify ShardedTensor replication accuracy by checking local shards and metadata.
+
+        Args:
+            replicated_dict: Replicated state_dict received from partner
+            rank: Current rank
+            partner_rank: Partner rank we should have received from
+        """
+        # Verify sharded tensor
+        if "sharded_tensor" in replicated_dict:
+            replicated_sharded = replicated_dict["sharded_tensor"]
+            self.assertIsInstance(
+                replicated_sharded, ShardedTensor, "Should receive ShardedTensor"
+            )
+
+            # Get local shard from replicated ShardedTensor
+            local_shards = replicated_sharded.local_shards()
+            self.assertEqual(
+                len(local_shards), 1, "Should have exactly one local shard"
+            )
+
+            local_shard = local_shards[0]
+            replicated_local = local_shard.tensor
+
+            # Create expected local shard (what partner rank would have)
+            world_size = dist.get_world_size()
+            global_size = 64
+            shard_size = global_size // world_size
+            start_idx = partner_rank * shard_size
+            end_idx = (partner_rank + 1) * shard_size
+
+            expected_local = torch.arange(
+                start_idx * 8, end_idx * 8, dtype=torch.float32, device="cpu"
+            ).reshape(shard_size, 8)
+
+            self.assertTrue(
+                torch.equal(replicated_local, expected_local),
+                "Sharded tensor value mismatch",
+            )
+
+            # Verify shard metadata is preserved
+            expected_metadata = ShardMetadata(
+                shard_offsets=[start_idx, 0],
+                shard_sizes=[shard_size, 8],
+                placement=f"rank:{partner_rank}/cpu",
+            )
+            self.assertEqual(
+                local_shard.metadata.shard_offsets,
+                expected_metadata.shard_offsets,
+                "Shard offsets should match",
+            )
+            self.assertEqual(
+                local_shard.metadata.shard_sizes,
+                expected_metadata.shard_sizes,
+                "Shard sizes should match",
+            )
+
+        # Verify regular tensors
+        if "rank_scalar" in replicated_dict:
+            self.assertEqual(
+                replicated_dict["rank_scalar"].item(),
+                float(partner_rank),
+                f"Rank scalar should be {partner_rank}, got {replicated_dict['rank_scalar'].item()}",
+            )
+
+    @with_comms
+    @skip_if_lt_x_gpu(4)
+    def test_replication_basic(self):
+        """Test basic replication functionality with world_size=16"""
+        world_size = dist.get_world_size()
+
+        current_rank = dist.get_rank()
+
+        # Create unique DTensor state_dict for this rank
+        state_dict = self._create_simple_state_dict(current_rank)
+
+        # Initialize replication stager
+        stager = _ReplicationStager(
+            pg=dist.new_group(backend=dist.Backend.GLOO),
+            timeout=timedelta(seconds=30),
+            device=torch.device("cpu"),
+        )
+
+        # Perform replication
+        replicated_dict = stager.stage(state_dict)
+
+        # Calculate expected partner rank
+        partner_rank = (current_rank + world_size // 2) % world_size
+
+        # Verify DTensor replication
+        self._verify_simple_state_dict_replication(
+            replicated_dict, current_rank, partner_rank
+        )
+
+        # Clean up
+        stager.close()
+
+    @with_comms
+    @skip_if_lt_x_gpu(4)
+    def test_replication_dtensors(self):
+        """Test replication with DTensor and mixed tensor types"""
+        world_size = dist.get_world_size()
+
+        current_rank = dist.get_rank()
+
+        # Create CPU-based DeviceMesh for DTensor
+        device_mesh = DeviceMesh("cpu", list(range(world_size)))
+
+        # Create DTensor state_dict which includes different tensor types
+        state_dict = self._create_dtensor_state_dict(current_rank, device_mesh)
+
+        # Initialize replication stager
+        stager = _ReplicationStager(
+            pg=dist.group.WORLD,
+            timeout=timedelta(seconds=30),
+            device=torch.device("cpu"),
+        )
+
+        # Perform replication
+        result = stager.stage(state_dict)
+
+        # Wait for completion
+        from concurrent.futures import Future
+
+        if isinstance(result, Future):
+            replicated_dict = result.result()
+        else:
+            replicated_dict = result
+
+        # Calculate expected partner
+        partner_rank = (current_rank + world_size // 2) % world_size
+
+        # Verify all DTensor types are correctly replicated
+        self._verify_dtensor_replication(replicated_dict, current_rank, partner_rank)
+
+        # Clean up
+        stager.close()
+
+    @with_comms
+    @skip_if_lt_x_gpu(4)
+    def test_replication_sharded_tensors(self):
+        """Test replication with ShardedTensor and mixed tensor types"""
+        world_size = dist.get_world_size()
+
+        current_rank = dist.get_rank()
+
+        # Create ShardedTensor state_dict for this rank
+        state_dict = self._create_sharded_tensor_state_dict(current_rank, world_size)
+
+        # Initialize replication stager
+        stager = _ReplicationStager(
+            pg=dist.group.WORLD,
+            timeout=timedelta(seconds=30),
+            device=torch.device("cpu"),
+        )
+
+        # Perform replication
+        result = stager.stage(state_dict)
+
+        # Wait for completion
+        from concurrent.futures import Future
+
+        if isinstance(result, Future):
+            replicated_dict = result.result()
+        else:
+            replicated_dict = result
+
+        # Calculate expected partner
+        partner_rank = (current_rank + world_size // 2) % world_size
+
+        # Verify all ShardedTensor types are correctly replicated
+        self._verify_sharded_tensor_replication(
+            replicated_dict, current_rank, partner_rank
+        )
+
+        # Clean up
+        stager.close()
+
+    @with_comms
+    @skip_if_lt_x_gpu(4)
+    def test_replication_persistence(self):
+        """Test persistence functionality in _ReplicationStager"""
+        world_size = dist.get_world_size()
+
+        current_rank = dist.get_rank()
+
+        # Test 1: Default storage directory (auto-generated tempdir)
+        with tempfile.TemporaryDirectory() as _:
+            # Create state_dict for this rank
+            state_dict = self._create_simple_state_dict(current_rank)
+
+            # Initialize stager with default storage_dir (None)
+            stager = _ReplicationStager(
+                pg=dist.group.WORLD,
+                timeout=timedelta(seconds=30),
+                device=torch.device("cpu"),
+                storage_dir=None,  # Let it create its own tempdir
+            )
+
+            # Perform replication to trigger persistence
+            stager.stage(state_dict)
+
+            # Calculate expected partner rank
+            partner_rank = (current_rank + world_size // 2) % world_size
+
+            # Verify file was created with correct naming convention
+            expected_path = stager._get_persisted_path(current_rank, partner_rank)
+
+            self.assertTrue(
+                os.path.exists(expected_path),
+                f"Persisted file should exist at {expected_path}",
+            )
+
+            # Verify the storage directory was created
+            self.assertTrue(
+                os.path.isdir(stager._storage_dir), "Storage directory should exist"
+            )
+            self.assertTrue(
+                stager._storage_dir.startswith(tempfile.gettempdir()),
+                "Default storage directory should be in system temp directory",
+            )
+
+            # Load and verify the persisted state_dict matches the received one
+            loaded_state_dict = torch.load(expected_path)
+            self._verify_simple_state_dict_replication(
+                loaded_state_dict, current_rank, partner_rank
+            )
+
+            # Clean up
+            stager.close()
+
+        # Test 2: Custom storage directory
+        with tempfile.TemporaryDirectory() as custom_storage_dir:
+            # Create custom subdirectory
+            custom_subdir = os.path.join(custom_storage_dir, "custom_replication_test")
+
+            # Create state_dict for this rank
+            state_dict = self._create_simple_state_dict(current_rank)
+
+            # Initialize stager with custom storage_dir
+            stager = _ReplicationStager(
+                pg=dist.group.WORLD,
+                timeout=timedelta(seconds=30),
+                device=torch.device("cpu"),
+                storage_dir=custom_subdir,
+            )
+
+            # Perform replication to trigger persistence
+            stager.stage(state_dict)
+
+            # Verify custom storage directory was created and used
+            self.assertEqual(
+                stager._storage_dir,
+                custom_subdir,
+                "Should use custom storage directory",
+            )
+            self.assertTrue(
+                os.path.isdir(custom_subdir),
+                "Custom storage directory should be created",
+            )
+
+            # Verify file was created in custom directory
+            expected_path = stager._get_persisted_path(current_rank, partner_rank)
+
+            self.assertTrue(
+                os.path.exists(expected_path),
+                f"Persisted file should exist in custom directory at {expected_path}",
+            )
+
+            # Load and verify the persisted state_dict
+            loaded_state_dict = torch.load(expected_path)
+            self._verify_simple_state_dict_replication(
+                loaded_state_dict, current_rank, partner_rank
+            )
+
+            # Clean up
+            stager.close()
 
 
 if __name__ == "__main__":
