@@ -2,9 +2,10 @@
 
 import bisect
 import sys
+from collections.abc import Callable
 from copy import deepcopy
 from enum import auto, Enum
-from typing import Any, Callable, Optional
+from typing import Any, Optional
 
 import torch
 import torch.nn as nn
@@ -38,7 +39,6 @@ from torch.testing._internal.common_utils import (
     instantiate_parametrized_tests,
     parametrize,
     run_tests,
-    skipIfRocm,
     TEST_WITH_DEV_DBG_ASAN,
 )
 
@@ -421,7 +421,7 @@ class TestFSDPOptimState(FSDPTest):
             return False
         for state_name, value1 in state1.items():
             value2 = state2[state_name]
-            if type(value1) != type(value2):
+            if type(value1) is not type(value2):
                 return False
             if torch.is_tensor(value1):  # tensor state
                 assert torch.is_tensor(value2)
@@ -514,7 +514,6 @@ class TestFSDPOptimState(FSDPTest):
                     continue
                 self.assertEqual(full_osd_value, ref_osd_pg[name])
 
-    @skipIfRocm
     @skip_if_lt_x_gpu(2)
     @parametrize("state_dict_type", STATE_DICT_TYPES)
     @parametrize("use_multiple_param_groups", [False, True])
