@@ -15,7 +15,9 @@ from torch.distributed.tensor.parallel.style import (
 )
 from torch.testing._internal.common_utils import run_tests
 from torch.testing._internal.distributed._tensor.common_dtensor import (
+    create_local_tensor_test_class,
     DTensorTestBase,
+    map_local_tensor_for_rank,
     MLPModule,
     MLPStacked,
     with_comms,
@@ -33,7 +35,7 @@ class DummyModule(torch.nn.Module):
 class TensorParallelAPITests(DTensorTestBase):
     @property
     def world_size(self):
-        gpu_num = torch.cuda.device_count()
+        gpu_num = torch.accelerator.device_count()
         return gpu_num if gpu_num % 2 == 0 and gpu_num > 4 else 4
 
     def _compare_params(
@@ -78,7 +80,14 @@ class TensorParallelAPITests(DTensorTestBase):
 
         # check forward correctness
         local_output = local_module(inp)
-        inp = inp.chunk(self.world_size, dim=-1)[self.rank] if rowwise else inp
+        inp = map_local_tensor_for_rank(
+            inp,
+            self.rank,
+            lambda inp, rank: inp.chunk(self.world_size, dim=-1)[rank]
+            if rowwise
+            else inp,
+        )
+        # inp = inp.chunk(self.world_size, dim=-1)[self.rank] if rowwise else inp
         dist_output = dist_module(inp)
         dist_output = (
             dist_output.redistribute(dist_output.device_mesh, [Replicate()]).to_local()
@@ -332,6 +341,49 @@ class TensorParallelAPITests(DTensorTestBase):
         )
         self._compare_module(model, model_tp, inp_size, rank0_only=False)
 
+    @with_comms
+    def test_parallelize_module_with_root_module(self):
+        inp_size = [16, 10]
+        model = MLPModule(self.device_type)
+        device_mesh = DeviceMesh(self.device_type, torch.arange(self.world_size))
+
+        model_tp = deepcopy(model)
+        model_tp = parallelize_module(
+            model_tp,
+            device_mesh,
+            {
+                "": PrepareModuleInputOutput(
+                    input_layouts=Replicate(),
+                    desired_input_layouts=Shard(0),
+                    output_layouts=Shard(0),
+                    desired_output_layouts=Replicate(),
+                ),
+                "net1": ColwiseParallel(input_layouts=Shard(0)),
+                "net2": RowwiseParallel(output_layouts=Shard(0)),
+            },
+        )
+        self._compare_module(model, model_tp, inp_size, rank0_only=False)
+
+    @with_comms
+    def test_parallelize_module_with_no_match(self):
+        inp_size = [16, 10]
+        model = MLPModule(self.device_type)
+        device_mesh = DeviceMesh(self.device_type, torch.arange(self.world_size))
+
+        model_tp = deepcopy(model)
+        with self.assertWarns(UserWarning):
+            model_tp = parallelize_module(
+                model_tp,
+                device_mesh,
+                {
+                    "net0.hello.world": ColwiseParallel(),
+                    "net1": ColwiseParallel(),
+                    "net2": RowwiseParallel(),
+                    "net3": ColwiseParallel(),
+                },
+            )
+        self._compare_module(model, model_tp, inp_size, rank0_only=False)
+
     @with_comms
     def test_under_devicemesh_context(self):
         # test ColwiseParallel
@@ -357,7 +409,17 @@ class TensorParallelAPITests(DTensorTestBase):
         # Call parallelize_module with empty plan.
         # Goal is not to crash.
         device_mesh = DeviceMesh(self.device_type, list(range(self.world_size)))
-        parallelize_module(model, device_mesh)
+        with self.assertWarns(UserWarning):
+            parallelize_module(model, device_mesh)
+
+
+TensorParallelAPITestsWithLocalTensor = create_local_tensor_test_class(
+    TensorParallelAPITests,
+    skipped_tests=[
+        # Uses mesh_scatter that has local rank dependent logic
+        "test_parallelize_module_src_data_rank",
+    ],
+)
 
 
 if __name__ == "__main__":
