@@ -2,9 +2,11 @@
 # Owner(s): ["oncall: distributed"]
 
 import torch
+from torch.distributed._local_tensor import maybe_run_for_local_tensor
 from torch.distributed.tensor import DeviceMesh, DTensor, Replicate, Shard, zeros
 from torch.testing._internal.common_utils import run_tests
 from torch.testing._internal.distributed._tensor.common_dtensor import (
+    create_local_tensor_test_class,
     DTensorTestBase,
     with_comms,
 )
@@ -77,8 +79,13 @@ class DTensorConstructorTest(DTensorTestBase):
                         dim=shard_dim,
                     )
                 )
-                if self.rank < len(exp_tensor_list):
-                    eq_op(exp_tensor_list[self.rank], dist_tensor.to_local())
+
+                @maybe_run_for_local_tensor
+                def check_per_rank_chunk(rank, local_tensor):
+                    if rank < len(exp_tensor_list):
+                        eq_op(exp_tensor_list[rank], local_tensor)
+
+                check_per_rank_chunk(self.rank, dist_tensor.to_local())
             else:
                 exp_tensor = init_op(tensor_size, *args, **kwargs)
                 eq_op(exp_tensor, dist_tensor.to_local())
@@ -150,12 +157,17 @@ class DTensorConstructorTest(DTensorTestBase):
         dist_tensor = zeros(size, device_mesh=mesh, placements=placements)
         self.assertEqual(dist_tensor.size(), torch.Size(size))
         local_tensor = dist_tensor.to_local()
-        if self.rank <= 2:
-            self.assertEqual(local_tensor.size(), torch.Size([8, 3]))
-            self.assertEqual(torch.zeros(8, 3), local_tensor)
-        else:
-            self.assertEqual(local_tensor.size(), torch.Size([7, 3]))
-            self.assertEqual(torch.zeros(7, 3), local_tensor)
+
+        @maybe_run_for_local_tensor
+        def check_per_rank_tensors(rank, local_tensor):
+            if rank <= 2:
+                self.assertEqual(local_tensor.size(), torch.Size([8, 3]))
+                self.assertEqual(torch.zeros(8, 3), local_tensor)
+            else:
+                self.assertEqual(local_tensor.size(), torch.Size([7, 3]))
+                self.assertEqual(torch.zeros(7, 3), local_tensor)
+
+        check_per_rank_tensors(self.rank, local_tensor)
 
         # construct a gpu device mesh with 2d: shard, replicate
         mesh = DeviceMesh(self.device_type, torch.arange(self.world_size).reshape(2, 2))
@@ -250,5 +262,13 @@ class DTensorConstructorTest(DTensorTestBase):
             self.assertEqual(local_tensor, torch.tensor([]))
 
 
+DTensorConstructorTestWithLocalTensor = create_local_tensor_test_class(
+    DTensorConstructorTest,
+    skipped_tests=[
+        # Non-contigous sub-meshes are not supported
+        "test_zeros_submesh",
+    ],
+)
+
 if __name__ == "__main__":
     run_tests()
