@@ -14,6 +14,7 @@ from torch.distributed.tensor import (
 )
 from torch.testing._internal.common_utils import run_tests
 from torch.testing._internal.distributed._tensor.common_dtensor import (
+    create_local_tensor_test_class,
     DTensorTestBase,
     MLPModule,
     with_comms,
@@ -716,5 +717,9 @@ class TestDTensorOptimizer(DTensorTestBase):
             self._assert_optimizer(None, mod, opt, mod_copy, dist_opt, inp)
 
 
+TestDTensorOptimizerWithLocalTensor = create_local_tensor_test_class(
+    TestDTensorOptimizer,
+)
+
 if __name__ == "__main__":
     run_tests()
