@@ -1,6 +1,9 @@
 # Copyright (c) Meta Platforms, Inc. and affiliates
 # Owner(s): ["oncall: distributed"]
 
+import itertools
+import unittest
+
 import torch
 from torch.distributed.tensor import (
     DeviceMesh,
@@ -11,10 +14,12 @@ from torch.distributed.tensor import (
     Replicate,
     Shard,
 )
+from torch.distributed.tensor._sharding_prop import ShardingPropagator
 from torch.distributed.tensor.debug import CommDebugMode
 from torch.testing._internal.common_distributed import skip_if_lt_x_gpu
-from torch.testing._internal.common_utils import run_tests, skipIfRocm
+from torch.testing._internal.common_utils import MI200_ARCH, run_tests, skipIfRocmArch
 from torch.testing._internal.distributed._tensor.common_dtensor import (
+    create_local_tensor_test_class,
     DTensorConverter,
     DTensorTestBase,
     with_comms,
@@ -93,6 +98,19 @@ class DistTensorOpsTest(DTensorTestBase):
             dst_tensor.copy_(src_tensor)
             self.assertEqual(dst_dtensor.full_tensor(), dst_tensor)
 
+        # as a pointwise op, need to keep Partial placements without redistribute
+        src_tensor = torch.randn((64, 1))
+        dst_tensor = torch.zeros(16, 32, 64, 128)
+        src_specs = [[Partial()]]
+        dst_specs = [[Partial()]]
+        for dst_spec, src_spec in zip(dst_specs, src_specs):
+            src_dtensor = DTensor.from_local(src_tensor, device_mesh, src_spec)
+            dst_dtensor = DTensor.from_local(dst_tensor, device_mesh, dst_spec)
+            dst_dtensor.copy_(src_dtensor)
+            dst_tensor.copy_(src_tensor)
+            self.assertEqual(dst_dtensor.placements, (Partial(),))
+            self.assertEqual(dst_dtensor._local_tensor, dst_tensor)
+
     @with_comms
     def test_contiguous(self):
         device_mesh = self.build_device_mesh()
@@ -178,6 +196,30 @@ class DistTensorOpsTest(DTensorTestBase):
         # empty is not deterministic, so we only check that the shard propagation worked
         self.assertEqual((4, 8), empty_like_dt.to_local().shape)
 
+    @with_comms
+    def test_meta_init_partial(self):
+        device_mesh = self.build_device_mesh()
+        partial_spec = [Partial()]
+
+        class ToyModule(torch.nn.Module):
+            def __init__(self):
+                super().__init__()
+                self.register_buffer(
+                    "scalar_buffer", torch.tensor(0.0, dtype=torch.float32)
+                )
+
+        with torch.device("meta"):
+            module = ToyModule()
+            module._buffers["scalar_buffer"] = DTensor.from_local(
+                module.scalar_buffer,
+                device_mesh=device_mesh,
+                placements=partial_spec,
+            )
+        module.to_empty(device=None)
+
+        # check that to_empty preserves partial
+        self.assertEqual(module.scalar_buffer.placements, (Partial(),))
+
     @with_comms
     def test_fill_inplace(self):
         device_mesh = self.build_device_mesh()
@@ -280,8 +322,8 @@ class DistTensorOpsTest(DTensorTestBase):
         self.assertEqual(dist_tensor.dtype, torch.float32)
         self.assertEqual(zeros_like_dt.dtype, torch.bfloat16)
 
-    @with_comms
     @skip_if_lt_x_gpu(4)
+    @with_comms
     def test_stack(self):
         mesh_2d = DeviceMesh(
             self.device_type, torch.arange(self.world_size).reshape(2, 2)
@@ -318,6 +360,34 @@ class DistTensorOpsTest(DTensorTestBase):
             torch.stack([global_input, global_input], dim=1),
         )
 
+    @with_comms
+    def test_stack_cache(self):
+        device_mesh = self.build_device_mesh()
+
+        shape = (4, 8)
+        placements = [Replicate()]
+        dtensor_list = []
+        for _ in range(3):
+            local_tensor = torch.randn(shape)
+            dt = DTensor.from_local(local_tensor, device_mesh, placements)
+            dtensor_list.append(dt)
+
+        _ = torch.stack(dtensor_list)
+
+        dtensor_list2 = []
+        for _ in range(3):
+            local_tensor = torch.randn(shape)
+            dt = DTensor.from_local(local_tensor, device_mesh, placements)
+            dtensor_list2.append(dt)
+
+        def error(*args, **kwargs):
+            raise AssertionError
+
+        with unittest.mock.patch.object(
+            ShardingPropagator, "_propagate_tensor_meta_non_cached", error
+        ):
+            _ = torch.stack(dtensor_list2)
+
     @with_comms
     def test_equal(self):
         device_mesh = self.build_device_mesh()
@@ -495,7 +565,7 @@ class DistTensorOpsTest(DTensorTestBase):
         # case 2 input sharding: input sharded, index replicated, output mask partial
         # only works when index has size 1 on the gather dimension and
         # input is sharded on the gather dimension
-        from torch.distributed.tensor._ops._embedding_ops import _MaskPartial
+        from torch.distributed.tensor.placement_types import MaskPartial
 
         gather_dim = 1
         global_input = torch.randn(12, 8, 16)
@@ -506,7 +576,7 @@ class DistTensorOpsTest(DTensorTestBase):
         with comm_mode:
             output_dt = torch.gather(input_dt, gather_dim, index_dt)
             self.assertEqual(comm_mode.get_total_counts(), 0)
-        self.assertIsInstance(output_dt.placements[0], _MaskPartial)
+        self.assertIsInstance(output_dt.placements[0], MaskPartial)
         self.assertEqual(output_dt.full_tensor(), global_output)
 
         # case 3 index sharding: input replicated, index sharded, output sharded
@@ -523,7 +593,7 @@ class DistTensorOpsTest(DTensorTestBase):
             self.assertEqual(output_dt.placements, [Shard(gather_dim)])
             self.assertEqual(output_dt.full_tensor(), global_output)
 
-    @skipIfRocm
+    @skipIfRocmArch(MI200_ARCH)
     @with_comms
     def test_index(self):
         meshes = [
@@ -689,6 +759,12 @@ class DistTensorOpsTest(DTensorTestBase):
 
     @with_comms
     def test_dtensor_dtype_conversion(self):
+        from torch.distributed.tensor.debug import (
+            _clear_fast_path_sharding_prop_cache,
+            _get_fast_path_sharding_prop_cache_stats,
+        )
+
+        _clear_fast_path_sharding_prop_cache()
         device_mesh = self.build_device_mesh()
         shard_spec = [Shard(0)]
         # by default we start from bf16 dtype
@@ -707,16 +783,14 @@ class DistTensorOpsTest(DTensorTestBase):
         self.assertEqual(bf16_sharded_dtensor1.dtype, torch.bfloat16)
         self.assertEqual(bf16_sharded_dtensor1.to_local().dtype, torch.bfloat16)
 
-        from torch.distributed.tensor.debug import _get_sharding_prop_cache_info
-
         # by this point we only have cache misses
-        hits, misses, _, _ = _get_sharding_prop_cache_info()
+        hits, misses = _get_fast_path_sharding_prop_cache_stats()
         self.assertEqual(hits, 0)
         self.assertEqual(misses, 2)
 
         # convert to fp32 again and see if there's cache hit
         bf16_sharded_dtensor1.float()
-        hits, misses, _, _ = _get_sharding_prop_cache_info()
+        hits, misses = _get_fast_path_sharding_prop_cache_stats()
         # by now we should have cache hit
         self.assertEqual(hits, 1)
         self.assertEqual(misses, 2)
@@ -760,7 +834,7 @@ class DistTensorOpsTest(DTensorTestBase):
         )
 
     def _test_split_on_partial(self, reduce_op: str, split_size: int, split_dim: int):
-        torch.manual_seed(self.rank)
+        self.init_manual_seed_for_rank()
         mesh = self.build_device_mesh()
 
         partial_tensor = torch.randn(8, 8, device=self.device_type)
@@ -776,6 +850,77 @@ class DistTensorOpsTest(DTensorTestBase):
             dim=split_dim,
         )
 
+    @with_comms
+    def test_unbind(self):
+        device_mesh = self.build_device_mesh()
+        shard_dims = [0, 1]
+        unbind_dims = [0, 1]
+        local_tensor = torch.randn(4, 8, requires_grad=True)
+        for shard_dim, unbind_dim in itertools.product(shard_dims, unbind_dims):
+            dist_tensor = distribute_tensor(
+                local_tensor, device_mesh, (Shard(shard_dim),)
+            )
+
+            if shard_dim == unbind_dim:
+                with self.assertRaisesRegex(
+                    RuntimeError, "Sharding propagation failed"
+                ):
+                    dist_tensor.unbind(dim=unbind_dim)
+            else:
+                unbinded_dist_tensors = dist_tensor.unbind(dim=unbind_dim)
+                new_shard_dim = shard_dim if shard_dim < unbind_dim else shard_dim - 1
+                self.assertTrue(
+                    all(
+                        elem.placements[0].is_shard(dim=new_shard_dim)
+                        for elem in unbinded_dist_tensors
+                    )
+                )
+                for x, y in zip(
+                    unbinded_dist_tensors, local_tensor.unbind(dim=unbind_dim)
+                ):
+                    self.assertEqual(x.full_tensor(), y)
+
+
+class DistArgMaxArgMinTest(DTensorTestBase):
+    _ops = [torch.argmax, torch.argmin]
+    sample = [
+        [0, 2, 1, 11, 5, 9, -2, -23],
+        [3, 5, 7, 9, 0, -1, 4, 2],
+        [8, 4, 6, -5, -10, 12, 7, 1],
+        [13, 6, 9, -5, 0, 4, 2, 8],
+        [4, 9, 2, 1, -6, -3, 5, 7],
+        [0, -4, -2, 8, 6, 3, 12, -7],
+        [20, 6, -3, 1, -8, 4, 2, 0],
+        [5, 9, 11, -1, -4, 2, 3, 8],
+    ]
+    placements_tuples = (
+        [Partial(), Shard(1)],
+        [Partial(), Shard(0)],
+        [Shard(0), Shard(1)],
+        [Replicate(), Shard(0)],
+        [Replicate(), Shard(1)],
+    )
+
+    @skip_if_lt_x_gpu(4)
+    @with_comms
+    def test_argmax_argmin_with_placements(self):
+        device_mesh = self.build_device_mesh()
+        local_tensor = torch.tensor(self.sample, device=self.device_type)
+        for placements in self.placements_tuples:
+            dtensor_input = distribute_tensor(local_tensor, device_mesh, placements)
+            for op in self._ops:
+                d_result = op(dtensor_input, dim=1)
+                full_dresult = d_result.full_tensor()
+                local_result = op(local_tensor, dim=1)
+                self.assertEqual(full_dresult, local_result)
+
+    def build_device_mesh(self):
+        return init_device_mesh(self.device_type, (2, 2))
+
+
+DistTensorOpsTestWithLocalTensor = create_local_tensor_test_class(
+    DistTensorOpsTest,
+)
 
 if __name__ == "__main__":
     run_tests()
