@@ -25,7 +25,7 @@ import torch.distributed as dist
 from torch.testing._internal.common_cuda import TEST_MULTIGPU
 from torch.testing._internal.common_distributed import (
     init_multigpu_helper,
-    MultiProcContinousTest,
+    MultiProcContinuousTest,
     requires_nccl,
     requires_nccl_version,
     sm_is_or_higher_than,
@@ -33,7 +33,6 @@ from torch.testing._internal.common_distributed import (
 from torch.testing._internal.common_utils import (
     run_tests,
     skip_but_pass_in_sandcastle_if,
-    skipIfRocm,
     TEST_WITH_DEV_DBG_ASAN,
 )
 
@@ -45,7 +44,7 @@ if TEST_WITH_DEV_DBG_ASAN:
     sys.exit(0)
 
 
-class ProcessGroupNCCLOpTest(MultiProcContinousTest):
+class ProcessGroupNCCLOpTest(MultiProcContinuousTest):
     @classmethod
     def backend_str(cls) -> str:
         return "nccl"
@@ -279,7 +278,7 @@ class ProcessGroupNCCLOpTest(MultiProcContinousTest):
                 tmp.append(torch.rand(10 ** (3 + i), device=local_device))
             race_tensors.append(tmp)
 
-        for i in range(10):
+        for _ in range(10):
             race_tensors.pop()
             work = pg.alltoall_base(output, input, [], [], opts)
             # this triggers cudaFree
@@ -319,7 +318,6 @@ class ProcessGroupNCCLOpTest(MultiProcContinousTest):
 
     @requires_nccl()
     @skip_but_pass_in_sandcastle_if(not TEST_MULTIGPU, "NCCL test requires 2+ GPUs")
-    @skipIfRocm()
     def test_nccl_watchdog_cudagraph(self):
         # test that the watchdog does not crash graphs with disallowed event query
         pg = self.pg
@@ -936,6 +934,25 @@ class ProcessGroupNCCLOpTest(MultiProcContinousTest):
         )
         torch.testing.assert_close(output_tensor, expected)
 
+    @skip_but_pass_in_sandcastle_if(not TEST_MULTIGPU, "NCCL test requires 2+ GPUs")
+    def test_reduce_scatter_bfloat16(self):
+        device = torch.device("cuda", self.rank_to_GPU[self.rank][0])
+
+        numel = 1024
+        output_tensor = torch.zeros(numel, dtype=torch.float32, device=device).to(
+            torch.bfloat16
+        )
+        input_tensor = torch.ones(
+            self.world_size * numel, dtype=torch.float32, device=device
+        ).to(torch.bfloat16)
+        # currently only reduce_scatter_tensor supports bfloat16
+        dist.reduce_scatter_tensor(output_tensor, input_tensor)
+
+        expected = (
+            torch.empty_like(output_tensor).fill_(self.world_size).to(torch.bfloat16)
+        )
+        torch.testing.assert_close(output_tensor, expected)
+
     @requires_nccl()
     @skip_but_pass_in_sandcastle_if(not TEST_MULTIGPU, "NCCL test requires 2+ GPUs")
     def test_barrier(self):
