@@ -4,6 +4,16 @@
 
 import sys
 
+# Import TEST_WITH_ROCM first to check for ROCm before importing NVSHMEM modules
+from torch.testing._internal.common_utils import TEST_WITH_ROCM
+
+
+# Skip entire module on ROCm before importing NVSHMEM-specific modules
+# NVSHMEM is NVIDIA-specific and can cause crashes during import on ROCm
+if TEST_WITH_ROCM:
+    print("NVSHMEM not available on ROCm, skipping tests")
+    sys.exit(0)
+
 import triton.language as tl
 
 import torch
@@ -61,8 +71,13 @@ def my_get_kernel(
     src,
     nelems,
     pe,
+    nbi: tl.constexpr,  # use nonblocking interface if True
 ):
-    nvshmem.get(dest, src, nelems, pe)
+    if nbi:
+        nvshmem.get_nbi(dest, src, nelems, pe)
+        nvshmem.quiet()
+    else:
+        nvshmem.get(dest, src, nelems, pe)
 
 
 @requires_nvshmem
@@ -327,7 +342,8 @@ class NVSHMEMTritonTest(MultiProcContinuousTest):
     @skipIfRocm
     @requires_triton()
     @requires_h100()
-    def test_triton_get(self) -> None:
+    @parametrize("nbi", [False, True])  # Test both blocking and nonblocking interfaces
+    def test_triton_get(self, nbi: bool) -> None:
         torch.manual_seed(42 + self.rank)
         self._init_device()
 
@@ -357,6 +373,7 @@ class NVSHMEMTritonTest(MultiProcContinuousTest):
                 inp,
                 numel,
                 peer,
+                nbi=nbi,
             )
         if rank == 1:
             torch.testing.assert_close(
@@ -397,6 +414,7 @@ class NVSHMEMTritonTest(MultiProcContinuousTest):
             inp,
             numel,
             peer,
+            nbi=False,
         )
 
         expected_value = peer
@@ -1128,9 +1146,8 @@ class NVSHMEMTritonTest(MultiProcContinuousTest):
         vals[0, ::2] = 1
         vals[0, 1::2] = 2
         vals[1] = 1
-        vals2 = vals[2].view(-1, 2, 2)
-        vals2[:, 0] = 1
-        vals2[:, 1] = 2
+        for rank in range(world_size):
+            vals[2, rank] = 1 if (rank // 2) % 2 == 0 else 2
         expected = vals.prod(-1).tolist()
 
         # Synchronize before reduction
