@@ -47,6 +47,11 @@ class TestModules(__TestCase):
             self.assertEqual(getattr(c_heapq, fname).__module__, '_heapq')
 
 
+@torch._dynamo.disable
+def randrange(*args):
+    return random.randrange(*args)
+
+
 class _TestHeap:
 
     def test_push_pop(self):
@@ -55,7 +60,8 @@ class _TestHeap:
         data = []
         self.check_invariant(heap)
         for i in range(256):
-            item = random.random()
+            with torch._dynamo.error_on_graph_break(False):
+                item = random.random()
             data.append(item)
             self.module.heappush(heap, item)
             self.check_invariant(heap)
@@ -86,14 +92,16 @@ class _TestHeap:
 
     def test_heapify(self):
         for size in list(range(30)) + [20000]:
-            heap = [random.random() for dummy in range(size)]
+            with torch._dynamo.error_on_graph_break(False):
+                heap = [random.random() for dummy in range(size)]
             self.module.heapify(heap)
             self.check_invariant(heap)
 
         self.assertRaises(TypeError, self.module.heapify, None)
 
     def test_naive_nbest(self):
-        data = [random.randrange(2000) for i in range(1000)]
+        with torch._dynamo.error_on_graph_break(False):
+            data = [randrange(2000) for i in range(1000)]
         heap = []
         for item in data:
             self.module.heappush(heap, item)
@@ -116,7 +124,8 @@ class _TestHeap:
         # heap instead of a min heap, it could go faster still via
         # heapify'ing all of data (linear time), then doing 10 heappops
         # (10 log-time steps).
-        data = [random.randrange(2000) for i in range(1000)]
+        with torch._dynamo.error_on_graph_break(False):
+            data = [randrange(2000) for i in range(1000)]
         heap = data[:10]
         self.module.heapify(heap)
         for item in data[10:]:
@@ -129,7 +138,8 @@ class _TestHeap:
         self.assertRaises(IndexError, self.module.heapreplace, [], None)
 
     def test_nbest_with_pushpop(self):
-        data = [random.randrange(2000) for i in range(1000)]
+        with torch._dynamo.error_on_graph_break(False):
+            data = [randrange(2000) for i in range(1000)]
         heap = data[:10]
         self.module.heapify(heap)
         for item in data[10:]:
@@ -166,8 +176,9 @@ class _TestHeap:
     def test_heapsort(self):
         # Exercise everything with repeated heapsort checks
         for trial in range(100):
-            size = random.randrange(50)
-            data = [random.randrange(25) for i in range(size)]
+            with torch._dynamo.error_on_graph_break(False):
+                size = randrange(50)
+                data = [randrange(25) for i in range(size)]
             if trial & 1:     # Half of the time, use heapify
                 heap = data[:]
                 self.module.heapify(heap)
@@ -180,12 +191,13 @@ class _TestHeap:
 
     def test_merge(self):
         inputs = []
-        for i in range(random.randrange(25)):
-            row = []
-            for j in range(random.randrange(100)):
-                tup = random.choice('ABC'), random.randrange(-500, 500)
-                row.append(tup)
-            inputs.append(row)
+        with torch._dynamo.error_on_graph_break(False):
+            for i in range(randrange(25)):
+                row = []
+                for j in range(randrange(100)):
+                    tup = random.choice('ABC'), randrange(-500, 500)
+                    row.append(tup)
+                inputs.append(row)
 
         for key in [None, itemgetter(0), itemgetter(1), itemgetter(1, 0)]:
             for reverse in [False, True]:
@@ -212,12 +224,14 @@ class _TestHeap:
             list(self.module.merge(iterable(), iterable()))
 
     def test_merge_stability(self):
-        class Int(int):
-            pass
+        with torch._dynamo.error_on_graph_break(False):
+            class Int(int):
+                pass
         inputs = [[], [], [], []]
         for i in range(20000):
-            stream = random.randrange(4)
-            x = random.randrange(500)
+            with torch._dynamo.error_on_graph_break(False):
+                stream = randrange(4)
+                x = randrange(500)
             obj = Int(x)
             obj.pair = (x, stream)
             inputs[stream].append(obj)
@@ -227,7 +241,8 @@ class _TestHeap:
         self.assertEqual(result, sorted(result))
 
     def test_nsmallest(self):
-        data = [(random.randrange(2000), i) for i in range(1000)]
+        with torch._dynamo.error_on_graph_break(False):
+            data = [(randrange(2000), i) for i in range(1000)]
         for f in (None, lambda x:  x[0] * 547 % 2000):
             for n in (0, 1, 2, 10, 100, 400, 999, 1000, 1100):
                 self.assertEqual(list(self.module.nsmallest(n, data)),
@@ -236,7 +251,8 @@ class _TestHeap:
                                  sorted(data, key=f)[:n])
 
     def test_nlargest(self):
-        data = [(random.randrange(2000), i) for i in range(1000)]
+        with torch._dynamo.error_on_graph_break(False):
+            data = [(randrange(2000), i) for i in range(1000)]
         for f in (None, lambda x:  x[0] * 547 % 2000):
             for n in (0, 1, 2, 10, 100, 400, 999, 1000, 1100):
                 self.assertEqual(list(self.module.nlargest(n, data)),
@@ -251,17 +267,18 @@ class _TestHeap:
             data = [comp(x) for x in data]
             self.module.heapify(data)
             return [self.module.heappop(data).x for i in range(len(data))]
-        class LT:
-            def __init__(self, x):
-                self.x = x
-            def __lt__(self, other):
-                return self.x > other.x
-        class LE:
-            def __init__(self, x):
-                self.x = x
-            def __le__(self, other):
-                return self.x >= other.x
-        data = [random.random() for i in range(100)]
+        with torch._dynamo.error_on_graph_break(False):
+            class LT:
+                def __init__(self, x):
+                    self.x = x
+                def __lt__(self, other):
+                    return self.x > other.x
+            class LE:
+                def __init__(self, x):
+                    self.x = x
+                def __le__(self, other):
+                    return self.x >= other.x
+            data = [random.random() for i in range(100)]
         target = sorted(data, reverse=True)
         self.assertEqual(hsort(data, LT), target)
         self.assertRaises(TypeError, data, LE)
@@ -438,10 +455,11 @@ class _TestErrorHandling:
     def test_comparison_operator_modifiying_heap(self):
         # See bpo-39421: Strong references need to be taken
         # when comparing objects as they can alter the heap
-        class EvilClass(int):
-            def __lt__(self, o):
-                heap.clear()
-                return NotImplemented
+        with torch._dynamo.error_on_graph_break(False):
+            class EvilClass(int):
+                def __lt__(self, o):
+                    heap.clear()
+                    return NotImplemented
 
         heap = []
         self.module.heappush(heap, EvilClass(0))
@@ -449,15 +467,16 @@ class _TestErrorHandling:
 
     def test_comparison_operator_modifiying_heap_two_heaps(self):
 
-        class h(int):
-            def __lt__(self, o):
-                list2.clear()
-                return NotImplemented
+        with torch._dynamo.error_on_graph_break(False):
+            class h(int):
+                def __lt__(self, o):
+                    list2.clear()
+                    return NotImplemented
 
-        class g(int):
-            def __lt__(self, o):
-                list1.clear()
-                return NotImplemented
+            class g(int):
+                def __lt__(self, o):
+                    list1.clear()
+                    return NotImplemented
 
         list1, list2 = [], []
 
