@@ -80,7 +80,7 @@ def fn():
         self.assertEqual(fn.__code__.co_lnotab, result[1].co_lnotab)
 
     @unittest.skipIf(
-        sys.version_info < (3, 10) or sys.version_info >= (3, 11),
+        sys.version_info >= (3, 11),
         "linetable test for Python 3.10",
     )
     def test_linetable_310_writer(self):
@@ -95,19 +95,6 @@ def fn():
         result = bytecode_transformation.assemble(inst, fn.__code__.co_firstlineno)
         self.assertTrue(result[1] == fn.__code__.co_linetable)
 
-    @unittest.skipIf(sys.version_info >= (3, 10), "use lnotab when python < 3.10")
-    def test_lnotab_writer(self):
-        def fn():
-            a = 10
-            b = 20
-            c = a + b
-            f = "lnotab_writer"
-            return f"Test if {f} generates correct co_lnotab: {c}"
-
-        inst = dis.get_instructions(fn)
-        result = bytecode_transformation.assemble(inst, fn.__code__.co_firstlineno)
-        self.assertTrue(result[1] == fn.__code__.co_lnotab)
-
     def test_if_tensor_is_none(self):
         """
         Python 3.11 adds new jump instructions that check if
@@ -500,6 +487,7 @@ def fn():
                 self.assertIn("JUMP", i1.opname)
                 self.assertIs(i1.target, insts[-1])
 
+    @unittest.skipIf(sys.version_info >= (3, 14), "3.14+ removed RETURN_CONST")
     @skipIfNotPy312
     def test_bytecode_from_template_noreturn_const(self):
         # Test 3.12+ RETURN_CONST
@@ -548,7 +536,9 @@ def fn():
     def test_extended_args_starts_line(self):
         # NOTE: need to LOAD_CONST i before LOAD_FAST x
         # in order to get an EXTENDED_ARG with starts_line set
-        lines = "\n".join(f"    x = {i} + x" for i in range(300))
+        # NOTE: 3.14+ introduced LOAD_SMALL_INT, so integers need to be >= 256
+        # in order for LOAD_CONST to be generated
+        lines = "\n".join(f"    x = {i + 1000} + x" for i in range(300))
         fn_str = f"def fn(x):\n{lines}"
         locals = {}
         exec(fn_str, {}, locals)
