@@ -12,7 +12,11 @@ from torch._C import (
     _push_on_torch_function_stack,
 )
 from torch._dynamo.utils import counters
-from torch.overrides import _get_current_function_mode_stack, BaseTorchFunctionMode
+from torch.overrides import (
+    _get_current_function_mode_stack,
+    BaseTorchFunctionMode,
+    TorchFunctionMode,
+)
 from torch.testing._internal.common_utils import skipIfXpu
 from torch.testing._internal.inductor_utils import GPU_TYPE
 from torch.testing._internal.triton_utils import requires_gpu
@@ -66,7 +70,7 @@ class TorchDispatchModeTests(torch._dynamo.test_case.TestCase):
         counters.clear()
         from torch.utils._python_dispatch import TorchDispatchMode
 
-        @torch.library.custom_op("mylib::foo", mutates_args=())
+        @torch.library.custom_op("mylib::modes_checksum", mutates_args=())
         def foo(x: torch.Tensor) -> torch.Tensor:
             return x.clone()
 
@@ -86,7 +90,7 @@ class TorchDispatchModeTests(torch._dynamo.test_case.TestCase):
             def __torch_dispatch__(self, func, types, args, kwargs=None):
                 kwargs = kwargs or {}
 
-                if func is torch.ops.mylib.foo.default:
+                if func is torch.ops.mylib.modes_checksum.default:
                     # Do some compute, smoketest to see if there's a bad interaction
                     _checksums.append(args[0].abs().sum())
 
@@ -134,12 +138,20 @@ class TorchDispatchModeTests(torch._dynamo.test_case.TestCase):
 class TorchFunctionModeTests(torch._dynamo.test_case.TestCase):
     @classmethod
     def setUpClass(cls):
-        cls.default_device_old = torch.get_default_device()
+        try:
+            cls.default_device_old = torch.get_default_device()
+        except AttributeError:
+            cls.default_device_old = torch.device("cpu")
+        global_default_ctx = getattr(
+            getattr(torch, "_GLOBAL_DEVICE_CONTEXT", None), "device_context", None
+        )
+        cls._had_global_default_device = global_default_ctx is not None
         super().setUpClass()
 
     @classmethod
     def tearDownClass(cls):
-        torch.set_default_device(cls.default_device_old)
+        if cls._had_global_default_device:
+            torch.set_default_device(cls.default_device_old)
         super().tearDownClass()
 
     def setUp(self):
@@ -190,6 +202,19 @@ class TorchFunctionModeTests(torch._dynamo.test_case.TestCase):
     def test_torch_function_mode_guards_cpp(self):
         self._run_torch_function_mode_guard_test()
 
+    @requires_gpu
+    def test_torch_function_mode_preserves_cuda_rng_state(self):
+        class ConstantReturnMode(TorchFunctionMode):
+            def __torch_function__(self, func, types, args=(), kwargs=None):
+                return -42
+
+        @torch._dynamo.optimize("eager")
+        def fn():
+            with ConstantReturnMode():
+                return 123
+
+        self.assertEqual(fn(), 123)
+
     def test_stack_state_mutation_default_device(self):
         m = BaseTorchFunctionMode()
         m1 = BaseTorchFunctionMode()
@@ -774,6 +799,23 @@ class TorchFunctionModeTests(torch._dynamo.test_case.TestCase):
                     )
 
 
+class TorchFunctionModeLifecycleTests(torch._dynamo.test_case.TestCase):
+    def test_default_device_restored_after_mode_tests(self):
+        case = TorchFunctionModeTests("test_stack_state_mutation_default_device")
+        TorchFunctionModeTests.setUpClass()
+        try:
+            case.setUp()
+            try:
+                case.test_stack_state_mutation_default_device()
+            finally:
+                case.tearDown()
+        finally:
+            TorchFunctionModeTests.tearDownClass()
+
+        stack = _get_current_function_mode_stack()
+        self.assertFalse(any(isinstance(mode, DeviceContext) for mode in stack))
+
+
 if __name__ == "__main__":
     from torch._dynamo.test_case import run_tests
 
