@@ -1,16 +1,9 @@
 # Owner(s): ["module: dynamo"]
-
-import pickle
-
 import torch
 import torch._dynamo
 import torch._dynamo.test_case
 import torch._functorch
-from torch._dynamo.precompile_context import (
-    EditablePrecompileCacheArtifact,
-    PrecompileCacheArtifact,
-    PrecompileContext,
-)
+from torch._dynamo.precompile_context import BackendCacheArtifact, PrecompileContext
 from torch._functorch import config as functorch_config
 from torch._functorch._aot_autograd.autograd_cache import (
     BundledAOTAutogradCacheArtifact,
@@ -47,32 +40,13 @@ class PrecompileContextTests(InductorTestCase):
         x = torch.randn(10, device=GPU_TYPE, requires_grad=True)
         result = compiled_fn(x)
         result.sum().backward()
-        self.assertEqual(len(PrecompileContext._new_cache_artifacts_by_key), 2)
-        self.assertEqual(len(PrecompileContext._new_cache_artifacts), 0)
-
-        result = PrecompileContext.serialize()
-        assert result is not None
-        serialized, cache_info = result
-        self.assertEqual(len(cache_info.precompile_aot_autograd_artifacts), 1)
-
-        artifacts = PrecompileContext.deserialize(serialized)
-        assert artifacts is not None
-        deserialized = artifacts["precompile_aot_autograd"]
-        assert len(deserialized) == 1
-        entry = deserialized[0]
-        assert isinstance(entry, BundledAOTAutogradCacheArtifact)
-        entry = entry.after_deserialization()
-        # Now that we've serialized, there should be no new cache artifacts
-        self.assertEqual(
-            len(PrecompileContext._new_cache_artifacts["precompile_aot_autograd"]), 0
-        )
+        self.assertEqual(len(PrecompileContext._dynamo_cache_entries), 1)
+        self.assertEqual(len(PrecompileContext._backend_artifacts_by_key), 1)
+        cache_entries, _ = PrecompileContext.create_cache_entries()
+        self.assertEqual(len(cache_entries), 1)
 
     @requires_triton()
     def test_serialize_by_key(self):
-        """
-        Test that after torch.compile, PrecompileContext._new_cache_artifacts length is 1
-        """
-
         def simple_function(x):
             return x.sin() + x.cos()
 
@@ -82,17 +56,16 @@ class PrecompileContextTests(InductorTestCase):
         x = torch.randn(10, device=GPU_TYPE, requires_grad=True)
         result = compiled_fn(x)
         result.sum().backward()
-        self.assertEqual(len(PrecompileContext._new_cache_artifacts_by_key), 2)
-        for key in PrecompileContext._new_cache_artifacts_by_key.keys():
+        self.assertEqual(len(PrecompileContext._dynamo_cache_entries), 1)
+        self.assertEqual(len(PrecompileContext._backend_artifacts_by_key), 1)
+        for key in PrecompileContext._backend_artifacts_by_key:
             result = PrecompileContext.serialize_artifact_by_key(key)
-            assert isinstance(result, PrecompileCacheArtifact)
+            assert isinstance(result, BackendCacheArtifact)
             self.assertEqual(result.key, key)
 
-        self.assertEqual(len(PrecompileContext._new_cache_artifacts), 0)
-        result = PrecompileContext.serialize()
-        assert result is not None
-        _, cache_info = result
-        self.assertEqual(len(cache_info.precompile_aot_autograd_artifacts), 1)
+        # This should still work
+        result, _ = PrecompileContext.create_cache_entries()
+        assert len(result) == 1
 
     @requires_triton()
     def test_editable(self):
@@ -109,13 +82,10 @@ class PrecompileContextTests(InductorTestCase):
         x = torch.randn(10, device=GPU_TYPE, requires_grad=True)
         result = compiled_fn(x)
         result.sum().backward()
-        self.assertEqual(len(PrecompileContext._new_cache_artifacts_by_key), 2)
+        self.assertEqual(len(PrecompileContext._dynamo_cache_entries), 1)
+        self.assertEqual(len(PrecompileContext._backend_artifacts_by_key), 1)
         # Find the key for the artifact of type "precompile_aot_autograd"
-        key = next(
-            k
-            for k, v in PrecompileContext._new_cache_artifacts_by_key.items()
-            if isinstance(v, EditablePrecompileCacheArtifact)
-        )
+        key = next(iter(PrecompileContext._backend_artifacts_by_key))
 
         def edit_fn(x):
             x._my_private_field = 42
@@ -127,24 +97,12 @@ class PrecompileContextTests(InductorTestCase):
         assert isinstance(result, BundledAOTAutogradCacheArtifact)
         self.assertEqual(result.key, key)
 
-        self.assertEqual(len(PrecompileContext._new_cache_artifacts), 0)
-        result = PrecompileContext.serialize()
-        assert result is not None
-        artifacts, cache_info = result
-        self.assertEqual(len(cache_info.precompile_aot_autograd_artifacts), 1)
-
-        deserialized = PrecompileContext.deserialize(artifacts)
-        assert deserialized is not None
-        aot_autograd_artifacts = deserialized["precompile_aot_autograd"]
+        result, _ = PrecompileContext.create_cache_entries()
+        assert len(result) == 1
+        aot_autograd_artifacts = next(iter(result.values())).backends
         assert len(aot_autograd_artifacts) == 1
-        entry = aot_autograd_artifacts[0]
-        assert isinstance(entry, BundledAOTAutogradCacheArtifact)
-        raw_entry = pickle.loads(entry.content)
-        self.assertEqual(raw_entry._my_private_field, 42)
-        # Now that we've serialized, there should be no new cache artifacts
-        self.assertEqual(
-            len(PrecompileContext._new_cache_artifacts["precompile_aot_autograd"]), 0
-        )
+        entry = next(iter(aot_autograd_artifacts.values())).content
+        self.assertEqual(entry._my_private_field, 42)
 
 
 if __name__ == "__main__":
