@@ -242,11 +242,12 @@ class RecompileUxTests(torch._dynamo.test_case.TestCase):
             opt_f(torch.randn(8 + i))
 
         failure_str = "\n".join(failure_reasons)
-        for line in """\
-tensor 'x' size mismatch at index 0. expected 11, actual 12
-tensor 'x' size mismatch at index 0. expected 10, actual 12
-tensor 'x' size mismatch at index 0. expected 9, actual 12
-tensor 'x' size mismatch at index 0. expected 8, actual 12""".split("\n"):
+        for line in [
+            "tensor 'x' size mismatch at index 0. expected 11, actual 12",
+            "tensor 'x' size mismatch at index 0. expected 10, actual 12",
+            "tensor 'x' size mismatch at index 0. expected 9, actual 12",
+            "tensor 'x' size mismatch at index 0. expected 8, actual 12",
+        ]:
             self.assertIn(
                 line,
                 failure_str,
@@ -281,16 +282,13 @@ tensor 'x' size mismatch at index 0. expected 8, actual 12""".split("\n"):
             failure_reasons.clear()
             opt_f([7, 8])
 
-            for line in """\
-len(x) == 3""".split("\n"):
+            for line in ["len(x) == 3"]:
                 self.assertIn(line, filter_reasons())
 
             failure_reasons.clear()
             opt_f([9])
 
-            for line in """\
-len(x) == 2
-len(x) == 3""".split("\n"):
+            for line in ["len(x) == 2", "len(x) == 3"]:
                 self.assertIn(line, filter_reasons())
 
     @torch._dynamo.config.patch(recompile_limit=1)
