@@ -8,6 +8,7 @@ from unittest import mock
 import torch
 import torch._dynamo.config as dynamo_config
 import torch._inductor.config as inductor_config
+import torch.compiler.config as compiler_config
 from torch._dynamo import utils
 from torch._inductor.test_case import TestCase
 
@@ -226,6 +227,11 @@ class TestDynamoTimed(TestCase):
     Test utilities surrounding dynamo_timed.
     """
 
+    def setUp(self):
+        super().setUp()
+        if hasattr(torch._dynamo, "reset_recompile_user_contexts"):
+            torch._dynamo.reset_recompile_user_contexts()
+
     def run_forward_backward(self):
         model = torch.compile(TestModel())
         x = torch.rand([3], requires_grad=True)
@@ -384,12 +390,21 @@ class TestDynamoTimed(TestCase):
             self.run_forward_backward()
             compilation_events = [arg[0][0] for arg in log_event.call_args_list]
 
+        def filter_expected(s: str) -> str:
+            d = eval(s)
+            if not dynamo_config.run_gc_after_compile:
+                d.pop("gc", None)
+                if "gc_time_us" in d:
+                    d["gc_time_us"] = None
+            return pprint.pformat(d)
+
         # Validate utils.compile_times(). Unfortunately, we can't test the output
         # reliably because it depends on whether 'tabulate' is installed. So we'll
         # directly inspect the dict it prints instead:
         self.assertExpectedInline(
             pprint.pformat(utils.compilation_time_metrics),
-            """\
+            filter_expected(
+                """\
 {'GraphLowering.codegen': [0.0, 0.0],
  'GraphLowering.compile_to_fn': [0.0, 0.0],
  'GraphLowering.compile_to_module': [0.0, 0.0],
@@ -419,8 +434,8 @@ class TestDynamoTimed(TestCase):
  'fx_codegen_and_compile': [0.0, 0.0],
  'gc': [0.0],
  'min_cut_rematerialization_partition': [0.0]}"""
-            if _IS_WINDOWS
-            else """\
+                if _IS_WINDOWS
+                else """\
 {'GraphLowering.codegen': [0.0, 0.0],
  'GraphLowering.compile_to_fn': [0.0, 0.0],
  'GraphLowering.compile_to_module': [0.0, 0.0],
@@ -450,7 +465,8 @@ class TestDynamoTimed(TestCase):
  'create_aot_dispatcher_function': [0.0],
  'fx_codegen_and_compile': [0.0, 0.0],
  'gc': [0.0],
- 'min_cut_rematerialization_partition': [0.0]}""",  # noqa: B950
+ 'min_cut_rematerialization_partition': [0.0]}"""
+            ),  # noqa: B950
         )
 
         # Now validate utils.calculate_time_spent(). Formatting the return
@@ -458,7 +474,8 @@ class TestDynamoTimed(TestCase):
         time_spent = utils.calculate_time_spent()
         self.assertExpectedInline(
             pprint.pformat(time_spent),
-            """\
+            filter_expected(
+                """\
 {'_recursive_joint_graph_passes': 0.0,
  '_recursive_post_grad_passes': 0.0,
  '_recursive_pre_grad_passes': 0.0,
@@ -469,8 +486,8 @@ class TestDynamoTimed(TestCase):
  'gc': 0.0,
  'inductor_compile': 0.0,
  'total_wall_time': 0.0}"""
-            if _IS_WINDOWS
-            else """\
+                if _IS_WINDOWS
+                else """\
 {'_recursive_joint_graph_passes': 0.0,
  '_recursive_post_grad_passes': 0.0,
  '_recursive_pre_grad_passes': 0.0,
@@ -481,7 +498,8 @@ class TestDynamoTimed(TestCase):
  'entire_frame_compile': 0.0,
  'gc': 0.0,
  'inductor_compile': 0.0,
- 'total_wall_time': 0.0}""",  # noqa: B950
+ 'total_wall_time': 0.0}"""
+            ),  # noqa: B950
         )
 
         # Now validate the CompilationMetrics logs. We expect a log for the
@@ -497,9 +515,11 @@ class TestDynamoTimed(TestCase):
             e.co_filename = None
             e.co_firstlineno = None
             e.inductor_config = None
+            e.compiler_config = None
             e.cuda_version = None
             e.triton_version = None
             e.python_version = None
+            e.pytorch_version = None
             e.stack_trace = None
             e.graph_node_shapes = None
             e.exception_stack_trace = None
@@ -509,6 +529,7 @@ class TestDynamoTimed(TestCase):
         raw = dataclasses.asdict(compilation_events[0])
         del raw["feature_usage"]
         del raw["ir_count"]
+        del raw["inductor_provenance"]
         del raw["param_numel"]
         del raw["param_bytes"]
         del raw["param_count"]
@@ -516,7 +537,8 @@ class TestDynamoTimed(TestCase):
         del raw["guard_latency_us"]
         self.assertExpectedInline(
             pprint.pformat(raw),
-            """\
+            filter_expected(
+                """\
 {'accumulated_cache_size': 0,
  'aot_autograd_cumulative_compile_time_us': 0,
  'backend_compile_time_s': 0.0,
@@ -528,6 +550,7 @@ class TestDynamoTimed(TestCase):
  'code_gen_time_s': 0.0,
  'compile_id': '1/0',
  'compile_time_autotune_time_us': None,
+ 'compiler_config': None,
  'compliant_custom_ops': set(),
  'config_inline_inbuilt_nn_modules': False,
  'config_suppress_errors': False,
@@ -552,92 +575,7 @@ class TestDynamoTimed(TestCase):
  'graph_node_count': 3,
  'graph_node_shapes': None,
  'graph_op_count': 1,
- 'guard_count': 9,
- 'has_guarded_code': True,
- 'inductor_code_gen_cumulative_compile_time_us': 0,
- 'inductor_compile_time_s': 0.0,
- 'inductor_config': None,
- 'inductor_cumulative_compile_time_us': 0,
- 'inductor_fx_remote_cache_backend_type': None,
- 'inductor_fx_remote_cache_hit_count': None,
- 'inductor_fx_remote_cache_hit_keys': None,
- 'inductor_fx_remote_cache_miss_count': None,
- 'inductor_fx_remote_cache_miss_keys': None,
- 'inline_inbuilt_nn_modules_candidate': False,
- 'is_forward': True,
- 'is_runtime': False,
- 'joint_graph_pass_time_us': 0,
- 'log_format_version': 3,
- 'non_compliant_ops': set(),
- 'num_graph_breaks': 0,
- 'num_triton_bundles': None,
- 'pgo_get_remote_code_state_time_us': None,
- 'pgo_put_remote_code_state_time_us': None,
- 'post_grad_pass_time_us': 0,
- 'pre_grad_pass_time_us': 0,
- 'python_version': None,
- 'recompile_reason': None,
- 'recompile_user_contexts': None,
- 'remote_cache_time_saved_s': None,
- 'remote_cache_version': None,
- 'remote_fx_graph_cache_get_time_ms': None,
- 'remote_fx_graph_cache_get_time_us': None,
- 'remote_fx_graph_cache_put_time_ms': None,
- 'remote_fx_graph_cache_put_time_us': None,
- 'restart_reasons': set(),
- 'runtime_cudagraphify_time_us': None,
- 'runtime_triton_autotune_time_us': None,
- 'shape_env_guard_count': 0,
- 'specialize_float': False,
- 'stack_trace': None,
- 'start_time': 0.0001,
- 'start_time_us': 100,
- 'structured_logging_overhead_s': 0.0,
- 'structured_logging_overhead_us': 0,
- 'tensorify_float_attempt': None,
- 'tensorify_float_failure': None,
- 'tensorify_float_success': None,
- 'triton_compile_time_us': None,
- 'triton_kernel_compile_times_us': None,
- 'triton_version': None}"""
-            if _IS_WINDOWS
-            else """\
-{'accumulated_cache_size': 0,
- 'aot_autograd_cumulative_compile_time_us': 0,
- 'backend_compile_time_s': 0.0,
- 'backward_cumulative_compile_time_us': None,
- 'cache_size': 0,
- 'co_filename': None,
- 'co_firstlineno': None,
- 'co_name': 'forward',
- 'code_gen_time_s': 0.0,
- 'compile_id': '1/0',
- 'compile_time_autotune_time_us': None,
- 'compliant_custom_ops': set(),
- 'config_inline_inbuilt_nn_modules': False,
- 'config_suppress_errors': False,
- 'cuda_version': None,
- 'cudagraph_skip_reason': None,
- 'distributed_ephemeral_timeout_us': None,
- 'duration_us': 0,
- 'dynamo_compile_time_before_restart_us': 0,
- 'dynamo_config': None,
- 'dynamo_cumulative_compile_time_us': 0,
- 'dynamo_time_before_restart_s': 0.0,
- 'end_time_us': 100,
- 'entire_frame_compile_time_s': 0.0,
- 'exception_stack_trace': None,
- 'fail_reason': None,
- 'fail_type': None,
- 'fail_user_frame_filename': None,
- 'fail_user_frame_lineno': None,
- 'frame_key': '1',
- 'gc_time_us': 0,
- 'graph_input_count': 1,
- 'graph_node_count': 3,
- 'graph_node_shapes': None,
- 'graph_op_count': 1,
- 'guard_count': 9,
+ 'guard_count': 10,
  'has_guarded_code': True,
  'inductor_code_gen_cumulative_compile_time_us': 0,
  'inductor_compile_time_s': 0.0,
@@ -661,6 +599,7 @@ class TestDynamoTimed(TestCase):
  'post_grad_pass_time_us': 0,
  'pre_grad_pass_time_us': 0,
  'python_version': None,
+ 'pytorch_version': None,
  'recompile_reason': None,
  'recompile_user_contexts': None,
  'remote_cache_time_saved_s': None,
@@ -684,20 +623,110 @@ class TestDynamoTimed(TestCase):
  'tensorify_float_success': None,
  'triton_compile_time_us': 0,
  'triton_kernel_compile_times_us': None,
- 'triton_version': None}""",  # noqa: B950
+ 'triton_version': None}"""
+                if _IS_WINDOWS
+                else """\
+{'accumulated_cache_size': 0,
+ 'aot_autograd_cumulative_compile_time_us': 0,
+ 'backend_compile_time_s': 0.0,
+ 'backward_cumulative_compile_time_us': None,
+ 'cache_size': 0,
+ 'co_filename': None,
+ 'co_firstlineno': None,
+ 'co_name': 'forward',
+ 'code_gen_time_s': 0.0,
+ 'compile_id': '1/0',
+ 'compile_time_autotune_time_us': None,
+ 'compiler_config': None,
+ 'compliant_custom_ops': set(),
+ 'config_inline_inbuilt_nn_modules': False,
+ 'config_suppress_errors': False,
+ 'cuda_version': None,
+ 'cudagraph_skip_reason': None,
+ 'distributed_ephemeral_timeout_us': None,
+ 'duration_us': 0,
+ 'dynamo_compile_time_before_restart_us': 0,
+ 'dynamo_config': None,
+ 'dynamo_cumulative_compile_time_us': 0,
+ 'dynamo_time_before_restart_s': 0.0,
+ 'end_time_us': 100,
+ 'entire_frame_compile_time_s': 0.0,
+ 'exception_stack_trace': None,
+ 'fail_reason': None,
+ 'fail_type': None,
+ 'fail_user_frame_filename': None,
+ 'fail_user_frame_lineno': None,
+ 'frame_key': '1',
+ 'gc_time_us': 0,
+ 'graph_input_count': 1,
+ 'graph_node_count': 3,
+ 'graph_node_shapes': None,
+ 'graph_op_count': 1,
+ 'guard_count': 10,
+ 'has_guarded_code': True,
+ 'inductor_code_gen_cumulative_compile_time_us': 0,
+ 'inductor_compile_time_s': 0.0,
+ 'inductor_config': None,
+ 'inductor_cumulative_compile_time_us': 0,
+ 'inductor_fx_remote_cache_backend_type': None,
+ 'inductor_fx_remote_cache_hit_count': None,
+ 'inductor_fx_remote_cache_hit_keys': None,
+ 'inductor_fx_remote_cache_miss_count': None,
+ 'inductor_fx_remote_cache_miss_keys': None,
+ 'inline_inbuilt_nn_modules_candidate': False,
+ 'is_forward': True,
+ 'is_runtime': False,
+ 'joint_graph_pass_time_us': 0,
+ 'log_format_version': 3,
+ 'non_compliant_ops': set(),
+ 'num_graph_breaks': 0,
+ 'num_triton_bundles': None,
+ 'pgo_get_remote_code_state_time_us': None,
+ 'pgo_put_remote_code_state_time_us': None,
+ 'post_grad_pass_time_us': 0,
+ 'pre_grad_pass_time_us': 0,
+ 'python_version': None,
+ 'pytorch_version': None,
+ 'recompile_reason': None,
+ 'recompile_user_contexts': None,
+ 'remote_cache_time_saved_s': None,
+ 'remote_cache_version': None,
+ 'remote_fx_graph_cache_get_time_ms': None,
+ 'remote_fx_graph_cache_get_time_us': None,
+ 'remote_fx_graph_cache_put_time_ms': None,
+ 'remote_fx_graph_cache_put_time_us': None,
+ 'restart_reasons': set(),
+ 'runtime_cudagraphify_time_us': None,
+ 'runtime_triton_autotune_time_us': None,
+ 'shape_env_guard_count': 0,
+ 'specialize_float': False,
+ 'stack_trace': None,
+ 'start_time': 0.0001,
+ 'start_time_us': 100,
+ 'structured_logging_overhead_s': 0.0,
+ 'structured_logging_overhead_us': 0,
+ 'tensorify_float_attempt': None,
+ 'tensorify_float_failure': None,
+ 'tensorify_float_success': None,
+ 'triton_compile_time_us': 0,
+ 'triton_kernel_compile_times_us': None,
+ 'triton_version': None}"""
+            ),  # noqa: B950
         )
 
         # Second event is for the backward
         raw = dataclasses.asdict(compilation_events[1])
         del raw["feature_usage"]
         del raw["ir_count"]
+        del raw["inductor_provenance"]
         del raw["guard_latency_us"]
         del raw["param_numel"]
         del raw["param_bytes"]
         del raw["param_count"]
         self.assertExpectedInline(
             pprint.pformat(raw),
-            """\
+            (
+                """\
 {'accumulated_cache_size': None,
  'aot_autograd_cumulative_compile_time_us': None,
  'backend_compile_time_s': None,
@@ -709,6 +738,7 @@ class TestDynamoTimed(TestCase):
  'code_gen_time_s': 0.0,
  'compile_id': '1/0',
  'compile_time_autotune_time_us': None,
+ 'compiler_config': None,
  'compliant_custom_ops': None,
  'config_inline_inbuilt_nn_modules': False,
  'config_suppress_errors': False,
@@ -757,6 +787,7 @@ class TestDynamoTimed(TestCase):
  'post_grad_pass_time_us': 0,
  'pre_grad_pass_time_us': None,
  'python_version': None,
+ 'pytorch_version': None,
  'recompile_reason': None,
  'recompile_user_contexts': None,
  'remote_cache_time_saved_s': None,
@@ -781,8 +812,8 @@ class TestDynamoTimed(TestCase):
  'triton_compile_time_us': None,
  'triton_kernel_compile_times_us': None,
  'triton_version': None}"""
-            if _IS_WINDOWS
-            else """\
+                if _IS_WINDOWS
+                else """\
 {'accumulated_cache_size': None,
  'aot_autograd_cumulative_compile_time_us': None,
  'backend_compile_time_s': None,
@@ -794,6 +825,7 @@ class TestDynamoTimed(TestCase):
  'code_gen_time_s': 0.0,
  'compile_id': '1/0',
  'compile_time_autotune_time_us': None,
+ 'compiler_config': None,
  'compliant_custom_ops': None,
  'config_inline_inbuilt_nn_modules': False,
  'config_suppress_errors': False,
@@ -842,6 +874,7 @@ class TestDynamoTimed(TestCase):
  'post_grad_pass_time_us': 0,
  'pre_grad_pass_time_us': None,
  'python_version': None,
+ 'pytorch_version': None,
  'recompile_reason': None,
  'recompile_user_contexts': None,
  'remote_cache_time_saved_s': None,
@@ -865,7 +898,27 @@ class TestDynamoTimed(TestCase):
  'tensorify_float_success': None,
  'triton_compile_time_us': 0,
  'triton_kernel_compile_times_us': None,
- 'triton_version': None}""",  # noqa: B950
+ 'triton_version': None}"""
+            ),  # noqa: B950
+        )
+
+    @dynamo_config.patch(
+        {
+            "log_compilation_metrics": True,
+        }
+    )
+    @compiler_config.patch({"job_id": "test_job_id"})
+    def test_compiler_config(self):
+        def test1(x):
+            return x * x
+
+        compilation_events = []
+        with mock.patch("torch._dynamo.utils.log_compilation_event") as log_event:
+            torch.compile(test1)(torch.randn(1))
+            compilation_events = [arg[0][0] for arg in log_event.call_args_list]
+        self.assertIn(
+            '"job_id": "test_job_id"',
+            compilation_events[0].compiler_config,
         )
 
     @dynamo_config.patch(
@@ -876,13 +929,14 @@ class TestDynamoTimed(TestCase):
     def test_ir_count(self):
         # Different python versions have different potential IR counts.
         version = (sys.version_info[0], sys.version_info[1])
-        self.assertIn(version, ((3, 9), (3, 10), (3, 11), (3, 12), (3, 13)))
+        self.assertIn(version, ((3, 9), (3, 10), (3, 11), (3, 12), (3, 13), (3, 14)))
         first, second = {
             (3, 9): (10, 6),
             (3, 10): (10, 6),
-            (3, 11): (10, 6),
+            (3, 11): (11, 7),
             (3, 12): (11, 7),
             (3, 13): (11, 7),
+            (3, 14): (11, 7),
         }[version]
 
         def test1(x):
@@ -906,6 +960,27 @@ class TestDynamoTimed(TestCase):
             compilation_events = [arg[0][0] for arg in log_event.call_args_list]
         self.assertEqual(compilation_events[0].ir_count, second)
 
+    @dynamo_config.patch(
+        {
+            "log_compilation_metrics": True,
+        }
+    )
+    @inductor_config.patch(
+        {"trace.enabled": True, "trace.provenance_tracking_level": 1},
+    )
+    def test_inductor_provenance(self):
+        module = torch.nn.Linear(6, 66)
+        graph_module = torch.fx.symbolic_trace(module)
+
+        compilation_events = []
+        with mock.patch("torch._dynamo.utils.log_compilation_event") as log_event:
+            torch.compile(graph_module)(torch.randn(6, 6))
+            compilation_events = [arg[0][0] for arg in log_event.call_args_list]
+        self.assertEqual(
+            compilation_events[0].inductor_provenance,
+            {'{"extern_kernels.addmm:1": []}'},
+        )
+
     @dynamo_config.patch({"log_compilation_metrics": True})
     @inductor_config.patch({"force_disable_caches": True})
     def test_dynamic_shape_feature_use(self):
