@@ -1,12 +1,28 @@
 # Owner(s): ["module: fx"]
 
+import dataclasses
 from collections import defaultdict
 
 import torch
+import torch.fx.passes.operator_support as op_support
+import torch.fx.passes.splitter_base as splitter_base
 from torch.fx.passes.split_utils import split_by_tags
 from torch.testing._internal.common_utils import TestCase
 
 
+@torch.jit.script
+@dataclasses.dataclass
+class DummyDataClass:
+    a: int
+    b: int
+    c: int
+
+
+@torch.fx.wrap
+def wrapped_add(_dataclass, y):
+    return _dataclass.c + y
+
+
 class TestFXSplit(TestCase):
     def test_split_preserve_node_meta(self):
         class TestModule(torch.nn.Module):
@@ -38,6 +54,65 @@ class TestFXSplit(TestCase):
                 self.assertIn("name", node.meta)
                 self.assertEqual(node.meta["name"], node.name)
 
+    def test_dataclass_as_graph_entry(self):
+        """
+        Test that splitting works when the graph entry is a dataclass instance
+        and a wrapped function is called with it, resulting in a call_function
+        node with no input dependencies. This tests the edge case fixed in D81232435
+        where call_function nodes with no dependencies should be handled properly
+        in the starter_nodes() method.
+
+        Graph visualization:
+        y (input)    DummyDataClass(2,3,4) (no input deps, result as a call_function_node)
+            \              /
+             \            /
+              wrapped_add
+                  |
+              z (output)
+        """  # noqa: W605
+
+        class TestModuleWithFunctionEntry(torch.nn.Module):
+            def forward(self, y):
+                # This creates a call_function node with no input dependencies
+                dummy_data_class = DummyDataClass(2, 3, 4)
+                z = wrapped_add(dummy_data_class, y)
+                return z
+
+        mod = TestModuleWithFunctionEntry()
+        gm = torch.fx.symbolic_trace(mod)
+
+        # Create custom operator support to mark wrapped_add as supported
+        class CustomOpSupport(op_support.OperatorSupportBase):
+            def is_node_supported(self, submodules, node) -> bool:
+                return node.target is wrapped_add
+
+        # Create a simple splitter to test the edge case
+        class TestSplitter(splitter_base._SplitterBase):
+            def __init__(self, module, sample_input, operator_support):
+                settings = splitter_base._SplitterSettingBase()
+                super().__init__(module, sample_input, operator_support, settings)
+
+        # Create splitter instance - this tests the fix where call_function nodes
+        # with no input dependencies are properly handled in starter_nodes()
+        splitter = TestSplitter(
+            module=gm,
+            sample_input=[torch.randn(2, 3)],
+            operator_support=CustomOpSupport(),
+        )
+
+        # This should not raise an exception (tests the fix from D81232435)
+        # The fix allows call_function nodes with no dependencies as valid starter nodes
+        split_result = splitter()
+
+        # Verify the splitting worked correctly
+        self.assertIsNotNone(split_result)
+
+        # Test that the split module produces the same result as the original
+        test_input = torch.randn(2, 3)
+        original_result = mod(test_input)
+        split_module_result = split_result(test_input)
+        self.assertTrue(torch.equal(original_result, split_module_result))
+
 
 class TestSplitByTags(TestCase):
     class TestModule(torch.nn.Module):
@@ -221,7 +296,7 @@ class TestSplitOutputType(TestCase):
         gm_output = module(inputs)
         split_gm_output = split_gm(inputs)
 
-        self.assertTrue(type(gm_output) == type(split_gm_output))
+        self.assertTrue(type(gm_output) is type(split_gm_output))
         self.assertTrue(torch.equal(gm_output, split_gm_output))
 
 
