@@ -222,7 +222,7 @@ class TestSourceMatcher(JitTestCase):
         class M(torch.nn.Module):
             def forward(self, x, y):
                 b = x.item()
-                torch._check_is_size(b)
+                torch._check(b >= 0)
                 torch._check(b + 1 < y.size(0))
                 return y[: b + 1]
 
