@@ -997,20 +997,21 @@ class TestSDPAPatternRewriterTemplate(TestCase):
             attn_weights = scores.float().softmax(dim=-1).type(value.dtype)
             return attn_weights.matmul(value)
 
-        tensor_shape = (4, 2, 16, 32)
-        attn_mask = torch.randn((1, 1, 1, 2), dtype=torch.float, device=self.device)
-        args = [
-            torch.randn(tensor_shape, device=self.device),
-            torch.randn(tensor_shape, device=self.device),
-            torch.randn(tensor_shape, device=self.device),
-            attn_mask,
-        ]
-        self._check_common(
-            dot_prod_attention,
-            args1=args,
-            has_dropout=False,
-            check_train=False,
-        )
+        tensor_shapes = [(4, 2, 16, 32), (1, 2, 16, 32)]
+        for tensor_shape in tensor_shapes:
+            attn_mask = torch.randn((1, 1, 1, 2), dtype=torch.float, device=self.device)
+            args = [
+                torch.randn(tensor_shape, device=self.device),
+                torch.randn(tensor_shape, device=self.device),
+                torch.randn(tensor_shape, device=self.device),
+                attn_mask,
+            ]
+            self._check_common(
+                dot_prod_attention,
+                args1=args,
+                has_dropout=False,
+                check_train=False,
+            )
 
     def _test_sdpa_rewriter_22(self):
         def dot_prod_attention(
@@ -1027,30 +1028,31 @@ class TestSDPAPatternRewriterTemplate(TestCase):
             attn_weights = scores.float().softmax(dim=-1).type(value.dtype)
             return attn_weights.matmul(value), key, value
 
-        tensor_shape = (4, 2, 16, 32)
-        attn_mask = torch.randn((1, 1, 2, 2), dtype=torch.float, device=self.device)
-        args = [
-            torch.randn(tensor_shape, device=self.device),
-            torch.randn(tensor_shape, device=self.device),
-            torch.randn(tensor_shape, device=self.device),
-            attn_mask,
-        ]
-        self._check_common(
-            dot_prod_attention,
-            args1=args,
-            has_dropout=False,
-            check_train=False,
-        )
-        # test attn_mask with stride of last dim != 1
-        attn_mask_ = attn_mask.transpose(2, 3)
-        args[3] = attn_mask_
-        self._check_common(
-            dot_prod_attention,
-            args1=args,
-            has_dropout=False,
-            check_train=False,
-            contains=self.device == "cpu",
-        )
+        tensor_shapes = [(4, 2, 16, 32), (1, 2, 16, 32)]
+        for tensor_shape in tensor_shapes:
+            attn_mask = torch.randn((1, 1, 2, 2), dtype=torch.float, device=self.device)
+            args = [
+                torch.randn(tensor_shape, device=self.device),
+                torch.randn(tensor_shape, device=self.device),
+                torch.randn(tensor_shape, device=self.device),
+                attn_mask,
+            ]
+            self._check_common(
+                dot_prod_attention,
+                args1=args,
+                has_dropout=False,
+                check_train=False,
+            )
+            # test attn_mask with stride of last dim != 1
+            attn_mask_ = attn_mask.transpose(2, 3)
+            args[3] = attn_mask_
+            self._check_common(
+                dot_prod_attention,
+                args1=args,
+                has_dropout=False,
+                check_train=False,
+                contains=self.device == "cpu",
+            )
 
     def _test_sdpa_rewriter_23(self):
         def dot_prod_attention(
@@ -1067,18 +1069,19 @@ class TestSDPAPatternRewriterTemplate(TestCase):
             attn_weights = scores.float().softmax(dim=-1).type(value.dtype)
             return attn_weights.matmul(value), key, value
 
-        tensor_shape = (4, 2, 16, 32)
-        args = [
-            torch.randn(tensor_shape, device=self.device),
-            torch.randn(tensor_shape, device=self.device),
-            torch.randn(tensor_shape, device=self.device),
-        ]
-        self._check_common(
-            dot_prod_attention,
-            args1=args,
-            has_dropout=False,
-            check_train=False,
-        )
+        tensor_shapes = [(4, 2, 16, 32), (1, 2, 16, 32)]
+        for tensor_shape in tensor_shapes:
+            args = [
+                torch.randn(tensor_shape, device=self.device),
+                torch.randn(tensor_shape, device=self.device),
+                torch.randn(tensor_shape, device=self.device),
+            ]
+            self._check_common(
+                dot_prod_attention,
+                args1=args,
+                has_dropout=False,
+                check_train=False,
+            )
 
     def _test_sdpa_rewriter_24(self):
         def dot_prod_attention(
