@@ -57,6 +57,7 @@ class TestJointOps(TestCase):
     basetype = OrderedSet
 
     def setUp(self):
+        super().setUp()
         self.word = word = "simsalabim"
         self.otherword = "madagascar"
         self.letters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
@@ -539,7 +540,7 @@ class TestSet(TestJointOps, TestCase):
         # s.discard(self.thetype(self.word))
 
     def test_pop(self):
-        for i in range(len(self.s)):
+        for _ in range(len(self.s)):
             elem = self.s.pop()
             self.assertNotIn(elem, self.s)
         self.assertRaises(KeyError, self.s.pop)
@@ -851,6 +852,7 @@ class TestBasicOps(TestCase):
 
 class TestBasicOpsEmpty(TestBasicOps, TestCase):
     def setUp(self):
+        super().setUp()
         self.case = "empty OrderedSet"
         self.values = []
         self.OrderedSet = OrderedSet(self.values)
@@ -864,6 +866,7 @@ class TestBasicOpsEmpty(TestBasicOps, TestCase):
 
 class TestBasicOpsSingleton(TestBasicOps, TestCase):
     def setUp(self):
+        super().setUp()
         self.case = "unit OrderedSet (number)"
         self.values = [3]
         self.OrderedSet = OrderedSet(self.values)
@@ -883,6 +886,7 @@ class TestBasicOpsSingleton(TestBasicOps, TestCase):
 
 class TestBasicOpsTuple(TestBasicOps, TestCase):
     def setUp(self):
+        super().setUp()
         self.case = "unit OrderedSet (tuple)"
         self.values = [(0, "zero")]
         self.OrderedSet = OrderedSet(self.values)
@@ -902,6 +906,7 @@ class TestBasicOpsTuple(TestBasicOps, TestCase):
 
 class TestBasicOpsTriple(TestBasicOps, TestCase):
     def setUp(self):
+        super().setUp()
         self.case = "triple OrderedSet"
         self.values = [0, "zero", operator.add]
         self.OrderedSet = OrderedSet(self.values)
@@ -915,6 +920,7 @@ class TestBasicOpsTriple(TestBasicOps, TestCase):
 
 class TestBasicOpsString(TestBasicOps, TestCase):
     def setUp(self):
+        super().setUp()
         self.case = "string OrderedSet"
         self.values = ["a", "b", "c"]
         self.OrderedSet = OrderedSet(self.values)
@@ -931,6 +937,7 @@ class TestBasicOpsString(TestBasicOps, TestCase):
 
 class TestBasicOpsBytes(TestBasicOps, TestCase):
     def setUp(self):
+        super().setUp()
         self.case = "bytes OrderedSet"
         self.values = [b"a", b"b", b"c"]
         self.OrderedSet = OrderedSet(self.values)
@@ -947,6 +954,7 @@ class TestBasicOpsBytes(TestBasicOps, TestCase):
 
 class TestBasicOpsMixedStringBytes(TestBasicOps, TestCase):
     def setUp(self):
+        super().setUp()
         warnings.simplefilter("ignore", BytesWarning)
         self.case = "string and bytes OrderedSet"
         self.values = ["a", "b", b"a", b"b"]
@@ -990,7 +998,7 @@ class TestExceptionPropagation(TestCase):
     def test_changingSizeWhileIterating(self):
         s = OrderedSet([1, 2, 3])
         try:
-            for i in s:
+            for _ in s:
                 s.update([4])  # noqa: B909
         except RuntimeError:
             pass
@@ -1018,6 +1026,7 @@ class TestSetOfSets(TestCase):
 
 class TestBinaryOps(TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet((2, 4, 6))
 
     def test_eq(self):  # SF bug 643115
@@ -1093,6 +1102,7 @@ class TestBinaryOps(TestCase):
 
 class TestUpdateOps(TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet((2, 4, 6))
 
     def test_union_subset(self):
@@ -1181,6 +1191,7 @@ class TestUpdateOps(TestCase):
 
 class TestMutate(TestCase):
     def setUp(self):
+        super().setUp()
         self.values = ["a", "b", "c"]
         self.OrderedSet = OrderedSet(self.values)
 
@@ -1469,6 +1480,7 @@ class TestOnlySetsInBinaryOps(TestCase):
 
 class TestOnlySetsNumeric(TestOnlySetsInBinaryOps, TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet((1, 2, 3))
         self.other = 19
         self.otherIsIterable = False
@@ -1479,6 +1491,7 @@ class TestOnlySetsNumeric(TestOnlySetsInBinaryOps, TestCase):
 
 class TestOnlySetsDict(TestOnlySetsInBinaryOps, TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet((1, 2, 3))
         self.other = {1: 2, 3: 4}
         self.otherIsIterable = True
@@ -1489,6 +1502,7 @@ class TestOnlySetsDict(TestOnlySetsInBinaryOps, TestCase):
 
 class TestOnlySetsOperator(TestOnlySetsInBinaryOps, TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet((1, 2, 3))
         self.other = operator.add
         self.otherIsIterable = False
@@ -1499,6 +1513,7 @@ class TestOnlySetsOperator(TestOnlySetsInBinaryOps, TestCase):
 
 class TestOnlySetsTuple(TestOnlySetsInBinaryOps, TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet((1, 2, 3))
         self.other = (2, 4, 6)
         self.otherIsIterable = True
@@ -1509,6 +1524,7 @@ class TestOnlySetsTuple(TestOnlySetsInBinaryOps, TestCase):
 
 class TestOnlySetsString(TestOnlySetsInBinaryOps, TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet((1, 2, 3))
         self.other = "abc"
         self.otherIsIterable = True
@@ -1519,6 +1535,8 @@ class TestOnlySetsString(TestOnlySetsInBinaryOps, TestCase):
 
 class TestOnlySetsGenerator(TestOnlySetsInBinaryOps, TestCase):
     def setUp(self):
+        super().setUp()
+
         def gen():
             for i in range(0, 10, 2):  # noqa: UP028
                 yield i
@@ -1556,6 +1574,7 @@ class TestCopying:
 
 class TestCopyingEmpty(TestCopying, TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet()
 
 
@@ -1564,6 +1583,7 @@ class TestCopyingEmpty(TestCopying, TestCase):
 
 class TestCopyingSingleton(TestCopying, TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet(["hello"])
 
 
@@ -1572,6 +1592,7 @@ class TestCopyingSingleton(TestCopying, TestCase):
 
 class TestCopyingTriple(TestCopying, TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet(["zero", 0, None])
 
 
@@ -1580,6 +1601,7 @@ class TestCopyingTriple(TestCopying, TestCase):
 
 class TestCopyingTuple(TestCopying, TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet([(1, 2)])
 
 
@@ -1588,6 +1610,7 @@ class TestCopyingTuple(TestCopying, TestCase):
 
 class TestCopyingNested(TestCopying, TestCase):
     def setUp(self):
+        super().setUp()
         self.OrderedSet = OrderedSet([((1, 2), (3, 4))])
 
 
@@ -1598,6 +1621,7 @@ del TestCopying
 
 class TestIdentities(TestCase):
     def setUp(self):
+        super().setUp()
         self.a = OrderedSet("abracadabra")
         self.b = OrderedSet("alacazam")
 
