@@ -13,10 +13,10 @@ from torch._inductor.runtime.triton_compat import CompiledKernel, tl, triton
 from torch._inductor.runtime.triton_helpers import libdevice
 from torch._inductor.test_case import TestCase
 from torch.testing._internal.common_utils import skipIfRocm
-from torch.testing._internal.triton_utils import requires_cuda
+from torch.testing._internal.triton_utils import requires_cuda_and_triton
 
 
-@requires_cuda
+@requires_cuda_and_triton
 class TestStaticCudaLauncher(TestCase):
     def setUp(self):
         super().setUp()
@@ -38,11 +38,10 @@ class TestStaticCudaLauncher(TestCase):
             return
         # Just used by tests for now.
         # TODO: derive cubin_path from wherever triton stores the cubin file on disk.
-        tmp_file = tempfile.NamedTemporaryFile(mode="wb", delete=False)
-        with tmp_file:
+        with tempfile.NamedTemporaryFile(mode="wb", delete=False) as tmp_file:
             tmp_file.write(kernel.asm["cubin"])
-        self.tmp_files.append(tmp_file)
-        return tmp_file.name
+            self.tmp_files.append(tmp_file)
+            return tmp_file.name
 
     def _make_launcher(
         self,
@@ -396,7 +395,7 @@ def kernel_many_args(out_tensor, {decl}):
         self.assertEqual(buf0, buf1)
 
 
-@requires_cuda
+@requires_cuda_and_triton
 @torch._inductor.config.patch(
     {"use_static_cuda_launcher": True, "strict_static_cuda_launcher": True}
 )
