@@ -1,6 +1,5 @@
 # Owner(s): ["module: functorch"]
 import json
-import tempfile
 import zipfile
 from pathlib import Path
 
@@ -11,8 +10,10 @@ import torch._inductor
 import torch._inductor.decomposition
 from torch._higher_order_ops.torchbind import CallTorchBind, enable_torchbind_tracing
 from torch._inductor import aot_compile, ir
+from torch._inductor.codecache import WritableTempFile
 from torch._inductor.package import package_aoti
 from torch._inductor.test_case import run_tests, TestCase
+from torch.testing._internal.common_utils import skipIfWindows
 from torch.testing._internal.inductor_utils import GPU_TYPE, requires_gpu
 from torch.testing._internal.torchbind_impls import (
     _empty_tensor_queue,
@@ -158,6 +159,7 @@ class TestTorchbind(TestCase):
             "call_torchbind(__torch__.torch.classes._TorchScriptTesting._TensorQueue _0, str method, Tensor _1) -> NoneType _0",
         )
 
+    @skipIfWindows(msg="AOTI is not fully support on Windows")
     def test_torchbind_aot_compile(self):
         ep, inputs, _, _ = self.get_exported_model()
         aoti_files = aot_compile(
@@ -172,7 +174,7 @@ class TestTorchbind(TestCase):
                 custom_objs_config = file
             elif file.endswith("/custom_obj_0"):
                 custom_obj_0 = file
-            elif file.endswith(".json") and "metadata" not in file:
+            elif file.endswith("wrapper.json") and "metadata" not in file:
                 extern_json = file
 
         self.assertIsNotNone(custom_objs_config)
@@ -190,7 +192,7 @@ class TestTorchbind(TestCase):
                 {
                     "nodes": [
                         {
-                            "name": "buf3",
+                            "name": "buf1",
                             "node": {
                                 "target": "_TorchScriptTesting::takes_foo_tuple_return",
                                 "inputs": [
@@ -206,20 +208,20 @@ class TestTorchbind(TestCase):
                                     },
                                     {
                                         "name": "x",
-                                        "arg": {"as_tensor": {"name": "buf2"}},
+                                        "arg": {"as_tensor": {"name": "buf0"}},
                                         "kind": 1,
                                     },
                                 ],
                                 "outputs": [
-                                    {"as_tensor": {"name": "buf4"}},
-                                    {"as_tensor": {"name": "buf5"}},
+                                    {"as_tensor": {"name": "buf2"}},
+                                    {"as_tensor": {"name": "buf3"}},
                                 ],
                                 "metadata": {},
                                 "is_hop_single_tensor_return": None,
                             },
                         },
                         {
-                            "name": "buf7",
+                            "name": "buf5",
                             "node": {
                                 "target": "_TorchScriptTesting::takes_foo",
                                 "inputs": [
@@ -235,17 +237,17 @@ class TestTorchbind(TestCase):
                                     },
                                     {
                                         "name": "x",
-                                        "arg": {"as_tensor": {"name": "buf6"}},
+                                        "arg": {"as_tensor": {"name": "buf4"}},
                                         "kind": 1,
                                     },
                                 ],
-                                "outputs": [{"as_tensor": {"name": "buf8"}}],
+                                "outputs": [{"as_tensor": {"name": "buf6"}}],
                                 "metadata": {},
                                 "is_hop_single_tensor_return": None,
                             },
                         },
                         {
-                            "name": "buf9",
+                            "name": "buf7",
                             "node": {
                                 "target": "call_torchbind",
                                 "inputs": [
@@ -266,11 +268,11 @@ class TestTorchbind(TestCase):
                                     },
                                     {
                                         "name": "_1",
-                                        "arg": {"as_tensor": {"name": "buf2"}},
+                                        "arg": {"as_tensor": {"name": "buf0"}},
                                         "kind": 1,
                                     },
                                 ],
-                                "outputs": [{"as_tensor": {"name": "buf10"}}],
+                                "outputs": [{"as_tensor": {"name": "buf8"}}],
                                 "metadata": {},
                                 "is_hop_single_tensor_return": None,
                             },
@@ -280,7 +282,7 @@ class TestTorchbind(TestCase):
             )
 
         # Test that the files are packaged
-        with tempfile.NamedTemporaryFile(suffix=".pt2") as f:
+        with WritableTempFile(suffix=".pt2") as f:
             package_path = package_aoti(f.name, aoti_files)
 
             with zipfile.ZipFile(package_path, "r") as zip_ref:
@@ -302,6 +304,7 @@ class TestTorchbind(TestCase):
         self.assertEqual(result, orig_res)
 
     @torch._inductor.config.patch("aot_inductor.use_runtime_constant_folding", True)
+    @skipIfWindows(msg="AOTI is not fully support on Windows")
     def test_torchbind_aot_compile_constant_folding(self):
         ep, inputs, orig_res, _ = self.get_exported_model()
         pt2_path = torch._inductor.aoti_compile_and_package(ep)
@@ -410,6 +413,30 @@ class TestTorchbind(TestCase):
         ):
             aot_compile(ep.module(), inputs, options={"aot_inductor.package": True})
 
+    def test_aoti_torchbind_name_collision(self):
+        class M(torch.nn.Module):
+            def __init__(self) -> None:
+                super().__init__()
+                self._torchbind_obj0 = torch.classes._TorchScriptTesting._Foo(2, 3)
+
+            def forward(self, x):
+                a = self._torchbind_obj0.add_tensor(x)
+                torchbind = torch.classes._TorchScriptTesting._Foo(4, 5)
+                b = torchbind.add_tensor(x)
+                return a + b
+
+        m = M()
+        inputs = (torch.ones(2, 3),)
+        orig_res = m(*inputs)
+
+        with enable_torchbind_tracing():
+            ep = torch.export.export(m, inputs, strict=False)
+
+        pt2_path = torch._inductor.aoti_compile_and_package(ep)
+        optimized = torch._inductor.aoti_load_package(pt2_path)
+        result = optimized(*inputs)
+        self.assertEqual(result, orig_res)
+
 
 if __name__ == "__main__":
     run_tests()
