@@ -19,18 +19,14 @@ from torch.testing import FileCheck
 # Make the helper files in test/ importable
 pytorch_test_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 sys.path.append(pytorch_test_dir)
-from torch.testing._internal.common_utils import skipIfTorchDynamo, TEST_CUDA
+from torch.testing._internal.common_utils import (
+    raise_on_run_directly,
+    skipIfTorchDynamo,
+    TEST_CUDA,
+)
 from torch.testing._internal.jit_utils import JitTestCase, make_global
 
 
-if __name__ == "__main__":
-    raise RuntimeError(
-        "This test file is not meant to be run directly, use:\n\n"
-        "\tpython test/test_jit.py TESTNAME\n\n"
-        "instead."
-    )
-
-
 class TestList(JitTestCase):
     def test_list_bool_conversion(self):
         def if_predicate(l: List[int]):
@@ -1777,7 +1773,7 @@ class TestDict(JitTestCase):
             return x
 
         self.checkScript(setdefault, (self.dict(), "a", torch.randn(2, 2)))
-        self.checkScript(setdefault, (self.dict(), "nonexistant", torch.randn(2, 2)))
+        self.checkScript(setdefault, (self.dict(), "nonexistent", torch.randn(2, 2)))
 
     @skipIfTorchDynamo("TorchDynamo fails for this test for unknown reason")
     def test_update(self):
@@ -1825,7 +1821,7 @@ class TestDict(JitTestCase):
     def test_popitem(self):
         @torch.jit.script
         def popitem(
-            x: Dict[str, Tensor]
+            x: Dict[str, Tensor],
         ) -> Tuple[Tuple[str, Tensor], Dict[str, Tensor]]:
             item = x.popitem()
             return item, x
@@ -1898,9 +1894,13 @@ class TestDict(JitTestCase):
 
         @torch.jit.script
         def missing_index(x: Dict[str, int]) -> int:
-            return x["dne"]
+            return x["dne"]  # codespell:ignore
 
-        with self.assertRaisesRegexWithHighlight(RuntimeError, "KeyError", 'x["dne"'):
+        with self.assertRaisesRegexWithHighlight(
+            RuntimeError,
+            "KeyError",
+            'x["dne"',  # codespell:ignore
+        ):
             missing_index({"item": 20, "other_item": 120})
 
         code = dedent(
@@ -2372,7 +2372,7 @@ class TestScriptDict(JitTestCase):
 
     The vast majority of tests are for making sure that objects returned
     by torch.jit.script behave like dictionaries do so that they are fungible
-    in almost all cirumstances with regular dictionaries.
+    in almost all circumstances with regular dictionaries.
     """
 
     def _script_dict_add(self, d: torch._C.ScriptDict, k: int, v: int):
@@ -2609,7 +2609,7 @@ class TestScriptList(JitTestCase):
 
     The vast majority of tests are for making sure that instances of
     torch._C.ScriptList behave like lists do so that they are fungible
-    in almost all cirumstances with regular list.
+    in almost all circumstances with regular list.
     """
 
     def _script_list_add(self, l: torch._C.ScriptList, e: int):
@@ -2983,7 +2983,7 @@ class TestScriptList(JitTestCase):
                 self.col2 = "b"
 
             def forward(self):
-                if self.col1 in self.segments_groupby_col.keys():
+                if self.col1 in self.segments_groupby_col:
                     return 1
                 else:
                     return 2
@@ -2993,6 +2993,10 @@ class TestScriptList(JitTestCase):
         test_script.segments_groupby_col
 
         # Smoketest for flakiness. Takes around 2s.
-        for i in range(300):
+        for _ in range(300):
             test = Test()
             test_script = torch.jit.script(test)
+
+
+if __name__ == "__main__":
+    raise_on_run_directly("test/test_jit.py")
