@@ -12,7 +12,11 @@ import torch.testing._internal.jit_utils
 from jit.test_module_interface import TestModuleInterface  # noqa: F401
 from torch import jit
 from torch.testing import FileCheck
-from torch.testing._internal.common_utils import freeze_rng_state
+from torch.testing._internal.common_utils import (
+    freeze_rng_state,
+    raise_on_run_directly,
+    skipIfTorchDynamo,
+)
 from torch.testing._internal.jit_utils import JitTestCase, make_global, RUN_CUDA_HALF
 
 
@@ -20,13 +24,6 @@ from torch.testing._internal.jit_utils import JitTestCase, make_global, RUN_CUDA
 pytorch_test_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 sys.path.append(pytorch_test_dir)
 
-if __name__ == "__main__":
-    raise RuntimeError(
-        "This test file is not meant to be run directly, use:\n\n"
-        "\tpython test/test_jit.py TESTNAME\n\n"
-        "instead."
-    )
-
 
 class TestMisc(JitTestCase):
     def test_joined_str(self):
@@ -129,7 +126,7 @@ class TestMisc(JitTestCase):
     def test_subexpression_Tuple_int_int_Future(self):
         @torch.jit.script
         def fn(
-            x: Tuple[int, int, torch.jit.Future[int]]
+            x: Tuple[int, int, torch.jit.Future[int]],
         ) -> Tuple[int, torch.jit.Future[int]]:
             return x[0], x[2]
 
@@ -147,7 +144,7 @@ class TestMisc(JitTestCase):
     def test_subexpression_Optional(self):
         @torch.jit.script
         def fn(
-            x: Optional[Dict[int, torch.jit.Future[int]]]
+            x: Optional[Dict[int, torch.jit.Future[int]]],
         ) -> Optional[torch.jit.Future[int]]:
             if x is not None:
                 return x[0]
@@ -440,6 +437,54 @@ class TestMisc(JitTestCase):
         self.assertTrue(ret.numel() == 1)
         self.assertTrue(len(ret.size()) == 1)
 
+    @skipIfTorchDynamo("The test case only test the parser. No need to wrap dynamo.")
+    def test_parse_ir_single_inf(self):
+        ir = """
+        graph():
+          %12 : float = prim::Constant[value=inf]()
+          return (%12)
+        """
+        graph = torch._C.parse_ir(ir, True)
+        func = torch._C._create_function_from_graph("forward", graph)
+        ret = func()
+        self.assertTrue(ret == float("inf"))
+
+    @skipIfTorchDynamo("The test case only test the parser. No need to wrap dynamo.")
+    def test_parse_ir_single_minus_inf(self):
+        ir = """
+        graph():
+          %12 : float = prim::Constant[value=-inf]()
+          return (%12)
+        """
+        graph = torch._C.parse_ir(ir, True)
+        func = torch._C._create_function_from_graph("forward", graph)
+        ret = func()
+        self.assertTrue(ret == float("-inf"))
+
+    @skipIfTorchDynamo("The test case only test the parser. No need to wrap dynamo.")
+    def test_parse_ir_bool_true(self):
+        ir = """
+        graph():
+          %12 : bool = prim::Constant[value=True]()
+          return (%12)
+        """
+        graph = torch._C.parse_ir(ir, True)
+        func = torch._C._create_function_from_graph("forward", graph)
+        ret = func()
+        self.assertTrue(ret == True)  # noqa: E712
+
+    @skipIfTorchDynamo("The test case only test the parser. No need to wrap dynamo.")
+    def test_parse_ir_bool_false(self):
+        ir = """
+        graph():
+          %12 : bool = prim::Constant[value=False]()
+          return (%12)
+        """
+        graph = torch._C.parse_ir(ir, True)
+        func = torch._C._create_function_from_graph("forward", graph)
+        ret = func()
+        self.assertTrue(ret == False)  # noqa: E712
+
     def test_script_many_decorators(self):
         def no_op_decorator(f):
             return f
@@ -473,7 +518,7 @@ class TestMisc(JitTestCase):
         ref = fn(x)
 
         script_fn = torch.jit.script(fn)
-        for i in range(4):
+        for _ in range(4):
             res = script_fn(x)
 
         self.assertEqual(ref, res)
@@ -504,3 +549,7 @@ class TestMisc(JitTestCase):
         self.assertTrue(len(complex_indices) > 0)
         self.assertTrue(len(Scalar_indices) > 0)
         self.assertTrue(complex_indices[0] > Scalar_indices[0])
+
+
+if __name__ == "__main__":
+    raise_on_run_directly("test/test_jit.py")
