@@ -78,7 +78,7 @@ class TestModuleContainers(JitTestCase):
                     x = mod(x)
                     values.append(x)
 
-                for key in self.moduledict.keys():
+                for key in self.moduledict:
                     names.append(key)
 
                 return x, names
@@ -300,13 +300,13 @@ class TestModuleContainers(JitTestCase):
 
                 # note: unable to index moduledict with a string variable currently
                 i = 0
-                for key in self.moduledict:
+                for _ in self.moduledict:
                     i += 1
                 assert i == len(self.moduledict), "iteration failing for ModuleDict"
 
                 assert "submod" in self.moduledict, "__contains__ fails for ModuleDict"
 
-                for key in self.moduledict.keys():
+                for key in self.moduledict:
                     assert key == "submod", "keys() fails for ModuleDict"
 
                 for item in self.moduledict.items():
