@@ -5,11 +5,9 @@ from __future__ import annotations
 
 import logging
 
-import onnxruntime
 import pytest
 import transformers
 from onnxscript import ir
-from packaging import version
 
 import torch
 from torch.onnx._internal.exporter import _testing as onnx_testing
@@ -17,12 +15,10 @@ from torch.testing._internal import common_utils
 from torch.utils import _pytree as torch_pytree
 
 
-def has_onnxruntime_opset_23() -> bool:
-    return version.parse(onnxruntime.__version__) >= version.parse("1.23")
-
-
 class _WithExport:
     def export(self, model, args=(), kwargs=None, **options) -> torch.onnx.ONNXProgram:
+        if isinstance(model, torch.nn.Module):
+            model = model.eval()
         onnx_program = torch.onnx.export(
             model,
             args,
@@ -746,13 +742,7 @@ class DynamoExporterNewOpsetsTest(common_utils.TestCase, _WithExport):
         onnx_program = self.export(Model(), (query, key, value), opset_version=23)
         self.assertEqual(["Attention"], [n.op_type for n in onnx_program.model.graph])
 
-        if has_onnxruntime_opset_23():
-            onnx_testing.assert_onnx_program(onnx_program, atol=1e-2, rtol=1)
-        else:
-            # Test with reference evaluator because ORT does not support the op as of version 1.22
-            onnx_testing.assert_onnx_program(
-                onnx_program, atol=1e-2, rtol=1, backend="reference"
-            )
+        onnx_testing.assert_onnx_program(onnx_program, atol=1e-2, rtol=1)
 
     def test_rms_norm(self):
         """Test RMS normalization with various configurations."""
@@ -763,7 +753,7 @@ class DynamoExporterNewOpsetsTest(common_utils.TestCase, _WithExport):
 
         x = torch.randn(2, 5, 3)
         onnx_program = self.export(RMSNormModel(), (x,), opset_version=23)
-        onnx_testing.assert_onnx_program(onnx_program, backend="reference")
+        onnx_testing.assert_onnx_program(onnx_program)
 
         # Test with multi-dimensional normalized_shape
         class RMSNormModel2D(torch.nn.Module):
@@ -772,7 +762,7 @@ class DynamoExporterNewOpsetsTest(common_utils.TestCase, _WithExport):
 
         x = torch.randn(2, 5, 7, 3)
         onnx_program = self.export(RMSNormModel2D(), (x,), opset_version=23)
-        onnx_testing.assert_onnx_program(onnx_program, backend="reference")
+        onnx_testing.assert_onnx_program(onnx_program)
 
     def test_rms_norm_with_weight(self):
         """Test RMS normalization with weight parameter."""
@@ -789,8 +779,7 @@ class DynamoExporterNewOpsetsTest(common_utils.TestCase, _WithExport):
 
         onnx_program = self.export(RMSNormWithWeight(), (x,), opset_version=23)
 
-        # Test with reference evaluator because ORT does not support the op as of version 1.22
-        onnx_testing.assert_onnx_program(onnx_program, backend="reference")
+        onnx_testing.assert_onnx_program(onnx_program)
 
     def test_rms_norm_with_eps(self):
         """Test RMS normalization with custom epsilon."""
@@ -803,8 +792,37 @@ class DynamoExporterNewOpsetsTest(common_utils.TestCase, _WithExport):
 
         onnx_program = self.export(RMSNormWithEps(), (x,), opset_version=23)
 
-        # Test with reference evaluator because ORT does not support the op as of version 1.22
-        onnx_testing.assert_onnx_program(onnx_program, backend="reference")
+        onnx_testing.assert_onnx_program(onnx_program)
+
+    def test_enable_gqa_in_attention_23_with_dropout(self):
+        class Model(torch.nn.Module):
+            def forward(self, q, k, v):
+                return torch.nn.functional.scaled_dot_product_attention(  # pylint: disable=not-callable
+                    q, k, v, enable_gqa=True, dropout_p=0.1
+                )
+
+        model = Model()
+
+        query = torch.randn(2, 4, 8, 16)
+        key = torch.randn(2, 2, 8, 16)
+        value = torch.randn(2, 2, 8, 16)
+
+        onnx_program = self.export(
+            model,
+            (
+                query,
+                key,
+                value,
+            ),
+            opset_version=23,
+        )
+        # opset23 only uses manually gqa path when dropout is enabled,
+        # and dropout makes the output non-deterministic,
+        # so we check for the presence of the ops used in that path.
+        all_ops = [node.op_type for node in onnx_program.model.graph]
+        self.assertIn("Unsqueeze", all_ops)
+        self.assertIn("Expand", all_ops)
+        self.assertIn("Reshape", all_ops)
 
 
 if __name__ == "__main__":
