@@ -2,6 +2,7 @@
 
 import json
 import os
+import sys
 import tempfile
 import unittest
 from typing import Any
@@ -32,6 +33,7 @@ from torch.testing._internal.common_utils import (
     run_tests,
     skipIfHpu,
     skipIfTorchDynamo,
+    TemporaryFileName,
     TEST_HPU,
     TEST_XPU,
     TestCase,
@@ -43,7 +45,7 @@ from torch.utils._triton import has_triton
 # This causes an issue in the multithreading test because we check all events
 # in that test with their tids. The events that correspond to these lingering
 # threads all have TID of (uint64_t)(-1) which is invalid.
-# The work around is turnning off monitoring thread when tqdm is loaded.
+# The work around is turning off monitoring thread when tqdm is loaded.
 # Since these are unit tests, it is safe to turn off monitor thread.
 try:
     import tqdm
@@ -147,23 +149,25 @@ class TestExecutionTrace(TestCase):
             or torch.profiler.ProfilerActivity.HPU in supported_activities()
         )
         # Create a temp file to save execution trace and kineto data.
-        fp = tempfile.NamedTemporaryFile("w+t", suffix=".et.json", delete=False)
-        fp.close()
-        kt = tempfile.NamedTemporaryFile(
-            mode="w+t", suffix=".kineto.json", delete=False
-        )
-        kt.close()
 
-        with profile(
-            activities=supported_activities(),
-            schedule=torch.profiler.schedule(
-                skip_first=3, wait=1, warmup=1, active=2, repeat=1
-            ),
-            on_trace_ready=trace_handler,
-            execution_trace_observer=(
-                ExecutionTraceObserver().register_callback(fp.name)
-            ),
-        ) as p:
+        with (
+            tempfile.NamedTemporaryFile("w+t", suffix=".et.json", delete=False) as fp,
+            tempfile.NamedTemporaryFile(
+                mode="w+t", suffix=".kineto.json", delete=False
+            ) as kt,
+            profile(
+                activities=supported_activities(),
+                schedule=torch.profiler.schedule(
+                    skip_first=3, wait=1, warmup=1, active=2, repeat=1
+                ),
+                on_trace_ready=trace_handler,
+                execution_trace_observer=(
+                    ExecutionTraceObserver().register_callback(fp.name)
+                ),
+            ) as p,
+        ):
+            trace_name = fp.name
+            kt_name = kt.name
             for idx in range(10):
                 with record_function(f"## LOOP {idx} ##"):
                     self.payload(device, use_device=use_device)
@@ -174,10 +178,11 @@ class TestExecutionTrace(TestCase):
         # print("Output kineto = ", kt.name)
         # print("Output ET = ", fp.name)
 
-        p.export_chrome_trace(kt.name)
+        p.export_chrome_trace(kt_name)
         self.assertEqual(trace_called_num, 1)
 
-        nodes = self.get_execution_trace_root(fp.name)
+        nodes = self.get_execution_trace_root(trace_name)
+        os.remove(trace_name)
         loop_count = 0
         found_root_node = False
         for n in nodes:
@@ -194,9 +199,10 @@ class TestExecutionTrace(TestCase):
         # in terms of record func ID (rf_id) and External IDs
         # both of these should match for the same trace window.
 
-        with open(kt.name) as f:
+        with open(kt_name) as f:
             kineto = json.load(f)
             events = kineto["traceEvents"]
+        os.remove(kt_name)
 
         # Look up rf_ids in both Execution and Kineto trace as two lists.
         rf_ids_et = self.get_execution_trace_rf_ids(nodes)
@@ -231,18 +237,20 @@ class TestExecutionTrace(TestCase):
             or torch.profiler.ProfilerActivity.HPU in supported_activities()
         )
         # Create a temp file to save kineto data.
-        kt = tempfile.NamedTemporaryFile(
-            mode="w+t", suffix=".kineto.json", delete=False
-        )
-        kt.close()
 
-        with profile(
-            activities=supported_activities(),
-            schedule=torch.profiler.schedule(
-                skip_first=3, wait=1, warmup=1, active=2, repeat=1
-            ),
-            on_trace_ready=trace_handler,
-        ) as p:
+        with (
+            tempfile.NamedTemporaryFile(
+                mode="w+t", suffix=".kineto.json", delete=False
+            ) as kt,
+            profile(
+                activities=supported_activities(),
+                schedule=torch.profiler.schedule(
+                    skip_first=3, wait=1, warmup=1, active=2, repeat=1
+                ),
+                on_trace_ready=trace_handler,
+            ) as p,
+        ):
+            kt_name = kt.name
             for idx in range(10):
                 with record_function(f"## LOOP {idx} ##"):
                     self.payload(device, use_device=use_device)
@@ -252,7 +260,8 @@ class TestExecutionTrace(TestCase):
         # print("Output kineto = ", kt.name)
         # print("Output ET = ", fp.name)
 
-        p.export_chrome_trace(kt.name)
+        p.export_chrome_trace(kt_name)
+
         self.assertEqual(trace_called_num, 1)
         et_path = p.execution_trace_observer.get_output_file_path()
         et_res_path = p.execution_trace_observer.get_resources_dir(et_path)
@@ -280,9 +289,10 @@ class TestExecutionTrace(TestCase):
         # in terms of record func ID (rf_id) and External IDs
         # both of these should match for the same trace window.
 
-        with open(kt.name) as f:
+        with open(kt_name) as f:
             kineto = json.load(f)
             events = kineto["traceEvents"]
+        os.remove(kt_name)
 
         # Look up rf_ids in both Execution and Kineto trace as two lists.
         rf_ids_et = self.get_execution_trace_rf_ids(nodes)
@@ -305,11 +315,11 @@ class TestExecutionTrace(TestCase):
         )
         # Create a temp file to save execution trace data.
         # Use a gzip file to test compression codepath
-        fp = tempfile.NamedTemporaryFile("w", suffix=".et.json.gz", delete=False)
-        fp.close()
+        with tempfile.NamedTemporaryFile("w", suffix=".et.json.gz", delete=False) as fp:
+            filename = fp.name
         expected_loop_events = 0
 
-        et = ExecutionTraceObserver().register_callback(fp.name)
+        et = ExecutionTraceObserver().register_callback(filename)
 
         et.start()
         for idx in range(5):
@@ -318,9 +328,10 @@ class TestExecutionTrace(TestCase):
                 self.payload(device, use_device=use_device)
         et.stop()
 
-        assert fp.name == et.get_output_file_path()
+        assert filename == et.get_output_file_path()
         et.unregister_callback()
-        nodes = self.get_execution_trace_root(fp.name)
+        nodes = self.get_execution_trace_root(filename)
+        os.remove(filename)
         loop_count = 0
         # Expected tensor object tuple size, in th form of:
         # [tensor_id, storage_id, offset, numel, itemsize, device_str]
@@ -364,6 +375,9 @@ class TestExecutionTrace(TestCase):
         self.assertTrue(p.execution_trace_observer is None)
 
     @unittest.skipIf(IS_WINDOWS, "torch.compile does not support WINDOWS")
+    @unittest.skipIf(
+        sys.version_info >= (3, 12), "torch.compile is not supported on python 3.12+"
+    )
     @unittest.skipIf(
         (not has_triton()) or (not TEST_CUDA and not TEST_XPU),
         "need triton and device(CUDA or XPU) availability to run",
@@ -383,10 +397,10 @@ class TestExecutionTrace(TestCase):
             fn(*inputs)
 
         # Create a temp file to save execution trace data.
-        fp = tempfile.NamedTemporaryFile("w+t", suffix="_et.json", delete=False)
-        fp.close()
+        with tempfile.NamedTemporaryFile("w+t", suffix="_et.json", delete=False) as fp:
+            filename = fp.name
         et = ExecutionTraceObserver()
-        et.register_callback(fp.name)
+        et.register_callback(filename)
         et.set_extra_resource_collection(True)
 
         with profile(
@@ -402,7 +416,8 @@ class TestExecutionTrace(TestCase):
                     fn(*inputs)
                 p.step()
 
-        nodes = self.get_execution_trace_root(fp.name)
+        nodes = self.get_execution_trace_root(filename)
+        os.remove(filename)
         found_captured_triton_kernel_node = False
         found_call_compiled_fx_graph = False
         for n in nodes:
@@ -419,6 +434,9 @@ class TestExecutionTrace(TestCase):
         assert found_call_compiled_fx_graph
 
     @unittest.skipIf(IS_WINDOWS, "torch.compile does not support WINDOWS")
+    @unittest.skipIf(
+        sys.version_info >= (3, 12), "torch.compile is not supported on python 3.12+"
+    )
     @unittest.skipIf(
         (not has_triton()) or (not TEST_CUDA and not TEST_XPU),
         "need triton and device(CUDA or XPU) availability to run",
@@ -426,7 +444,7 @@ class TestExecutionTrace(TestCase):
     @skipCPUIf(True, "skip CPU device for testing profiling triton")
     def test_execution_trace_env_enabled_with_pt2(self, device):
         # clean up the local cache for triton kernel
-        from torch._inductor.codecache import PyCodeCache as PyCodeCache
+        from torch._inductor.codecache import PyCodeCache
 
         PyCodeCache.cache_clear(purge=True)
 
@@ -482,13 +500,13 @@ class TestExecutionTrace(TestCase):
 
     @unittest.skipIf(IS_WINDOWS, "torch.compile does not support WINDOWS")
     @unittest.skipIf(
-        (not has_triton()) or (not TEST_CUDA and not TEST_XPU),
-        "need triton and device(CUDA or XPU) availability to run",
+        (not has_triton()) or (not TEST_CUDA),
+        "need triton and device CUDA availability to run",
     )
     @skipCPUIf(True, "skip CPU device for testing profiling triton")
     def test_triton_fx_graph_with_et(self, device):
         # clean up the local cache for triton kernel
-        from torch._inductor.codecache import PyCodeCache as PyCodeCache
+        from torch._inductor.codecache import PyCodeCache
 
         PyCodeCache.cache_clear(purge=True)
 
@@ -512,10 +530,11 @@ class TestExecutionTrace(TestCase):
         ):
             fn(*inputs)
 
-        fp = tempfile.NamedTemporaryFile("w+t", suffix="fx_graph_et.json", delete=False)
-        fp.close()
         et = ExecutionTraceObserver()
-        et.register_callback(fp.name)
+        with tempfile.NamedTemporaryFile(
+            "w+t", suffix="fx_graph_et.json", delete=False
+        ) as fp:
+            et.register_callback(fp.name)
         et.set_extra_resource_collection(True)
         with profile(
             activities=torch.profiler.supported_activities(),
@@ -584,6 +603,7 @@ class TestExecutionTrace(TestCase):
                             == '#   %cos : Tensor "f32[4, 4][1, 4]cuda:0"[num_users=1] = call_function[target=torch.ops.aten.cos.default](args = (%add,), kwargs = {})'  # noqa: B950
                         )
                         assert fx_graph[7] == "#   return %cos"
+                os.remove(file_path)
 
     def test_execution_trace_start_stop(self, device):
         use_device = (
@@ -592,10 +612,10 @@ class TestExecutionTrace(TestCase):
             or torch.profiler.ProfilerActivity.HPU in supported_activities()
         )
         # Create a temp file to save execution trace data.
-        fp = tempfile.NamedTemporaryFile("w+t", suffix=".et.json", delete=False)
-        fp.close()
+        with tempfile.NamedTemporaryFile("w+t", suffix=".et.json", delete=False) as fp:
+            filename = fp.name
         expected_loop_events = 0
-        et = ExecutionTraceObserver().register_callback(fp.name)
+        et = ExecutionTraceObserver().register_callback(filename)
         for idx in range(10):
             if idx == 3:
                 et.start()
@@ -610,9 +630,10 @@ class TestExecutionTrace(TestCase):
             with record_function(f"## LOOP {idx} ##"):
                 self.payload(device, use_device=use_device)
 
-        assert fp.name == et.get_output_file_path()
+        assert filename == et.get_output_file_path()
         et.unregister_callback()
-        nodes = self.get_execution_trace_root(fp.name)
+        nodes = self.get_execution_trace_root(filename)
+        os.remove(filename)
         loop_count = 0
         found_root_node = False
         for n in nodes:
@@ -636,10 +657,11 @@ class TestExecutionTrace(TestCase):
         for idx in range(10):
             if idx in iter_list:
                 # Create a temp file to save execution trace data.
-                fp = tempfile.NamedTemporaryFile("w+t", suffix=".et.json", delete=False)
-                fp.close()
-                output_files.append(fp.name)
-                et = ExecutionTraceObserver().register_callback(fp.name)
+                with tempfile.NamedTemporaryFile(
+                    "w+t", suffix=".et.json", delete=False
+                ) as fp:
+                    output_files.append(fp.name)
+                    et = ExecutionTraceObserver().register_callback(fp.name)
                 et.start()
             with record_function(f"## LOOP {idx} ##"):
                 self.payload(device, use_device=use_device)
@@ -662,29 +684,27 @@ class TestExecutionTrace(TestCase):
         assert event_count == expected_loop_events
 
     def test_execution_trace_no_capture(self):
-        fp = tempfile.NamedTemporaryFile("w+t", suffix=".et.json", delete=False)
-        fp.close()
-        et = ExecutionTraceObserver().register_callback(fp.name)
+        with TemporaryFileName("w+t", suffix=".et.json") as file_name:
+            et = ExecutionTraceObserver().register_callback(file_name)
 
-        assert fp.name == et.get_output_file_path()
-        et.unregister_callback()
-        nodes = self.get_execution_trace_root(fp.name)
-        for n in nodes:
-            assert "name" in n
-            if "[pytorch|profiler|execution_trace|process]" in n["name"]:
-                found_root_node = True
-        assert found_root_node
+            assert file_name == et.get_output_file_path()
+            et.unregister_callback()
+            nodes = self.get_execution_trace_root(file_name)
+            found_root_node = False
+            for n in nodes:
+                assert "name" in n
+                if "[pytorch|profiler|execution_trace|process]" in n["name"]:
+                    found_root_node = True
+            assert found_root_node
 
     @skipIfTorchDynamo("https://github.com/pytorch/pytorch/issues/124500")
     def test_execution_trace_nested_tensor(self):
-        fp = tempfile.NamedTemporaryFile("w+t", suffix=".et.json", delete=False)
-        fp.close()
-
-        observer = ExecutionTraceObserver().register_callback(fp.name)
-
         def fn(nt):
             return nt.sin().cos()
 
+        with tempfile.NamedTemporaryFile("w+t", suffix=".et.json", delete=False) as fp:
+            observer = ExecutionTraceObserver().register_callback(fp.name)
+            filename = fp.name
         with torch.profiler.profile(execution_trace_observer=observer):
             for i in range(3):
                 values = torch.rand((8 + i, 4 + i))
@@ -692,7 +712,8 @@ class TestExecutionTrace(TestCase):
                 nt = torch.nested.nested_tensor_from_jagged(values, offsets)
                 fn(nt)
 
-        nodes = self.get_execution_trace_root(fp.name)
+        nodes = self.get_execution_trace_root(filename)
+        os.remove(filename)
         found_cos = False
         for n in nodes:
             assert "name" in n
@@ -705,26 +726,28 @@ class TestExecutionTrace(TestCase):
         "need CUDA device availability to run",
     )
     def test_execution_trace_record_integral_tensor_range(self):
-        fp = tempfile.NamedTemporaryFile("w+t", suffix=".et.json", delete=False)
-        fp.close()
-
         os.environ["ENABLE_PYTORCH_EXECUTION_TRACE_SAVE_INTEGRAL_TENSOR_RANGE"] = "1"
         t1 = torch.tensor([[1, 2], [3, 4]]).cuda()
         t2 = torch.tensor([[0, 0], [1, 0]]).cuda()
-        with profile(
-            activities=supported_activities(),
-            schedule=torch.profiler.schedule(
-                skip_first=0, wait=0, warmup=0, active=1, repeat=1
-            ),
-            record_shapes=True,
-            execution_trace_observer=(
-                ExecutionTraceObserver().register_callback(fp.name)
-            ),
-        ) as p:
+        with (
+            tempfile.NamedTemporaryFile("w+t", suffix=".et.json", delete=False) as fp,
+            profile(
+                activities=supported_activities(),
+                schedule=torch.profiler.schedule(
+                    skip_first=0, wait=0, warmup=0, active=1, repeat=1
+                ),
+                record_shapes=True,
+                execution_trace_observer=(
+                    ExecutionTraceObserver().register_callback(fp.name)
+                ),
+            ) as p,
+        ):
+            filename = fp.name
             torch.gather(t1, 1, t2)
             p.step()
 
-        nodes = self.get_execution_trace_root(fp.name)
+        nodes = self.get_execution_trace_root(filename)
+        os.remove(filename)
         for n in nodes:
             assert "name" in n
             if "aten::gather" in n["name"]:
