@@ -4,12 +4,8 @@ import sys
 import unittest
 
 import torch
-from torch.testing._internal.common_cuda import (
-    _get_torch_cuda_version,
-    TEST_CUDA,
-    TEST_MULTIGPU,
-)
-from torch.testing._internal.common_utils import NoTest, run_tests, TestCase
+from torch.testing._internal.common_cuda import TEST_CUDA, TEST_MULTIGPU
+from torch.testing._internal.common_utils import NoTest, run_tests, skipIfRocm, TestCase
 
 
 # NOTE: this needs to be run in a brand new process
@@ -28,6 +24,7 @@ class TestCudaPrimaryCtx(TestCase):
     )
 
     def setUp(self):
+        super().setUp()
         for device in range(torch.cuda.device_count()):
             # Ensure context has not been created beforehand
             self.assertFalse(
@@ -35,18 +32,17 @@ class TestCudaPrimaryCtx(TestCase):
                 TestCudaPrimaryCtx.CTX_ALREADY_CREATED_ERR_MSG,
             )
 
+    @skipIfRocm(
+        msg="last checked in ROCm 7, HIP runtime doesn't create context for hipSetDevice()"
+    )
     def test_set_device_0(self):
         # In CUDA 12 the behavior of cudaSetDevice has changed. It eagerly creates context on target.
         # The behavior of `torch.cuda.set_device(0)` should also create context on the device 0.
         # Initially, we should not have any context on device 0.
         self.assertFalse(torch._C._cuda_hasPrimaryContext(0))
         torch.cuda.set_device(0)
-        if _get_torch_cuda_version() >= (12, 0):
-            # Now after the device was set, the context should present in CUDA 12.
-            self.assertTrue(torch._C._cuda_hasPrimaryContext(0))
-        else:
-            # In CUDA 11 the context should not be created.
-            self.assertFalse(torch._C._cuda_hasPrimaryContext(0))
+        # Now after the device was set, the context should present in CUDA 12.
+        self.assertTrue(torch._C._cuda_hasPrimaryContext(0))
 
     @unittest.skipIf(not TEST_MULTIGPU, "only one GPU detected")
     def test_str_repr(self):
