@@ -199,7 +199,7 @@ class TestFunctionalization(TestCase):
             y.set_(x.storage())
             return y
 
-        # We should probaby get the crossref test to work,
+        # We should probably get the crossref test to work,
         # but fixing it for Storage() objects is annoying.
         r = _functionalize(f, reapply_views=True, crossref=False)(torch.ones(2))
         self.assertEqual(str(r.device), "cpu")
@@ -2318,7 +2318,7 @@ def forward(self, arg0_1):
     ]
 )
 @unittest.skipIf(
-    TEST_WITH_TORCHDYNAMO, "dynamo-ing code with proxy + fake doesnt work well"
+    TEST_WITH_TORCHDYNAMO, "dynamo-ing code with proxy + fake doesn't work well"
 )
 class TestCrossRefFunctionalization(TestFunctionalization):
     crossref = True
