@@ -1,3 +1,5 @@
+# Owner(s): ["oncall: jit"]
+
 import sys
 import os
 import contextlib
@@ -44,9 +46,10 @@ class TestJitDisabled(TestCase):
     def test_attribute(self):
         _program_string = """
 import torch
+
 class Foo(torch.jit.ScriptModule):
     def __init__(self, x):
-        super(Foo, self).__init__()
+        super().__init__()
         self.x = torch.jit.Attribute(x, torch.Tensor)
 
     def forward(self, input):
@@ -62,8 +65,6 @@ print(s.x)
 import torch
 
 class AModule(torch.jit.ScriptModule):
-    def __init__(self):
-        super(AModule, self).__init__()
     @torch.jit.script_method
     def forward(self, input):
         pass
@@ -78,9 +79,6 @@ print("Didn't throw exception")
 import torch
 
 class AModule(torch.nn.Module):
-    def __init__(self):
-        super(AModule, self).__init__()
-
     def forward(self, input):
         pass
 
@@ -90,4 +88,5 @@ print("Didn't throw exception")
         self.compare_enabled_disabled(_program_string)
 
 if __name__ == '__main__':
-    run_tests()
+    if sys.version_info < (3, 14):
+        run_tests()
