@@ -280,6 +280,11 @@ class TestNamedTensor(TestCase):
         self.assertEqual(named_tensor.diagonal(outdim='E', dim1='B', dim2='D').names,
                          ['A', 'C', 'E'])
 
+    def test_empty_names(self):
+        ref_tensor = torch.tensor([[1, 2, 3, 4], [4, 3, 2, 1]])
+        empty_named_tensor = torch.tensor([[1, 2, 3, 4], [4, 3, 2, 1]], names=[])
+        self.assertEqual(ref_tensor, empty_named_tensor)
+
     def test_max_pooling(self):
         def check_tuple_return(op, inputs, expected_names):
             values, indices = op(*inputs)
@@ -997,7 +1002,7 @@ class TestNamedTensor(TestCase):
         def test_ops(op):
             for device in get_all_device_types():
                 names = ('N', 'D')
-                tensor = torch.rand(2, 3, names=names)
+                tensor = torch.rand(2, 3, names=names, device=device)
                 result = op(tensor, 0)
                 self.assertEqual(result[0].names, names)
                 self.assertEqual(result[1].names, names)
@@ -1007,15 +1012,15 @@ class TestNamedTensor(TestCase):
     def test_logcumsumexp(self):
         for device in get_all_device_types():
             names = ('N', 'D')
-            tensor = torch.rand(2, 3, names=names)
+            tensor = torch.rand(2, 3, names=names, device=device)
             result = torch.logcumsumexp(tensor, 'D')
             self.assertEqual(result.names, names)
 
     def test_bitwise_not(self):
         for device in get_all_device_types():
             names = ('N', 'D')
-            tensor = torch.zeros(2, 3, names=names, dtype=torch.bool)
-            result = torch.empty(0, dtype=torch.bool)
+            tensor = torch.zeros(2, 3, names=names, dtype=torch.bool, device=device)
+            result = torch.empty(0, dtype=torch.bool, device=device)
 
             self.assertEqual(tensor.bitwise_not().names, names)
             self.assertEqual(torch.bitwise_not(tensor, out=result).names, names)
@@ -1024,8 +1029,8 @@ class TestNamedTensor(TestCase):
     def test_logical_not(self):
         for device in get_all_device_types():
             names = ('N', 'D')
-            tensor = torch.zeros(2, 3, names=names, dtype=torch.bool)
-            result = torch.empty(0, dtype=torch.bool)
+            tensor = torch.zeros(2, 3, names=names, dtype=torch.bool, device=device)
+            result = torch.empty(0, dtype=torch.bool, device=device)
 
             self.assertEqual(tensor.logical_not().names, names)
             self.assertEqual(torch.logical_not(tensor, out=result).names, names)
@@ -1034,8 +1039,8 @@ class TestNamedTensor(TestCase):
     def test_bernoulli(self):
         for device in get_all_device_types():
             names = ('N', 'D')
-            tensor = torch.rand(2, 3, names=names)
-            result = torch.empty(0)
+            tensor = torch.rand(2, 3, names=names, device=device)
+            result = torch.empty(0, device=device)
             self.assertEqual(tensor.bernoulli().names, names)
 
             torch.bernoulli(tensor, out=result)
