@@ -22,6 +22,7 @@ from torch.testing._internal.common_utils import (
     parametrize,
     run_tests,
     subtest,
+    TEST_WITH_TORCHDYNAMO,
     TestCase,
 )
 
@@ -52,6 +53,14 @@ class GlobalDummyType:
         self.x = x
         self.y = y
 
+    def __eq__(self, other):
+        if not isinstance(other, GlobalDummyType):
+            return NotImplemented
+        return self.x == other.x and self.y == other.y
+
+    def __hash__(self):
+        return hash((self.x, self.y))
+
 
 cxx_pytree.register_pytree_node(
     GlobalDummyType,
@@ -65,9 +74,6 @@ class TestEnum(enum.Enum):
     A = auto()
 
 
-python_leafspec = python_pytree.LeafSpec()
-
-
 class TestGenericPytree(TestCase):
     def test_aligned_public_apis(self):
         public_apis = python_pytree.__all__
@@ -197,7 +203,7 @@ class TestGenericPytree(TestCase):
         def run_test_with_leaf(leaf):
             values, treespec = pytree.tree_flatten(leaf)
             self.assertEqual(values, [leaf])
-            self.assertEqual(treespec, pytree.LeafSpec())
+            self.assertEqual(treespec, pytree.treespec_leaf())
 
             unflattened = pytree.tree_unflatten(values, treespec)
             self.assertEqual(unflattened, leaf)
@@ -215,7 +221,7 @@ class TestGenericPytree(TestCase):
                 (
                     python_pytree,
                     lambda tup: python_pytree.TreeSpec(
-                        tuple, None, [python_leafspec for _ in tup]
+                        tuple, None, [python_pytree.treespec_leaf() for _ in tup]
                     ),
                 ),
                 name="python",
@@ -250,7 +256,7 @@ class TestGenericPytree(TestCase):
                 (
                     python_pytree,
                     lambda lst: python_pytree.TreeSpec(
-                        list, None, [python_leafspec for _ in lst]
+                        list, None, [python_pytree.treespec_leaf() for _ in lst]
                     ),
                 ),
                 name="python",
@@ -286,7 +292,7 @@ class TestGenericPytree(TestCase):
                     lambda dct: python_pytree.TreeSpec(
                         dict,
                         list(dct.keys()),
-                        [python_leafspec for _ in dct.values()],
+                        [python_pytree.treespec_leaf() for _ in dct.values()],
                     ),
                 ),
                 name="python",
@@ -327,7 +333,7 @@ class TestGenericPytree(TestCase):
                     lambda odict: python_pytree.TreeSpec(
                         OrderedDict,
                         list(odict.keys()),
-                        [python_leafspec for _ in odict.values()],
+                        [python_pytree.treespec_leaf() for _ in odict.values()],
                     ),
                 ),
                 name="python",
@@ -371,7 +377,7 @@ class TestGenericPytree(TestCase):
                     lambda ddct: python_pytree.TreeSpec(
                         defaultdict,
                         [ddct.default_factory, list(ddct.keys())],
-                        [python_leafspec for _ in ddct.values()],
+                        [python_pytree.treespec_leaf() for _ in ddct.values()],
                     ),
                 ),
                 name="python",
@@ -413,7 +419,7 @@ class TestGenericPytree(TestCase):
                 (
                     python_pytree,
                     lambda deq: python_pytree.TreeSpec(
-                        deque, deq.maxlen, [python_leafspec for _ in deq]
+                        deque, deq.maxlen, [python_pytree.treespec_leaf() for _ in deq]
                     ),
                 ),
                 name="python",
@@ -453,7 +459,7 @@ class TestGenericPytree(TestCase):
         def run_test(tup):
             if pytree is python_pytree:
                 expected_spec = python_pytree.TreeSpec(
-                    namedtuple, Point, [python_leafspec for _ in tup]
+                    namedtuple, Point, [python_pytree.treespec_leaf() for _ in tup]
                 )
             else:
                 expected_spec = cxx_pytree.tree_structure(Point(0, 1))
@@ -604,6 +610,24 @@ class TestGenericPytree(TestCase):
         for case in cases:
             run_test(case)
 
+    @parametrize_pytree_module
+    def test_tree_map_dict_order(self, pytree):
+        d = {"b": 2, "a": 1, "c": 3}
+        od = OrderedDict([("b", 2), ("a", 1), ("c", 3)])
+        dd = defaultdict(int, {"b": 2, "a": 1, "c": 3})
+        for tree in (d, od, dd):
+            result = pytree.tree_map(lambda x: x, tree)
+            self.assertEqual(
+                list(result.keys()),
+                list(tree.keys()),
+                msg=f"Dictionary keys order changed in tree_map: {tree!r} vs. {result!r}",
+            )
+            self.assertEqual(
+                list(result.values()),
+                list(tree.values()),
+                msg=f"Dictionary keys order changed in tree_map: {tree!r} vs. {result!r}",
+            )
+
     @parametrize_pytree_module
     def test_tree_map_only(self, pytree):
         self.assertEqual(pytree.tree_map_only(int, lambda x: x + 2, [0, "a"]), [2, "a"])
@@ -848,16 +872,16 @@ if "optree" in sys.modules:
 
     def test_treespec_equality(self):
         self.assertEqual(
-            python_pytree.LeafSpec(),
-            python_pytree.LeafSpec(),
+            python_pytree.treespec_leaf(),
+            python_pytree.treespec_leaf(),
         )
         self.assertEqual(
             python_pytree.TreeSpec(list, None, []),
             python_pytree.TreeSpec(list, None, []),
         )
         self.assertEqual(
-            python_pytree.TreeSpec(list, None, [python_pytree.LeafSpec()]),
-            python_pytree.TreeSpec(list, None, [python_pytree.LeafSpec()]),
+            python_pytree.TreeSpec(list, None, [python_pytree.treespec_leaf()]),
+            python_pytree.TreeSpec(list, None, [python_pytree.treespec_leaf()]),
         )
         self.assertFalse(
             python_pytree.TreeSpec(tuple, None, [])
@@ -892,24 +916,32 @@ if "optree" in sys.modules:
             # python_pytree.tree_structure({})
             python_pytree.TreeSpec(dict, [], []),
             # python_pytree.tree_structure([0])
-            python_pytree.TreeSpec(list, None, [python_leafspec]),
+            python_pytree.TreeSpec(list, None, [python_pytree.treespec_leaf()]),
             # python_pytree.tree_structure([0, 1])
             python_pytree.TreeSpec(
                 list,
                 None,
-                [python_leafspec, python_leafspec],
+                [python_pytree.treespec_leaf(), python_pytree.treespec_leaf()],
             ),
             # python_pytree.tree_structure((0, 1, 2))
             python_pytree.TreeSpec(
                 tuple,
                 None,
-                [python_leafspec, python_leafspec, python_leafspec],
+                [
+                    python_pytree.treespec_leaf(),
+                    python_pytree.treespec_leaf(),
+                    python_pytree.treespec_leaf(),
+                ],
             ),
             # python_pytree.tree_structure({"a": 0, "b": 1, "c": 2})
             python_pytree.TreeSpec(
                 dict,
                 ["a", "b", "c"],
-                [python_leafspec, python_leafspec, python_leafspec],
+                [
+                    python_pytree.treespec_leaf(),
+                    python_pytree.treespec_leaf(),
+                    python_pytree.treespec_leaf(),
+                ],
             ),
             # python_pytree.tree_structure(OrderedDict([("a", (0, 1)), ("b", 2), ("c", {"a": 3, "b": 4, "c": 5})])
             python_pytree.TreeSpec(
@@ -919,13 +951,17 @@ if "optree" in sys.modules:
                     python_pytree.TreeSpec(
                         tuple,
                         None,
-                        [python_leafspec, python_leafspec],
+                        [python_pytree.treespec_leaf(), python_pytree.treespec_leaf()],
                     ),
-                    python_leafspec,
+                    python_pytree.treespec_leaf(),
                     python_pytree.TreeSpec(
                         dict,
                         ["a", "b", "c"],
-                        [python_leafspec, python_leafspec, python_leafspec],
+                        [
+                            python_pytree.treespec_leaf(),
+                            python_pytree.treespec_leaf(),
+                            python_pytree.treespec_leaf(),
+                        ],
                     ),
                 ],
             ),
@@ -938,12 +974,15 @@ if "optree" in sys.modules:
                         tuple,
                         None,
                         [
-                            python_leafspec,
-                            python_leafspec,
+                            python_pytree.treespec_leaf(),
+                            python_pytree.treespec_leaf(),
                             python_pytree.TreeSpec(
                                 list,
                                 None,
-                                [python_leafspec, python_leafspec],
+                                [
+                                    python_pytree.treespec_leaf(),
+                                    python_pytree.treespec_leaf(),
+                                ],
                             ),
                         ],
                     ),
@@ -957,12 +996,12 @@ if "optree" in sys.modules:
                     python_pytree.TreeSpec(
                         list,
                         None,
-                        [python_leafspec, python_leafspec],
+                        [python_pytree.treespec_leaf(), python_pytree.treespec_leaf()],
                     ),
                     python_pytree.TreeSpec(
                         list,
                         None,
-                        [python_leafspec, python_leafspec],
+                        [python_pytree.treespec_leaf(), python_pytree.treespec_leaf()],
                     ),
                     python_pytree.TreeSpec(dict, [], []),
                 ],
@@ -991,7 +1030,7 @@ if "optree" in sys.modules:
                     list,
                     None,
                     [
-                        python_leafspec,
+                        python_pytree.treespec_leaf(),
                     ],
                 ),
             ],
@@ -1000,7 +1039,7 @@ if "optree" in sys.modules:
         self.assertIsInstance(serialized_spec, str)
 
     def test_pytree_serialize_enum(self):
-        spec = python_pytree.TreeSpec(dict, TestEnum.A, [python_leafspec])
+        spec = python_pytree.TreeSpec(dict, TestEnum.A, [python_pytree.treespec_leaf()])
 
         serialized_spec = python_pytree.treespec_dumps(spec)
         self.assertIsInstance(serialized_spec, str)
@@ -1163,12 +1202,20 @@ if "optree" in sys.modules:
             OrderedDict,
             [1, 2, 3],
             [
-                python_pytree.TreeSpec(tuple, None, [python_leafspec, python_leafspec]),
-                python_leafspec,
+                python_pytree.TreeSpec(
+                    tuple,
+                    None,
+                    [python_pytree.treespec_leaf(), python_pytree.treespec_leaf()],
+                ),
+                python_pytree.treespec_leaf(),
                 python_pytree.TreeSpec(
                     dict,
                     [4, 5, 6],
-                    [python_leafspec, python_leafspec, python_leafspec],
+                    [
+                        python_pytree.treespec_leaf(),
+                        python_pytree.treespec_leaf(),
+                        python_pytree.treespec_leaf(),
+                    ],
                 ),
             ],
         )
@@ -1452,8 +1499,27 @@ class TestCxxPytree(TestCase):
         if IS_FBCODE:
             raise unittest.SkipTest("C++ pytree tests are not supported in fbcode")
 
+    def assertEqual(self, x, y, *args, **kwargs):
+        x_typename, y_typename = type(x).__name__, type(y).__name__
+        if not ("treespec" in x_typename.lower() or "treespec" in y_typename.lower()):
+            super().assertEqual(x, y, *args, **kwargs)
+
+        # The Dynamo polyfill returns a polyfilled Python class for C++ PyTreeSpec instead of the
+        # C++ class. So we compare the type names and reprs instead because the types themselves
+        # won't be equal.
+        super().assertEqual(x_typename, y_typename, *args, **kwargs)
+        if not TEST_WITH_TORCHDYNAMO or type(x) is type(y):
+            super().assertEqual(x, y, *args, **kwargs)
+        else:
+            super().assertEqual(
+                x.unflatten(range(x.num_leaves)),
+                y.unflatten(range(y.num_leaves)),
+                *args,
+                **kwargs,
+            )
+
     def test_treespec_equality(self):
-        self.assertEqual(cxx_pytree.LeafSpec(), cxx_pytree.LeafSpec())
+        self.assertEqual(cxx_pytree.treespec_leaf(), cxx_pytree.treespec_leaf())
 
     def test_treespec_repr(self):
         # Check that it looks sane
@@ -1492,7 +1558,9 @@ class TestCxxPytree(TestCase):
 
         serialized_spec = cxx_pytree.treespec_dumps(spec)
         self.assertIsInstance(serialized_spec, str)
-        self.assertEqual(spec, cxx_pytree.treespec_loads(serialized_spec))
+
+        roundtrip_spec = cxx_pytree.treespec_loads(serialized_spec)
+        self.assertEqual(roundtrip_spec, spec)
 
     def test_pytree_serialize_namedtuple(self):
         python_pytree._register_namedtuple(
@@ -1525,6 +1593,14 @@ class TestCxxPytree(TestCase):
                 self.x = x
                 self.y = y
 
+            def __eq__(self, other):
+                if not isinstance(other, LocalDummyType):
+                    return NotImplemented
+                return self.x == other.x and self.y == other.y
+
+            def __hash__(self):
+                return hash((self.x, self.y))
+
         cxx_pytree.register_pytree_node(
             LocalDummyType,
             lambda dummy: ([dummy.x, dummy.y], None),
