@@ -367,6 +367,31 @@ class TestConvolutionNNDeviceType(NNTestCase):
                     no_weight,
                 )
 
+    @dtypes(torch.float)
+    def test_conv1d_large_input(self, device, dtype):
+        N, C_in, L = 4, 512, 441
+        C_out, K, P = 512, 3, 1
+        torch.manual_seed(42)
+
+        conv_cpu = (
+            nn.Conv1d(C_in, C_out, kernel_size=K, padding=P, bias=True)
+            .to(torch.float32)
+            .requires_grad_()
+        )
+        x_cpu = torch.randn(N, C_in, L, dtype=torch.float32)
+        out_cpu = conv_cpu(x_cpu)
+
+        conv_dev = nn.Conv1d(C_in, C_out, kernel_size=K, padding=P, bias=True).to(
+            device, dtype
+        )
+        conv_dev.weight.data.copy_(conv_cpu.weight.data.to(dtype))
+        conv_dev.bias.data.copy_(conv_cpu.bias.data.to(dtype))
+
+        x_dev = x_cpu.to(device, dtype).requires_grad_()
+        out_dev = conv_dev(x_dev)
+
+        self.assertEqual(out_cpu, out_dev, atol=1e-5, rtol=1e-5, exact_device=False)
+
     @dtypes(torch.float)
     def test_conv1d_same_padding(self, device, dtype):
         test_args = [
