@@ -1,22 +1,23 @@
 from dataclasses import dataclass, field
 from datetime import datetime
-from typing import Optional
 
-from dataclasses_json import DataClassJsonMixin
+#  pyrefly: ignore [missing-import]
+from dataclasses_json import DataClassJsonMixin  # type: ignore[import-not-found]
 
 
-_DATA_MODEL_VERSION = 1.0
+_DATA_MODEL_VERSION = 1.5
 
 
 # data model for test log usage
 @dataclass
 class UtilizationStats:
-    avg: Optional[float] = None
-    max: Optional[float] = None
+    avg: float | None = None
+    max: float | None = None
+    raw: list[float] | None = None
 
 
 @dataclass
-class UtilizationMetadata(DataClassJsonMixin):
+class UtilizationMetadata(DataClassJsonMixin):  # type: ignore[misc, no-any-unimported]
     level: str
     workflow_id: str
     job_id: str
@@ -25,41 +26,44 @@ class UtilizationMetadata(DataClassJsonMixin):
     usage_collect_interval: float
     data_model_version: float
     start_at: int
-    gpu_count: Optional[int] = None
-    cpu_count: Optional[int] = None
-    gpu_type: Optional[str] = None
-    error: Optional[str] = None
+    gpu_count: int | None = None
+    cpu_count: int | None = None
+    gpu_type: str | None = None
+    error: str | None = None
 
 
 @dataclass
-class GpuUsage(DataClassJsonMixin):
-    uuid: Optional[str] = None
-    util_percent: Optional[UtilizationStats] = None
-    mem_util_percent: Optional[UtilizationStats] = None
+class GpuUsage(DataClassJsonMixin):  # type: ignore[misc, no-any-unimported]
+    uuid: str | None = None
+    util_percent: UtilizationStats | None = None
+    mem_util_percent: UtilizationStats | None = None
+    allocated_mem_percent: UtilizationStats | None = None
+    allocated_mem_value: UtilizationStats | None = None
+    total_mem_value: float | None = None
 
 
 @dataclass
-class RecordData(DataClassJsonMixin):
-    cpu: Optional[UtilizationStats] = None
-    memory: Optional[UtilizationStats] = None
-    gpu_usage: Optional[list[GpuUsage]] = None
+class RecordData(DataClassJsonMixin):  # type: ignore[misc, no-any-unimported]
+    cpu: UtilizationStats | None = None
+    memory: UtilizationStats | None = None
+    gpu_usage: list[GpuUsage] | None = None
 
 
 @dataclass
-class UtilizationRecord(DataClassJsonMixin):
+class UtilizationRecord(DataClassJsonMixin):  # type: ignore[misc, no-any-unimported]
     level: str
     timestamp: int
-    data: Optional[RecordData] = None
-    cmd_names: Optional[list[str]] = None
-    error: Optional[str] = None
-    log_duration: Optional[str] = None
-    logs: Optional[list[str]] = None
+    data: RecordData | None = None
+    cmd_names: list[str] | None = None
+    error: str | None = None
+    log_duration: str | None = None
+    logs: list[str] | None = None
 
 
 # the db schema related to this is:
 # https://github.com/pytorch/test-infra/blob/main/clickhouse_db_schema/oss_ci_utilization/oss_ci_utilization_metadata_schema.sql
 @dataclass
-class OssCiSegmentV1(DataClassJsonMixin):
+class OssCiSegmentV1(DataClassJsonMixin):  # type: ignore[misc, no-any-unimported]
     level: str
     name: str
     start_at: int
