@@ -5,13 +5,12 @@
 
 #include <vector>
 
-namespace torch {
-namespace nn {
+namespace torch::nn {
 class ParameterListImpl : public Cloneable<ParameterListImpl> {
  public:
-  using Iterator = typename std::vector<
-      OrderedDict<std::string, torch::Tensor>::Item>::iterator;
-  using ConstIterator = typename std::vector<
+  using Iterator =
+      std::vector<OrderedDict<std::string, torch::Tensor>::Item>::iterator;
+  using ConstIterator = std::vector<
       OrderedDict<std::string, torch::Tensor>::Item>::const_iterator;
 
   ParameterListImpl() = default;
@@ -35,37 +34,37 @@ class ParameterListImpl : public Cloneable<ParameterListImpl> {
 
   /// Pretty prints the `ParameterList` module into the given `stream`.
   void pretty_print(std::ostream& stream) const override {
-    stream << "torch::nn::ParameterList(" << std::endl;
+    stream << "torch::nn::ParameterList(" << '\n';
     for (const auto& pair : parameters_) {
-      stream << "(" << pair.key() << ")"
-             << ": Parameter containing: [" << pair.value().scalar_type()
-             << " of size " << pair.value().sizes() << "]";
+      stream << '(' << pair.key() << ')' << ": Parameter containing: ["
+             << pair.value().scalar_type() << " of size "
+             << pair.value().sizes() << ']';
       ;
-      stream << std::endl;
+      stream << '\n';
     }
-    stream << ")";
+    stream << ')';
   }
 
   /// push the a given parameter at the end of the list
   void append(torch::Tensor&& param) {
     bool requires_grad = param.requires_grad();
     register_parameter(
-        c10::to_string(parameters_.size()), std::move(param), requires_grad);
+        std::to_string(parameters_.size()), std::move(param), requires_grad);
   }
 
   /// push the a given parameter at the end of the list
   void append(const torch::Tensor& param) {
     bool requires_grad = param.requires_grad();
     register_parameter(
-        c10::to_string(parameters_.size()), param, requires_grad);
+        std::to_string(parameters_.size()), param, requires_grad);
   }
 
   /// push the a given parameter at the end of the list
   /// And the key of the pair will be discarded, only the value
-  /// will be added into the `ParameterList`  
+  /// will be added into the `ParameterList`
   void append(const OrderedDict<std::string, torch::Tensor>::Item& pair) {
     register_parameter(
-        c10::to_string(parameters_.size()),
+        std::to_string(parameters_.size()),
         pair.value(),
         pair.value().requires_grad());
   }
@@ -111,7 +110,7 @@ class ParameterListImpl : public Cloneable<ParameterListImpl> {
   /// for a non-throwing way of access
   at::Tensor& at(size_t idx) {
     TORCH_CHECK(idx < size(), "Index out of range");
-    return parameters_[c10::to_string(idx)];
+    return parameters_[std::to_string(idx)];
   }
 
   /// Returns the value associated with the given `key`. Throws an exception if
@@ -119,7 +118,7 @@ class ParameterListImpl : public Cloneable<ParameterListImpl> {
   /// for a non-throwing way of access
   const at::Tensor& at(size_t idx) const {
     TORCH_CHECK(idx < size(), "Index out of range");
-    return parameters_[c10::to_string(idx)];
+    return parameters_[std::to_string(idx)];
   }
 
   /// Returns the value associated with the given `key`. Throws an exception if
@@ -165,5 +164,4 @@ class ParameterListImpl : public Cloneable<ParameterListImpl> {
   void push_back_var() {}
 };
 TORCH_MODULE(ParameterList);
-} // namespace nn
-} // namespace torch
+} // namespace torch::nn
