@@ -1,12 +1,11 @@
 #pragma once
 
 #include <torch/arg.h>
+#include <torch/csrc/Export.h>
 #include <torch/enum.h>
-#include <torch/csrc/WindowsTorchApiMacro.h>
 #include <torch/types.h>
 
-namespace torch {
-namespace nn {
+namespace torch::nn {
 
 /// Options for the `L1Loss` module.
 ///
@@ -15,7 +14,8 @@ namespace nn {
 /// L1Loss model(L1LossOptions(torch::kNone));
 /// ```
 struct TORCH_API L1LossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   TORCH_OPTIONS_CTOR_VARIANT_ARG3(L1LossOptions, reduction, kNone, kMean, kSum)
 
@@ -43,12 +43,24 @@ using L1LossFuncOptions = L1LossOptions;
 ///
 /// Example:
 /// ```
-/// KLDivLoss model(KLDivLossOptions().reduction(torch::kNone).log_target(false));
+/// KLDivLoss
+/// model(KLDivLossOptions().reduction(torch::kNone).log_target(false));
 /// ```
 struct TORCH_API KLDivLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kBatchMean, enumtype::kSum, enumtype::kMean> reduction_t;
+  typedef std::variant<
+      enumtype::kNone,
+      enumtype::kBatchMean,
+      enumtype::kSum,
+      enumtype::kMean>
+      reduction_t;
 
-  TORCH_OPTIONS_CTOR_VARIANT_ARG4(KLDivLossOptions, reduction, kNone, kBatchMean, kSum, kMean)
+  TORCH_OPTIONS_CTOR_VARIANT_ARG4(
+      KLDivLossOptions,
+      reduction,
+      kNone,
+      kBatchMean,
+      kSum,
+      kMean)
 
   /// Specifies the reduction to apply to the output.
   /// ``'none'`` | ``'batchmean'`` | ``'sum'`` | ``'mean'``. Default: ``'mean'``
@@ -67,7 +79,8 @@ namespace functional {
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::kl_div(input, target, F::KLDivFuncOptions().reduction(torch::kNone).log_target(false));
+/// F::kl_div(input, target,
+/// F::KLDivFuncOptions().reduction(torch::kNone).log_target(false));
 /// ```
 using KLDivFuncOptions = KLDivLossOptions;
 } // namespace functional
@@ -81,7 +94,8 @@ using KLDivFuncOptions = KLDivLossOptions;
 /// MSELoss model(MSELossOptions(torch::kNone));
 /// ```
 struct TORCH_API MSELossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   TORCH_OPTIONS_CTOR_VARIANT_ARG3(MSELossOptions, reduction, kNone, kMean, kSum)
 
@@ -113,10 +127,11 @@ using MSELossFuncOptions = MSELossOptions;
 /// BCELoss model(BCELossOptions().reduction(torch::kNone).weight(weight));
 /// ```
 struct TORCH_API BCELossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   /// A manual rescaling weight given to the loss of each batch element.
-  TORCH_ARG(Tensor, weight) = {};
+  TORCH_ARG(Tensor, weight);
   /// Specifies the reduction to apply to the output.
   /// ``'none'`` | ``'mean'`` | ``'sum'``. Default: ``'mean'``
   TORCH_ARG(reduction_t, reduction) = torch::kMean;
@@ -131,7 +146,8 @@ namespace functional {
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::binary_cross_entropy(input, target, F::BinaryCrossEntropyFuncOptions().weight(weight));
+/// F::binary_cross_entropy(input, target,
+/// F::BinaryCrossEntropyFuncOptions().weight(weight));
 /// ```
 using BinaryCrossEntropyFuncOptions = BCELossOptions;
 } // namespace functional
@@ -142,10 +158,12 @@ using BinaryCrossEntropyFuncOptions = BCELossOptions;
 ///
 /// Example:
 /// ```
-/// HingeEmbeddingLoss model(HingeEmbeddingLossOptions().margin(4).reduction(torch::kNone));
+/// HingeEmbeddingLoss
+/// model(HingeEmbeddingLossOptions().margin(4).reduction(torch::kNone));
 /// ```
 struct TORCH_API HingeEmbeddingLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   /// Specifies the threshold for which the distance of a negative sample must
   /// reach in order to incur zero loss. Default: 1
@@ -157,13 +175,14 @@ struct TORCH_API HingeEmbeddingLossOptions {
 namespace functional {
 /// Options for `torch::nn::functional::hinge_embedding_loss`.
 ///
-/// See the documentation for `torch::nn::HingeEmbeddingLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::HingeEmbeddingLossOptions` class to
+/// learn what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::hinge_embedding_loss(input, target, F::HingeEmbeddingLossFuncOptions().margin(2));
+/// F::hinge_embedding_loss(input, target,
+/// F::HingeEmbeddingLossFuncOptions().margin(2));
 /// ```
 using HingeEmbeddingLossFuncOptions = HingeEmbeddingLossOptions;
 } // namespace functional
@@ -177,7 +196,8 @@ using HingeEmbeddingLossFuncOptions = HingeEmbeddingLossOptions;
 /// MultiMarginLoss model(MultiMarginLossOptions().margin(2).weight(weight));
 /// ```
 struct TORCH_API MultiMarginLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   /// Has a default value of :math:`1`. :math:`1` and :math:`2`
   /// are the only supported values.
@@ -187,24 +207,27 @@ struct TORCH_API MultiMarginLossOptions {
   /// A manual rescaling weight given to each
   /// class. If given, it has to be a Tensor of size `C`. Otherwise, it is
   /// treated as if having all ones.
-  TORCH_ARG(Tensor, weight) = Tensor();
+  TORCH_ARG(Tensor, weight);
   /// Specifies the reduction to apply to the output:
-  /// ``'none'`` | ``'mean'`` | ``'sum'``. ``'none'``: no reduction will be applied,
+  /// ``'none'`` | ``'mean'`` | ``'sum'``. ``'none'``: no reduction will be
+  /// applied,
   /// ``'mean'``: the sum of the output will be divided by the number of
-  /// elements in the output, ``'sum'``: the output will be summed. Default: ``'mean'``
+  /// elements in the output, ``'sum'``: the output will be summed. Default:
+  /// ``'mean'``
   TORCH_ARG(reduction_t, reduction) = torch::kMean;
 };
 
 namespace functional {
 /// Options for `torch::nn::functional::multi_margin_loss`.
 ///
-/// See the documentation for `torch::nn::MultiMarginLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::MultiMarginLossOptions` class to learn
+/// what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::multi_margin_loss(input, target, F::MultiMarginLossFuncOptions().margin(2).weight(weight));
+/// F::multi_margin_loss(input, target,
+/// F::MultiMarginLossFuncOptions().margin(2).weight(weight));
 /// ```
 using MultiMarginLossFuncOptions = MultiMarginLossOptions;
 } // namespace functional
@@ -218,7 +241,8 @@ using MultiMarginLossFuncOptions = MultiMarginLossOptions;
 /// CosineEmbeddingLoss model(CosineEmbeddingLossOptions().margin(0.5));
 /// ```
 struct TORCH_API CosineEmbeddingLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   /// Specifies the threshold for which the distance of a negative sample must
   /// reach in order to incur zero loss. Should be a number from -1 to 1, 0
@@ -231,13 +255,14 @@ struct TORCH_API CosineEmbeddingLossOptions {
 namespace functional {
 /// Options for `torch::nn::functional::cosine_embedding_loss`.
 ///
-/// See the documentation for `torch::nn::CosineEmbeddingLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::CosineEmbeddingLossOptions` class to
+/// learn what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::cosine_embedding_loss(input1, input2, target, F::CosineEmbeddingLossFuncOptions().margin(0.5));
+/// F::cosine_embedding_loss(input1, input2, target,
+/// F::CosineEmbeddingLossFuncOptions().margin(0.5));
 /// ```
 using CosineEmbeddingLossFuncOptions = CosineEmbeddingLossOptions;
 } // namespace functional
@@ -251,9 +276,15 @@ using CosineEmbeddingLossFuncOptions = CosineEmbeddingLossOptions;
 /// MultiLabelMarginLoss model(MultiLabelMarginLossOptions(torch::kNone));
 /// ```
 struct TORCH_API MultiLabelMarginLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
-  TORCH_OPTIONS_CTOR_VARIANT_ARG3(MultiLabelMarginLossOptions, reduction, kNone, kMean, kSum)
+  TORCH_OPTIONS_CTOR_VARIANT_ARG3(
+      MultiLabelMarginLossOptions,
+      reduction,
+      kNone,
+      kMean,
+      kSum)
 
   /// Specifies the reduction to apply to the output: 'none' | 'mean' | 'sum'.
   /// 'none': no reduction will be applied, 'mean': the sum of the output will
@@ -265,13 +296,14 @@ struct TORCH_API MultiLabelMarginLossOptions {
 namespace functional {
 /// Options for `torch::nn::functional::multilabel_margin_loss`.
 ///
-/// See the documentation for `torch::nn::MultiLabelMarginLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::MultiLabelMarginLossOptions` class to
+/// learn what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::multilabel_margin_loss(input, target, F::MultilabelMarginLossFuncOptions(torch::kNone));
+/// F::multilabel_margin_loss(input, target,
+/// F::MultilabelMarginLossFuncOptions(torch::kNone));
 /// ```
 using MultilabelMarginLossFuncOptions = MultiLabelMarginLossOptions;
 } // namespace functional
@@ -285,9 +317,15 @@ using MultilabelMarginLossFuncOptions = MultiLabelMarginLossOptions;
 /// SoftMarginLoss model(SoftMarginLossOptions(torch::kNone));
 /// ```
 struct TORCH_API SoftMarginLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
-  TORCH_OPTIONS_CTOR_VARIANT_ARG3(SoftMarginLossOptions, reduction, kNone, kMean, kSum)
+  TORCH_OPTIONS_CTOR_VARIANT_ARG3(
+      SoftMarginLossOptions,
+      reduction,
+      kNone,
+      kMean,
+      kSum)
 
   /// Specifies the reduction to apply to the output: 'none' | 'mean' | 'sum'.
   /// 'none': no reduction will be applied, 'mean': the sum of the output will
@@ -299,13 +337,14 @@ struct TORCH_API SoftMarginLossOptions {
 namespace functional {
 /// Options for `torch::nn::functional::soft_margin_loss`.
 ///
-/// See the documentation for `torch::nn::SoftMarginLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::SoftMarginLossOptions` class to learn
+/// what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::soft_margin_loss(input, target, F::SoftMarginLossFuncOptions(torch::kNone));
+/// F::soft_margin_loss(input, target,
+/// F::SoftMarginLossFuncOptions(torch::kNone));
 /// ```
 using SoftMarginLossFuncOptions = SoftMarginLossOptions;
 } // namespace functional
@@ -316,15 +355,17 @@ using SoftMarginLossFuncOptions = SoftMarginLossOptions;
 ///
 /// Example:
 /// ```
-/// MultiLabelSoftMarginLoss model(MultiLabelSoftMarginLossOptions().reduction(torch::kNone).weight(weight));
+/// MultiLabelSoftMarginLoss
+/// model(MultiLabelSoftMarginLossOptions().reduction(torch::kNone).weight(weight));
 /// ```
 struct TORCH_API MultiLabelSoftMarginLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   /// A manual rescaling weight given to each
   /// class. If given, it has to be a Tensor of size `C`. Otherwise, it is
   /// treated as if having all ones.
-  TORCH_ARG(Tensor, weight) = Tensor();
+  TORCH_ARG(Tensor, weight);
 
   /// Specifies the reduction to apply to the output: 'none' | 'mean' | 'sum'.
   /// 'none': no reduction will be applied, 'mean': the sum of the output will
@@ -336,13 +377,14 @@ struct TORCH_API MultiLabelSoftMarginLossOptions {
 namespace functional {
 /// Options for `torch::nn::functional::multilabel_soft_margin_loss`.
 ///
-/// See the documentation for `torch::nn::MultiLabelSoftMarginLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::MultiLabelSoftMarginLossOptions` class
+/// to learn what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::multilabel_soft_margin_loss(input, target, F::MultilabelSoftMarginLossFuncOptions().reduction(torch::kNone).weight(weight));
+/// F::multilabel_soft_margin_loss(input, target,
+/// F::MultilabelSoftMarginLossFuncOptions().reduction(torch::kNone).weight(weight));
 /// ```
 using MultilabelSoftMarginLossFuncOptions = MultiLabelSoftMarginLossOptions;
 } // namespace functional
@@ -353,10 +395,12 @@ using MultilabelSoftMarginLossFuncOptions = MultiLabelSoftMarginLossOptions;
 ///
 /// Example:
 /// ```
-/// TripletMarginLoss model(TripletMarginLossOptions().margin(3).p(2).eps(1e-06).swap(false));
+/// TripletMarginLoss
+/// model(TripletMarginLossOptions().margin(3).p(2).eps(1e-06).swap(false));
 /// ```
 struct TORCH_API TripletMarginLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   /// Specifies the threshold for which the distance of a negative sample must
   /// reach in order to incur zero loss. Default: 1
@@ -375,13 +419,14 @@ struct TORCH_API TripletMarginLossOptions {
 namespace functional {
 /// Options for `torch::nn::functional::triplet_margin_loss`.
 ///
-/// See the documentation for `torch::nn::TripletMarginLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::TripletMarginLossOptions` class to
+/// learn what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::triplet_margin_loss(anchor, positive, negative, F::TripletMarginLossFuncOptions().margin(1.0));
+/// F::triplet_margin_loss(anchor, positive, negative,
+/// F::TripletMarginLossFuncOptions().margin(1.0));
 /// ```
 using TripletMarginLossFuncOptions = TripletMarginLossOptions;
 } // namespace functional
@@ -392,16 +437,20 @@ using TripletMarginLossFuncOptions = TripletMarginLossOptions;
 ///
 /// Example:
 /// ```
-/// TripletMarginWithDistanceLoss model(TripletMarginWithDistanceLossOptions().margin(3).swap(false));
+/// TripletMarginWithDistanceLoss
+/// model(TripletMarginWithDistanceLossOptions().margin(3).swap(false));
 /// ```
 struct TORCH_API TripletMarginWithDistanceLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
-  typedef std::function<Tensor(const Tensor&, const Tensor&)> distance_function_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
+  typedef std::function<Tensor(const Tensor&, const Tensor&)>
+      distance_function_t;
 
   /// Specifies a nonnegative, real-valued function that quantifies the
   /// closeness of two tensors. If not specified, `F::pairwise_distance` will
   /// be used. Default: nullopt
-  TORCH_ARG(c10::optional<distance_function_t>, distance_function) = c10::nullopt;
+  TORCH_ARG(std::optional<distance_function_t>, distance_function) =
+      std::nullopt;
   /// Specifies a nonnegative margin representing the minimum difference
   /// between the positive and negative distances required for the loss to be 0.
   /// Larger margins penalize cases where the negative examples are not distance
@@ -420,15 +469,17 @@ struct TORCH_API TripletMarginWithDistanceLossOptions {
 namespace functional {
 /// Options for `torch::nn::functional::triplet_margin_with_distance_loss`.
 ///
-/// See the documentation for `torch::nn::TripletMarginWithDistanceLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::TripletMarginWithDistanceLossOptions`
+/// class to learn what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::triplet_margin_with_distance_loss(anchor, positive, negative, F::TripletMarginWithDistanceLossFuncOptions().margin(1.0));
+/// F::triplet_margin_with_distance_loss(anchor, positive, negative,
+/// F::TripletMarginWithDistanceLossFuncOptions().margin(1.0));
 /// ```
-using TripletMarginWithDistanceLossFuncOptions = TripletMarginWithDistanceLossOptions;
+using TripletMarginWithDistanceLossFuncOptions =
+    TripletMarginWithDistanceLossOptions;
 } // namespace functional
 
 // ============================================================================
@@ -437,10 +488,12 @@ using TripletMarginWithDistanceLossFuncOptions = TripletMarginWithDistanceLossOp
 ///
 /// Example:
 /// ```
-/// CTCLoss model(CTCLossOptions().blank(42).zero_infinity(false).reduction(torch::kSum));
+/// CTCLoss
+/// model(CTCLossOptions().blank(42).zero_infinity(false).reduction(torch::kSum));
 /// ```
 struct TORCH_API CTCLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   /// blank label. Default `0`.
   TORCH_ARG(int64_t, blank) = 0;
@@ -461,7 +514,8 @@ namespace functional {
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::ctc_loss(log_probs, targets, input_lengths, target_lengths, F::CTCLossFuncOptions().reduction(torch::kNone));
+/// F::ctc_loss(log_probs, targets, input_lengths, target_lengths,
+/// F::CTCLossFuncOptions().reduction(torch::kNone));
 /// ```
 using CTCLossFuncOptions = CTCLossOptions;
 } // namespace functional
@@ -472,25 +526,35 @@ using CTCLossFuncOptions = CTCLossOptions;
 ///
 /// Example:
 /// ```
-/// SmoothL1Loss model(SmoothL1LossOptions(torch::kNone));
+/// SmoothL1Loss model(SmoothL1LossOptions().reduction(torch::kNone).beta(0.5));
 /// ```
 struct TORCH_API SmoothL1LossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
-  TORCH_OPTIONS_CTOR_VARIANT_ARG3(SmoothL1LossOptions, reduction, kNone, kMean, kSum)
+  TORCH_OPTIONS_CTOR_VARIANT_ARG3(
+      SmoothL1LossOptions,
+      reduction,
+      kNone,
+      kMean,
+      kSum)
 
   /// Specifies the reduction to apply to the output: 'none' | 'mean' | 'sum'.
   /// 'none': no reduction will be applied, 'mean': the sum of the output will
   /// be divided by the number of elements in the output, 'sum': the output will
   /// be summed. Default: 'mean'
   TORCH_ARG(reduction_t, reduction) = torch::kMean;
+  /// Specifies the threshold at which to change between L1 and L2 loss.
+  /// If beta is not specified, a value of 1.0 will be used.
+  /// Default: nullopt
+  TORCH_ARG(std::optional<double>, beta) = std::nullopt;
 };
 
 namespace functional {
 /// Options for `torch::nn::functional::smooth_l1_loss`.
 ///
-/// See the documentation for `torch::nn::SmoothL1LossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::SmoothL1LossOptions` class to learn
+/// what arguments are supported.
 ///
 /// Example:
 /// ```
@@ -502,14 +566,59 @@ using SmoothL1LossFuncOptions = SmoothL1LossOptions;
 
 // ============================================================================
 
+/// Options for the `HuberLoss` module.
+///
+/// Example:
+/// ```
+/// HuberLoss model(HuberLossOptions().reduction(torch::kNone).delta(0.5));
+/// ```
+struct TORCH_API HuberLossOptions {
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
+
+  TORCH_OPTIONS_CTOR_VARIANT_ARG3(
+      HuberLossOptions,
+      reduction,
+      kNone,
+      kMean,
+      kSum)
+
+  /// Specifies the reduction to apply to the output: 'none' | 'mean' | 'sum'.
+  /// 'none': no reduction will be applied, 'mean': the sum of the output will
+  /// be divided by the number of elements in the output, 'sum': the output will
+  /// be summed. Default: 'mean'
+  TORCH_ARG(reduction_t, reduction) = torch::kMean;
+  /// Specifies the threshold at which to change between L1 and L2 loss.
+  /// Default: 1.0
+  TORCH_ARG(double, delta) = 1.0;
+};
+
+namespace functional {
+/// Options for `torch::nn::functional::huber_loss`.
+///
+/// See the documentation for `torch::nn::HuberLossOptions` class to learn what
+/// arguments are supported.
+///
+/// Example:
+/// ```
+/// namespace F = torch::nn::functional;
+/// F::huber_loss(input, target, F::HuberLossFuncOptions(torch::kNone));
+/// ```
+using HuberLossFuncOptions = HuberLossOptions;
+} // namespace functional
+
+// ============================================================================
+
 /// Options for the `PoissonNLLLoss` module.
 ///
 /// Example:
 /// ```
-/// PoissonNLLLoss model(PoissonNLLLossOptions().log_input(false).full(true).eps(0.42).reduction(torch::kSum));
+/// PoissonNLLLoss
+/// model(PoissonNLLLossOptions().log_input(false).full(true).eps(0.42).reduction(torch::kSum));
 /// ```
 struct TORCH_API PoissonNLLLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   /// if true the loss is computed as `exp(input) - target * input`,
   /// if false the loss is `input - target * log(input + eps)`.
@@ -527,13 +636,14 @@ struct TORCH_API PoissonNLLLossOptions {
 namespace functional {
 /// Options for `torch::nn::functional::poisson_nll_loss`.
 ///
-/// See the documentation for `torch::nn::PoissonNLLLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::PoissonNLLLossOptions` class to learn
+/// what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::poisson_nll_loss(input, target, F::PoissonNLLLossFuncOptions().reduction(torch::kNone));
+/// F::poisson_nll_loss(input, target,
+/// F::PoissonNLLLossFuncOptions().reduction(torch::kNone));
 /// ```
 using PoissonNLLLossFuncOptions = PoissonNLLLossOptions;
 } // namespace functional
@@ -544,10 +654,12 @@ using PoissonNLLLossFuncOptions = PoissonNLLLossOptions;
 ///
 /// Example:
 /// ```
-/// MarginRankingLoss model(MarginRankingLossOptions().margin(0.5).reduction(torch::kSum));
+/// MarginRankingLoss
+/// model(MarginRankingLossOptions().margin(0.5).reduction(torch::kSum));
 /// ```
 struct TORCH_API MarginRankingLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   /// Has a default value of `0`.
   TORCH_ARG(double, margin) = 0;
@@ -558,13 +670,14 @@ struct TORCH_API MarginRankingLossOptions {
 namespace functional {
 /// Options for `torch::nn::functional::margin_ranking_loss`.
 ///
-/// See the documentation for `torch::nn::MarginRankingLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::MarginRankingLossOptions` class to
+/// learn what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::margin_ranking_loss(input1, input2, target, F::MarginRankingLossFuncOptions().margin(0.5).reduction(torch::kSum));
+/// F::margin_ranking_loss(input1, input2, target,
+/// F::MarginRankingLossFuncOptions().margin(0.5).reduction(torch::kSum));
 /// ```
 using MarginRankingLossFuncOptions = MarginRankingLossOptions;
 } // namespace functional
@@ -578,12 +691,13 @@ using MarginRankingLossFuncOptions = MarginRankingLossOptions;
 /// NLLLoss model(NLLLossOptions().ignore_index(-100).reduction(torch::kMean));
 /// ```
 struct TORCH_API NLLLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
   /// A manual rescaling weight given to each
   /// class. If given, it has to be a Tensor of size `C`. Otherwise, it is
   /// treated as if having all ones.
-  TORCH_ARG(Tensor, weight) = {};
+  TORCH_ARG(Tensor, weight);
   /// Specifies a target value that is ignored
   /// and does not contribute to the input gradient.
   TORCH_ARG(int64_t, ignore_index) = -100;
@@ -600,7 +714,8 @@ namespace functional {
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::nll_loss(input, target, F::NLLLossFuncOptions().ignore_index(-100).reduction(torch::kMean));
+/// F::nll_loss(input, target,
+/// F::NLLLossFuncOptions().ignore_index(-100).reduction(torch::kMean));
 /// ```
 using NLLLossFuncOptions = NLLLossOptions;
 } // namespace functional
@@ -611,31 +726,36 @@ using NLLLossFuncOptions = NLLLossOptions;
 ///
 /// Example:
 /// ```
-/// CrossEntropyLoss model(CrossEntropyLossOptions().ignore_index(-100).reduction(torch::kMean));
+/// CrossEntropyLoss
+/// model(CrossEntropyLossOptions().ignore_index(-100).reduction(torch::kMean));
 /// ```
 struct TORCH_API CrossEntropyLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
 
-  /// A manual rescaling weight given to each class. If given, has to be a Tensor
-  /// of size C
-  TORCH_ARG(Tensor, weight) = {};
+  /// A manual rescaling weight given to each class. If given, has to be a
+  /// Tensor of size C
+  TORCH_ARG(Tensor, weight);
   /// Specifies a target value that is ignored
   /// and does not contribute to the input gradient.
   TORCH_ARG(int64_t, ignore_index) = -100;
   /// Specifies the reduction to apply to the output. Default: Mean
   TORCH_ARG(reduction_t, reduction) = torch::kMean;
+  /// Specifies the amount of smoothing when computing the loss. Default: 0.0
+  TORCH_ARG(double, label_smoothing) = 0.0;
 };
 
 namespace functional {
 /// Options for `torch::nn::functional::cross_entropy`.
 ///
-/// See the documentation for `torch::nn::CrossEntropyLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::CrossEntropyLossOptions` class to
+/// learn what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::cross_entropy(input, target, F::CrossEntropyFuncOptions().ignore_index(-100).reduction(torch::kMean));
+/// F::cross_entropy(input, target,
+/// F::CrossEntropyFuncOptions().ignore_index(-100).reduction(torch::kMean));
 /// ```
 using CrossEntropyFuncOptions = CrossEntropyLossOptions;
 } // namespace functional
@@ -646,33 +766,35 @@ using CrossEntropyFuncOptions = CrossEntropyLossOptions;
 ///
 /// Example:
 /// ```
-/// BCEWithLogitsLoss model(BCEWithLogitsLossOptions().reduction(torch::kNone).weight(weight));
+/// BCEWithLogitsLoss
+/// model(BCEWithLogitsLossOptions().reduction(torch::kNone).weight(weight));
 /// ```
 struct TORCH_API BCEWithLogitsLossOptions {
-  typedef c10::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum> reduction_t;
+  typedef std::variant<enumtype::kNone, enumtype::kMean, enumtype::kSum>
+      reduction_t;
   /// A manual rescaling weight given to the loss of each batch element.
   /// If given, has to be a Tensor of size `nbatch`.
-  TORCH_ARG(Tensor, weight) = {};
+  TORCH_ARG(Tensor, weight);
   /// Specifies the reduction to apply to the output. Default: Mean
   TORCH_ARG(reduction_t, reduction) = torch::kMean;
   /// A weight of positive examples.
   /// Must be a vector with length equal to the number of classes.
-  TORCH_ARG(Tensor, pos_weight) = {};
+  TORCH_ARG(Tensor, pos_weight);
 };
 
 namespace functional {
 /// Options for `torch::nn::functional::binary_cross_entropy_with_logits`.
 ///
-/// See the documentation for `torch::nn::BCEWithLogitsLossOptions` class to learn what
-/// arguments are supported.
+/// See the documentation for `torch::nn::BCEWithLogitsLossOptions` class to
+/// learn what arguments are supported.
 ///
 /// Example:
 /// ```
 /// namespace F = torch::nn::functional;
-/// F::binary_cross_entropy_with_logits(input, target, F::BinaryCrossEntropyWithLogitsFuncOptions().pos_weight(pos_weight).reduction(torch::kSum));
+/// F::binary_cross_entropy_with_logits(input, target,
+/// F::BinaryCrossEntropyWithLogitsFuncOptions().pos_weight(pos_weight).reduction(torch::kSum));
 /// ```
 using BinaryCrossEntropyWithLogitsFuncOptions = BCEWithLogitsLossOptions;
 } // namespace functional
 
-} // namespace nn
-} // namespace torch
+} // namespace torch::nn
