@@ -1,13 +1,10 @@
 #include <torch/nn/modules/fold.h>
 
 #include <torch/expanding_array.h>
-#include <torch/types.h>
-#include <torch/utils.h>
 
 namespace F = torch::nn::functional;
 
-namespace torch {
-namespace nn {
+namespace torch::nn {
 
 FoldImpl::FoldImpl(const FoldOptions& options_) : options(options_) {}
 
@@ -17,19 +14,18 @@ void FoldImpl::pretty_print(std::ostream& stream) const {
   stream << "torch::nn::Fold(output_size=" << options.output_size()
          << ", kernel_size=" << options.kernel_size()
          << ", dilation=" << options.dilation()
-         << ", padding=" << options.padding()
-         << ", stride=" << options.stride()
-         << ")";
+         << ", padding=" << options.padding() << ", stride=" << options.stride()
+         << ')';
 }
 
 Tensor FoldImpl::forward(const Tensor& input) {
   return F::detail::fold(
-    input,
-    options.output_size(),
-    options.kernel_size(),
-    options.dilation(),
-    options.padding(),
-    options.stride());
+      input,
+      options.output_size(),
+      options.kernel_size(),
+      options.dilation(),
+      options.padding(),
+      options.stride());
 }
 
 // ============================================================================
@@ -41,14 +37,17 @@ void UnfoldImpl::reset() {}
 void UnfoldImpl::pretty_print(std::ostream& stream) const {
   stream << "torch::nn::Unfold(kernel_size=" << options.kernel_size()
          << ", dilation=" << options.dilation()
-         << ", padding=" << options.padding()
-         << ", stride=" << options.stride()
-         << ")";
+         << ", padding=" << options.padding() << ", stride=" << options.stride()
+         << ')';
 }
 
 Tensor UnfoldImpl::forward(const Tensor& input) {
-  return F::detail::unfold(input, options.kernel_size(), options.dilation(), options.padding(), options.stride());
+  return F::detail::unfold(
+      input,
+      options.kernel_size(),
+      options.dilation(),
+      options.padding(),
+      options.stride());
 }
 
-} // namespace nn
-} // namespace torch
+} // namespace torch::nn
