@@ -309,8 +309,12 @@ PyObject* THPAutograd_initExtension(PyObject* _unused, PyObject* unused) {
           })
       .def("nbytes", [](const KinetoEvent& e) { return e.nBytes(); })
       // whether the event is hidden
-      .def("is_hidden_event", [](const KinetoEvent& e) {
-        return e.isHiddenEvent();
+      .def(
+          "is_hidden_event",
+          [](const KinetoEvent& e) { return e.isHiddenEvent(); })
+      // KinetoEvent metadata
+      .def("metadata_json", [](const KinetoEvent& e) {
+        return e.metadataJson();
       });
 
   m.def("_soft_assert_raises", &setSoftAssertRaises);
@@ -386,31 +390,25 @@ PyObject* THPAutograd_initExtension(PyObject* _unused, PyObject* unused) {
   m.def("_supported_activities", []() {
     std::set<torch::profiler::impl::ActivityType> activities{
         torch::profiler::impl::ActivityType::CPU};
-#if defined(USE_KINETO) && \
-    (!defined(LIBKINETO_NOCUPTI) || !defined(LIBKINETO_NOROCTRACER))
-    if (at::hasMTIA()) {
-      activities.insert(torch::profiler::impl::ActivityType::MTIA);
-    }
-    if (at::hasHPU()) {
-      activities.insert(torch::profiler::impl::ActivityType::HPU);
-    }
+#if defined(USE_KINETO)
+#if (!defined(LIBKINETO_NOCUPTI) || !defined(LIBKINETO_NOROCTRACER))
     if (at::getNumGPUs() > 0) {
       activities.insert(torch::profiler::impl::ActivityType::CUDA);
     }
-#elif defined(USE_KINETO)
+#endif // (!defined(LIBKINETO_NOCUPTI) || !defined(LIBKINETO_NOROCTRACER))
     if (at::hasXPU()) {
       activities.insert(torch::profiler::impl::ActivityType::XPU);
     }
-    if (at::hasHPU()) {
-      activities.insert(torch::profiler::impl::ActivityType::HPU);
-    }
     if (at::hasMTIA()) {
       activities.insert(torch::profiler::impl::ActivityType::MTIA);
     }
+    if (at::hasHPU()) {
+      activities.insert(torch::profiler::impl::ActivityType::HPU);
+    }
     if (c10::get_privateuse1_backend() != "privateuseone") {
       activities.insert(torch::profiler::impl::ActivityType::PrivateUse1);
     }
-#endif
+#endif // defined(USE_KINETO)
     return activities;
   });
 
@@ -594,6 +592,33 @@ PyObject* THPAutograd_initExtension(PyObject* _unused, PyObject* unused) {
             s.register_hooks(
                 std::make_unique<torch::autograd::PySavedVariableHooks>(
                     pack_hook, unpack_hook));
+          })
+      .def_property_readonly(
+          "data",
+          [](const torch::autograd::SavedVariable& s) -> py::object {
+            if (s.has_hooks()) {
+              auto opt = s.retrieve_unpack_hook_data();
+              TORCH_INTERNAL_ASSERT(opt.has_value());
+              py::gil_scoped_acquire gil;
+              const auto& [_unpack_fn, data_obj] = *opt;
+              PyObject* raw = data_obj.ptr(getPyInterpreter());
+              TORCH_INTERNAL_ASSERT(raw != nullptr);
+              return py::reinterpret_borrow<py::object>(raw);
+            } else {
+              return py::cast(s.get_raw_data().value());
+            }
+          })
+      .def_property_readonly(
+          "unpack_hook",
+          [](const torch::autograd::SavedVariable& s) -> py::object {
+            auto opt = s.retrieve_unpack_hook_data();
+            if (!opt.has_value()) {
+              return py::none();
+            }
+            py::gil_scoped_acquire gil;
+            const auto& [unpack_safe, _unused_data] = *opt;
+            auto* unpack_ptr = unpack_safe.ptr(getPyInterpreter());
+            return py::reinterpret_borrow<py::function>(unpack_ptr);
           });
 
   torch::autograd::profiler::python_tracer::init();
@@ -1082,7 +1107,7 @@ static PyObject* any_output_is_alias_to_input_or_output(
     if (!t.storage()) {
       return false;
     }
-    auto* cp = t.storage().data_ptr().get_context();
+    auto* cp = t.storage().unsafeGetStorageImpl();
     if (cp) {
       s.insert(cp);
     }
@@ -1093,7 +1118,7 @@ static PyObject* any_output_is_alias_to_input_or_output(
     if (!t.storage()) {
       return false;
     }
-    auto* cp = t.storage().data_ptr().get_context();
+    auto* cp = t.storage().unsafeGetStorageImpl();
     if (!cp) {
       return false;
     }
@@ -1187,6 +1212,33 @@ static PyObject* is_view_replay_enabled(PyObject* self, PyObject* args) {
   END_HANDLE_TH_ERRORS
 }
 
+static PyObject* set_graph_exec_group(PyObject* self, PyObject* obj) {
+  HANDLE_TH_ERRORS
+  if (obj == Py_None) {
+    c10::AutogradState::get_tls_state().set_graph_exec_group(std::nullopt);
+  } else {
+    Py_INCREF(obj);
+    c10::AutogradState::get_tls_state().set_graph_exec_group(
+        c10::SafePyObject(obj, getPyInterpreter()));
+  }
+  Py_RETURN_NONE;
+  END_HANDLE_TH_ERRORS
+}
+
+static PyObject* get_graph_exec_group(PyObject* self, PyObject* args) {
+  HANDLE_TH_ERRORS
+  const auto& group =
+      c10::AutogradState::get_tls_state().get_graph_exec_group();
+  if (group.has_value()) {
+    PyObject* obj = group->ptr(getPyInterpreter());
+    Py_INCREF(obj);
+    return obj;
+  } else {
+    Py_RETURN_NONE;
+  }
+  END_HANDLE_TH_ERRORS
+}
+
 static PyObject* is_inference_mode_enabled(PyObject* _unused, PyObject* arg) {
   HANDLE_TH_ERRORS
   if (c10::InferenceMode::is_enabled()) {
@@ -1567,6 +1619,8 @@ static PyMethodDef methods[] = {
      castPyCFunctionWithKeywords(set_view_replay_enabled),
      METH_VARARGS | METH_KEYWORDS,
      nullptr},
+    {"_set_graph_exec_group", set_graph_exec_group, METH_O, nullptr},
+    {"_get_graph_exec_group", get_graph_exec_group, METH_NOARGS, nullptr},
     {"_enter_dual_level", python_enter_dual_level, METH_NOARGS, nullptr},
     {"_exit_dual_level",
      castPyCFunctionWithKeywords(python_exit_dual_level),
