@@ -57,10 +57,6 @@ PythonEngine::~PythonEngine() {
   Engine::stop();
 }
 
-#if PY_MAJOR_VERSION == 3 && PY_MINOR_VERSION >= 9
-#define IS_PYTHON_3_9_PLUS
-#endif
-
 void PythonEngine::thread_init(
     int device,
     const std::shared_ptr<ReadyQueue>& ready_queue,
@@ -72,11 +68,7 @@ void PythonEngine::thread_init(
   // Create a PyThreadState, but release the GIL. This lets
   // pybind11::gil_scoped_acquire calls inside thread_main acquire the GIL
   // without having to create a new PyThreadState each time.
-#if defined(IS_PYTHON_3_9_PLUS)
   auto gil = std::make_unique<pybind11::gil_scoped_acquire>();
-#else
-  pybind11::gil_scoped_acquire gil;
-#endif
   pybind11::gil_scoped_release no_gil;
   Engine::thread_init(device, ready_queue, false);
 
@@ -85,7 +77,6 @@ void PythonEngine::thread_init(
     decrement_non_reentrant_thread_count();
   }
 
-#if defined(IS_PYTHON_3_9_PLUS)
   // Do not call PyEval_RestoreThread, PyThreadState_[Clear|DeleteCurrent] if
   // runtime is finalizing
   if (!Py_IsInitialized()) {
@@ -96,7 +87,6 @@ void PythonEngine::thread_init(
     auto ptr = gil.release();
     operator delete(ptr);
   }
-#endif
 }
 
 void PythonEngine::thread_on_exception(
@@ -160,8 +150,6 @@ c10::intrusive_ptr<at::ivalue::Future> PythonEngine::execute_with_graph_task(
 }
 } // namespace torch::autograd::python
 
-PyObject* THPEngineClass = nullptr;
-
 static Edge parseGradientEdge(PyObject* obj, int64_t index) {
   PyObject* grad_fn = PyTuple_GetItem(obj, 0);
   auto output_nr = THPUtils_unpackLong(PyTuple_GetItem(obj, 1));
@@ -181,7 +169,7 @@ static Edge parseGradientEdge(PyObject* obj, int64_t index) {
 }
 
 // Implementation of torch._C._EngineBase.run_backward
-PyObject* THPEngine_run_backward(
+static PyObject* THPEngine_run_backward(
     PyObject* self,
     PyObject* args,
     PyObject* kwargs) {
@@ -396,7 +384,7 @@ PyObject* THPEngine_run_backward(
   END_HANDLE_TH_ERRORS
 }
 
-PyObject* THPEngine_queue_callback(PyObject* self, PyObject* _callback) {
+static PyObject* THPEngine_queue_callback(PyObject* self, PyObject* _callback) {
   HANDLE_TH_ERRORS
   auto& engine = python::PythonEngine::get_python_engine();
   std::shared_ptr<PyObject> callback(_callback, [](PyObject* obj) {
@@ -431,7 +419,9 @@ PyObject* THPEngine_queue_callback(PyObject* self, PyObject* _callback) {
   END_HANDLE_TH_ERRORS
 }
 
-PyObject* THPEngine_is_checkpoint_valid(PyObject* self, PyObject* noargs) {
+static PyObject* THPEngine_is_checkpoint_valid(
+    PyObject* self,
+    PyObject* noargs) {
   HANDLE_TH_ERRORS
   auto& engine = python::PythonEngine::get_python_engine();
   if (engine.is_checkpoint_valid()) {
@@ -442,24 +432,27 @@ PyObject* THPEngine_is_checkpoint_valid(PyObject* self, PyObject* noargs) {
   END_HANDLE_TH_ERRORS
 }
 
-PyObject* THPEngine_new(PyTypeObject* type, PyObject* args, PyObject* kwargs) {
+static PyObject* THPEngine_new(
+    PyTypeObject* type,
+    PyObject* args,
+    PyObject* kwargs) {
   return type->tp_alloc(type, 0);
 }
 
 // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,modernize-avoid-c-arrays,cppcoreguidelines-avoid-non-const-global-variables)
 static struct PyMethodDef THPEngine_methods[] = {
-    {(char*)"run_backward",
+    {"run_backward",
      castPyCFunctionWithKeywords(THPEngine_run_backward),
      METH_VARARGS | METH_KEYWORDS,
      nullptr},
-    {(char*)"queue_callback", THPEngine_queue_callback, METH_O, nullptr},
-    {(char*)"is_checkpoint_valid",
+    {"queue_callback", THPEngine_queue_callback, METH_O, nullptr},
+    {"is_checkpoint_valid",
      THPEngine_is_checkpoint_valid,
      METH_NOARGS,
      nullptr},
     {nullptr}};
 
-PyTypeObject THPEngineType = {
+static PyTypeObject THPEngineType = {
     PyVarObject_HEAD_INIT(nullptr, 0)
     "torch._C._EngineBase", /* tp_name */
     sizeof(THPEngine), /* tp_basicsize */
@@ -507,9 +500,9 @@ static void child_atfork() {
 
 bool THPEngine_initModule(PyObject* module) {
 #ifndef _WIN32
-  if (pthread_atfork(nullptr, nullptr, child_atfork) != 0) {
-    throw std::runtime_error("unable to set pthread_atfork handler");
-  }
+  TORCH_CHECK(
+      pthread_atfork(nullptr, nullptr, child_atfork) == 0,
+      "unable to set pthread_atfork handler");
 #endif
   if (PyType_Ready(&THPEngineType) < 0)
     return false;
