@@ -289,9 +289,7 @@ static variable_list unwrap_variables(PyObject* py_variables) {
       results[i] = THPVariable_Unpack(item);
     } else {
       // this should never happen, but just in case...
-      std::stringstream ss;
-      ss << "expected variable but got " << Py_TYPE(item)->tp_name;
-      throw std::runtime_error(ss.str());
+      TORCH_CHECK(false, "expected variable but got ", Py_TYPE(item)->tp_name);
     }
   }
   return results;
@@ -308,14 +306,16 @@ static void check_result(PyObject* prev, PyObject* result, PyObject* hook) {
 
   auto prev_size = PyTuple_GET_SIZE(prev);
   auto result_size = PyTuple_GET_SIZE(result);
-  if (prev_size != result_size) {
-    std::stringstream ss;
-    auto name = hook_name(hook);
-    ss << "hook '" << name << "' has returned an incorrect number ";
-    ss << "of values (got " << result_size << ", but expected ";
-    ss << prev_size << ")";
-    throw std::runtime_error(ss.str());
-  }
+
+  TORCH_CHECK(
+      prev_size == result_size,
+      "hook '",
+      hook_name(hook),
+      "' has returned an incorrect number of values (got ",
+      result_size,
+      ", but expected ",
+      prev_size,
+      ")");
 
   for (const auto i : c10::irange(prev_size)) {
     check_single_result(
@@ -330,10 +330,9 @@ static void check_single_result(
   if (_result == Py_None)
     return;
 
-  if (_original == Py_None) {
-    throw std::runtime_error(
-        "can't replace a None gradient with a non-None value");
-  }
+  TORCH_CHECK(
+      _original != Py_None,
+      "can't replace a None gradient with a non-None value");
 
   if (!PyObject_IsInstance(_result, THPVariableClass)) {
     PyErr_Format(
