@@ -1,13 +1,9 @@
 #include <torch/csrc/distributed/autograd/rpc_messages/rpc_with_autograd.h>
-#include <c10/util/C++17.h>
 #include <torch/csrc/distributed/rpc/rpc_agent.h>
 #include <torch/csrc/distributed/rpc/utils.h>
 #include <torch/csrc/jit/serialization/pickle.h>
-#include <torch/csrc/utils/byte_order.h>
 
-namespace torch {
-namespace distributed {
-namespace autograd {
+namespace torch::distributed::autograd {
 
 using rpc::Message;
 using rpc::MessageType;
@@ -18,8 +14,8 @@ RpcWithAutograd::RpcWithAutograd(
     worker_id_t fromWorkerId,
     MessageType messageType,
     const AutogradMetadata& autogradMetadata,
-    rpc::Message&& wrappedMessage,
-    std::unordered_map<c10::DeviceIndex, c10::DeviceIndex> deviceMap)
+    c10::intrusive_ptr<rpc::Message> wrappedMessage,
+    rpc::DeviceMap deviceMap)
     : fromWorkerId_(fromWorkerId),
       messageType_(messageType),
       autogradMetadata_(autogradMetadata),
@@ -28,8 +24,8 @@ RpcWithAutograd::RpcWithAutograd(
   TORCH_INTERNAL_ASSERT(
       messageType_ == MessageType::FORWARD_AUTOGRAD_REQ ||
       messageType_ == MessageType::FORWARD_AUTOGRAD_RESP);
-  tensors_ = wrappedMessage_.tensors();
-  wrappedMessageType_ = wrappedMessage_.type();
+  tensors_ = wrappedMessage_->tensors();
+  wrappedMessageType_ = wrappedMessage_->type();
 }
 
 RpcWithAutograd::RpcWithAutograd(
@@ -39,7 +35,7 @@ RpcWithAutograd::RpcWithAutograd(
     std::unique_ptr<RpcCommandBase> wrappedRpc,
     MessageType wrappedMessageType,
     std::vector<torch::Tensor> tensors,
-    std::unordered_map<c10::DeviceIndex, c10::DeviceIndex> deviceMap)
+    rpc::DeviceMap deviceMap)
     : fromWorkerId_(fromWorkerId),
       messageType_(messageType),
       autogradMetadata_(autogradMetadata),
@@ -53,24 +49,25 @@ RpcWithAutograd::RpcWithAutograd(
       messageType_ == MessageType::FORWARD_AUTOGRAD_RESP);
 }
 
-Message RpcWithAutograd::toMessageImpl() && {
-  auto messageId = wrappedMessage_.id();
-  auto wrappedMessageType = wrappedMessage_.type();
+c10::intrusive_ptr<Message> RpcWithAutograd::toMessageImpl() && {
+  auto messageId = wrappedMessage_->id();
+  auto wrappedMessageType = wrappedMessage_->type();
 
-  auto payload = std::move(wrappedMessage_).movePayload();
+  auto payload = std::move(*wrappedMessage_).movePayload();
   TORCH_INTERNAL_ASSERT(!payload.empty());
 
   // Convert deviceMap to c10::Dict for serialization.
-  c10::Dict<int64_t, int64_t> deviceMap;
+  c10::Dict<std::string, std::string> deviceMap;
   for (const auto& mapEntry : deviceMap_) {
-    deviceMap.insert(mapEntry.first, mapEntry.second);
+    deviceMap.insert(mapEntry.first.str(), mapEntry.second.str());
   }
 
-  std::vector<at::IValue> ivalues{wrappedMessageType,
-                                  autogradMetadata_.autogradContextId,
-                                  autogradMetadata_.autogradMessageId,
-                                  fromWorkerId_,
-                                  deviceMap};
+  std::vector<at::IValue> ivalues{
+      wrappedMessageType,
+      autogradMetadata_.autogradContextId,
+      autogradMetadata_.autogradMessageId,
+      fromWorkerId_,
+      deviceMap};
 
   // Now pickle using JIT pickler.
   std::vector<torch::Tensor> tensorTable;
@@ -84,7 +81,7 @@ Message RpcWithAutograd::toMessageImpl() && {
   // encoding.
   rpc::writeWrappedPayload(payload, additionalPayload);
 
-  return Message(
+  return c10::make_intrusive<Message>(
       std::move(payload), std::move(tensors_), messageType_, messageId);
 }
 
@@ -108,24 +105,25 @@ std::unique_ptr<RpcWithAutograd> RpcWithAutograd::fromMessage(
       static_cast<MessageType>(tupleElements[0].toInt());
   AutogradMetadata autogradMetadata(
       tupleElements[1].toInt(), tupleElements[2].toInt());
-  worker_id_t workerId = tupleElements[3].toInt();
-  auto c10DeviceMap = tupleElements[4].to<c10::Dict<int64_t, int64_t>>();
+  worker_id_t workerId = static_cast<worker_id_t>(tupleElements[3].toInt());
+  auto c10DeviceMap =
+      tupleElements[4].to<c10::Dict<std::string, std::string>>();
 
   // Convert to regular map.
-  std::unordered_map<c10::DeviceIndex, c10::DeviceIndex> deviceMap;
+  rpc::DeviceMap deviceMap;
   for (const auto& mapEntry : c10DeviceMap) {
     deviceMap.insert({mapEntry.key(), mapEntry.value()});
   }
 
   // Create new message type and build wrapped RPC.
-  Message wrappedMessage(
+  auto wrappedMessage = c10::make_intrusive<Message>(
       std::move(payload), std::move(tensors), wrappedMessageType, messageId);
 
   std::unique_ptr<RpcCommandBase> wrappedRpc;
   if (originalMessageType == MessageType::FORWARD_AUTOGRAD_REQ) {
-    wrappedRpc = deserializeRequest(wrappedMessage);
+    wrappedRpc = deserializeRequest(*wrappedMessage);
   } else {
-    wrappedRpc = deserializeResponse(wrappedMessage, wrappedMessageType);
+    wrappedRpc = deserializeResponse(*wrappedMessage, wrappedMessageType);
   }
 
   return std::make_unique<RpcWithAutograd>(
@@ -134,7 +132,7 @@ std::unique_ptr<RpcWithAutograd> RpcWithAutograd::fromMessage(
       autogradMetadata,
       std::move(wrappedRpc),
       wrappedMessageType,
-      wrappedMessage.tensors(),
+      wrappedMessage->tensors(),
       deviceMap);
 }
 
@@ -169,11 +167,8 @@ rpc::worker_id_t RpcWithAutograd::fromWorkerId() const {
   return fromWorkerId_;
 }
 
-const std::unordered_map<c10::DeviceIndex, c10::DeviceIndex>& RpcWithAutograd::
-    deviceMap() {
+const rpc::DeviceMap& RpcWithAutograd::deviceMap() {
   return deviceMap_;
 }
 
-} // namespace autograd
-} // namespace distributed
-} // namespace torch
+} // namespace torch::distributed::autograd
