@@ -1,9 +1,9 @@
 #include <torch/csrc/distributed/c10d/NCCLUtils.hpp>
 
-#include <c10/util/env.h>
-
 #ifdef USE_C10D_NCCL
+#include <fmt/format.h>
 #include <mutex>
+#include <thread>
 #include <vector>
 
 namespace c10d {
@@ -39,8 +39,24 @@ NCCLComm::NCCLComm(NCCLComm&& other) {
   std::swap(deviceIndex_, other.deviceIndex_);
 }
 
-ncclUniqueId NCCLComm::getNcclId() {
-  return ncclId_;
+void NCCLComm::setUniqueHash(ncclUniqueId ncclId) {
+  const uint8_t* bytes = reinterpret_cast<const uint8_t*>(&ncclId);
+
+  fmt::memory_buffer buf;
+  buf.reserve(NCCL_UNIQUE_ID_BYTES * 2); // 2 hex chars per byte
+  for (int i = 0; i < NCCL_UNIQUE_ID_BYTES; ++i) {
+    fmt::format_to(
+        std::back_inserter(buf), "{:02x}", static_cast<int>(bytes[i]));
+  }
+  this->uniqueHash_ = fmt::to_string(buf);
+}
+
+void NCCLComm::setUniqueHash(std::string hash) {
+  this->uniqueHash_ = std::move(hash);
+}
+
+std::string NCCLComm::getUniqueHash() {
+  return uniqueHash_;
 }
 
 std::shared_ptr<NCCLComm> NCCLComm::create(
@@ -53,7 +69,7 @@ std::shared_ptr<NCCLComm> NCCLComm::create(
   C10D_NCCL_CHECK(
       ncclCommInitRank(&(comm->ncclComm_), numRanks, commId, rank),
       std::nullopt);
-  comm->ncclId_ = commId;
+  comm->setUniqueHash(commId);
   comm->rank_ = rank;
   comm->deviceIndex_ = deviceIndex;
   comm->initialized_ = true;
@@ -78,7 +94,7 @@ std::shared_ptr<NCCLComm> NCCLComm::create(
       ncclCommInitRankConfig(
           &(comm->ncclComm_), numRanks, commId, rank, &config),
       std::nullopt);
-  comm->ncclId_ = commId;
+  comm->setUniqueHash(commId);
   comm->rank_ = rank;
   comm->deviceIndex_ = deviceIndex;
   // Under blocking mode, comm is initialized immediately after NCCL init
@@ -112,7 +128,7 @@ std::shared_ptr<NCCLComm> NCCLComm::create_scalable(
   // Only the first ncclUniqueId will be used to create the
   // communicator hash id, which is used to identify the communicator
   // in the log file and in the replay tool.
-  comm->ncclId_ = commIds[0];
+  comm->setUniqueHash(commIds[0]);
   comm->rank_ = rank;
   comm->deviceIndex_ = deviceIndex;
   comm->initialized_ = !comm->nonBlocking_;
@@ -166,10 +182,34 @@ void NCCLComm::waitReady(bool longInterval) {
   if (aborted_)
     return;
   // If timeout is reached, throw an exception.
-  if (longInterval) {
-    C10D_NCCL_CHECK_TIMEOUT_SLEEP(ncclInProgress, ncclComm_, std::nullopt);
-  } else {
-    C10D_NCCL_CHECK_TIMEOUT(ncclInProgress, ncclComm_, std::nullopt);
+  // (RohitRathore1) Note: We already hold the mutex_ lock here, so the direct
+  // call to ncclCommGetAsyncError is safe from concurrent access.
+  ncclResult_t result = ncclInProgress;
+  auto startTimepoint = std::chrono::steady_clock::now();
+  auto timeout = nccl_nonblocking_timeout();
+  while (result == ncclInProgress) {
+    auto currentTime = std::chrono::steady_clock::now();
+    auto timeElapsed = std::chrono::duration_cast<std::chrono::seconds>(
+                           currentTime - startTimepoint)
+                           .count();
+    if (timeElapsed > timeout) {
+      std::string err = "NCCL timeout in: " + std::string(__FILE__) + ":" +
+          std::to_string(__LINE__);
+      TORCH_CHECK_WITH(DistBackendError, false, err);
+    }
+    if (longInterval) {
+      std::this_thread::sleep_for(
+          std::chrono::milliseconds(kCommInitBusyWaitMillis));
+    } else {
+      sched_yield();
+    }
+    ncclCommGetAsyncError(ncclComm_, &result);
+  }
+  if (result != ncclSuccess) {
+    std::string err = "NCCL error in: " + std::string(__FILE__) + ":" +
+        std::to_string(__LINE__) + ", " + ncclGetErrorWithVersion(result) +
+        "\n" + getNcclErrorDetailStr(result, std::nullopt);
+    TORCH_CHECK_WITH(DistBackendError, false, err);
   }
 }
 
@@ -178,16 +218,12 @@ std::optional<std::string> NCCLComm::getNcclCommFailureReason() const {
   return commFailureReason_;
 }
 
-// TODO: why do we have `!defined(FBCODE_CAFFE2)` here?
-#if defined(NCCL_HAS_COMM_SPLIT) && !defined(FBCODE_CAFFE2)
-// last argument to split() API is not used to support
-// multiple implementations
+#if defined(NCCL_HAS_COMM_SPLIT)
 std::shared_ptr<NCCLComm> NCCLComm::split(
     NCCLComm* source,
     int color_id,
     int rank,
-    ncclConfig_t& config,
-    std::vector<uint64_t>& ranks_ull) {
+    ncclConfig_t& config) {
   TORCH_CHECK(
       color_id >= NCCL_SPLIT_NOCOLOR,
       "Color must be a non-negative value or NCCL_SPLIT_NOCOLOR (-1)"
@@ -216,7 +252,7 @@ std::shared_ptr<NCCLComm> NCCLComm::split(
   //   https://github.com/NVIDIA/nccl/issues/1472
   C10D_NCCL_CHECK_TIMEOUT_SLEEP(
       ncclCommSplit(sourceComm, color_id, rank, &(comm->ncclComm_), &config),
-      sourceComm, // wait on parent comm
+      source, // wait on parent comm
       std::nullopt);
   if (color_id >= 0) {
     // Waiting for parent comm above still does not seem to guarantee the child
@@ -237,12 +273,74 @@ std::shared_ptr<NCCLComm> NCCLComm::split(
   // Child comm should be on the same device as parent comm
   comm->deviceIndex_ = source->deviceIndex_;
   comm->nonBlocking_ = config.blocking == 0;
+  comm->setUniqueHash(
+      source->getUniqueHash() + ":" +
+      std::to_string(source->ncclCommSplitCounter_));
   LOG(INFO) << "Rank " << source->rank_ << ": created child comm "
             << comm->repr() << " with color_id " << color_id;
   return comm;
 }
 #endif
 
+#ifdef NCCL_HAS_COMM_SHRINK
+std::shared_ptr<NCCLComm> NCCLComm::shrink(
+    NCCLComm* source,
+    std::vector<int>& ranks_to_exclude,
+    ncclConfig_t* config,
+    int shrinkFlags) {
+  // Preconditions are validated in ProcessGroupNCCL::shrink
+
+  LOG(INFO) << "Rank " << source->rank_ << ": shrinking comm " << source->repr()
+            << " excluding " << ranks_to_exclude.size() << " ranks";
+
+  at::cuda::OptionalCUDAGuard gpuGuard(source->deviceIndex_);
+  auto comm = std::make_shared<NCCLComm>();
+
+  // This call will block until the source communicator is initialized
+  auto sourceComm = source->getNcclComm();
+
+  C10D_NCCL_CHECK_NONBLOCKING(
+      ncclCommShrink(
+          sourceComm,
+          ranks_to_exclude.data(),
+          ranks_to_exclude.size(),
+          reinterpret_cast<ncclComm_t*>(&(comm->ncclComm_)),
+          config,
+          shrinkFlags),
+      source->getNcclCommFailureReason());
+
+  // Wait for the child communicator to be ready
+  source->waitReady(true);
+  comm->initialized_ = true;
+
+  // NCCL automatically assigns rank during shrink - query it efficiently
+  int assigned_rank;
+  try {
+    C10D_NCCL_CHECK(
+        ncclCommUserRank(comm->ncclComm_, &assigned_rank), std::nullopt);
+    comm->rank_ = assigned_rank;
+  } catch (const std::exception& e) {
+    // Fallback: if ncclCommUserRank fails, we can't determine the rank
+    LOG(ERROR) << "Failed to query NCCL-assigned rank: " << e.what();
+    throw;
+  }
+
+  // Child comm should be on the same device as parent comm
+  comm->deviceIndex_ = source->deviceIndex_;
+  if (config != nullptr) {
+    comm->nonBlocking_ = config->blocking == 0;
+  } else {
+    // Inherit parent behavior if no config provided
+    comm->nonBlocking_ = source->nonBlocking_;
+  }
+
+  LOG(INFO) << "Rank " << source->rank_ << ": created shrunken comm "
+            << comm->repr() << " with NCCL-assigned rank " << assigned_rank;
+
+  return comm;
+}
+#endif
+
 void NCCLComm::finalize() {
   LockType lock(mutex_);
   if (aborted_) {
@@ -302,8 +400,30 @@ void NCCLComm::abort(std::optional<std::string> commFailureReason) {
 #ifndef NCCL_HAS_COMM_NONBLOCKING
   C10D_NCCL_CHECK(::ncclCommAbort(ncclComm_), commFailureReason_);
 #else
-  C10D_NCCL_CHECK_TIMEOUT(
-      ::ncclCommAbort(ncclComm_), ncclComm_, commFailureReason_);
+  // Note: We already hold the mutex_ lock here, so the direct call to
+  // ncclCommGetAsyncError is safe from concurrent access.
+  ncclResult_t result = ::ncclCommAbort(ncclComm_);
+  auto startTimepoint = std::chrono::steady_clock::now();
+  auto timeout = nccl_nonblocking_timeout();
+  while (result == ncclInProgress) {
+    auto currentTime = std::chrono::steady_clock::now();
+    auto timeElapsed = std::chrono::duration_cast<std::chrono::seconds>(
+                           currentTime - startTimepoint)
+                           .count();
+    if (timeElapsed > timeout) {
+      std::string err = "NCCL timeout in: " + std::string(__FILE__) + ":" +
+          std::to_string(__LINE__);
+      TORCH_CHECK_WITH(DistBackendError, false, err);
+    }
+    sched_yield();
+    ncclCommGetAsyncError(ncclComm_, &result);
+  }
+  if (result != ncclSuccess) {
+    std::string err = "NCCL error in: " + std::string(__FILE__) + ":" +
+        std::to_string(__LINE__) + ", " + ncclGetErrorWithVersion(result) +
+        "\n" + getNcclErrorDetailStr(result, commFailureReason_);
+    TORCH_CHECK_WITH(DistBackendError, false, err);
+  }
 #endif
   aborted_ = true;
   ncclComm_ = nullptr;
@@ -347,6 +467,11 @@ ncclResult_t NCCLComm::checkForNcclError() {
 #endif
 }
 
+ncclResult_t NCCLComm::getAsyncError(ncclResult_t* asyncError) {
+  LockType lock(mutex_);
+  return ncclCommGetAsyncError(ncclComm_, asyncError);
+}
+
 ncclResult_t NCCLComm::registerSegment(
     void* ptr,
     size_t size,
@@ -557,6 +682,27 @@ size_t hashTensors(const std::vector<at::Tensor>& tensors) {
   return hash;
 }
 
+// NCCL uses Non-negative int to represent in-group according to API
+// requirement. We take a list of ranks and generate a hash value based on the
+// list and ensure its range of 32-bit int.
+int genNcclSplitColor(const std::vector<int>& ranks) {
+  // Combine the hash values using a simple reducer (std::hash + fold)
+  std::size_t combined_hash = std::accumulate(
+      ranks.begin(),
+      ranks.end(),
+      std::size_t(0),
+      [](std::size_t acc, int rank) {
+        return acc ^
+            (std::hash<int>{}(rank) + 0x9e3779b9 + (acc << 6) + (acc >> 2));
+      });
+
+  // max positive value of int32_t
+  constexpr int32_t max_c_int = std::numeric_limits<int32_t>::max();
+  int color = static_cast<int>(
+      std::abs(static_cast<int64_t>(combined_hash)) % max_c_int);
+  return color;
+}
+
 // Default value: 30 minutes
 int nccl_nonblocking_timeout() {
   static int timeout = -2; // -2 means not initialized
