@@ -7,7 +7,6 @@
 #include <c10/util/error.h>
 #include <torch/csrc/distributed/c10d/socket.h>
 
-#include <cstring>
 #include <optional>
 #include <system_error>
 #include <utility>
@@ -25,7 +24,6 @@
 #include <netinet/tcp.h>
 #include <poll.h>
 #include <sys/socket.h>
-#include <sys/types.h>
 #include <unistd.h>
 #endif
 
@@ -220,7 +218,7 @@ std::string formatSockAddr(const struct ::sockaddr* addr, socklen_t len) {
   }
   // if we can't resolve the hostname, display the IP address
   if (addr->sa_family == AF_INET) {
-    struct sockaddr_in* psai = (struct sockaddr_in*)&addr;
+    struct sockaddr_in* psai = reinterpret_cast<struct sockaddr_in*>(&addr);
     // NOLINTNEXTLINE(*array*)
     char ip[INET_ADDRSTRLEN];
     if (inet_ntop(addr->sa_family, &(psai->sin_addr), ip, INET_ADDRSTRLEN) !=
@@ -228,7 +226,7 @@ std::string formatSockAddr(const struct ::sockaddr* addr, socklen_t len) {
       return fmt::format("{}:{}", ip, psai->sin_port);
     }
   } else if (addr->sa_family == AF_INET6) {
-    struct sockaddr_in6* psai = (struct sockaddr_in6*)&addr;
+    struct sockaddr_in6* psai = reinterpret_cast<struct sockaddr_in6*>(&addr);
     // NOLINTNEXTLINE(*array*)
     char ip[INET6_ADDRSTRLEN];
     if (inet_ntop(addr->sa_family, &(psai->sin6_addr), ip, INET6_ADDRSTRLEN) !=
@@ -247,12 +245,12 @@ namespace fmt {
 
 template <>
 struct formatter<::addrinfo> {
-  constexpr decltype(auto) parse(format_parse_context& ctx) const {
+  constexpr auto parse(format_parse_context& ctx) const {
     return ctx.begin();
   }
 
   template <typename FormatContext>
-  decltype(auto) format(const ::addrinfo& addr, FormatContext& ctx) const {
+  auto format(const ::addrinfo& addr, FormatContext& ctx) const {
     return fmt::format_to(
         ctx.out(),
         "{}",
@@ -262,14 +260,13 @@ struct formatter<::addrinfo> {
 
 template <>
 struct formatter<c10d::detail::SocketImpl> {
-  constexpr decltype(auto) parse(format_parse_context& ctx) const {
+  constexpr auto parse(format_parse_context& ctx) const {
     return ctx.begin();
   }
 
   template <typename FormatContext>
-  decltype(auto) format(
-      const c10d::detail::SocketImpl& socket,
-      FormatContext& ctx) const {
+  auto format(const c10d::detail::SocketImpl& socket, FormatContext& ctx)
+      const {
     ::sockaddr_storage addr_s{};
 
     auto addr_ptr = reinterpret_cast<::sockaddr*>(&addr_s);
@@ -532,8 +529,8 @@ class SocketListenOp {
 
   std::string port_;
   const SocketOptions* opts_;
-  std::vector<std::string> errors_{};
-  std::unique_ptr<SocketImpl> socket_{};
+  std::vector<std::string> errors_;
+  std::unique_ptr<SocketImpl> socket_;
 };
 
 SocketListenOp::SocketListenOp(std::uint16_t port, const SocketOptions& opts)
@@ -772,9 +769,9 @@ class SocketConnectOp {
   const char* host_;
   std::string port_;
   const SocketOptions* opts_;
-  TimePoint deadline_{};
-  std::vector<std::string> errors_{};
-  std::unique_ptr<SocketImpl> socket_{};
+  TimePoint deadline_;
+  std::vector<std::string> errors_;
+  std::unique_ptr<SocketImpl> socket_;
 };
 
 SocketConnectOp::SocketConnectOp(
