@@ -1,6 +1,4 @@
 #include <ATen/ThreadLocalState.h>
-#include <c10/util/C++17.h>
-#include <torch/csrc/distributed/autograd/context/container.h>
 #include <torch/csrc/distributed/autograd/utils.h>
 #include <torch/csrc/distributed/rpc/message.h>
 #include <torch/csrc/distributed/rpc/python_call.h>
@@ -9,18 +7,15 @@
 #include <torch/csrc/distributed/rpc/python_resp.h>
 #include <torch/csrc/distributed/rpc/python_rpc_handler.h>
 #include <torch/csrc/distributed/rpc/rref_context.h>
-#include <torch/csrc/distributed/rpc/rref_proto.h>
 #include <torch/csrc/distributed/rpc/script_call.h>
 #include <torch/csrc/distributed/rpc/script_remote_call.h>
 #include <torch/csrc/distributed/rpc/script_resp.h>
 #include <torch/csrc/distributed/rpc/torchscript_functions.h>
 #include <torch/csrc/distributed/rpc/utils.h>
 #include <torch/csrc/jit/runtime/operator.h>
-#include <torch/csrc/utils/python_compat.h>
+#include <exception>
 
-namespace torch {
-namespace distributed {
-namespace rpc {
+namespace torch::distributed::rpc {
 
 namespace {
 
@@ -45,9 +40,9 @@ IValue toPyIValue(const Message& message) {
       auto& pythonRpcHandler = PythonRpcHandler::getInstance();
       // Need GIL to destruct the py::object returned by deserialize()
       py::gil_scoped_acquire acquire;
-      return jit::toIValue(
-          pythonRpcHandler.deserialize(resp.serializedPyObj()),
-          PyObjectType::get());
+      py::object value = pythonRpcHandler.deserialize(resp.serializedPyObj());
+      pythonRpcHandler.handleException(value);
+      return jit::toIValue(value, PyObjectType::get());
     }
     default: {
       TORCH_CHECK(false, "Unrecognized response message type ", msgType);
@@ -89,8 +84,8 @@ std::shared_ptr<Operator> matchBuiltinOp(
     } catch (const std::runtime_error& e) {
       opWithStack = torch::jit::getOpWithStack(ops, args, kwargs);
     }
-    matchedOperator = std::get<0>(opWithStack);
-    stack = std::get<1>(opWithStack);
+    matchedOperator = std::move(std::get<0>(opWithStack));
+    stack = std::move(std::get<1>(opWithStack));
   }
 
   // We should never hit this path, since if !matchedOperator, then the last
@@ -109,7 +104,7 @@ std::shared_ptr<Operator> matchBuiltinOp(
   return matchedOperator;
 }
 
-std::shared_ptr<JitFuture> sendPythonRemoteCall(
+c10::intrusive_ptr<JitFuture> sendPythonRemoteCall(
     const WorkerInfo& dst,
     SerializedPyObj serializedPyObj,
     const IValue& rrefId,
@@ -135,27 +130,57 @@ std::shared_ptr<JitFuture> sendPythonRemoteCall(
 using namespace torch::distributed::autograd;
 
 c10::intrusive_ptr<JitFuture> toPyJitFuture(
-    const std::shared_ptr<JitFuture>& messageJitFuture,
+    const c10::intrusive_ptr<JitFuture>& messageJitFuture,
     bool hasValue) {
   if (hasValue) {
-    std::weak_ptr<JitFuture> wp = messageJitFuture;
-    return messageJitFuture->then(
-        at::wrapPropagateTLSState<IValue>([wp]() {
-          auto future = wp.lock();
-          if (future->hasError()) {
-            std::rethrow_exception(future->exception_ptr());
+    auto child = messageJitFuture->createInstance(PyObjectType::get());
+    messageJitFuture->addCallback(
+        at::wrapPropagateTLSState([child](JitFuture& future) {
+          if (future.hasError()) {
+            child->setError(future.exception_ptr());
           } else {
-            return toPyIValue(*future->value().toCustomClass<Message>());
+            const Message& message = *future.value().toCustomClass<Message>();
+
+            // toPyIValue might throw and we need to record the appropriate
+            // exception.
+            IValue ivalue;
+            try {
+              ivalue = toPyIValue(message);
+            } catch (py::error_already_set& e) {
+              py::gil_scoped_acquire acquire;
+              // FIXME: this is a temporary solution to add a special-case for
+              // ValueError and TypeError, as those are already used in our
+              // tests. We should have a more comprehensive coverage for other
+              // types of exceptions as well.
+              if (e.matches(PyExc_ValueError)) {
+                child->setErrorIfNeeded(
+                    std::make_exception_ptr(pybind11::value_error(e.what())));
+              } else if (e.matches(PyExc_TypeError)) {
+                child->setErrorIfNeeded(
+                    std::make_exception_ptr(pybind11::type_error(e.what())));
+              } else {
+                // py::error_already_set requires GIL to destruct, take special
+                // care.
+                child->setErrorIfNeeded(
+                    std::make_exception_ptr(std::runtime_error(e.what())));
+              }
+              e.restore();
+              PyErr_Clear();
+              return;
+            } catch (std::exception& e) {
+              child->setErrorIfNeeded(std::current_exception());
+              return;
+            }
+
+            child->markCompleted(ivalue, future.storages());
           }
-        }),
-        PyObjectType::get());
+        }));
+    return child;
   } else {
-    std::weak_ptr<JitFuture> wp = messageJitFuture;
     return messageJitFuture->then(
-        at::wrapPropagateTLSState<IValue>([wp]() {
-          auto future = wp.lock();
-          if (future->hasError()) {
-            std::rethrow_exception(future->exception_ptr());
+        at::wrapPropagateTLSState([](JitFuture& future) {
+          if (future.hasError()) {
+            std::rethrow_exception(future.exception_ptr());
           } else {
             return IValue();
           }
@@ -231,14 +256,14 @@ c10::intrusive_ptr<JitFuture> pyRpcTorchscript(
         functionSchema,
         argsTuple.cast<py::args>(),
         kwargsDict.cast<py::kwargs>(),
-        c10::nullopt);
+        std::nullopt);
   }
   DCHECK(!PyGILState_Check());
   c10::intrusive_ptr<c10::ivalue::Future> fut = rpcTorchscript(
       dstWorkerName,
       qualifiedName,
       functionSchema,
-      stack,
+      std::move(stack),
       rpcTimeoutSeconds,
       isAsyncExecution);
   return fut;
@@ -275,10 +300,9 @@ PyRRef pyRemoteBuiltin(
 
     userRRef->registerOwnerCreationFuture(jitFuture);
     ctx.addPendingUser(userRRef->forkId(), userRRef);
-    std::weak_ptr<JitFuture> wp = jitFuture;
-    jitFuture->addCallback(
-        at::wrapPropagateTLSState<void>([wp, forkId{userRRef->forkId()}]() {
-          callback::confirmPendingUser(*wp.lock(), forkId);
+    jitFuture->addCallback(at::wrapPropagateTLSState(
+        [forkId{userRRef->forkId()}](JitFuture& future) {
+          callback::confirmPendingUser(future, forkId);
         }));
     return PyRRef(userRRef);
   } else {
@@ -298,10 +322,9 @@ PyRRef pyRemoteBuiltin(
     ownerRRef->registerOwnerCreationFuture(jitFuture);
     // Builtin operators does not return py::object, and hence does not require
     // GIL for destructing the potentially deleted OwerRRef.
-    std::weak_ptr<JitFuture> wp = jitFuture;
-    jitFuture->addCallback(at::wrapPropagateTLSState<void>(
-        [wp, ownerRRefId = ownerRRef->rrefId()]() {
-          callback::finishCreatingOwnerRRef(*wp.lock(), ownerRRefId);
+    jitFuture->addCallback(at::wrapPropagateTLSState(
+        [ownerRRefId = ownerRRef->rrefId()](JitFuture& future) {
+          callback::finishCreatingOwnerRRef(future, ownerRRefId);
         }));
     return PyRRef(ownerRRef);
   }
@@ -330,10 +353,9 @@ PyRRef pyRemotePythonUdf(
 
     userRRef->registerOwnerCreationFuture(jitFuture);
     ctx.addPendingUser(userRRef->forkId(), userRRef);
-    std::weak_ptr<JitFuture> wp = jitFuture;
-    jitFuture->addCallback(
-        at::wrapPropagateTLSState<void>([wp, forkId{userRRef->forkId()}]() {
-          callback::confirmPendingUser(*wp.lock(), forkId);
+    jitFuture->addCallback(at::wrapPropagateTLSState(
+        [forkId{userRRef->forkId()}](JitFuture& future) {
+          callback::confirmPendingUser(future, forkId);
         }));
     return PyRRef(userRRef);
   } else {
@@ -350,11 +372,10 @@ PyRRef pyRemotePythonUdf(
         isAsyncExecution);
 
     ownerRRef->registerOwnerCreationFuture(jitFuture);
-    std::weak_ptr<JitFuture> wp = jitFuture;
-    jitFuture->addCallback(at::wrapPropagateTLSState<void>(
-        [wp, ownerRRefId = ownerRRef->rrefId()]() {
+    jitFuture->addCallback(at::wrapPropagateTLSState(
+        [ownerRRefId = ownerRRef->rrefId()](JitFuture& future) {
           auto deletedRRef =
-              callback::finishCreatingOwnerRRef(*wp.lock(), ownerRRefId);
+              callback::finishCreatingOwnerRRef(future, ownerRRefId);
           if (deletedRRef && deletedRRef->isPyObj()) {
             py::gil_scoped_acquire ag;
             deletedRRef.reset();
@@ -382,7 +403,7 @@ PyRRef pyRemoteTorchscript(
     // Acquire GIL for py::args and py::kwargs processing.
     py::gil_scoped_acquire ag;
     stack = torch::jit::createStackForSchema(
-        functionSchema, args, kwargs, c10::nullopt);
+        functionSchema, args, kwargs, std::nullopt);
   }
   DCHECK(!PyGILState_Check());
   auto rrefPtr = remoteTorchscript(
@@ -395,6 +416,4 @@ PyRRef pyRemoteTorchscript(
   return PyRRef(rrefPtr);
 }
 
-} // namespace rpc
-} // namespace distributed
-} // namespace torch
+} // namespace torch::distributed::rpc
