@@ -1,3 +1,6 @@
+#include <torch/csrc/distributed/rpc/request_callback_no_python.h>
+
+#include <c10/core/StreamGuard.h>
 #include <torch/csrc/distributed/autograd/context/container.h>
 #include <torch/csrc/distributed/autograd/engine/dist_engine.h>
 #include <torch/csrc/distributed/autograd/rpc_messages/cleanup_autograd_context_req.h>
@@ -7,18 +10,18 @@
 #include <torch/csrc/distributed/autograd/rpc_messages/rpc_with_autograd.h>
 #include <torch/csrc/distributed/autograd/utils.h>
 #include <torch/csrc/distributed/rpc/profiler/server_process_global_profiler.h>
-#include <torch/csrc/distributed/rpc/request_callback_no_python.h>
 #include <torch/csrc/distributed/rpc/rpc_agent.h>
 #include <torch/csrc/distributed/rpc/rref_context.h>
 #include <torch/csrc/distributed/rpc/rref_proto.h>
 #include <torch/csrc/distributed/rpc/script_resp.h>
 #include <torch/csrc/distributed/rpc/utils.h>
 
-namespace torch {
-namespace distributed {
-namespace rpc {
+#include <utility>
+
+namespace torch::distributed::rpc {
 
 using namespace torch::distributed::autograd;
+using namespace torch::autograd::profiler;
 
 // When request message has autograd info, processMessage() will set up valid
 // current context id properly. This struct is used to clean up current context
@@ -29,6 +32,10 @@ struct DistAutogradContextGuard {
     prevCtxId_ = container.currentContextId();
     container.forceCurrentContextId(ctxId);
   }
+  DistAutogradContextGuard(const DistAutogradContextGuard&) = delete;
+  DistAutogradContextGuard(DistAutogradContextGuard&&) = delete;
+  DistAutogradContextGuard& operator=(const DistAutogradContextGuard&) = delete;
+  DistAutogradContextGuard& operator=(DistAutogradContextGuard&&) = delete;
   ~DistAutogradContextGuard() {
     auto& container = DistAutogradContainer::getInstance();
     container.forceCurrentContextId(prevCtxId_);
@@ -48,13 +55,13 @@ std::unique_ptr<RpcCommandBase> RequestCallbackNoPython::
   return rpc;
 }
 
-std::shared_ptr<JitFuture> RequestCallbackNoPython::processMessage(
-    Message& request) const {
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::processMessage(
+    Message& request,
+    std::vector<c10::Stream> streams) const {
   // We need two futures here because it could pause twice when processing a
   // RPC message:
   //  1) waiting for all RRefs in the arguments to become confirmed;
   //  2) waiting for processRpc to finish.
-  auto retFuture = std::make_shared<JitFuture>(at::AnyClassType::get());
   auto& rrefContext = RRefContext::getInstance();
   try {
     rrefContext.recordThreadLocalPendingRRefs();
@@ -63,257 +70,196 @@ std::shared_ptr<JitFuture> RequestCallbackNoPython::processMessage(
         deserializeRequest(request), request.type());
     auto rrefsReadyFuture = rrefContext.waitForThreadLocalPendingRRefs();
 
-    rrefsReadyFuture->addCallback(
+    auto retFuture = rrefsReadyFuture->thenAsync(
         [this,
-         retFuture,
-         // std::function must be copyable, hence hae to cast the unique_ptr to
+         // std::function must be copyable, hence has to cast the unique_ptr to
          // a shared_ptr here.
-         rpc = (std::shared_ptr<RpcCommandBase>)std::move(rpc),
+         rpc = std::shared_ptr<RpcCommandBase>(std::move(rpc)),
          messageType = request.type(),
-         id = request.id()]() {
+         streams = std::move(streams)](JitFuture& /* unused */) mutable {
           // The cost of pre-request check is minimal thanks to
           // std::shared_lock. The cost is in magnitude
           // of 10us.
           auto serverProcessGlobalProfilerStateStackEntryPtr =
               profiler::processglobal::StateStackEntry::current();
-          // If server global profiler is enabled, we futher pay the
+          // If server global profiler is enabled, we further pay the
           // cost of thread local profiler state initialization.
           if (serverProcessGlobalProfilerStateStackEntryPtr) {
             // Initialize thread-local profiler state from process-global
             // profiler state.
-            ::torch::autograd::profiler::enableProfilerLegacy(
+            enableProfilerLegacy(
                 serverProcessGlobalProfilerStateStackEntryPtr->statePtr()
                     ->config());
           }
 
-          processRpcWithErrors(*rpc, messageType, id, retFuture);
+          auto retFuture = processRpcWithErrors(*rpc, messageType, streams);
 
           // Response message has been sent at this moment, this post-response
           // work doesn't affect RPC trip time.
           if (serverProcessGlobalProfilerStateStackEntryPtr) {
             // Restore thread-local profiler state.
-            ::torch::autograd::profiler::thread_event_lists event_lists =
-                ::torch::autograd::profiler::disableProfilerLegacy();
+            thread_event_lists event_lists = disableProfilerLegacy();
             // Put thread_local event_lists into the process-global profiler
             // state.
             profiler::processglobal::pushResultRecursive(
                 serverProcessGlobalProfilerStateStackEntryPtr, event_lists);
           }
-        });
+
+          return retFuture;
+        },
+        c10::getCustomClassType<c10::intrusive_ptr<Message>>());
+
+    auto retFutureWithMessageId = retFuture->then(
+        [id = request.id()](JitFuture& future) {
+          c10::intrusive_ptr<Message> message =
+              future.value().toCustomClass<Message>();
+          message->setId(id);
+          return withStorages(message);
+        },
+        c10::getCustomClassType<c10::intrusive_ptr<Message>>());
+
+    return retFutureWithMessageId;
   } catch (std::exception& e) {
-    retFuture->markCompleted(handleError(e, request.type(), request.id()));
     rrefContext.clearRecordedPendingRRefsOnError();
+    return asFuture(handleError(e, request.type(), request.id()));
   }
-  return retFuture;
 }
 
-void RequestCallbackNoPython::processRpcWithErrors(
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::processRpcWithErrors(
     RpcCommandBase& rpc,
     const MessageType& messageType,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& responseFuture) const {
+    const std::vector<c10::Stream>& streams) const {
   try {
-    processRpc(rpc, messageType, messageId, responseFuture);
+    return processRpc(rpc, messageType, streams);
   } catch (std::exception& e) {
-    responseFuture->markCompleted(handleError(e, messageType, messageId));
+    // Pass a dummy message ID since it will be overwritten anyways.
+    return asFuture(handleError(e, messageType, -1));
   }
 }
 
-void RequestCallbackNoPython::processScriptCall(
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::processScriptCall(
     RpcCommandBase& rpc,
-    const std::function<void(Message)>& markComplete,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& /* unused */) const {
+    const std::vector<c10::Stream>& streams) const {
   auto& scriptCall = static_cast<ScriptCall&>(rpc);
-  auto& stack = scriptCall.stackRef();
+
   TORCH_CHECK(
       scriptCall.hasOp(), "Only supports the case where ScriptCall has an op");
-  processScriptCallOp(scriptCall, markComplete, stack);
+  auto future =
+      runJitOperator(*scriptCall.op(), scriptCall.stackRef(), streams);
+
+  return future->then(
+      [](JitFuture& future) {
+        return withStorages(ScriptResp(future.value()).toMessage());
+      },
+      c10::getCustomClassType<c10::intrusive_ptr<Message>>());
 }
 
-bool RequestCallbackNoPython::processScriptCallOp(
-    ScriptCall& scriptCall,
-    const std::function<void(Message)>& markComplete,
-    std::vector<at::IValue>& stack) const {
-  if (scriptCall.hasOp()) {
-    scriptCall.op()->getOperation()(&stack);
-    TORCH_INTERNAL_ASSERT(
-        stack.size() == 1,
-        "Return value of a builtin operator or a "
-        "TorchScript function should be a single IValue, got a vector of "
-        "size ",
-        stack.size());
-    markComplete(std::move(ScriptResp(std::move(stack.front()))).toMessage());
-    return true;
-  }
-  return false;
-}
-
-TypePtr RequestCallbackNoPython::getScriptRemoteCallType(
-    ScriptRemoteCall& scriptRemoteCall) const {
-  TORCH_CHECK(
-      scriptRemoteCall.hasOp(),
-      "Only supports the case where ScriptCall has an op");
-  return scriptRemoteCall.op()->schema().returns()[0].type();
-}
-
-void RequestCallbackNoPython::processPythonCall(
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::processPythonCall(
     RpcCommandBase& rpc,
-    const std::function<void(Message)>& markComplete,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& /* unused */) const {
+    const std::vector<c10::Stream>& /* unused */) const {
   C10_THROW_ERROR(Error, "Python call not supported!");
 }
 
-void RequestCallbackNoPython::processPythonRemoteCall(
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::processPythonRemoteCall(
     RpcCommandBase& rpc,
-    const std::function<void(Message)>& markComplete,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& /* unused */) const {
+    const std::vector<c10::Stream>& /* unused */) const {
   C10_THROW_ERROR(Error, "Python call not supported!");
 }
 
-void RequestCallbackNoPython::processScriptRemoteCall(
-    ScriptRemoteCall& scriptRemoteCall,
-    const std::function<void(void)>& postProcessing,
-    std::vector<at::IValue>& stack,
-    const c10::intrusive_ptr<OwnerRRef>& ownerRRef) const {
-  TORCH_CHECK(
-      scriptRemoteCall.hasOp(), "ScriptRemoteCall needs to have an op!");
-  processScriptRemoteCallOp(scriptRemoteCall, postProcessing, stack, ownerRRef);
-}
-
-void RequestCallbackNoPython::processBaseScriptRemoteCall(
-    RpcCommandBase& rpc,
-    const std::function<void(Message)>& markComplete,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& responseFuture) const {
-  auto& scriptRemoteCall = static_cast<ScriptRemoteCall&>(rpc);
-  auto rrefId = scriptRemoteCall.retRRefId();
-  auto forkId = scriptRemoteCall.retForkId();
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::assignOwnerRRef(
+    const RRefId& rrefId,
+    const RRefId& forkId,
+    const c10::intrusive_ptr<JitFuture>& valueFuture) const {
   auto& ctx = RRefContext::getInstance();
 
-  auto postProcessing = [rrefId, forkId, messageId, responseFuture]() {
-    if (rrefId != forkId) {
-      // Caller is a user and callee is the owner, add fork
-      //
-      // NB: rrefId == forkId is true if and only if calling remote to
-      // self. In that case both the caller and the callee will access
-      // the OwnerRRef. Hence, on the callee side (here), it should not
-      // call addForkOfOwner as it is not a fork. To allow callee to
-      // distinguish when this request is sent to self, the caller will
-      // set forkId using rrefId (OwnerRRef does not have a forkId
-      // anyway).
-      RRefContext::getInstance().addForkOfOwner(rrefId, forkId);
-    }
-    Message m = RemoteRet(rrefId, forkId).toMessage();
-    m.setId(messageId);
-    responseFuture->markCompleted(
-        IValue(c10::make_intrusive<Message>(std::move(m))));
-  };
-
-  // scriptRemoteCall is only alive within this block, use reference to
-  // avoid copy. If the underlying code runs with a continuation, runAsync()
-  // below will std::move the appropriate portion of the stack.
-  TypePtr returnType = getScriptRemoteCallType(scriptRemoteCall);
   c10::intrusive_ptr<OwnerRRef> ownerRRef;
   if (rrefId == forkId) {
     // Creating an owner RRef on self, should already exist in owners map
-    ownerRRef = ctx.getOwnerRRef(rrefId, /* forceCreated */ true)->constValue();
+    ownerRRef =
+        fromRRefInterface(ctx.getOwnerRRef(rrefId, /* forceCreated */ true)
+                              ->constValue()
+                              .toRRef());
   } else {
-    ownerRRef = ctx.getOrCreateOwnerRRef(rrefId, returnType);
+    ownerRRef = ctx.getOrCreateOwnerRRef(rrefId, valueFuture->elementType());
+    // Caller is a user and callee is the owner, add fork
+    //
+    // NB: rrefId == forkId is true if and only if calling remote to self.
+    // In that case both the caller and the callee will access the
+    // OwnerRRef. Hence, on the callee side (here), it should not call
+    // addForkOfOwner as it is not a fork. To allow callee to distinguish
+    // when this request is sent to self, the caller will set forkId using
+    // rrefId (OwnerRRef does not have a forkId anyway).
+    ctx.addForkOfOwner(rrefId, forkId);
   }
 
-  auto& stack = scriptRemoteCall.stackRef();
-  processScriptRemoteCall(scriptRemoteCall, postProcessing, stack, ownerRRef);
+  return valueFuture->then(
+      [ownerRRef, rrefId, forkId](JitFuture& future) {
+        if (future.hasError()) {
+          ownerRRef->setError(future.exception_ptr());
+        } else {
+          ownerRRef->setValue(future.value());
+        }
+        return withStorages(RemoteRet(rrefId, forkId).toMessage());
+      },
+      c10::getCustomClassType<c10::intrusive_ptr<Message>>());
 }
 
-bool RequestCallbackNoPython::processScriptRemoteCallOp(
-    ScriptRemoteCall& scriptRemoteCall,
-    const std::function<void(void)>& postProcessing,
-    std::vector<at::IValue>& stack,
-    const c10::intrusive_ptr<OwnerRRef>& ownerRRef) const {
-  if (scriptRemoteCall.hasOp()) {
-    try {
-      scriptRemoteCall.op()->getOperation()(&stack);
-    } catch (const std::exception& e) {
-      // Don't throw in this call, but rather transfer the exception
-      // to the rref.
-      ownerRRef->setError(std::current_exception());
-      postProcessing();
-      return true;
-    }
-    TORCH_INTERNAL_ASSERT(
-        stack.size() == 1,
-        "Return value of a builtin operator or a "
-        "TorchScript function should be a single IValue, got a vector of "
-        "size ",
-        stack.size());
-    ownerRRef->setValue(std::move(stack.front()));
-    postProcessing();
-    return true;
-  }
-  return false;
-}
-
-void RequestCallbackNoPython::processScriptRRefFetchCall(
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::processScriptRemoteCall(
     RpcCommandBase& rpc,
-    const std::function<void(Message)>& markComplete,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& responseFuture) const {
-  auto& srf = static_cast<ScriptRRefFetchCall&>(rpc);
+    const std::vector<c10::Stream>& streams) const {
+  auto& scriptRemoteCall = static_cast<ScriptRemoteCall&>(rpc);
+
+  TORCH_CHECK(
+      scriptRemoteCall.hasOp(), "ScriptRemoteCall needs to have an op!");
+  auto future = runJitOperator(
+      *scriptRemoteCall.op(), scriptRemoteCall.stackRef(), streams);
+
+  return assignOwnerRRef(
+      scriptRemoteCall.retRRefId(), scriptRemoteCall.retForkId(), future);
+}
+
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::retrieveOwnerRRef(
+    const RRefId& rrefId) const {
   auto& ctx = RRefContext::getInstance();
 
-  auto futureOwner = ctx.getOwnerRRef(srf.rrefId());
+  auto rrefFuture = ctx.getOwnerRRef(rrefId);
 
-  if (futureOwner->completed()) { // optional fast-path
-    // the OwnerRRef has been created
-    const auto& rref = futureOwner->constValue();
-    if (rref->hasValue()) {
-      markComplete(ScriptRRefFetchRet({rref->getValue()}).toMessage());
-      return;
-    }
-  }
-
-  futureOwner->addCallback([responseFuture, messageId, futureOwner]() {
-    const auto& rref = futureOwner->constValue();
-    auto whenValueSet = rref->getFuture();
-
-    // Our response is satisfied when the rpc.remote() request
-    // finishes executing on the owner.
-    whenValueSet->addCallback(
-        [responseFuture, messageId, rref, whenValueSet]() {
-          if (whenValueSet->hasError()) {
-            responseFuture->setError(whenValueSet->exception_ptr());
-            return;
-          }
-          try {
-            Message m = ScriptRRefFetchRet({rref->getValue()}).toMessage();
-            m.setId(messageId);
-            responseFuture->markCompleted(
-                IValue(c10::make_intrusive<Message>(std::move(m))));
-          } catch (const std::exception& /* unused */) {
-            responseFuture->setError(std::current_exception());
-          }
-        });
-  });
+  at::TypePtr type = rrefFuture->elementType();
+  TORCH_INTERNAL_ASSERT(type->kind() == at::RRefType::Kind);
+  return rrefFuture->thenAsync(
+      [](JitFuture& rrefFuture) {
+        c10::intrusive_ptr<OwnerRRef> rref =
+            fromRRefInterface(rrefFuture.value().toRRef());
+        return rref->getFuture();
+      },
+      type->cast<at::RRefType>()->getElementType());
 }
 
-void RequestCallbackNoPython::processPythonRRefFetchCall(
-    RpcCommandBase& rpc,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& /* unused */) const {
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::
+    processScriptRRefFetchCall(RpcCommandBase& rpc) const {
+  auto& srf = static_cast<ScriptRRefFetchCall&>(rpc);
+
+  auto future = retrieveOwnerRRef(srf.rrefId());
+
+  return future->then(
+      [](JitFuture& future) {
+        return withStorages(ScriptRRefFetchRet({future.value()}).toMessage());
+      },
+      c10::getCustomClassType<c10::intrusive_ptr<Message>>());
+}
+
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::
+    processPythonRRefFetchCall(RpcCommandBase& rpc) const {
   C10_THROW_ERROR(Error, "Python call not supported!");
 }
 
-void RequestCallbackNoPython::processRRefUserDelete(
-    RpcCommandBase& rpc,
-    const std::function<void(Message)>& markComplete) const {
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::processRRefUserDelete(
+    RpcCommandBase& rpc) const {
   auto& rud = static_cast<RRefUserDelete&>(rpc);
   auto& ctx = RRefContext::getInstance();
   auto deletedRRef = ctx.delForkOfOwner(rud.rrefId(), rud.forkId());
   handleRRefDelete(deletedRRef);
-  markComplete(std::move(RRefAck()).toMessage());
+  return asFuture(RRefAck().toMessage());
 }
 
 void RequestCallbackNoPython::handleRRefDelete(
@@ -321,33 +267,31 @@ void RequestCallbackNoPython::handleRRefDelete(
   TORCH_CHECK(!rref->isPyObj(), "RRefs with python objects not supported!");
 }
 
-void RequestCallbackNoPython::processRRefChildAccept(
-    RpcCommandBase& rpc,
-    const std::function<void(Message)>& markComplete) const {
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::processRRefChildAccept(
+    RpcCommandBase& rpc) const {
   auto& rca = static_cast<RRefChildAccept&>(rpc);
   auto& ctx = RRefContext::getInstance();
   ctx.delPendingChild(rca.forkId());
-  markComplete(std::move(RRefAck()).toMessage());
+  return asFuture(RRefAck().toMessage());
 }
 
-void RequestCallbackNoPython::processRRefForkRequest(
-    RpcCommandBase& rpc,
-    const std::function<void(Message)>& markComplete) const {
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::processRRefForkRequest(
+    RpcCommandBase& rpc) const {
   auto& rfr = static_cast<RRefForkRequest&>(rpc);
   auto& ctx = RRefContext::getInstance();
   ctx.addForkOfOwnerIfNotPresent(rfr.rrefId(), rfr.forkId());
-  markComplete(RRefAck().toMessage());
+  return asFuture(RRefAck().toMessage());
 }
 
-void RequestCallbackNoPython::processForwardAutogradReq(
-    RpcCommandBase& rpc,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& responseFuture) const {
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::
+    processForwardAutogradReq(
+        RpcCommandBase& rpc,
+        const std::vector<c10::Stream>& streams) const {
   auto& rpcWithAutograd = static_cast<RpcWithAutograd&>(rpc);
 
   // Need to reverse the device map for the backward pass of distributed
   // autograd.
-  std::unordered_map<c10::DeviceIndex, c10::DeviceIndex> reverseDeviceMap;
+  DeviceMap reverseDeviceMap;
   for (const auto& mapEntry : rpcWithAutograd.deviceMap()) {
     reverseDeviceMap.insert({mapEntry.second, mapEntry.first});
   }
@@ -371,27 +315,17 @@ void RequestCallbackNoPython::processForwardAutogradReq(
 
   // Process the original RPC.
   auto wrappedMessageType = rpcWithAutograd.wrappedMessageType();
-  // Make an overall future for the wrapped response.
-  auto wrappedRpcResponseFuture =
-      std::make_shared<JitFuture>(at::AnyClassType::get());
   // Kick off processing for the nested RPC command.
   // wrappedRpcResponseFuture will be a Future<T> to the result.
-  processRpc(
-      rpcWithAutograd.wrappedRpc(),
-      wrappedMessageType,
-      messageId,
-      wrappedRpcResponseFuture);
+  auto wrappedRpcResponseFuture =
+      processRpc(rpcWithAutograd.wrappedRpc(), wrappedMessageType, streams);
 
   auto fromWorkerId = rpcWithAutograd.fromWorkerId();
   // The original future needs to be marked as completed when the wrapped
   // one completes, with the autograd context information wrapped.
-  // Uses weak_ptr so we can std::move the value.
-  wrappedRpcResponseFuture->addCallback(
-      [responseFuture,
-       messageId,
-       fromWorkerId,
-       weak = std::weak_ptr<JitFuture>(wrappedRpcResponseFuture),
-       ctxId = autogradContext->contextId()]() {
+  auto responseFuture = wrappedRpcResponseFuture->then(
+      [fromWorkerId, ctxId = autogradContext->contextId()](
+          JitFuture& wrappedRpcResponseFuture) {
         // As this callback can be invoked by a different thread, we have to
         // make sure that the thread_local states in the previous thread is
         // correctly propagated.
@@ -404,28 +338,27 @@ void RequestCallbackNoPython::processForwardAutogradReq(
         // https://github.com/pytorch/pytorch/issues/38510
         DistAutogradContextGuard cbCtxGuard(ctxId);
 
-        auto wrappedRpcResponseFuture = weak.lock();
-        TORCH_INTERNAL_ASSERT(wrappedRpcResponseFuture);
-        if (wrappedRpcResponseFuture->hasError()) {
+        if (wrappedRpcResponseFuture.hasError()) {
           // Propagate error to responseFuture if we had one.
-          responseFuture->setError(wrappedRpcResponseFuture->exception_ptr());
+          std::rethrow_exception(wrappedRpcResponseFuture.exception_ptr());
         } else {
           auto msg = getMessageWithAutograd(
               fromWorkerId,
-              std::move(
-                  *wrappedRpcResponseFuture->value().toCustomClass<Message>()),
+              wrappedRpcResponseFuture.value().toCustomClass<Message>(),
               MessageType::FORWARD_AUTOGRAD_RESP);
-          msg.setId(messageId);
-          responseFuture->markCompleted(
-              IValue(c10::make_intrusive<Message>(std::move(msg))));
+          return withStorages(std::move(msg));
         }
-      });
+      },
+      c10::getCustomClassType<c10::intrusive_ptr<Message>>());
+
+  return responseFuture;
 }
 
-void RequestCallbackNoPython::processBackwardAutogradReq(
-    RpcCommandBase& rpc,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& responseFuture) const {
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::
+    processBackwardAutogradReq(
+        RpcCommandBase& rpc,
+        const std::vector<c10::Stream>& streams) const {
+  c10::MultiStreamGuard guard(streams);
   auto& gradientsCall = static_cast<PropagateGradientsReq&>(rpc);
   const auto& autogradMetadata = gradientsCall.getAutogradMetadata();
 
@@ -445,21 +378,19 @@ void RequestCallbackNoPython::processBackwardAutogradReq(
       autogradContext, sendFunction, gradientsCall.retainGraph());
 
   // Our response is satisfied when the rpcs come back.
-  execFuture->addCallback([responseFuture, messageId, execFuture]() {
-    if (!execFuture->hasError()) {
-      Message m = std::move(PropagateGradientsResp()).toMessage();
-      m.setId(messageId);
-      responseFuture->markCompleted(
-          IValue(c10::make_intrusive<Message>(std::move(m))));
-    } else {
-      responseFuture->setError(execFuture->exception_ptr());
-    }
-  });
+  return execFuture->then(
+      [](JitFuture& execFuture) {
+        if (execFuture.hasError()) {
+          std::rethrow_exception(execFuture.exception_ptr());
+        } else {
+          return withStorages(PropagateGradientsResp().toMessage());
+        }
+      },
+      c10::getCustomClassType<c10::intrusive_ptr<Message>>());
 }
 
-void RequestCallbackNoPython::processCleanupAutogradContextReq(
-    RpcCommandBase& rpc,
-    const std::function<void(Message)>& markComplete) const {
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::
+    processCleanupAutogradContextReq(RpcCommandBase& rpc) const {
   auto& cleanupContextReq = static_cast<CleanupAutogradContextReq&>(rpc);
   auto cleanupContextId = cleanupContextReq.getContextId();
   // release the context if it still exists on this thread. We need to
@@ -468,22 +399,29 @@ void RequestCallbackNoPython::processCleanupAutogradContextReq(
   // notified to clean up their context.
   DistAutogradContainer::getInstance().releaseContextIfPresent(
       cleanupContextId);
-  markComplete(std::move(CleanupAutogradContextResp()).toMessage());
+  return asFuture(CleanupAutogradContextResp().toMessage());
 }
 
-void RequestCallbackNoPython::processRunWithProfilingReq(
-    RpcCommandBase& rpc,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& responseFuture) const {
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::
+    processRunWithProfilingReq(RpcCommandBase& rpc) const {
   auto& rpcWithProfilingReq = static_cast<RpcWithProfilingReq&>(rpc);
   auto wrappedMsgType = rpcWithProfilingReq.wrappedMessageType();
   auto profilingConfig = rpcWithProfilingReq.getProfilingConfig();
+
+  if (profilingConfig.state == ProfilerState::KINETO ||
+      profilingConfig.state == ProfilerState::KINETO_GPU_FALLBACK ||
+      profilingConfig.state == ProfilerState::KINETO_PRIVATEUSE1_FALLBACK) {
+    profilingConfig = ProfilerConfig(
+        ProfilerState::CPU,
+        profilingConfig.report_input_shapes,
+        profilingConfig.profile_memory);
+  }
+
   // If requested with CUDA from caller but CUDA is not available on this
   // machine, fallback to CPU and log a warning instead of crashing.
-  if (profilingConfig.state == torch::autograd::profiler::ProfilerState::CUDA &&
-      !this->cudaAvailable()) {
-    profilingConfig = torch::autograd::profiler::ProfilerConfig(
-        torch::autograd::profiler::ProfilerState::CPU,
+  if (profilingConfig.state == ProfilerState::CUDA && !this->cudaAvailable()) {
+    profilingConfig = ProfilerConfig(
+        ProfilerState::CPU,
         profilingConfig.report_input_shapes,
         profilingConfig.profile_memory);
 
@@ -492,90 +430,74 @@ void RequestCallbackNoPython::processRunWithProfilingReq(
                  << "Falling back to CPU profiling only.";
   }
   TORCH_INTERNAL_ASSERT(
-      profilingConfig.state != torch::autograd::profiler::ProfilerState::CUDA ||
-          this->cudaAvailable(),
+      profilingConfig.state != ProfilerState::CUDA || this->cudaAvailable(),
       "Profiler state set to CUDA but CUDA not available.");
   const auto profilingKeyId = rpcWithProfilingReq.getProfilingId();
-  auto wrappedRpcResponseFuture =
-      std::make_shared<JitFuture>(at::AnyClassType::get());
   // Enable the profiler with the config from the sender.
   // When enabling on the main thread, ensure profiler states are cleaned
   // up, but defer consolidation of all profiled events to the continuation
   // below.
-  torch::autograd::profiler::ProfilerDisableOptions requestThreadOptions(
+  ProfilerDisableOptions requestThreadOptions(
       true /* cleanup TLS state */, false /* consolidate events */);
   {
-    torch::autograd::profiler::TLSProfilerGuard g(
-        profilingConfig, c10::nullopt, requestThreadOptions);
+    TLSLegacyProfilerGuard g(
+        profilingConfig, std::nullopt, requestThreadOptions);
     TORCH_INTERNAL_ASSERT(
-        torch::autograd::profiler::profilerEnabled(),
-        "Expected profiler to be enabled!");
+        profilerEnabled(), "Expected profiler to be enabled!");
     // Kick off processing for nested work and get Future<T> result in
     // wrappedRpcResponseFuture
-    processRpc(
+    auto wrappedRpcResponseFuture = processRpc(
         rpcWithProfilingReq.wrappedRpc(),
         wrappedMsgType,
-        messageId,
-        wrappedRpcResponseFuture);
+        {}); // TODO: https://github.com/pytorch/pytorch/issues/55757
 
-    wrappedRpcResponseFuture->addCallback(
-        at::wrapPropagateTLSState<void>([wrappedRpcResponseFuture,
-                                         responseFuture,
-                                         profilingKeyId,
-                                         profilingConfig] {
-          std::vector<torch::autograd::profiler::LegacyEvent> profiledEvents;
+    auto responseFuture = wrappedRpcResponseFuture->then(
+        at::wrapPropagateTLSState([profilingKeyId, profilingConfig](
+                                      JitFuture& wrappedRpcResponseFuture) {
+          std::vector<LegacyEvent> profiledEvents;
           // Defer consolidation of profiler events until async work has
           // completed (such as async UDF)
 
           TORCH_INTERNAL_ASSERT(
-              torch::autograd::profiler::profilerEnabled(),
-              "Expected profiler to be enabled!");
+              profilerEnabled(), "Expected profiler to be enabled!");
 
           // On continuation thread, don't clean up profiler states, since
           // they will be cleaned up by main thread, and consolidate all
           // events so we obtain asynchronously run events.
-          torch::autograd::profiler::ProfilerDisableOptions opts(false, true);
-          auto event_lists =
-              torch::autograd::profiler::disableProfilerLegacy(opts);
-          if (wrappedRpcResponseFuture->hasError()) {
+          ProfilerDisableOptions opts(false, true);
+          auto event_lists = disableProfilerLegacy(opts);
+          if (wrappedRpcResponseFuture.hasError()) {
             // Propagate error
             // No need to propagate remote events in the case of an error.
-            responseFuture->setError(wrappedRpcResponseFuture->exception_ptr());
+            std::rethrow_exception(wrappedRpcResponseFuture.exception_ptr());
           } else {
             populateRemoteProfiledEvents(
                 profiledEvents, profilingConfig, event_lists);
             auto rpcWithProfilingResp = std::make_unique<RpcWithProfilingResp>(
                 MessageType::RUN_WITH_PROFILING_RESP,
-                std::move(*wrappedRpcResponseFuture->value()
-                               .toCustomClass<Message>()),
+                wrappedRpcResponseFuture.value().toCustomClass<Message>(),
                 profiledEvents,
                 profilingKeyId);
-            responseFuture->markCompleted(IValue(c10::make_intrusive<Message>(
-                std::move(*rpcWithProfilingResp).toMessage())));
+            return withStorages(std::move(*rpcWithProfilingResp).toMessage());
           }
-        }));
+        }),
+        c10::getCustomClassType<c10::intrusive_ptr<Message>>());
+
+    return responseFuture;
     // Exiting the scope will disable the profiler on this thread with the
     // options specified above.
   }
 }
 
-void RequestCallbackNoPython::processRRefBackward(
-    RpcCommandBase& rpc,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& /* unused */) const {
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::processRRefBackward(
+    RpcCommandBase& rpc) const {
   C10_THROW_ERROR(Error, "Python call not supported!");
 }
 
-void RequestCallbackNoPython::processRpc(
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::processRpc(
     RpcCommandBase& rpc,
     const MessageType& messageType,
-    const int64_t messageId,
-    const std::shared_ptr<JitFuture>& responseFuture) const {
-  auto markComplete = [messageId, &responseFuture](Message m) {
-    m.setId(messageId);
-    responseFuture->markCompleted(
-        IValue(c10::make_intrusive<Message>(std::move(m))));
-  };
+    const std::vector<c10::Stream>& streams) const {
   // TODO: RpcCommandBase should have an abstract execute() method that we can
   // call here instead of having another switch statement here. Even better we
   // could have abstract classes RpcRequest and RpcResp which inherit from
@@ -584,60 +506,46 @@ void RequestCallbackNoPython::processRpc(
   // to a python object.
   switch (messageType) {
     case MessageType::SCRIPT_CALL: {
-      processScriptCall(rpc, markComplete, messageId, responseFuture);
-      return;
+      return processScriptCall(rpc, streams);
     }
     case MessageType::PYTHON_CALL: {
-      processPythonCall(rpc, markComplete, messageId, responseFuture);
-      return;
+      return processPythonCall(rpc, streams);
     }
     case MessageType::SCRIPT_REMOTE_CALL: {
-      processBaseScriptRemoteCall(rpc, markComplete, messageId, responseFuture);
-      return;
+      return processScriptRemoteCall(rpc, streams);
     }
     case MessageType::PYTHON_REMOTE_CALL: {
-      processPythonRemoteCall(rpc, markComplete, messageId, responseFuture);
-      return;
+      return processPythonRemoteCall(rpc, streams);
     }
     case MessageType::SCRIPT_RREF_FETCH_CALL: {
-      processScriptRRefFetchCall(rpc, markComplete, messageId, responseFuture);
-      return;
+      return processScriptRRefFetchCall(rpc);
     }
     case MessageType::PYTHON_RREF_FETCH_CALL: {
-      processPythonRRefFetchCall(rpc, messageId, responseFuture);
-      return;
+      return processPythonRRefFetchCall(rpc);
     }
     case MessageType::RREF_USER_DELETE: {
-      processRRefUserDelete(rpc, markComplete);
-      return;
+      return processRRefUserDelete(rpc);
     }
     case MessageType::RREF_CHILD_ACCEPT: {
-      processRRefChildAccept(rpc, markComplete);
-      return;
+      return processRRefChildAccept(rpc);
     }
     case MessageType::RREF_FORK_REQUEST: {
-      processRRefForkRequest(rpc, markComplete);
-      return;
+      return processRRefForkRequest(rpc);
     }
     case MessageType::FORWARD_AUTOGRAD_REQ: {
-      processForwardAutogradReq(rpc, messageId, responseFuture);
-      return;
+      return processForwardAutogradReq(rpc, streams);
     }
     case MessageType::BACKWARD_AUTOGRAD_REQ: {
-      processBackwardAutogradReq(rpc, messageId, responseFuture);
-      return;
+      return processBackwardAutogradReq(rpc, streams);
     };
     case MessageType::CLEANUP_AUTOGRAD_CONTEXT_REQ: {
-      processCleanupAutogradContextReq(rpc, markComplete);
-      return;
+      return processCleanupAutogradContextReq(rpc);
     }
     case MessageType::RUN_WITH_PROFILING_REQ: {
-      processRunWithProfilingReq(rpc, messageId, responseFuture);
-      return;
+      return processRunWithProfilingReq(rpc);
     }
     case MessageType::RREF_BACKWARD_REQ: {
-      processRRefBackward(rpc, messageId, responseFuture);
-      return;
+      return processRRefBackward(rpc);
     }
     default: {
       TORCH_INTERNAL_ASSERT(
@@ -646,7 +554,7 @@ void RequestCallbackNoPython::processRpc(
   }
 }
 
-IValue RequestCallbackNoPython::handleError(
+c10::intrusive_ptr<Message> RequestCallbackNoPython::handleError(
     const std::exception& e,
     const MessageType messageType,
     int64_t messageId) const {
@@ -659,8 +567,7 @@ IValue RequestCallbackNoPython::handleError(
       DistAutogradContainer::getInstance().getWorkerId(),
       ": ",
       e.what());
-  return IValue(c10::make_intrusive<Message>(
-      createExceptionResponse(errorMsg, messageId)));
+  return createExceptionResponse(errorMsg, messageId);
 }
 
 bool RequestCallbackNoPython::cudaAvailable() const {
@@ -671,6 +578,51 @@ bool RequestCallbackNoPython::cudaAvailable() const {
 #endif
 }
 
-} // namespace rpc
-} // namespace distributed
-} // namespace torch
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::runJitOperator(
+    const jit::Operator& op,
+    std::vector<at::IValue>& stack,
+    const std::vector<c10::Stream>& streams) const {
+  c10::MultiStreamGuard guard(streams);
+  try {
+    op.getOperation()(stack);
+  } catch (const std::exception&) {
+    return asFuture(std::current_exception());
+  }
+  TORCH_INTERNAL_ASSERT(
+      stack.size() == 1,
+      "Return value of a builtin operator or a TorchScript function should be "
+      "a single IValue, got a vector of size ",
+      stack.size());
+  TypePtr type = stack.front().type();
+  return asFuture(std::move(stack.front()), std::move(type));
+}
+
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::asFuture(
+    IValue value,
+    TypePtr type) const {
+  auto future = c10::make_intrusive<JitFuture>(
+      std::move(type), RpcAgent::getCurrentRpcAgent()->getDevices());
+  future->markCompleted(std::move(value));
+  return future;
+}
+
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::asFuture(
+    c10::intrusive_ptr<Message> message) const {
+  auto future = c10::make_intrusive<JitFuture>(
+      at::getCustomClassType<c10::intrusive_ptr<Message>>(),
+      RpcAgent::getCurrentRpcAgent()->getDevices());
+  std::vector<c10::weak_intrusive_ptr<c10::StorageImpl>> storages =
+      message->getStorages();
+  future->markCompleted(std::move(message), std::move(storages));
+  return future;
+}
+
+c10::intrusive_ptr<JitFuture> RequestCallbackNoPython::asFuture(
+    std::exception_ptr err) const {
+  auto future = c10::make_intrusive<JitFuture>(
+      at::NoneType::get(), RpcAgent::getCurrentRpcAgent()->getDevices());
+  future->setError(std::move(err));
+  return future;
+}
+
+} // namespace torch::distributed::rpc
