@@ -1,11 +1,6 @@
 #include <torch/csrc/distributed/rpc/unpickled_python_remote_call.h>
 
-#include <c10/util/C++17.h>
-#include <torch/csrc/distributed/rpc/python_rpc_handler.h>
-
-namespace torch {
-namespace distributed {
-namespace rpc {
+namespace torch::distributed::rpc {
 
 UnpickledPythonRemoteCall::UnpickledPythonRemoteCall(
     const SerializedPyObj& serializedPyObj,
@@ -24,6 +19,4 @@ const ForkId& UnpickledPythonRemoteCall::forkId() const {
   return forkId_;
 }
 
-} // namespace rpc
-} // namespace distributed
-} // namespace torch
+} // namespace torch::distributed::rpc
