@@ -1,7 +1,6 @@
 #include <torch/csrc/distributed/rpc/utils.h>
 
 #include <fmt/format.h>
-#include <torch/csrc/autograd/profiler.h>
 #include <torch/csrc/distributed/autograd/rpc_messages/cleanup_autograd_context_req.h>
 #include <torch/csrc/distributed/autograd/rpc_messages/cleanup_autograd_context_resp.h>
 #include <torch/csrc/distributed/autograd/rpc_messages/propagate_gradients_req.h>
@@ -23,35 +22,34 @@
 #include <torch/csrc/jit/serialization/pickler.h>
 #include <torch/csrc/jit/serialization/unpickler.h>
 
-namespace torch {
-namespace distributed {
-namespace rpc {
+#include <c10/util/irange.h>
+
+using namespace torch::autograd::profiler;
+
+namespace torch::distributed::rpc {
 namespace {
 void processRemoteProfiledEvents(
     autograd::RpcWithProfilingResp& rpcWithProfilingResp) {
   // Check if the profiler is enabled
-  auto enabled = torch::autograd::profiler::profilerEnabled();
+  auto enabled = profilerEnabled();
   TORCH_CHECK(
       enabled,
       "Profiler was expected to be enabled. This can happen in callback "
-      " continutations that run in different threads, and the TLS of the "
+      " continuations that run in different threads, and the TLS of the "
       " profiler was not propagated.");
-  std::vector<torch::autograd::profiler::LegacyEvent> events =
-      rpcWithProfilingResp.getProfiledEvents();
+  std::vector<LegacyEvent> events = rpcWithProfilingResp.getProfiledEvents();
   const auto& profilingId = rpcWithProfilingResp.getProfilingId();
   auto& remoteProfilerManager = RemoteProfilerManager::getInstance();
   auto key = remoteProfilerManager.retrieveRPCProfilingKey(profilingId);
   remoteProfilerManager.eraseKey(profilingId);
   auto keyPrefixStr = key + rpc::REMOTE_PROFILING_KEY_PREFIX;
   std::for_each(
-      events.begin(),
-      events.end(),
-      [&keyPrefixStr](torch::autograd::profiler::LegacyEvent& event) {
+      events.begin(), events.end(), [&keyPrefixStr](LegacyEvent& event) {
         std::string name = keyPrefixStr + std::string(event.name());
         event.setName(at::StringView(name));
       });
   // Add event list to the thread local profiler.
-  torch::autograd::profiler::addEventList(std::move(events));
+  addEventList(std::move(events));
 }
 
 } // namespace
@@ -91,7 +89,7 @@ std::string makeRPCError(
   return fmt::format(
       "{}:{}:{}",
       torch::distributed::rpc::kRPCErrorPrefix,
-      errorType,
+      static_cast<int>(errorType),
       rpcErrorStr);
 }
 
@@ -176,7 +174,7 @@ std::unique_ptr<RpcCommandBase> deserializeResponse(
 
       // Need to reverse the device map for the backward pass of distributed
       // autograd.
-      std::unordered_map<c10::DeviceIndex, c10::DeviceIndex> reverseDeviceMap;
+      DeviceMap reverseDeviceMap;
       for (const auto& mapEntry : rpcWithAutograd.deviceMap()) {
         reverseDeviceMap.insert({mapEntry.second, mapEntry.first});
       }
@@ -296,12 +294,12 @@ parseWireSections(const void* data, size_t data_size) {
     if (ptr == endp) {
       break;
     }
-    size_t sz = c10::stoll(std::string(sizePtr, ptr - sizePtr));
-    headerEnts.emplace_back(std::make_pair(name, sz));
+    size_t sz = std::stoll(std::string(sizePtr, ptr - sizePtr));
+    headerEnts.emplace_back(name, sz);
     ++ptr; // past the '\n'
   }
   if (!ok) {
-    throw std::runtime_error("failed parse");
+    TORCH_CHECK(false, "failed parse");
   }
 
   std::unordered_map<std::string, std::pair<const char*, size_t>> out;
@@ -310,14 +308,14 @@ parseWireSections(const void* data, size_t data_size) {
     ptr += headerEnt.second;
   }
   if (ptr != endp) {
-    throw std::runtime_error("failed bounds");
+    TORCH_CHECK(false, "failed bounds");
   }
   return out;
 }
 
-static const char* kMeta = "meta";
-static const char* kPayload = "payload";
-}; // namespace
+static constexpr const char* kMeta = "meta";
+static constexpr const char* kPayload = "payload";
+} // namespace
 
 c10::List<at::Tensor> cloneSparseTensors(
     const std::vector<at::Tensor>& tensors) {
@@ -331,7 +329,7 @@ c10::List<at::Tensor> cloneSparseTensors(
     auto storageSize = t.storage().nbytes();
     auto usefulSize = t.element_size() * t.numel();
     constexpr size_t kMinMultiple = 2;
-    constexpr size_t kMinRecopyBytes = 8 * 1024;
+    constexpr size_t kMinRecopyBytes = 8ull * 1024;
     return storageSize >= kMinRecopyBytes &&
         storageSize >= usefulSize * kMinMultiple;
   };
@@ -378,7 +376,7 @@ std::string wireSerialize(
     pickler.stop();
     tensorData = pickler.tensorData();
     entries.push_back({kMeta, metaEntry.data(), metaEntry.size()});
-    for (size_t i = 0; i < tensorData.size(); i++) {
+    for (const auto i : c10::irange(tensorData.size())) {
       // Construct WritableTensorData for each tensor in the pickler tensorData
       // Since tensorData is in function scope, and getWritableTensorData just
       // record the tensors, the data() pointers stay valid for CPU tensors
@@ -388,7 +386,7 @@ std::string wireSerialize(
       // out of scope of this loop.
       auto writeableTensorData = jit::getWriteableTensorData(tensorData[i]);
       entries.push_back(
-          {c10::to_string(i),
+          {std::to_string(i),
            writeableTensorData.data(),
            writeableTensorData.sizeInBytes()});
     }
@@ -400,7 +398,7 @@ std::string wireSerialize(
     tot += e.size;
     header.append(e.name)
         .append(" ")
-        .append(c10::to_string(e.size))
+        .append(std::to_string(e.size))
         .append("\n");
   }
   header.push_back('\n');
@@ -444,7 +442,7 @@ std::pair<std::vector<char>, std::vector<at::Tensor>> wireDeserialize(
     auto sectionReadFunc = [&](const std::string& ename) -> at::DataPtr {
       auto it = sections.find(ename);
       if (it == sections.end()) {
-        throw std::runtime_error("Couldn't find entity " + ename);
+        TORCH_CHECK(false, "Couldn't find entity " + ename);
       }
       const auto& idat = it->second;
       auto dptr = at::getCPUAllocator()->allocate(idat.second);
@@ -475,10 +473,11 @@ void writeWrappedPayload(
       additionalPayload.end());
 
   // Add size of the additional payload
-  int64_t indexToWrite = originalPayload.size();
+  int64_t indexToWrite = static_cast<int64_t>(originalPayload.size());
   originalPayload.resize(originalPayload.size() + sizeof(int64_t));
-  const int64_t additionalPayloadSize = additionalPayload.size();
-  torch::utils::THP_encodeInt64Buffer(
+  const int64_t additionalPayloadSize =
+      static_cast<int64_t>(additionalPayload.size());
+  torch::utils::THP_encodeBuffer(
       reinterpret_cast<uint8_t*>(originalPayload.data()) + indexToWrite,
       &additionalPayloadSize,
       torch::utils::THPByteOrder::THP_BIG_ENDIAN,
@@ -489,10 +488,10 @@ std::vector<at::IValue> readWrappedPayload(
     std::vector<char>& payload,
     const rpc::Message& message) {
   // Read the additional payload remove it from the payload.
-  int64_t additionalPayloadSize;
+  TORCH_INTERNAL_ASSERT(payload.size() >= sizeof(int64_t));
   size_t indexToRead = payload.size() - sizeof(int64_t);
-  TORCH_INTERNAL_ASSERT(indexToRead >= 0);
-  torch::utils::THP_decodeInt64Buffer(
+  int64_t additionalPayloadSize = 0;
+  torch::utils::THP_decodeBuffer(
       &additionalPayloadSize,
       reinterpret_cast<uint8_t*>(payload.data()) + indexToRead,
       torch::utils::THPByteOrder::THP_BIG_ENDIAN,
@@ -500,104 +499,80 @@ std::vector<at::IValue> readWrappedPayload(
   payload.resize(indexToRead);
 
   TORCH_INTERNAL_ASSERT(
-      payload.size() > additionalPayloadSize,
+      additionalPayloadSize > 0 &&
+          static_cast<int64_t>(payload.size()) > additionalPayloadSize,
       "Wrong payload sizes: payload.size() is ",
       payload.size(),
       " but additional payload size is ",
       additionalPayloadSize);
   auto wrappedPayloadBegin =
-      static_cast<const char*>(message.payload().data()) + payload.size() -
-      additionalPayloadSize;
+      message.payload().data() + payload.size() - additionalPayloadSize;
   std::vector<torch::Tensor> tensorTable;
   IValue tuple = jit::unpickle(
       wrappedPayloadBegin,
       additionalPayloadSize,
       *rpc::RpcAgent::getCurrentRpcAgent()->getTypeResolver(),
-      &tensorTable);
-  std::vector<at::IValue> tupleElements = tuple.toTuple()->elements();
+      tensorTable);
+  std::vector<at::IValue> tupleElements = tuple.toTupleRef().elements().vec();
   payload.resize(payload.size() - additionalPayloadSize);
   return tupleElements;
 }
 
 void populateRemoteProfiledEvents(
-    std::vector<torch::autograd::profiler::LegacyEvent>& profiledEvents,
-    const torch::autograd::profiler::ProfilerConfig& profilingConfig,
-    const std::vector<std::vector<torch::autograd::profiler::LegacyEvent>>&
-        eventLists) {
+    std::vector<LegacyEvent>& profiledEvents,
+    const ProfilerConfig& profilingConfig,
+    const std::vector<std::vector<LegacyEvent>>& eventLists) {
   // Gather all events into a vector
   for (auto& l : eventLists) {
     for (auto& e : l) {
       profiledEvents.push_back(e);
     }
   }
-  // find __start_profile event and __cuda_start_event.
-  bool cudaProfilingEnabled =
-      profilingConfig.state == torch::autograd::profiler::ProfilerState::CUDA;
-  bool foundCpuStart = false;
-  const torch::autograd::profiler::LegacyEvent* profilerStart = nullptr;
-  // Each device has its own cudaProfilerStart, so we must take
-  // care to use the correct one depending on the device the
-  // operation ran on.
-  std::unordered_map<int, const torch::autograd::profiler::LegacyEvent*>
-      cudaProfilerStarts;
-  for (auto& e : profiledEvents) {
-    if (!foundCpuStart && 0 == strcmp(e.name(), "__start_profile")) {
-      profilerStart = &e;
-      foundCpuStart = true;
-    } else if (
-        cudaProfilingEnabled && 0 == strcmp(e.name(), "__cuda_start_event")) {
-      e.setCudaUs(e.cpuUs());
-      auto device = e.device();
-      TORCH_CHECK(
-          device != -1,
-          "CUDA profiling was enabled but could not find CUDA device.");
-      TORCH_CHECK(
-          cudaProfilerStarts.find(device) == cudaProfilerStarts.end(),
-          c10::str("Duplicate __cuda_start_event found for ", device));
-      cudaProfilerStarts[device] = &e;
-    }
+  // find __start_profile event
+  bool cudaProfilingEnabled = profilingConfig.state == ProfilerState::CUDA;
+  const LegacyEvent* profilerStart = nullptr;
 
-    // TODO: determine no. of CUDA devices and break here if we have
-    // a cudaProfilerStart for all of them, in the case of cuda
-    // profiling.
-    if (foundCpuStart && !cudaProfilingEnabled) {
+  for (auto& e : profiledEvents) {
+    if (std::string(e.name()) == "__start_profile") {
+      profilerStart = &e;
       break;
     }
   }
   // We should always find __start_profile.
   TORCH_CHECK(
       profilerStart != nullptr, "Expected to find __start_profile event.");
-  // Should have >= 1 CUDA start event if cudaProfilingEnabled.
-  // TODO: we can enhance this assert by ensuring we have found a
-  // start for every available CUDA device.
-  TORCH_CHECK(
-      !cudaProfilingEnabled || cudaProfilerStarts.size() > 0,
-      "Profiler was enabled with CUDA recording, but did not find __cuda_start_event.");
 
   if (cudaProfilingEnabled) {
-    // Compute and set global time for when this CUDA kernel was
-    // launched/ended, since deserialized event will not have a
-    // corresponding CUDA event.
+    // Deserialized events don't have the corresponding CUDA events, making it
+    // impossible to use cudaEventElapsedTime the receiving end. To avoid this,
+    // find all push/pop pairs of CUDA events and set the corresponding CUDA
+    // time to zero for the push event and to the elapsed time for the pop
+    // event, to be used later for the elapsed CUDA time computation.
+    std::unordered_map<at::RecordFunctionHandle, const LegacyEvent*>
+        startEvents;
     for (auto& e : profiledEvents) {
       if (e.hasCuda()) {
-        auto cudaDevice = e.device();
-        TORCH_CHECK(
-            cudaDevice != -1,
-            "CUDA profiling was enabled but could not find CUDA device.");
-        auto it = cudaProfilerStarts.find(cudaDevice);
-        TORCH_CHECK(
-            it != cudaProfilerStarts.end(),
-            c10::str(
-                "Failed to find __cuda_start_event for device ", cudaDevice));
-        auto cudaProfilerStartEvent = it->second;
-        double cudaElapsedUs = cudaProfilerStartEvent->cudaElapsedUs(e);
-        int64_t cudaUs = cudaElapsedUs + cudaProfilerStartEvent->cpuUs();
-        e.setCudaUs(cudaUs);
+        if (e.kind() == EventKind::PushRange) {
+          startEvents[e.handle()] = &e;
+        }
+      }
+    }
+    for (auto& e : profiledEvents) {
+      if (e.hasCuda()) {
+        if (e.kind() == EventKind::PopRange) {
+          auto it = startEvents.find(e.handle());
+          if (it != startEvents.end()) {
+            e.setCudaUs(static_cast<int64_t>(it->second->cudaElapsedUs(e)));
+          } else {
+            TORCH_WARN("Found a pop event without a corresponding push event");
+            e.setCudaUs(0);
+          }
+        } else {
+          e.setCudaUs(0);
+        }
       }
     }
   }
 }
 
-} // namespace rpc
-} // namespace distributed
-} // namespace torch
+} // namespace torch::distributed::rpc
