@@ -65,7 +65,9 @@ struct TORCH_API Method : public torch::IMethod {
   }
 
  private:
-  void setArgumentNames(std::vector<std::string>&) const override;
+  void setArgumentNames(
+      std::vector<std::string>& /*argumentNames*/ /*argumentNamesOut*/)
+      const override;
 
   // Methods are uniqued owned by a single module. This raw pointer allows
   // looking up the module.
