@@ -1,8 +1,6 @@
 #include <torch/csrc/jit/frontend/builtin_functions.h>
 
 #include <ATen/code_template.h>
-#include <caffe2/serialize/versions.h>
-#include <torch/csrc/api/include/torch/jit.h>
 #include <torch/csrc/jit/frontend/resolver.h>
 
 namespace torch::jit {
