@@ -1,7 +1,5 @@
 #include <torch/csrc/jit/frontend/error_report.h>
 
-#include <torch/csrc/jit/frontend/tree.h>
-
 namespace torch::jit {
 
 // Avoid storing objects with destructor in thread_local for mobile build.
@@ -99,7 +97,7 @@ std::string ErrorReport::current_call_stack() {
 
 const char* ErrorReport::what() const noexcept {
   std::stringstream msg;
-  msg << "\n" << ss.str();
+  msg << '\n' << ss.str();
   msg << ":\n";
   context.highlight(msg);
 
