@@ -1,23 +1,31 @@
 #include <torch/csrc/jit/frontend/schema_matching.h>
 
+#include <ATen/core/interned_strings.h>
 #include <ATen/core/jit_type.h>
+#include <c10/util/Exception.h>
+#include <c10/util/irange.h>
 #include <torch/csrc/jit/frontend/builtin_functions.h>
 #include <torch/csrc/jit/frontend/error_report.h>
+#include <torch/csrc/jit/frontend/function_schema_parser.h>
+#include <torch/csrc/jit/ir/ir.h>
+#include <torch/csrc/jit/operator_upgraders/utils.h>
+#include <torch/csrc/jit/operator_upgraders/version_map.h>
 #include <torch/csrc/jit/runtime/operator.h>
+#include <optional>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 
-static inline TypePtr unwrapOptional(TypePtr opt_type) {
+static TypePtr unwrapOptional(TypePtr opt_type) {
+  if (auto dyn = opt_type->castRaw<c10::DynamicType>()) {
+    return unwrapOptional(dyn->fallback());
+  }
   if (auto unwrap_list_type = opt_type->cast<OptionalType>()) {
     return unwrap_list_type->getElementType();
   }
   return opt_type;
 }
 
-static inline bool isIntOrFloatUsedAsList(
-    const Value* value,
-    const Argument& arg) {
+static bool isIntOrFloatUsedAsList(const Value* value, const Argument& arg) {
   // Look for int[N] or float[N]
   const auto& v_type = value->type();
   if (v_type != FloatType::get() && v_type != IntType::get())
@@ -29,21 +37,21 @@ static inline bool isIntOrFloatUsedAsList(
 
 /// Returns true if `type` is a Tuple in which all the elements have the
 /// same type or if it's a subtype of `list_type_`.
-inline bool convertibleToList(const TypePtr& type, const TypePtr& list_type_) {
-  auto list_type = list_type_->cast<ListType>();
+bool convertibleToList(const TypePtr& type, const TypePtr& list_type_) {
+  auto list_type = list_type_->castRaw<ListType>();
   if (!list_type) {
     return false;
   }
-  if (type->isSubtypeOf(list_type_)) {
+  if (type->isSubtypeOf(*list_type_)) {
     return true;
   }
-  if (auto tuple = type->cast<TupleType>()) {
+  if (auto tuple = type->castRaw<TupleType>()) {
     return std::all_of(
         tuple->elements().begin(),
         tuple->elements().end(),
         [&](const TypePtr& t) {
           // TODO: resolve VarType if necessary
-          return t->isSubtypeOf(list_type->getElementType());
+          return t->isSubtypeOf(*list_type->getElementType());
         });
   }
   return false;
@@ -60,12 +68,22 @@ Value* tryConvertToType(
   // treat conversion to Optional[T] as conversions to T
   if (OptionalTypePtr op = concrete_type->cast<OptionalType>()) {
     if (value->type()->kind() != OptionalType::Kind &&
-        !value->type()->isSubtypeOf(NoneType::get())) {
+        !value->type()->isSubtypeOf(*NoneType::get())) {
       return tryConvertToType(
           loc, graph, op->getElementType(), value, allow_conversions);
     }
   }
 
+  // allow temporary, unannotated list literals `[]` to match to arbitrary list
+  // types
+  if (value->node()->kind() == prim::EmptyListLiteral &&
+      concrete_type->cast<ListType>()) {
+    value = graph
+                .insertNode(graph.createList(
+                    concrete_type->cast<ListType>()->getElementType(), {}))
+                ->output();
+  }
+
   if (auto value_tuple = value->type()->cast<TupleType>()) {
     // Allow homogeneous tuples to be casted implicitly to lists of appropriate
     // types
@@ -78,7 +96,7 @@ Value* tryConvertToType(
 
     // inductively apply implicit conversions to tuples
     if (auto concrete_tuple = concrete_type->cast<TupleType>()) {
-      if (!value_tuple->isSubtypeOf(concrete_tuple) &&
+      if (!value_tuple->isSubtypeOf(*concrete_tuple) &&
           concrete_tuple->elements().size() == value_tuple->elements().size()) {
         auto unpacked = createTupleUnpack(value);
         std::vector<Value*> converted;
@@ -98,14 +116,17 @@ Value* tryConvertToType(
   // implicit conversions
   if (allow_conversions) {
     // Convert tensor or number to concrete int/float types
-    bool value_isa_tensor = value->type()->isSubtypeOf(TensorType::get());
+    bool value_isa_tensor = value->type()->isSubtypeOf(*TensorType::get());
     bool value_equals_number = *value->type() == *NumberType::get();
     bool concrete_float = *concrete_type == *FloatType::get();
+    bool concrete_complex = *concrete_type == *ComplexType::get();
     bool concrete_int = *concrete_type == *IntType::get();
     bool concrete_number = *concrete_type == *NumberType::get();
     if (value_isa_tensor) {
       if (concrete_float) {
         value = graph.insert(aten::FloatImplicit, {value}, {}, loc);
+      } else if (concrete_complex) {
+        value = graph.insert(aten::ComplexImplicit, {value}, {}, loc);
       } else if (concrete_int) {
         value = graph.insert(aten::IntImplicit, {value}, {}, loc);
       } else if (concrete_number) {
@@ -114,14 +135,22 @@ Value* tryConvertToType(
     } else if (value_equals_number) {
       if (concrete_float) {
         value = graph.insert(aten::Float, {value}, {}, loc);
+      } else if (concrete_complex) {
+        value = graph.insert(aten::Complex, {value}, {}, loc);
       } else if (concrete_int) {
         value = graph.insert(aten::Int, {value}, {}, loc);
       }
+    } else if (*value->type() == *BoolType::get()) {
+      if (concrete_float) {
+        value = graph.insert(aten::Float, {value}, {}, loc);
+      } else if (concrete_int || concrete_number) {
+        value = graph.insert(aten::Int, {value}, {}, loc);
+      }
     }
 
     // Convert strings to device
-    if (value->type()->isSubtypeOf(StringType::get()) &&
-        concrete_type->isSubtypeOf(DeviceObjType::get())) {
+    if (value->type()->isSubtypeOf(*StringType::get()) &&
+        concrete_type->isSubtypeOf(*DeviceObjType::get())) {
       return graph.insert(aten::device, {value}, {}, loc);
     }
   }
@@ -179,7 +208,7 @@ static Value* tryMatchArgument(
   value = tryConvertToType(loc, graph, concrete_type, value, allow_conversions);
   std::stringstream ss;
   if (!value->type()->isSubtypeOfExt(
-          concrete_type, /*why_not=*/(failure_messages) ? &ss : nullptr)) {
+          *concrete_type, /*why_not=*/failure_messages ? &ss : nullptr)) {
     if (failure_messages) {
       auto& ostream = err()
           << arg.formatTypeMismatchMsg(value->type()->repr_str());
@@ -197,7 +226,7 @@ static Value* tryMatchArgument(
       }
 
       if (auto v = value->type()->cast<ListType>()) {
-        if (v->getElementType()->isSubtypeOf(TensorType::get())) {
+        if (v->getElementType()->isSubtypeOf(*TensorType::get())) {
           ostream << "Empty lists default to List[Tensor]. Add a variable "
                      "annotation to the assignment to create an empty list "
                      "of another type (torch.jit.annotate(List[T, []]) where T "
@@ -213,14 +242,21 @@ static Value* tryMatchArgument(
   return value;
 }
 
-c10::optional<size_t> findInputWithName(
+std::optional<size_t> findInputWithName(
     const std::string& name,
-    at::ArrayRef<NamedValue> kwargs) {
-  for (size_t i = 0; i < kwargs.size(); ++i) {
-    if (kwargs[i].name() == name)
+    at::ArrayRef<NamedValue> kwargs,
+    bool is_aten) {
+  for (const auto i : c10::irange(kwargs.size())) {
+    // TS doesn't understand that the self argument in function
+    // scheams is renamed to input for the functional variant
+    if (is_aten && name == "self" && kwargs[i].name() == "input") {
       return i;
+    }
+    if (kwargs[i].name() == name) {
+      return i;
+    }
   }
-  return c10::nullopt;
+  return std::nullopt;
 }
 
 /// Creates a list with the provided values if each value's type can be matched
@@ -270,12 +306,17 @@ static bool varargsCanBeUsedAsList(
   bool is_last_argument = arg_index + 1 == schema.arguments().size() ||
       schema.arguments()[arg_index + 1].kwarg_only();
 
+  auto arg_type = arg.type();
+  if (auto dyn = arg_type->castRaw<c10::DynamicType>()) {
+    arg_type = dyn->fallback();
+  }
+
   // The formal must be a list
-  bool argument_is_list = arg.type()->kind() == TypeKind::ListType;
+  bool argument_is_list = arg_type->kind() == TypeKind::ListType;
 
   // matching varargs of typevar list nyi
   bool typevar_list = argument_is_list &&
-      arg.type()->castRaw<ListType>()->getElementType()->cast<VarType>();
+      arg_type->castRaw<ListType>()->getElementType()->cast<VarType>();
 
   // it must not be a broadcasting list like int[3],
   // otherwise a single int is a valid input
@@ -285,34 +326,44 @@ static bool varargsCanBeUsedAsList(
       !typevar_list;
 }
 
-// Note (@zasdfgbnm):
-// This is a workaround for https://github.com/pytorch/pytorch/issues/47964
-// Currently JIT does not distinguish ScalarType vs int, so there is really
-// no way to distinguish x.view(1) vs x.view(torch.int8). So we have to hardcode
-// the aten::view.dtype here to block this overload. This blocklist should be
-// removed when JIT fully suports ScalarType as its own type.
 bool isBlockListedSchema(const FunctionSchema& schema) {
+  // Note (@zasdfgbnm):
+  // This is a workaround for https://github.com/pytorch/pytorch/issues/47964
+  // Currently JIT does not distinguish ScalarType vs int, so there is really
+  // no way to distinguish x.view(1) vs x.view(torch.int8). So we have to
+  // hardcode the aten::view.dtype here to block this overload. This blocklist
+  // should be removed when JIT fully supports ScalarType as its own type.
   if (schema.name() == "aten::view" && schema.overload_name() == "dtype") {
     return true;
   }
+  // Note (@tugsbayasgalan)
+  // TorchScript doesn't support kwargs so this op collides with aten.max.others
+  // since both of them have 2 Tensor inputs. Since we don't expect users to
+  // use this op in TS, we just skip it
+  if (schema.name() == "aten::max" && schema.overload_name() == "unary_out") {
+    return true;
+  }
+  if (schema.name() == "aten::min" && schema.overload_name() == "unary_out") {
+    return true;
+  }
   return false;
 }
 
-static c10::optional<MatchedSchema> tryMatchSchema(
+static std::optional<MatchedSchema> tryMatchSchema(
     const FunctionSchema& schema,
     const SourceRange& loc,
     Graph& graph,
     at::ArrayRef<NamedValue> args,
     at::ArrayRef<NamedValue> kwargs,
-    c10::optional<NamedValue> self,
+    std::optional<NamedValue> self,
     std::ostream* failure_messages,
     bool allow_conversions) {
   if (isBlockListedSchema(schema)) {
-    return c10::nullopt;
+    return std::nullopt;
   }
 
   auto err = [&]() -> std::ostream& {
-    *failure_messages << "\n" << schema << ":\n";
+    *failure_messages << '\n' << schema << ":\n";
     return *failure_messages;
   };
 
@@ -322,14 +373,21 @@ static c10::optional<MatchedSchema> tryMatchSchema(
   std::vector<Value*> positional_inputs;
   std::vector<bool> used_kwarg(kwargs.size(), false);
 
+  auto schema_namespace = schema.operator_name().getNamespace();
+  bool is_aten = false;
+  if (schema_namespace.has_value()) {
+    if (schema_namespace.value() == "aten") {
+      is_aten = true;
+    }
+  }
   // if we finish the loop will we have consumed all arguments?
   size_t used_args = 0;
-  for (size_t schema_i = 0; schema_i < schema.arguments().size(); ++schema_i) {
+  for (const auto schema_i : c10::irange(schema.arguments().size())) {
     const auto& arg = schema.arguments()[schema_i];
-    c10::optional<NamedValue> actual_named_value;
+    std::optional<NamedValue> actual_named_value;
     if (arg.name() == "self" && self) {
       actual_named_value = self;
-      self = c10::nullopt;
+      self = std::nullopt;
     } else if (!arg.kwarg_only() && used_args < args.size()) {
       // Try to convert all the remaining non-kwarg arguments (used_args) to a
       // list. Allow zeros(IntArrayRef sizes) to work with zeros(1, 2) or
@@ -354,7 +412,7 @@ static c10::optional<MatchedSchema> tryMatchSchema(
               allow_conversions,
               type_env);
           if (!list) {
-            return c10::nullopt;
+            return std::nullopt;
           }
           used_args = args.size();
           positional_inputs.push_back(list);
@@ -366,14 +424,15 @@ static c10::optional<MatchedSchema> tryMatchSchema(
       // used
       actual_named_value = args[used_args];
       used_args++;
-    } else if (auto kwarg_idx = findInputWithName(arg.name(), kwargs)) {
+    } else if (
+        auto kwarg_idx = findInputWithName(arg.name(), kwargs, is_aten)) {
       const NamedValue& nv = kwargs[*kwarg_idx];
       if (used_kwarg[*kwarg_idx]) {
         if (failure_messages) {
           err() << "Argument " << nv.name()
                 << " specified twice in schema, submit a bug report!\n";
         }
-        return c10::nullopt;
+        return std::nullopt;
       }
       used_kwarg[*kwarg_idx] = true;
       actual_named_value = nv;
@@ -386,7 +445,7 @@ static c10::optional<MatchedSchema> tryMatchSchema(
         err() << "Argument " << schema.arguments()[schema_i].name()
               << " not provided.\n";
       }
-      return c10::nullopt;
+      return std::nullopt;
     }
 
     // Make sure the actual_named_value found matches the type of arg
@@ -400,13 +459,16 @@ static c10::optional<MatchedSchema> tryMatchSchema(
         allow_conversions,
         type_env);
     if (!positional) {
-      return c10::nullopt;
+      return std::nullopt;
     }
     positional_inputs.push_back(positional);
   }
   // check for unused self argument
-  if (self != c10::nullopt && failure_messages) {
-    err() << "Provided self argument not used in schema.\n";
+  if (self != std::nullopt) {
+    if (failure_messages) {
+      err() << "Provided self argument not used in schema.\n";
+    }
+    return std::nullopt;
   }
 
   if (schema.is_vararg()) {
@@ -421,10 +483,10 @@ static c10::optional<MatchedSchema> tryMatchSchema(
       err() << "Expected at most " << used_args << " arguments "
             << "but found " << args.size() << " positional arguments.\n";
     }
-    return c10::nullopt;
+    return std::nullopt;
   }
   // check for unused kwargs
-  for (size_t i = 0; i < kwargs.size(); ++i) {
+  for (const auto i : c10::irange(kwargs.size())) {
     const auto& nv = kwargs[i];
     if (!used_kwarg[i]) {
       if (failure_messages) {
@@ -434,7 +496,7 @@ static c10::optional<MatchedSchema> tryMatchSchema(
           err() << "Keyword argument " << nv.name() << " specified twice.\n";
         }
       }
-      return c10::nullopt;
+      return std::nullopt;
     }
   }
 
@@ -449,17 +511,22 @@ static c10::optional<MatchedSchema> tryMatchSchema(
   // Therefore, either all or none returns has field names.
   bool return_has_field_names =
       std::all_of(returns.begin(), returns.end(), [&](const Argument& r) {
-        return r.name().length() > 0;
+        return !r.name().empty();
       });
-  c10::OptNameList return_field_names = c10::nullopt;
+  c10::OptNameList return_field_names = std::nullopt;
   if (return_has_field_names) {
     return_field_names =
         fmap(returns, [&](const Argument& r) { return r.name(); });
   }
+
+  // construct the full name of the schema for easier look up
+  auto schema_name = getFullSchemaName(schema);
+
   return MatchedSchema{
       std::move(positional_inputs),
       std::move(return_types),
-      std::move(return_field_names)};
+      std::move(return_field_names),
+      schema_name};
 }
 
 MatchedSchema matchSchema(
@@ -468,7 +535,7 @@ MatchedSchema matchSchema(
     Graph& graph,
     at::ArrayRef<NamedValue> args,
     at::ArrayRef<NamedValue> kwargs,
-    const c10::optional<NamedValue>& self) {
+    const std::optional<NamedValue>& self) {
   std::stringstream failure_messages;
   if (auto result = tryMatchSchema(
           schema,
@@ -481,18 +548,7 @@ MatchedSchema matchSchema(
           /*allow_conversions=*/true)) {
     return *result;
   }
-  throw ErrorReport(loc) << failure_messages.str();
-}
-
-MatchedSchema matchSchema(
-    const ::c10::FunctionSchema& schema,
-    const SourceRange& loc,
-    Graph& graph,
-    at::ArrayRef<Value*> args,
-    at::ArrayRef<NamedValue> kwargs) {
-  std::vector<NamedValue> named_args =
-      fmap(args, [](Value* v) { return NamedValue(v); });
-  return matchSchema(schema, loc, graph, named_args, kwargs);
+  throw(ErrorReport(loc) << failure_messages.str());
 }
 
 static std::string prefixLine(
@@ -515,9 +571,9 @@ std::pair<size_t, MatchedSchema> matchSchemas(
     Graph& graph,
     at::ArrayRef<NamedValue> args,
     at::ArrayRef<NamedValue> kwargs,
-    const c10::optional<NamedValue>& self,
+    const std::optional<NamedValue>& self,
     bool render_errors) {
-  TORCH_INTERNAL_ASSERT(schemas.size() > 0);
+  TORCH_INTERNAL_ASSERT(!schemas.empty());
   // if there is only one schema, we do not need to try without conversions
   // first. this is faster and puts less dead code in the graph.
   if (schemas.size() == 1) {
@@ -528,7 +584,7 @@ std::pair<size_t, MatchedSchema> matchSchemas(
   for (bool allow_conversions : {false, true}) {
     // clear previous error messages
     failure_messages.str("");
-    for (size_t i = 0; i < schemas.size(); ++i) {
+    for (const auto i : c10::irange(schemas.size())) {
       const auto matched_schema = tryMatchSchema(
           *schemas[i],
           loc,
@@ -551,11 +607,12 @@ std::pair<size_t, MatchedSchema> matchSchemas(
         schemas, loc, graph, args, kwargs, self, /*render_errors=*/true);
   }
 
-  throw ErrorReport(loc) << "Arguments for call are not valid.\n"
-                         << "The following variants are available:\n"
-                         << prefixLine(failure_messages.str(), "  ")
-                         << "\nThe original call is";
-  throw ErrorReport(loc) << failure_messages.str();
+  throw(
+      ErrorReport(loc) << "Arguments for call are not valid.\n"
+                       << "The following variants are available:\n"
+                       << prefixLine(failure_messages.str(), "  ")
+                       << "\nThe original call is");
+  throw(ErrorReport(loc) << failure_messages.str());
 }
 
 // pack outputs of a function following python rules. If there is a single value
@@ -572,7 +629,7 @@ static Value* packOutputs(
   if (field_names) {
     auto types = fmap(values, [](Value* v) { return v->type(); });
     named_tuple =
-        TupleType::createNamed(c10::nullopt, field_names.value(), types);
+        TupleType::createNamed(std::nullopt, field_names.value(), types);
   }
   return g.insertNode(g.createTuple(values, named_tuple))->output();
 }
@@ -583,7 +640,8 @@ static Value* emitBuiltinNode(
     const MatchedSchema& matched_schema,
     const SourceRange& loc,
     Graph& graph,
-    Symbol name) {
+    Symbol name,
+    std::optional<size_t> version) {
   auto n = graph.insertNode(graph.create(name, matched_schema.inputs, 0))
                ->setSourceRange(loc);
 
@@ -592,12 +650,25 @@ static Value* emitBuiltinNode(
   }
 
   // assert that we did indeed create an op that has implementation
-  // otherwise schema and dispatch are not in sync
-  n->getOperation();
+  // otherwise schema and dispatch are not in sync ONLY if the op is up
+  // to date with the server version
+  if (!version.has_value() ||
+      isOpSymbolCurrent(matched_schema.schema_name, version.value())) {
+    n->getOperation();
+  } else {
+    n->setHistoricSchemaName(matched_schema.schema_name);
+  }
 
   return packOutputs(graph, n->outputs(), matched_schema.return_field_names);
 }
 
+std::string getFullSchemaName(const ::c10::FunctionSchema& schema) {
+  if (!schema.overload_name().empty()) {
+    return schema.operator_name().name + "." + schema.overload_name();
+  }
+  return schema.operator_name().name;
+}
+
 // Search for operators matching the provided symbol name and input types.
 // If one is found, emit a node to the graph for that operator.
 Value* emitBuiltinCall(
@@ -606,52 +677,98 @@ Value* emitBuiltinCall(
     Symbol name,
     at::ArrayRef<NamedValue> args,
     at::ArrayRef<NamedValue> kwargs,
-    const c10::optional<NamedValue>& self) {
+    const std::optional<NamedValue>& self) {
   const auto& variants = getAllOperatorsFor(name);
   const auto& builtin_functions = getAllBuiltinFunctionsFor(name);
 
-  std::stringstream failure_messages;
+  // first let's set the graph's version
+  auto graph_version = graph.get_op_version();
+
   std::vector<const FunctionSchema*> schemas;
+  // we append them later to schemas because
+  // parseSchema returns rvalue which can not
+  // be casted to const pointer.
+  std::vector<FunctionSchema> upgrader_schemas;
   schemas.reserve(variants.size());
   for (const std::shared_ptr<Operator>& op : variants) {
-    schemas.push_back(&op->schema());
+    bool found_upgrader = false;
+    auto op_name = getFullSchemaName(op->schema());
+    if (graph_version.has_value()) {
+      auto version_entry = get_operator_version_map().find(op_name);
+      if (version_entry != get_operator_version_map().end()) {
+        auto old_schema_entry =
+            findUpgrader(version_entry->second, graph_version.value());
+        if (old_schema_entry.has_value()) {
+          FunctionSchema old_schema =
+              parseSchema(old_schema_entry.value().old_schema);
+          upgrader_schemas.push_back(old_schema);
+          found_upgrader = true;
+        } else {
+          if (!isOpCurrentBasedOnUpgraderEntries(
+                  version_entry->second, graph_version.value())) {
+            TORCH_INTERNAL_ASSERT(false, "Valid upgrader must be present");
+          }
+        }
+      }
+    }
+    if (!found_upgrader)
+      schemas.push_back(&op->schema());
   }
+
+  // we might have seen old historic
+  // ops that are deprecated
+  if (variants.empty()) {
+    auto oldSchemas =
+        loadPossibleHistoricOps(name.toQualString(), graph_version);
+    upgrader_schemas.reserve(oldSchemas.size());
+    for (const auto& old_schema_entry : oldSchemas) {
+      FunctionSchema old_schema = parseSchema(old_schema_entry);
+      upgrader_schemas.emplace_back(old_schema);
+    }
+  }
+
+  // TODO (tugsuu): make sure this is optimized later
+  for (const auto& schema : upgrader_schemas) {
+    schemas.push_back(&schema);
+  }
+
   for (const auto method : builtin_functions) {
     method->ensure_defined();
     schemas.push_back(&method->getSchema());
   }
 
   // no operators found with the same name, print out similarly named operators
-  if (schemas.size() == 0) {
+  if (schemas.empty()) {
     const auto close_symbols = findSimilarOperators(name);
     auto error = ErrorReport(loc);
     const auto& user_function_name = name.toQualString();
     error << "Unknown builtin op: " << user_function_name << ".\n";
-    if (close_symbols.size() == 0) {
+    if (close_symbols.empty()) {
       error
           << "Could not find any similar ops to " << user_function_name
           << ". This op may not exist or may not be currently supported in TorchScript.\n";
     } else {
       error << "Here are some suggestions: \n";
       for (const auto& sym : close_symbols) {
-        error << "\t" << sym.toQualString() << "\n";
+        error << '\t' << sym.toQualString() << '\n';
       }
       error << "\nThe original call is";
     }
-    throw error;
+    throw ErrorReport(error);
   }
 
   auto matched = matchSchemas(schemas, loc, graph, args, kwargs, self);
 
-  if (matched.first < variants.size()) {
-    return emitBuiltinNode(matched.second, loc, graph, name);
+  if (matched.first < variants.size() + upgrader_schemas.size()) {
+    return emitBuiltinNode(matched.second, loc, graph, name, graph_version);
   } else {
-    Function* fn = builtin_functions[matched.first - variants.size()];
+    auto& fn = *builtin_functions[matched.first - variants.size()];
     // we inline builtin calls because they are normally very small
     // wrappers and are not useful for keeping around to debug
-    return insertGraph(graph, *fn->graph(), matched.second.inputs).at(0);
+    return insertGraph(
+               graph, *toGraphFunction(fn).graph(), matched.second.inputs)
+        .at(0);
   }
 }
 
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
