@@ -1,24 +1,22 @@
 #pragma once
 
 #include <ATen/core/Dimname.h>
+#include <ATen/core/class_type.h>
 #include <ATen/core/jit_type.h>
 #include <ATen/core/stack.h>
+#include <ATen/core/symbol.h>
 #include <c10/util/Exception.h>
-#include <torch/csrc/WindowsTorchApiMacro.h>
+#include <torch/csrc/Export.h>
 
-#include <torch/csrc/jit/api/object.h>
 #include <torch/csrc/jit/frontend/source_range.h>
 #include <torch/csrc/utils/variadic.h>
 
 #include <cstdint>
-#include <iostream>
 #include <memory>
-#include <mutex>
 #include <unordered_map>
 #include <vector>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 struct Node;
 struct Value;
 struct Graph;
@@ -39,15 +37,22 @@ using ::c10::ivalue::ConstantString;
 using torch::autograd::Variable;
 using variable_list = std::vector<Variable>;
 
+TORCH_API std::atomic<bool>& getTracerStateWarnMode();
+
 struct TORCH_API TracingState
     : public std::enable_shared_from_this<TracingState> {
   TracingState();
   ~TracingState();
 
+  // NOLINTNEXTLINE(cppcoreguidelines-non-private-member-variables-in-classes)
   std::shared_ptr<Graph> graph;
-  bool warn = true;
+  // NOLINTNEXTLINE(cppcoreguidelines-non-private-member-variables-in-classes)
+  bool warn = getTracerStateWarnMode();
+  // NOLINTNEXTLINE(cppcoreguidelines-non-private-member-variables-in-classes)
   bool strict = true;
+  // NOLINTNEXTLINE(cppcoreguidelines-non-private-member-variables-in-classes)
   bool force_outplace = false;
+  // NOLINTNEXTLINE(cppcoreguidelines-non-private-member-variables-in-classes)
   std::function<std::string(const Variable& var)> lookup_var_name_fn =
       [](const Variable& var) { return ""; };
 
@@ -65,6 +70,9 @@ struct TORCH_API TracingState
   Value* getOutput(const IValue& var, size_t i);
   bool hasValue(const IValue& var) const;
 
+  Node* createNode(c10::Symbol op_name, size_t num_outputs);
+  void insertNode(Node* node);
+
  private:
   using WeakIValue = at::WeakIValue;
 
@@ -94,7 +102,7 @@ struct TORCH_API TracingState
 // all information. To prevent this, we temporarily stash it in here.
 struct ArgumentStash {
   struct IntArrayRefTrace : std::vector<Value*> {
-    IntArrayRefTrace(int size) : std::vector<Value*>(size, nullptr) {}
+    IntArrayRefTrace(size_t size) : std::vector<Value*>(size, nullptr) {}
   };
 
   static bool empty() {
@@ -179,7 +187,7 @@ struct TORCH_API NoWarn {
     }
   }
   std::shared_ptr<TracingState> state;
-  bool prev;
+  bool prev{false};
 };
 
 struct WithNestedTracingFrame {
@@ -214,42 +222,57 @@ TORCH_API std::pair<std::shared_ptr<TracingState>, Stack> trace(
     std::function<std::string(const Variable&)> var_name_lookup_fn,
     bool strict = true,
     bool force_outplace = false,
-    Module* self = nullptr);
+    Module* self = nullptr,
+    const std::vector<std::string>& argument_names = {});
 
 TORCH_API void abandon();
 
 // NB: those serve both as an intermediate steps in addInputs below,
 // as well as the overloads that terminate template recursion
 TORCH_API void addInputs(Node* n, const char* name, int64_t value);
+TORCH_API void addInputs(Node* n, const char* name, const c10::SymInt& value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    c10::optional<int64_t> value);
+    std::optional<int64_t> value);
 TORCH_API void addInputs(Node* n, const char* name, bool value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<bool>& value);
+    const std::optional<bool>& value);
 TORCH_API void addInputs(Node* n, const char* name, double value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<double>& value);
+    const std::optional<double>& value);
 TORCH_API void addInputs(Node* n, const char* name, const at::Scalar& value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<at::Scalar>& value);
+    const std::optional<at::Scalar>& value);
 TORCH_API void addInputs(Node* n, const char* name, const at::Tensor& value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<at::Tensor>& value);
+    const std::optional<at::Tensor>& value);
 TORCH_API void addInputs(Node* n, const char* name, ArrayRef<int64_t> value);
+TORCH_API void addInputs(Node* n, const char* name, c10::SymIntArrayRef value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<ArrayRef<int64_t>>& value);
+    std::optional<c10::SymInt> value);
+TORCH_API void addInputs(
+    Node* n,
+    const char* name,
+    const std::optional<ArrayRef<int64_t>>& value);
+TORCH_API void addInputs(
+    Node* n,
+    const char* name,
+    const at::OptionalIntArrayRef& opt_value);
+TORCH_API void addInputs(
+    Node* n,
+    const char* name,
+    const at::OptionalSymIntArrayRef& opt_value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
@@ -258,22 +281,35 @@ TORCH_API void addInputs(
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const List<c10::optional<at::Tensor>>& value);
+    const std::vector<at::Tensor>& value,
+    bool allow_undefined = false);
+TORCH_API void addInputs(
+    Node* n,
+    const char* name,
+    at::ITensorListRef value,
+    bool allow_undefined = false);
+TORCH_API void addInputs(
+    Node* n,
+    const char* name,
+    const List<std::optional<at::Tensor>>& value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
     ArrayRef<c10::intrusive_ptr<c10::ivalue::Object>> value,
-    const ClassTypePtr& class_type);
+    const c10::ClassTypePtr& class_type);
 TORCH_API void addInputs(Node* n, const char* name, ArrayRef<double> value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<ArrayRef<double>>& value);
-TORCH_API void addInputs(Node* n, const char* name, const std::string& value);
+    const std::optional<ArrayRef<double>>& value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<std::string>& value);
+    const std::string_view value);
+TORCH_API void addInputs(
+    Node* n,
+    const char* name,
+    const std::optional<std::string_view>& value);
 TORCH_API void addInputs(Node* n, const char* name, at::Device value);
 TORCH_API void addInputs(Node* n, const char* name, c10::Stream stream);
 TORCH_API void addInputs(Node* n, const char* name, at::Layout value);
@@ -281,46 +317,48 @@ TORCH_API void addInputs(Node* n, const char* name, at::ScalarType value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<at::ScalarType>& value);
+    const std::optional<at::ScalarType>& value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<at::Device>& value);
+    const std::optional<at::Device>& value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<at::Layout>& value);
+    const std::optional<at::Layout>& value);
 TORCH_API void addInputs(Node* n, const char* name, at::MemoryFormat value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    c10::optional<at::DimnameList> value);
+    std::optional<at::DimnameList> value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<at::MemoryFormat>& value);
+    const std::optional<at::MemoryFormat>& value);
 TORCH_API void addInputs(
     Node* n,
     const char* name,
-    const c10::optional<at::Generator>& value);
+    const std::optional<at::Generator>& value);
 
 inline void addInputs(
     Node* n,
     const char* name,
     const std::vector<bool>& value) {
-  AT_ERROR("Tracing a list of bool type is currently not supported!");
+  TORCH_CHECK(false, "Tracing a list of bool type is currently not supported!");
 }
 
 template <typename T>
 void addInputs(Node* n, const char* name, ArrayRef<T> value) {
-  AT_ERROR("Tracing a list of arbitrary type is currently not supported!");
+  TORCH_CHECK(
+      false, "Tracing a list of arbitrary type is currently not supported!");
 }
 template <typename K, typename V>
 void addInputs(
     Node* n,
     const char* name,
     const std::unordered_map<K, V>& value) {
-  AT_ERROR("Tracing a dict of arbitrary types is currently not supported!");
+  TORCH_CHECK(
+      false, "Tracing a dict of arbitrary types is currently not supported!");
 }
 
 template <size_t N>
@@ -339,19 +377,20 @@ TORCH_API void ensureUniqueIfOutOfPlaced(
     const at::Tensor& tensor);
 TORCH_API void ensureUniqueIfOutOfPlaced(
     const char* name,
-    const c10::optional<at::Tensor>& tensor);
+    const std::optional<at::Tensor>& tensor);
 
 template <
     typename T,
-    typename = torch::enable_if_t<(
-        !std::is_convertible<torch::decay_t<T>, at::TensorList>::value &&
-        !std::is_convertible<torch::decay_t<T>, c10::List<at::Tensor>>::value &&
-        !std::is_convertible<torch::decay_t<T>, at::Tensor>::value &&
-        !std::is_convertible<
-            torch::decay_t<T>,
-            c10::intrusive_ptr<c10::ivalue::Object>>::value)>>
-void addOutput(Node* node, T&&) {
-  AT_ERROR(
+    typename = std::enable_if_t<
+        (!std::is_convertible_v<std::decay_t<T>, at::TensorList> &&
+         !std::is_convertible_v<std::decay_t<T>, c10::List<at::Tensor>> &&
+         !std::is_convertible_v<std::decay_t<T>, at::Tensor> &&
+         !std::is_convertible_v<
+             std::decay_t<T>,
+             c10::intrusive_ptr<c10::ivalue::Object>>)>>
+void addOutput(Node* node, T&& /*unused*/) {
+  TORCH_CHECK(
+      false,
       "Found an unsupported argument type ",
       c10::demangle_type<T>(),
       " in the JIT tracer. File a bug report.");
@@ -368,6 +407,7 @@ TORCH_API autograd::Variable getSizeOf(
     const autograd::Variable& var,
     int64_t dim);
 
+TORCH_API autograd::Variable getNumelOf(const autograd::Variable& var);
+
 } // namespace tracer
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
