@@ -3,6 +3,7 @@
 #include <c10/util/ThreadLocalDebugInfo.h>
 #include <string>
 #include <unordered_map>
+#include <vector>
 
 namespace torch {
 
@@ -32,8 +33,6 @@ class MobileDebugInfo : public c10::DebugInfoBase {
     op_idx_ = op_idx;
   }
 
-  virtual ~MobileDebugInfo() {}
-
  private:
   std::string model_name_;
   std::string method_name_;
@@ -68,21 +67,31 @@ class MobileModuleObserver {
  public:
   virtual ~MobileModuleObserver() = default;
 
-  virtual void onEnterRunMethod(
-      const std::unordered_map<std::string, std::string>&,
-      const int32_t,
-      const std::string&) {}
-  virtual void onExitRunMethod(const int32_t) {}
-  virtual void onFailRunMethod(const int32_t, const char*) {}
-  virtual void onEnterLoadModel(const int32_t) {}
+  virtual void onEnterRunMethod(const int32_t /*unused*/) {}
+  virtual void onExitRunMethod(
+      const std::unordered_map<std::string, std::string>& /*unused*/,
+      const std::string& /*unused*/,
+      const int32_t /*unused*/) {}
+  virtual void onFailRunMethod(
+      const std::unordered_map<std::string, std::string>& /*unused*/,
+      const std::string& /*unused*/,
+      const int32_t /*unused*/,
+      const char* /*unused*/) {}
+  virtual void onEnterLoadModel(const int32_t /*unused*/) {}
   virtual void onExitLoadModel(
-      const int32_t,
-      const std::unordered_map<std::string, std::string>&) {}
-  virtual void onFailLoadModel(const int32_t, const char*) {}
+      const int32_t /*unused*/,
+      const std::unordered_map<std::string, std::string>& /*unused*/) {
+  } // key: filename, value: file content
   virtual void onFailLoadModel(
-      const int32_t,
-      const char*,
-      const std::unordered_map<std::string, std::string>&) {}
+      const int32_t /*unused*/,
+      const char* /*unused*/) {}
+  virtual void onFailLoadModel(
+      const int32_t /*unused*/,
+      const char* /*unused*/,
+      const std::unordered_map<std::string, std::string>& /*unused*/) {}
+  virtual std::vector<std::string> getDefaultExtraFiles() = 0;
+  virtual std::unordered_map<std::string, std::string> processMetadataFromExtra(
+      const std::unordered_map<std::string, std::string>&) = 0;
 };
 
 class MobileObserverConfig {
