@@ -1,9 +1,8 @@
 #include <torch/csrc/jit/passes/constant_propagation.h>
 
-#include <ATen/core/functional.h>
 #include <ATen/core/ivalue.h>
 #include <c10/util/Exception.h>
-#include <torch/csrc/autograd/variable.h>
+#include <c10/util/irange.h>
 #include <torch/csrc/jit/ir/alias_analysis.h>
 #include <torch/csrc/jit/ir/constants.h>
 #include <torch/csrc/jit/ir/ir.h>
@@ -12,34 +11,35 @@
 #include <torch/csrc/jit/passes/dead_code_elimination.h>
 #include <torch/csrc/jit/runtime/operator.h>
 #include <torch/csrc/jit/runtime/vararg_functions.h>
-#include <torch/csrc/utils/memory.h>
 
-namespace torch {
-namespace jit {
+#include <utility>
 
-c10::optional<std::vector<IValue>> runNodeIfInputsAreConstant(
+namespace torch::jit {
+
+std::optional<std::vector<IValue>> runNodeIfInputsAreConstant(
     const Node* n,
-    bool ignore_custom_classes) {
+    bool ignore_custom_classes,
+    AliasDb* db) {
   Stack stack;
   for (auto input : n->inputs()) {
     if (auto ival = toIValue(input)) {
       stack.push_back(*ival);
     } else {
-      return c10::nullopt;
+      return std::nullopt;
     }
   }
 
   switch (n->kind()) {
     case prim::ListUnpack: {
       if (stack.back().toList().size() != n->outputs().size()) {
-        return c10::nullopt;
+        return std::nullopt;
       }
       listUnpack(stack, n->outputs().size());
     } break;
     case prim::TupleConstruct: {
       auto tt = n->output()->type()->expect<TupleType>();
       if (tt->name()) {
-        namedTupleConstruct(stack, tt, n->inputs().size());
+        namedTupleConstruct(stack, std::move(tt), n->inputs().size());
       } else {
         tupleConstruct(stack, n->inputs().size());
       }
@@ -52,10 +52,15 @@ c10::optional<std::vector<IValue>> runNodeIfInputsAreConstant(
     } break;
     case prim::DictConstruct: {
       dictConstruct(
-          stack, n->output()->type()->expect<DictType>(), n->inputs().size());
+          stack,
+          n->output()->type()->expectRef<DictType>(),
+          n->inputs().size());
     } break;
     case prim::CreateObject: {
-      createObject(stack, n->output()->type()->expect<ClassType>());
+      createObject(
+          stack,
+          n->output()->type()->expect<ClassType>(),
+          /*use_weak_ref*/ true);
     } break;
     case prim::GetAttr: {
       auto attr = pop(stack).toObject()->getAttr(n->s(attr::name));
@@ -70,30 +75,52 @@ c10::optional<std::vector<IValue>> runNodeIfInputsAreConstant(
         // vararg schemas require the number of inputs at the top of the stack
         // but this is broken in other places in constant prop, so disable it
         // for now
-        return c10::nullopt;
+        return std::nullopt;
       }
 
       try {
         auto op = n->getOperation();
-        op(&stack);
+        op(stack);
       } catch (...) {
-        return c10::nullopt;
+        return std::nullopt;
       }
     } break;
   }
 
-  for (const IValue& v : stack) {
+  for (IValue& v : stack) {
     if (v.isTensor()) {
-      at::Tensor t = v.toTensor();
+      const at::Tensor& t = v.toTensor();
       if (t.defined() && t.requires_grad()) {
         // requires grad tensors cannot be constants
-        return c10::nullopt;
+        return std::nullopt;
       }
     }
     // Weak form of const propagation
     if (ignore_custom_classes) {
       if (v.isCustomClass()) {
-        return c10::nullopt;
+        return std::nullopt;
+      }
+    }
+    // see [Constant Object Weak CompilationUnit Reference]
+    if (v.isCustomClass()) {
+      if (v.toObject()->is_weak_compilation_ref()) {
+        continue;
+      }
+      if (!db) {
+        continue;
+      }
+      // NOLINTNEXTLINE(cppcoreguidelines-pro-type-const-cast)
+      Node* n_non_const = const_cast<Node*>(n);
+      if (db->mayContainAlias(
+              n_non_const->inputs(), {n_non_const->outputs()})) {
+        continue;
+      }
+      auto obj = v.toObject();
+      obj->unsafe_make_weak_compilation_ref();
+    }
+    if (v.isObject()) {
+      if (!v.toObject()->is_weak_compilation_ref()) {
+        return std::nullopt;
       }
     }
   }
@@ -113,6 +140,8 @@ std::unordered_set<Symbol> skip_list = {
     prim::profile,
     prim::profile_ivalue,
     prim::unchecked_unwrap_optional, // TODO remove
+    prim::awaitable,
+    aten::dequantize,
     // TODO (zach): we should consider skipping tensor factories in the cases
     // where the constant tensor would be large but cheap to create.
 };
@@ -132,8 +161,9 @@ struct ConstantPropagator {
     return ConstantPropagator(std::move(graph), false, false);
   }
 
-  void run() {
+  bool run() {
     ConstantPropagation(graph_->block());
+    return made_change_;
   }
 
  private:
@@ -141,14 +171,9 @@ struct ConstantPropagator {
       std::shared_ptr<Graph> graph,
       bool aliasing_types,
       bool ignore_custom_classes)
-      : graph_(std::move(graph)) {
-    if (aliasing_types) {
-      aliasDb_ = torch::make_unique<AliasDb>(graph_);
-    } else {
-      aliasDb_ = nullptr;
-    }
-    ignore_custom_classes_ = ignore_custom_classes;
-  }
+      : graph_(std::move(graph)),
+        aliasing_types_(aliasing_types),
+        ignore_custom_classes_(ignore_custom_classes) {}
 
   void propagateNode(Node* n) {
     std::vector<IValue> outputs;
@@ -161,9 +186,10 @@ struct ConstantPropagator {
     }
     auto graph = n->owningGraph();
     WithInsertPoint guard(n);
-    for (size_t i = 0; i < outputs.size(); ++i) {
+    for (const auto i : c10::irange(outputs.size())) {
       auto new_output = tryInsertConstant(*graph, outputs[i]);
       if (new_output) {
+        made_change_ = true;
         GRAPH_UPDATE(
             "Folding %",
             n->outputs()[i]->debugName(),
@@ -185,6 +211,7 @@ struct ConstantPropagator {
       n->outputs().at(i)->replaceAllUsesWith(
           n->inputs().at(i + loop_input_offset));
     }
+    made_change_ = true;
     n->destroy();
   }
 
@@ -236,6 +263,7 @@ struct ConstantPropagator {
     size_t block_index = *input_bool ? 0 : 1;
     ConstantPropagation(n->blocks().at(block_index));
     inlineIfBody(n->blocks().at(block_index));
+    made_change_ = true;
   }
 
   void replaceAndRemoveIfOutput(Node* n, size_t i, Value* replacement) {
@@ -246,7 +274,7 @@ struct ConstantPropagator {
   }
 
   // remove extra outputs from the node
-  bool removeExtraIfOutputs(Node* n) {
+  void removeExtraIfOutputs(Node* n) {
     TORCH_CHECK(n->kind() == prim::If, "Only supported for If nodes");
     auto true_block = n->blocks()[0];
     auto false_block = n->blocks()[1];
@@ -274,12 +302,12 @@ struct ConstantPropagator {
 
       i++; // increment bc we didn't remove current index
     }
-    // an output was removed
-    return initial_outputs != true_block->outputs().size();
+    made_change_ |= initial_outputs != true_block->outputs().size();
   }
 
   // remove extra outputs from the node
   void removeExtraLoopOutputs(Node* node) {
+    auto initial_outputs = node->outputs().size();
     auto loop_body = node->blocks().at(0);
     auto loop_input_offset = 2; // offset of loop carried deps in input list
     auto loop_body_offset =
@@ -300,40 +328,33 @@ struct ConstantPropagator {
         loop_body->eraseOutput(loop_body_offset + i);
       }
     }
+    made_change_ |= initial_outputs != node->outputs().size();
   }
 
-  // An Op has runnable inputs if:
-  // - All inputs are constants.
-  // - It is an op that forwards tuples, and all inputs are constants
-  // or tuples that we know the ivalue for. We can't use known tuple ivalues
-  // for non-forwarding ops because that Tuple could contain an ivalue that is
-  // not allowed as a constant, for instance, a Tensor with a gradient.
-  bool runnableInputs(Node* n) {
-    if (std::all_of(n->inputs().begin(), n->inputs().end(), [&](Value* v) {
-          return v->node()->kind() == prim::Constant;
-        })) {
-      return true;
-    }
-    return false;
-  };
-
   bool noMutableValues(at::ArrayRef<Value*> values) {
     return std::none_of(values.begin(), values.end(), [](Value* v) {
       return AliasDb::isMutableType(v);
     });
   }
 
+  AliasDb* getOrCreateAliasDb() {
+    if (!aliasDb_) {
+      aliasDb_ = std::make_unique<AliasDb>(graph_);
+    }
+    return aliasDb_.get();
+  }
+
   bool supportedNode(Node* n) {
-    bool no_mutation;
-    if (aliasDb_) {
-      no_mutation = !aliasDb_->hasWriters(n);
+    bool no_mutation = false;
+    if (aliasing_types_) {
+      no_mutation = !getOrCreateAliasDb()->hasWriters(n);
     } else {
       no_mutation =
           noMutableValues(n->inputs()) && noMutableValues(n->outputs());
     }
     return no_mutation && !n->kind().is_onnx() &&
         skip_list.count(n->kind()) == 0 && !n->isNondeterministic() &&
-        !n->hasSideEffects() && n->blocks().size() == 0;
+        !n->hasSideEffects() && n->blocks().empty();
   }
 
   void ConstantPropagation(at::ArrayRef<Block*> blocks) {
@@ -343,10 +364,13 @@ struct ConstantPropagator {
   }
 
   void ConstantPropagation(Node* n) {
-    bool runnable_inputs = runnableInputs(n);
+    bool constant_inputs =
+        std::all_of(n->inputs().begin(), n->inputs().end(), [&](Value* v) {
+          return v->node()->kind() == prim::Constant;
+        });
     if (n->kind() == prim::If) {
       // inline node if we can, otherwise check for simplified outputs
-      if (runnable_inputs) {
+      if (constant_inputs) {
         inlineIf(n);
       } else {
         ConstantPropagation(n->blocks());
@@ -359,7 +383,7 @@ struct ConstantPropagator {
         ConstantPropagation(n->blocks());
         removeExtraLoopOutputs(n);
       }
-    } else if (runnable_inputs && supportedNode(n)) {
+    } else if (constant_inputs && supportedNode(n)) {
       propagateNode(n);
     } else {
       ConstantPropagation(n->blocks());
@@ -375,27 +399,35 @@ struct ConstantPropagator {
   }
 
   std::shared_ptr<Graph> graph_;
-  std::unique_ptr<AliasDb> aliasDb_;
+  // lazily initialized if using aliasing_types, otherwise not initialized
+  std::unique_ptr<AliasDb> aliasDb_ = nullptr;
+  bool aliasing_types_;
+  bool made_change_ = false;
   bool ignore_custom_classes_;
 };
 } // anonymous namespace
 
-void ConstantPropagation(
+bool ConstantPropagation(
     std::shared_ptr<Graph>& graph,
     bool ignore_custom_classes) {
   ConstantPropagator cp =
       ConstantPropagator::WithAliasDb(graph, ignore_custom_classes);
-  cp.run();
-  EliminateDeadCode(graph);
+  bool made_change = cp.run();
+  if (made_change) {
+    EliminateDeadCode(graph);
+  }
   GRAPH_DUMP("After ConstantPropagation: ", graph);
+  return made_change;
 }
 
-void ConstantPropagationImmutableTypes(std::shared_ptr<Graph>& graph) {
+bool ConstantPropagationImmutableTypes(std::shared_ptr<Graph>& graph) {
   ConstantPropagator cp = ConstantPropagator::NoAliasDb(graph);
-  cp.run();
-  EliminateDeadCode(graph);
-  GRAPH_DUMP("After ConstantPropagation: ", graph);
+  bool made_change = cp.run();
+  if (made_change) {
+    EliminateDeadCode(graph);
+  }
+  GRAPH_DUMP("After ConstantPropagationImmutableTypes: ", graph);
+  return made_change;
 }
 
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
