@@ -1,18 +1,16 @@
 #include <torch/csrc/jit/passes/liveness.h>
 
-#include <torch/csrc/jit/ir/alias_analysis.h>
 #include <torch/csrc/jit/ir/ir_views.h>
-#include <torch/csrc/jit/passes/constant_pooling.h>
+#include <iostream>
 #include <memory>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 
 // LivenessAnalyzer computes "bailout" liveness which is equivalent to
 // "{LIVE_IN} or {GEN}" or "{LIVE_OUT} - {KILL}"
 struct LivenessAnalyzer {
   explicit LivenessAnalyzer(std::shared_ptr<Graph> graph)
-      : graph_(std::move(graph)), changed_(false) {}
+      : graph_(std::move(graph)) {}
 
   std::unordered_map<Node*, std::vector<Value*>> run() {
     std::vector<Node*> counters;
@@ -68,14 +66,14 @@ struct LivenessAnalyzer {
       const std::unordered_map<Node*, std::vector<Value*>>& liveness_sets) {
     std::cout << "Liveness info:\n";
     for (auto e : liveness_sets) {
-      if (e.first->outputs().size() > 0) {
+      if (!e.first->outputs().empty()) {
         std::cout << e.first->outputs()[0]->debugName();
       }
 
-      std::cout << " " << e.first->kind().toQualString();
+      std::cout << ' ' << e.first->kind().toQualString();
       std::cout << " = ";
       dump(e.second);
-      std::cout << std::endl;
+      std::cout << '\n';
     }
     std::cout << "graph :\n";
     graph_->dump();
@@ -83,16 +81,16 @@ struct LivenessAnalyzer {
 
   void dump(const std::vector<Value*>& set) {
     bool first = true;
-    std::cout << "[";
+    std::cout << '[';
     for (auto el : set) {
       if (first) {
         first = false;
       } else {
         std::cout << ", ";
       }
-      std::cout << el->debugName() << "(" << el->unique() << ")";
+      std::cout << el->debugName() << '(' << el->unique() << ')';
     }
-    std::cout << "]";
+    std::cout << ']';
   }
 
  private:
@@ -148,7 +146,7 @@ struct LivenessAnalyzer {
   }
 
   std::shared_ptr<Graph> graph_;
-  bool changed_;
+  bool changed_{false};
   std::map<Node*, SparseBitVector> liveness_sets_;
   std::map<size_t, Value*> ids_to_values_;
 };
@@ -159,5 +157,4 @@ std::unordered_map<Node*, std::vector<Value*>> BuildLivenessSets(
   return la.run();
 }
 
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
