@@ -1,7 +1,5 @@
 #include <torch/csrc/jit/frontend/error_report.h>
 #include <torch/csrc/jit/jit_log.h>
-#include <torch/csrc/jit/passes/dead_code_elimination.h>
-#include <torch/csrc/jit/passes/onnx/helper.h>
 #include <torch/csrc/jit/passes/onnx/list_model_parameters.h>
 
 namespace torch::jit {
@@ -22,7 +20,7 @@ using namespace ::c10::onnx;
 //   ...
 //   %weight = prim::GetAttr[name="scale"](%B)
 //   ...
-std::deque<std::string> findSubModuleAttr(
+static std::deque<std::string> findSubModuleAttr(
     Value* input,
     std::string& name,
     Module& attrModule,
@@ -48,7 +46,10 @@ std::deque<std::string> findSubModuleAttr(
   return moduleNames;
 }
 
-Value* addParamAsArgument(Function* function, std::string& name, IValue& attr) {
+static Value* addParamAsArgument(
+    Function* function,
+    std::string& name,
+    IValue& attr) {
   auto schema = function->getSchema();
   auto args = schema.arguments();
   args.emplace_back(name, nullptr, std::nullopt, attr);
@@ -64,7 +65,7 @@ Value* addParamAsArgument(Function* function, std::string& name, IValue& attr) {
       attr.type());
 }
 
-std::vector<IValue> getParamAttributes(
+static std::vector<IValue> getParamAttributes(
     Block* block,
     std::shared_ptr<Graph>& graph,
     const Module& module_,
@@ -163,7 +164,7 @@ std::vector<IValue> getParamAttributes(
   return parameterIValues;
 }
 
-void insertMainModuleAsConstant(const std::shared_ptr<Graph>& graph) {
+static void insertMainModuleAsConstant(const std::shared_ptr<Graph>& graph) {
   auto* constNode = graph->create(prim::CreateObject);
   constNode->output()->setType(graph->inputs().at(0)->type());
   auto it = graph->nodes().begin();
