@@ -1,10 +1,11 @@
 #include <torch/csrc/jit/passes/onnx/preprocess_for_onnx.h>
 
-#include <torch/csrc/jit/jit_log.h>
-#include <torch/csrc/jit/passes/onnx/helper.h>
+#include <ATen/ScalarOps.h>
+#include <c10/util/irange.h>
 
-namespace torch {
-namespace jit {
+#include <torch/csrc/jit/jit_log.h>
+
+namespace torch::jit {
 
 namespace onnx {
 using namespace ::c10::onnx;
@@ -12,18 +13,18 @@ using namespace ::c10::onnx;
 
 namespace {
 
-at::optional<Node*> FindFusibleListUnpack(Node* n) {
+std::optional<Node*> FindFusibleListUnpack(Node* n) {
   // 1. number of outputs is restricted to 1.
   // 2. output is only used by prim::ListUnpack.
   if (n->outputs().size() != 1) {
-    return at::nullopt;
+    return std::nullopt;
   }
   if (n->output()->uses().size() != 1) {
-    return at::nullopt;
+    return std::nullopt;
   }
   auto listUnpackNode = n->output()->uses()[0].user;
   if (listUnpackNode->kind() != prim::ListUnpack) {
-    return at::nullopt;
+    return std::nullopt;
   }
   return listUnpackNode;
 }
@@ -35,7 +36,7 @@ at::optional<Node*> FindFusibleListUnpack(Node* n) {
 // that the symbolic function is aware of the number of outputs.
 //
 // Example IR
-//  split.Tensor(Tensor(a) self, int split_size, int dim=0) -> Tensor(a)[]
+//  split.Tensor(Tensor(a -> *) self, int split_size, int dim=0) -> Tensor[]
 //  split_with_sizes(Tensor self, int[] split_sizes, int dim=0) -> Tensor[]
 //
 // graph(%input : Float(5, 4, 3, strides=[12, 3, 1])):
@@ -50,7 +51,7 @@ at::optional<Node*> FindFusibleListUnpack(Node* n) {
 // graph(%input : Float(5, 4, 3, strides=[12, 3, 1])):
 //   %13 : int[] = prim::Constant[value=[2, 1, 2]]()
 //   %7 : int = prim::Constant[value=0]()
-//   %8 : int = prim::Constant[value=3]()  # Adding addtional input of value 3
+//   %8 : int = prim::Constant[value=3]()  # Adding additional input of value 3
 //      representing the number of outputs.
 //   %14 : Float(2, 4, 3, strides=[12, 3, 1]), %15 : Float(1, 4, 3, strides=[12,
 //      3, 1]), %16 : Float(2, 4, 3, strides=[12, 3, 1] =
@@ -65,10 +66,9 @@ void FuseWithListUnpack(Node* n) {
 
   TORCH_INTERNAL_ASSERT(n->outputs().size() == 1);
   // 1. Add internal input "_outputs" to node, so that later symbolic function
-  // conversion
-  //    is aware of the number of outputs.
+  //    conversion is aware of the number of outputs.
   // 2. Add the exact number of outputs to n, copy metadata and replace uses of
-  // listUnpack outputs.
+  //    listUnpack outputs.
   n->i_(
       Symbol::fromQualString("attr::_outputs"),
       static_cast<int64_t>(listUnpack_node->outputs().size()));
@@ -144,7 +144,7 @@ static void ReplaceAddWithConcat(Block* b) {
         continue;
       }
 
-      TypePtr elem =
+      const auto& elem =
           it->input(0)->type()->castRaw<ListType>()->getElementType();
       if (elem->cast<IntType>()) {
         Node* concat_node = b->owningGraph()->create(onnx::Concat, 1);
@@ -152,8 +152,8 @@ static void ReplaceAddWithConcat(Block* b) {
         concat_node->insertBefore(*it);
         concat_node->addInput(it->input(0));
         concat_node->addInput(it->input(1));
-        concat_node->outputs()[0]->setType(
-            TensorType::fromNumberType(std::move(elem)));
+        concat_node->outputs()[0]->setType(TensorType::fromNumberType(*elem));
+        concat_node->copyMetadata(*it);
         it->replaceAllUsesWith(concat_node);
         it->removeAllInputs();
         it.destroyCurrent();
@@ -163,36 +163,34 @@ static void ReplaceAddWithConcat(Block* b) {
 }
 
 // This pass also covers the case when the input to ListUnpack
-// is int[] comming from some other op than ListConstruct (like Slice or Shape)
+// is int[] coming from some other op than ListConstruct (like Slice or Shape)
 //
 // before the pass
 // graph(%x.1 : Float(2, 3, strides=[3, 1], requires_grad=0, device=cpu)):
 //   %1 : None = prim::Constant()
-//   %2 : int[] = aten::size(%x.1) # <string>:7:9
+//   %2 : int[] = aten::size(%x.1)
 //   %a.1 : int, %b.1 : int = prim::ListUnpack(%2)
 //   %5 : int[] = prim::ListConstruct(%a.1, %b.1)
-//   %6 : Tensor = aten::new_zeros(%x.1, %5, %1, %1, %1, %1) #
-//   test/onnx/test_pytorch_onnx_onnxruntime.py:1757:23 return (%6)
+//   %6 : Tensor = aten::new_zeros(%x.1, %5, %1, %1, %1, %1)
 //
 // after the pass:
 // graph(%x.1 : Float(2, 3, strides=[3, 1], requires_grad=0, device=cpu)):
 //   %1 : None = prim::Constant()
-//   %2 : int[] = aten::size(%x.1) # <string>:7:9
+//   %2 : int[] = aten::size(%x.1)
 //   %7 : Tensor = onnx::Constant[value={0}]()
 //   %8 : Tensor = onnx::Gather(%2, %7)
 //   %9 : Tensor = onnx::Constant[value={1}]()
 //   %10 : Tensor = onnx::Gather(%2, %9)
 //   %a.1 : int, %b.1 : int = prim::ListUnpack(%2)
 //   %5 : int[] = prim::ListConstruct(%8, %10)
-//   %6 : Tensor = aten::new_zeros(%x.1, %5, %1, %1, %1, %1) #
-//   test/onnx/test_pytorch_onnx_onnxruntime.py:1757:23 return (%6)
+//   %6 : Tensor = aten::new_zeros(%x.1, %5, %1, %1, %1, %1)
 static void fuseListAndListUnpack(Block* b) {
   for (auto it = b->nodes().begin(), end = b->nodes().end(); it != end; ++it) {
     for (auto* child_block : it->blocks()) {
       fuseListAndListUnpack(child_block);
     }
     if (it->kind() == prim::ListUnpack) {
-      for (size_t i = 0; i < it->outputs().size(); i++) {
+      for (const auto i : c10::irange(it->outputs().size())) {
         auto output = it->outputs().at(i);
         if (it->inputs().size() == 1 &&
             it->input()->node()->kind() != prim::ListConstruct &&
@@ -210,6 +208,7 @@ static void fuseListAndListUnpack(Block* b) {
           gather_node->insertBefore(*it);
           gather_node->addInput(it->input());
           gather_node->addInput(gather_indices->output());
+          gather_node->copyMetadata(*it);
           output->replaceAllUsesWith(gather_node->output());
         }
       }
@@ -217,88 +216,15 @@ static void fuseListAndListUnpack(Block* b) {
   }
 }
 
-static void decomposeLinear(Block* b) {
-  std::vector<Node*> linear_nodes;
-  for (auto it = b->nodes().begin(), end = b->nodes().end(); it != end; ++it) {
-    for (auto* child_block : it->blocks()) {
-      decomposeLinear(child_block);
-    }
-    if (it->kind() == aten::linear) {
-      linear_nodes.push_back(*it);
-    }
-  }
-  for (Node* node : linear_nodes) {
-    auto g = b->owningGraph();
-
-    if (node->inputs()[2]->mustBeNone()) {
-      auto t_weight_n =
-          g->create(aten::t, {node->inputs()[1]}, 1)->insertBefore(node);
-      auto matmul_n =
-          g->create(aten::matmul, {node->inputs()[0], t_weight_n->output()}, 1)
-              ->insertBefore(node);
-      node->output()->replaceAllUsesWith(matmul_n->output());
-      node->destroy();
-    } else {
-      auto dim_n =
-          g->create(aten::dim, {node->inputs()[0]}, 1)->insertBefore(node);
-      auto const_2 = g->insertConstant(IValue(2));
-      const_2->node()->moveBefore(node);
-      auto eq_n = g->create(aten::eq, {dim_n->output(), const_2}, 1)
-                      ->insertBefore(node);
-
-      auto if_n = g->create(prim::If, {eq_n->output()}, 1)->insertBefore(node);
-
-      auto true_block = if_n->addBlock();
-      auto false_block = if_n->addBlock();
-
-      {
-        WithInsertPoint guard(true_block->return_node());
-        auto const_1 = g->insertConstant(IValue(1.0));
-        auto t_weight_n = g->create(aten::t, {node->inputs()[1]}, 1)
-                              ->insertBefore(true_block->return_node());
-        auto addmm_n = g->create(
-                            aten::addmm,
-                            {node->inputs()[2],
-                             node->inputs()[0],
-                             t_weight_n->output(),
-                             const_1,
-                             const_1},
-                            1)
-                           ->insertBefore(true_block->return_node());
-        true_block->registerOutput(addmm_n->output());
-      }
-
-      {
-        WithInsertPoint guard(false_block->return_node());
-        auto const_1 = g->insertConstant(IValue(1.0));
-        auto t_weight_n = g->create(aten::t, {node->inputs()[1]}, 1)
-                              ->insertBefore(false_block->return_node());
-        auto matmul_n =
-            g->create(
-                 aten::matmul, {node->inputs()[0], t_weight_n->output()}, 1)
-                ->insertBefore(false_block->return_node());
-        auto add_n =
-            g->create(
-                 aten::add, {matmul_n->output(), node->inputs()[2], const_1}, 1)
-                ->insertBefore(false_block->return_node());
-        false_block->registerOutput(add_n->output());
-      }
-      node->output()->replaceAllUsesWith(if_n->output());
-      node->destroy();
-    }
-  }
-}
-
 } // namespace
 
 void PreprocessForONNX(std::shared_ptr<Graph>& graph) {
-  GRAPH_DEBUG("priot to decompose linear", graph);
-  decomposeLinear(graph->block());
-  GRAPH_DEBUG("after decompose linear", graph);
   FuseWithListUnpack(graph->block());
+  GRAPH_DUMP("After FuseWithListUnpack: ", graph);
   ReplaceAddWithConcat(graph->block());
+  GRAPH_DUMP("After ReplaceAddWithConcat: ", graph);
   fuseListAndListUnpack(graph->block());
+  GRAPH_DUMP("After fuseListAndListUnpack: ", graph);
 }
 
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
