@@ -1,12 +1,6 @@
-#include <ATen/core/jit_type.h>
 #include <torch/csrc/jit/ir/alias_analysis.h>
-#include <torch/csrc/jit/ir/ir_views.h>
 #include <torch/csrc/jit/jit_log.h>
-#include <torch/csrc/jit/passes/dead_code_elimination.h>
-#include <torch/csrc/jit/passes/peephole.h>
 #include <torch/csrc/jit/passes/peephole_alias_sensitive.h>
-#include <torch/csrc/jit/runtime/graph_executor.h>
-#include <unordered_set>
 
 namespace torch::jit {
 
