@@ -1,28 +1,28 @@
+#include <c10/util/irange.h>
 #include <torch/csrc/jit/passes/quantization/insert_observers.h>
 
 #include <torch/csrc/jit/frontend/schema_matching.h>
 #include <torch/csrc/jit/ir/subgraph_matcher.h>
 #include <torch/csrc/jit/jit_log.h>
-#include <torch/csrc/jit/passes/constant_pooling.h>
-#include <torch/csrc/jit/passes/constant_propagation.h>
 #include <torch/csrc/jit/passes/fuse_linear.h>
 #include <torch/csrc/jit/passes/graph_rewrite_helper.h>
 #include <torch/csrc/jit/passes/inline_fork_wait.h>
 #include <torch/csrc/jit/passes/quantization/helper.h>
 #include <torch/csrc/jit/passes/remove_mutation.h>
 
-#include <regex>
+#include <memory>
 #include <stack>
+#include <string>
+#include <utility>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 
-using ModuleQConfigMap = std::unordered_map<ModulePtr, c10::optional<QConfig>>;
+using ModuleQConfigMap = std::unordered_map<ModulePtr, std::optional<QConfig>>;
 
 namespace {
 
 struct OptionalQConfigHash {
-  inline size_t operator()(const c10::optional<QConfig>& qconfig_opt) const {
+  inline size_t operator()(const std::optional<QConfig>& qconfig_opt) const {
     if (qconfig_opt.has_value()) {
       const auto& m1 = std::get<0>(*qconfig_opt);
       const auto& m2 = std::get<1>(*qconfig_opt);
@@ -33,9 +33,9 @@ struct OptionalQConfigHash {
   }
 };
 using QConfigTypePtrMap =
-    std::unordered_map<c10::optional<QConfig>, TypePtr, OptionalQConfigHash>;
+    std::unordered_map<std::optional<QConfig>, TypePtr, OptionalQConfigHash>;
 using NameModuleVector = std::vector<std::pair<std::string, Module>>;
-using OptionalModuleVector = std::vector<c10::optional<Module>>;
+using OptionalModuleVector = std::vector<std::optional<Module>>;
 using ModuleMethodVector = std::vector<std::pair<Module, std::string>>;
 using graph_rewrite_helper::PatternInfo;
 using graph_rewrite_helper::replaceConvolutionWithAtenConv;
@@ -46,8 +46,8 @@ void fillQConfigMap(
     const QConfigDict& qconfig_dict,
     ModuleQConfigMap& map,
     const std::string& key = "",
-    const c10::optional<QConfig>& parent_qconfig = c10::nullopt) {
-  c10::optional<QConfig> qconfig;
+    const std::optional<QConfig>& parent_qconfig = std::nullopt) {
+  std::optional<QConfig> qconfig;
   if (qconfig_dict.find(key) != qconfig_dict.end()) {
     GRAPH_DEBUG("Got module config for key:", key);
     qconfig = qconfig_dict.at(key);
@@ -59,7 +59,7 @@ void fillQConfigMap(
 
   for (const NameModule& s : module.named_children()) {
     std::string child_key;
-    if (key == "") {
+    if (key.empty()) {
       child_key = s.name;
     } else {
       child_key = key + "." + s.name;
@@ -89,8 +89,9 @@ class ModuleCloneHelper {
       const ModuleQConfigMap& module_qconfig_map,
       bool inplace = false) {
     std::unordered_map<TypePtr, QConfigTypePtrMap> type_remap;
-    IValue::HashAliasedIValueMap memo;
-    return clone_impl(module, module_qconfig_map, type_remap, inplace, memo);
+    IValue::HashIdentityIValueMap memo;
+    return clone_impl(
+        module, module_qconfig_map, type_remap, inplace, std::move(memo));
   }
 
  private:
@@ -99,7 +100,7 @@ class ModuleCloneHelper {
       const ModuleQConfigMap& module_qconfig_map,
       std::unordered_map<TypePtr, QConfigTypePtrMap>& type_remap,
       bool inplace,
-      IValue::HashAliasedIValueMap memo) {
+      IValue::HashIdentityIValueMap memo) {
     auto qconfig = module_qconfig_map.at(module._ivalue());
     auto type = module.type();
     // Create a new _ivalue in the same compilation unit.
@@ -124,7 +125,7 @@ class ModuleCloneHelper {
     }
     // Copy slots. If a slot is a module - recursively clone it.
     size_t N = type->numAttributes();
-    for (size_t i = 0; i < N; ++i) {
+    for (const auto i : c10::irange(N)) {
       IValue s = module._ivalue()->getSlot(i);
       std::string attr_name = type->getAttributeName(i);
       TypePtr attr_type = type->getAttribute(i);
@@ -171,7 +172,7 @@ class ModuleCloneHelper {
         auto getstate_method = r.find_method("__getstate__");
         TORCH_INTERNAL_ASSERT(getstate_method, "expect __getstate__");
         auto state = (*getstate_method)(Stack{});
-        (*setstate_method)(Stack{state});
+        (*setstate_method)(Stack{std::move(state)});
       }
     }
     return r;
@@ -183,7 +184,7 @@ class ModuleCloneHelper {
       const Module& source,
       Module& target,
       const ModuleQConfigMap& module_qconfig_map,
-      const std::function<TypePtr(TypePtr, c10::optional<QConfig>)>&
+      const std::function<TypePtr(TypePtr, std::optional<QConfig>)>&
           type_remap_fn) {
     // remap of %self will be done outside of the function
     // and we don't support the case when people pass in
@@ -235,7 +236,7 @@ class ModuleCloneHelper {
       const Module& source,
       Module& target,
       const ModuleQConfigMap& module_qconfig_map,
-      const std::function<TypePtr(TypePtr, c10::optional<QConfig>)>&
+      const std::function<TypePtr(TypePtr, std::optional<QConfig>)>&
           type_remap_fn) {
     remapTypes(
         graph->block(),
@@ -253,7 +254,7 @@ class ModuleCloneHelper {
       const ModuleQConfigMap& module_qconfig_map,
       const std::unordered_map<TypePtr, QConfigTypePtrMap>& type_remap) {
     auto type_remap_fn = [&](TypePtr type_ptr,
-                             const c10::optional<QConfig>& qconfig) {
+                             const std::optional<QConfig>& qconfig) {
       if (type_remap.find(type_ptr) != type_remap.end()) {
         const auto& qconfig_map = type_remap.at(type_ptr);
         if (qconfig_map.find(qconfig) != qconfig_map.end()) {
@@ -262,20 +263,21 @@ class ModuleCloneHelper {
       }
       return type_ptr;
     };
-    auto graph = method.graph()->copy();
+    auto graph = toGraphFunction(method).graph()->copy();
     remapTypes(graph.get(), source, target, module_qconfig_map, type_remap_fn);
     // remap self
     graph->inputs()[0]->setType(target.type());
     // we only support %self being Module in the arguments of function
     auto schema_type_remap_fn = [&](TypePtr type_ptr) {
-      return type_remap_fn(type_ptr, module_qconfig_map.at(source._ivalue()));
+      return type_remap_fn(
+          std::move(type_ptr), module_qconfig_map.at(source._ivalue()));
     };
     auto schema =
         method.getSchema().cloneWithRemappedTypes(schema_type_remap_fn);
     const auto this_method_name =
         c10::QualifiedName(*target.type()->name(), method.name());
     auto copied = target._ivalue()->compilation_unit()->create_function(
-        this_method_name, graph);
+        this_method_name, std::move(graph));
     target.type()->addMethod(copied);
     copied->setSchema(std::move(schema));
   }
@@ -301,6 +303,8 @@ class InsertObserversHelper {
   // be used in insert observers
   void analyze(Module& module, const std::string& method_name);
 
+  void removeActivationObservers();
+
   /**
    * Recursively insert observers for the method, also we'll process
    * the nodes in the graph in the order of execution of these nodes
@@ -308,7 +312,7 @@ class InsertObserversHelper {
    * observe/quantize a value a not, we don't want to observe a value multiple
    * times.
    *
-   * arguemnt: is_entry_point means whether the current method is the forward
+   * argument: is_entry_point means whether the current method is the forward
    * method of the top level module.
    *
    * Since we want to insert observers in the call site instead of in the called
@@ -334,6 +338,13 @@ class InsertObserversHelper {
       std::unordered_set<Value*> graph_observed_values =
           std::unordered_set<Value*>());
 
+  void setInsertResetObserverMethod(
+      bool insert_reset_observer_method,
+      const std::string& method_name) {
+    insert_reset_observer_method_ = insert_reset_observer_method;
+    reset_observer_method_name_ = "reset_observers_" + method_name;
+  }
+
  private:
   std::tuple<OptionalModuleVector, OptionalModuleVector, std::vector<size_t>>
   insertObserversFor(
@@ -381,11 +392,15 @@ class InsertObserversHelper {
       const Module& observer_module,
       NameModuleVector& observer_name_and_modules);
 
+  void insertObserverResetMinMax(
+      Module& module,
+      const NameModuleVector& observer_name_and_modules);
+
   // Uses the state created by fillBoundaryValueMap and fillValueObserverMap
   // to return an observer configured for a value, if it is needed.
-  c10::optional<Module> getObserverFor(Value* v);
+  std::optional<Module> getObserverFor(Value* v);
 
-  // Uses the state created by fillPassThroughValueMap to propage observed
+  // Uses the state created by fillPassThroughValueMap to propagage observed
   // property which should pass through from inputs to outputs.
   void propagateObservedProperty(
       Value* output,
@@ -433,6 +448,10 @@ class InsertObserversHelper {
   // property to it
   void fillPassThroughValueMap(const std::shared_ptr<Graph>& graph);
 
+  bool insertResetObserverMethod() {
+    return insert_reset_observer_method_;
+  }
+
   const ModuleQConfigMap& module_qconfig_map_;
 
   // Values we want to delay observation, used to delay the observation for
@@ -549,42 +568,6 @@ graph(%input, %linear, %relu):
     return (%second_output) )",
       {is_linear_module});
 
-  // F.linear + nn.ReLU
-  const PatternInfo f_linear_nn_relu = PatternInfo::parse_from_str(
-      R"(
-graph(%input, %weight, %bias, %linear, %relu):
-    %first_output = prim::CallFunction(%linear, %input, %weight, %bias)
-    %second_output = prim::CallMethod[name="forward\\d*"](%relu, %first_output)
-    return (%second_output) )",
-      {is_functional_linear, is_relu_module});
-
-  // F.linear + F.relu
-  const PatternInfo f_linear_f_relu = PatternInfo::parse_from_str(
-      R"(
-graph(%input, %weight, %bias, %linear, %relu, %inplace):
-    %first_output = prim::CallFunction(%linear, %input, %weight, %bias)
-    %second_output = prim::CallFunction(%relu, %first_output, %inplace)
-    return (%second_output) )",
-      {is_functional_linear, is_functional_relu});
-
-  // F.linear + aten::relu
-  const PatternInfo f_linear_aten_relu = PatternInfo::parse_from_str(
-      R"(
-graph(%input, %weight, %bias, %linear, %relu):
-    %first_output = prim::CallFunction(%linear, %input, %weight, %bias)
-    %second_output = aten::relu(%first_output)
-    return (%second_output) )",
-      {is_functional_linear});
-
-  // F.linear + aten::relu_
-  const PatternInfo f_linear_aten_relu_ = PatternInfo::parse_from_str(
-      R"(
-graph(%input, %weight, %bias, %linear, %relu):
-    %first_output = prim::CallFunction(%linear, %input, %weight, %bias)
-    %second_output = aten::relu_(%first_output)
-    return (%second_output) )",
-      {is_functional_linear});
-
   // aten::linear + nn.ReLU
   const PatternInfo aten_linear_nn_relu = PatternInfo::parse_from_str(
       R"(
@@ -895,8 +878,6 @@ graph(%self, %a, %b):
       {
           nn_linear_f_relu,      nn_linear_nn_relu,
           nn_linear_aten_relu,   nn_linear_aten_relu_,
-          f_linear_f_relu,       f_linear_nn_relu,
-          f_linear_aten_relu,    f_linear_aten_relu_,
           aten_linear_f_relu,    aten_linear_nn_relu,
           aten_linear_aten_relu, aten_linear_aten_relu_,
 
@@ -922,6 +903,9 @@ graph(%self, %a, %b):
           mul_aten_relu,         mul_aten_relu_,
           inplace_mul_aten_relu, inplace_mul_aten_relu_,
   };
+
+  bool insert_reset_observer_method_{false};
+  std::string reset_observer_method_name_;
 };
 
 ModuleMethodVector InsertObserversHelper::getInvokedMethods(
@@ -944,7 +928,7 @@ ModuleMethodVector InsertObserversHelper::getInvokedMethods(
       if (n->kind() == prim::CallMethod) {
         auto m_opt = getInvokedModuleOpt(module, n, graph->inputs()[0]);
         if (m_opt.has_value()) {
-          invoked_methods.push_back(std::make_pair(*m_opt, n->s(attr::name)));
+          invoked_methods.emplace_back(*m_opt, n->s(attr::name));
         }
       }
 
@@ -966,12 +950,12 @@ void InsertObserversHelper::insertObserverFor(
   }
   GRAPH_DEBUG("Inserting observer for:", v->debugName());
   Module observer = observer_module.deepcopy();
-  std::string observer_name = "_observer_" + c10::to_string(uid_++);
+  std::string observer_name = "_observer_" + std::to_string(uid_++);
   while (module.hasattr(observer_name)) {
-    observer_name = "_observer_" + c10::to_string(uid_++);
+    observer_name = "_observer_" + std::to_string(uid_++);
   }
   module.register_module(observer_name, observer);
-  observer_name_and_modules.push_back(std::make_pair(observer_name, observer));
+  observer_name_and_modules.emplace_back(observer_name, observer);
 
   auto* g = v->owningGraph();
   // Get handle of observer module
@@ -1002,6 +986,55 @@ void InsertObserversHelper::insertObserverFor(
   }
 }
 
+void InsertObserversHelper::insertObserverResetMinMax(
+    Module& module,
+    const NameModuleVector& observer_name_and_modules) {
+  if (observer_name_and_modules.empty()) {
+    return;
+  }
+  auto reset_min_max_opt = module.find_method(reset_observer_method_name_);
+  if (!reset_min_max_opt.has_value()) {
+    std::shared_ptr<Graph> reset_observer_graph = std::make_shared<Graph>();
+    Value* module_value = reset_observer_graph->addInput("self");
+    Node* output_node = reset_observer_graph->createNone();
+    reset_observer_graph->insertNode(output_node);
+    reset_observer_graph->registerOutput(output_node->output());
+    module_value->setType(module._ivalue()->type());
+    const auto method_name = c10::QualifiedName(
+        *(module.type()->name()), reset_observer_method_name_);
+    auto reset_observer_fn =
+        module._ivalue()->compilation_unit()->create_function(
+            method_name, std::move(reset_observer_graph));
+    auto self_arg = c10::Argument("self", module.type());
+    auto output_arg = c10::Argument("none", output_node->output()->type());
+    auto schema = c10::FunctionSchema(
+        reset_observer_method_name_,
+        "",
+        {std::move(self_arg)},
+        {std::move(output_arg)});
+    reset_observer_fn->setSchema(std::move(schema));
+    module.type()->addMethod(reset_observer_fn);
+  }
+  auto reset_min_max_graph =
+      module.get_method(reset_observer_method_name_).graph();
+  Value* self = reset_min_max_graph->inputs()[0];
+
+  for (const auto& pair : observer_name_and_modules) {
+    const auto& observer_name = pair.first;
+    const auto& observer = pair.second;
+    Value* observer_value =
+        reset_min_max_graph->insertGetAttr(self, observer_name);
+    MatchedSchema reset_minmax_schema = matchSchema(
+        observer.get_method("reset_min_max_vals").function().getSchema(),
+        observer_value->node()->sourceRange(),
+        *reset_min_max_graph,
+        {observer_value},
+        {});
+    reset_min_max_graph->insertMethodCall(
+        "reset_min_max_vals", reset_minmax_schema);
+  }
+}
+
 void InsertObserversHelper::delayObservingValuesInPattern(
     Graph& graph,
     const PatternInfo& pattern) {
@@ -1084,7 +1117,7 @@ void InsertObserversHelper::fillBoundaryValueMap(
         // offset of input for the caller node, since the first
         // input of CallFunction is the function node and the graph
         // for CallFunction start with actual input
-        size_t input_offset;
+        size_t input_offset = 0;
         if (n->kind() == prim::CallMethod) {
           auto m_opt = getInvokedModuleOpt(module, n, self);
           if (!m_opt.has_value()) {
@@ -1183,8 +1216,8 @@ bool InsertObserversHelper::valueNeedsToBeQuantized(
     Value* v,
     const QConfig& qconfig) {
   if (isBiasOfConvOrLinear(v) ||
-      !(v->type()->isSubtypeOf(TensorType::get()) ||
-        v->type()->isSubtypeOf(ListType::ofTensors())) ||
+      !(v->type()->isSubtypeOf(*TensorType::get()) ||
+        v->type()->isSubtypeOf(*ListType::ofTensors())) ||
       isEmbeddingBagNonInput(v)) {
     return false;
   }
@@ -1216,6 +1249,20 @@ bool InsertObserversHelper::valueNeedsToBeQuantized(
   return false;
 }
 
+void InsertObserversHelper::removeActivationObservers() {
+  std::vector<std::unordered_map<Value*, Module>::iterator>
+      values_to_be_removed;
+  for (auto it = observer_for_value_.begin(); it != observer_for_value_.end();
+       it++) {
+    if (!isWeight(it->first)) {
+      values_to_be_removed.push_back(it);
+    }
+  }
+  for (auto it : values_to_be_removed) {
+    observer_for_value_.erase(it);
+  }
+}
+
 void InsertObserversHelper::fillValueObserverMap(
     Module& module,
     const std::string& method_name) {
@@ -1261,13 +1308,13 @@ void InsertObserversHelper::fillValueObserverMap(
   }
 }
 
-c10::optional<Module> InsertObserversHelper::getObserverFor(Value* v) {
+std::optional<Module> InsertObserversHelper::getObserverFor(Value* v) {
   if (observer_for_value_.count(v)) {
     auto observer = observer_for_value_.at(v);
     GRAPH_DEBUG("Got observer module config for:", v->debugName());
     return observer;
   }
-  c10::optional<Module> result;
+  std::optional<Module> result;
   if (boundary_value_map_.count(v)) {
     for (Value* next : boundary_value_map_.at(v)) {
       GRAPH_DEBUG(
@@ -1333,9 +1380,9 @@ InsertObserversHelper::insertObserversFor(
   // the graph itself can be shared
   std::unordered_set<Value*> inputs_outputs;
   // list of observer modules for input values
-  std::vector<c10::optional<Module>> block_input_observers;
+  std::vector<std::optional<Module>> block_input_observers;
   // list of observer modules for output values
-  std::vector<c10::optional<Module>> block_output_observers;
+  std::vector<std::optional<Module>> block_output_observers;
 
   // if the current block is the block for entry point graph(the forward graph
   // of the top level module), we can insert observers in the block directly
@@ -1357,13 +1404,13 @@ InsertObserversHelper::insertObserversFor(
     }
 
     for (auto* v : block->outputs()) {
-      // we need explictly skip the values that are already observed
+      // we need explicitly skip the values that are already observed
       // this might happen in subblocks for `if` since
       // these subblock has access to all values before the `if` node
       if (!isObserved(v, block_observed_values)) {
         block_output_observers.emplace_back(getObserverFor(v));
       } else {
-        block_output_observers.emplace_back(c10::nullopt);
+        block_output_observers.emplace_back(std::nullopt);
       }
     }
   }
@@ -1418,7 +1465,7 @@ InsertObserversHelper::insertObserversFor(
       if (n->kind() == prim::CallMethod || userDefinedCallFunction(n)) {
         script::Module m;
         std::shared_ptr<Graph> g;
-        size_t input_offset;
+        size_t input_offset = 0;
         bool is_udf_for_subblock = is_user_defined_function;
         if (n->kind() == prim::CallMethod) {
           auto m_opt = getInvokedModuleOpt(module, n, self);
@@ -1509,7 +1556,7 @@ InsertObserversHelper::insertObserversFor(
             subblock_output_observe_state.push_back(
                 isObserved(output, block_observed_values));
           }
-          if (aggregated_output_observe_state.size() > 0) {
+          if (!aggregated_output_observe_state.empty()) {
             TORCH_CHECK(
                 aggregated_output_observe_state ==
                     subblock_output_observe_state,
@@ -1565,6 +1612,9 @@ InsertObserversHelper::insertObserversFor(
           "supported right now");
       insertObserverFor(v, module, observer, observer_name_and_modules);
     }
+    if (insertResetObserverMethod()) {
+      insertObserverResetMinMax(module, observer_name_and_modules);
+    }
     block_observer_map_[block] = observer_name_and_modules;
   }
   return std::make_tuple(
@@ -1603,6 +1653,7 @@ Module InsertObservers(
   fillQConfigMap(input_module, qconfig_dict, map_before_clone);
   ModuleCloneHelper mh;
   Module module = mh.clone(input_module, map_before_clone, inplace);
+  SwapFunctionalLinear(module);
   ModuleQConfigMap module_qconfig_map;
   // Since the types are changed after clone, we need to fill
   // the qconfig map again
@@ -1618,5 +1669,52 @@ Module InsertObservers(
   helper.insertObservers(module, method_name, /* is_entry_point */ true);
   return module;
 }
-} // namespace jit
-} // namespace torch
+
+Module InsertObserversForOnDevicePTQ(
+    Module& input_module,
+    const std::string& method_name,
+    const QConfigDict& qconfig_dict,
+    bool inplace,
+    QuantType quant_type) {
+  ModuleQConfigMap map_before_clone;
+  fillQConfigMap(input_module, qconfig_dict, map_before_clone);
+  ModuleCloneHelper mh;
+  Module cloned_module = mh.clone(input_module, map_before_clone, inplace);
+  std::shared_ptr<Graph> g = cloned_module.get_method(method_name).graph();
+  SwapFunctionalLinear(g);
+  std::string observer_method_name = "observe_" + method_name;
+  cloneMethod(cloned_module, method_name, observer_method_name);
+  ModuleQConfigMap module_qconfig_map;
+  // Since the types are changed after clone, we need to fill
+  // the qconfig map again
+  fillQConfigMap(cloned_module, qconfig_dict, module_qconfig_map);
+  GRAPH_DEBUG("Quant type:", quant_type);
+  InsertObserversHelper helper(module_qconfig_map, quant_type);
+  // Removes list mutation part is not clear. Is it needed
+  helper.preprocess(cloned_module, observer_method_name);
+  // Since we expect the graph to be inlined this should not have any use
+  // However, this function does handle if blocks
+  // Although as far as I understood If blocks are not really handled
+  // in JIT quantization. Should we just protect against this. That is if we
+  // find observable value inside If block? Also side effect of inlining is that
+  // you will have multiple getattrs for the same attribute and thus potentially
+  // multiple observers observing the same value. This will also lead to
+  // increased size of the packed param struct. I dont expect this to be a
+  // common pattern but something to be aware of Note that current quant
+  // workflow does not prevent this anyway since during inset quant dequant
+  // things are inlined anyway
+  helper.fillBoundaryValueMap(cloned_module, observer_method_name);
+  // analyze needs to run after fillBoundaryValueMap
+  // since we need to know the boundary value mapping to trace
+  // through the calls
+  helper.analyze(cloned_module, observer_method_name);
+  // Remove activation observer if quant_type is dynamic
+  if (quant_type == QuantType::DYNAMIC) {
+    helper.removeActivationObservers();
+  }
+  helper.setInsertResetObserverMethod(true, method_name);
+  helper.insertObservers(
+      cloned_module, observer_method_name, /* is_entry_point */ true);
+  return cloned_module;
+}
+} // namespace torch::jit
