@@ -1,7 +1,7 @@
 #include <torch/csrc/jit/passes/remove_inplace_ops.h>
+#include <iostream>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 namespace {
 static const std::unordered_map<NodeKind, NodeKind> inPlaceToOutOfPlace = {
     {aten::add_, aten::add},
@@ -47,7 +47,7 @@ void RemoveInplaceOps(Block* block) {
       // create a replacement out of place op
       auto newNode = graph->create(inPlaceToOutOfPlace.at(node->kind()));
       newNode->insertBefore(node);
-      newNode->setScope(node->scope());
+      newNode->copyMetadata(node);
       // copy inputs
       for (auto input : node->inputs()) {
         newNode->addInput(input);
@@ -87,6 +87,8 @@ void RemoveInplaceOps(Block* block) {
 // Before:
 // graph(%0 : Float),
 //        %1 : Half):
+//   # Should result in a Half, but after translation to out-of-place,
+//   # would become a Float b/c Half+Float -> Float.
 //   %4 : Float = onnx::Cast[to=1](%1)
 //   %5 : Float = onnx::Add(%4, %0)
 //   ...
@@ -106,22 +108,32 @@ void ImplicitCastForBinaryInplaceOps(Block* b) {
     // Check type if inplace operation is a binary node
     if ((it->kind() == aten::add_) || (it->kind() == aten::sub_) ||
         (it->kind() == aten::mul_) || (it->kind() == aten::div_)) {
-      auto orignalInputs = it->inputs();
-      if (orignalInputs.at(0) == orignalInputs.at(1)) {
+      auto originalInputs = it->inputs();
+      if (originalInputs.at(0) == originalInputs.at(1)) {
         continue;
       }
+
+      auto shape_node = originalInputs.at(0)->node();
+      if ((shape_node->kind() == prim::NumToTensor) &&
+          (shape_node->inputs().at(0)->node()->kind() == aten::size)) {
+        std::cerr
+            << "In-place op on output of tensor.shape. See https://pytorch.org/docs/main/onnx.html#"
+            << "avoid-inplace-operations-when-using-tensor-shape-in-tracing-mode"
+            << '\n';
+      }
+
       TensorTypePtr firstInp_tensor =
-          orignalInputs.at(0)->type()->cast<TensorType>();
+          originalInputs.at(0)->type()->cast<TensorType>();
       TensorTypePtr secondInp_tensor =
-          orignalInputs.at(1)->type()->cast<TensorType>();
-      if (!(firstInp_tensor) || !(secondInp_tensor) ||
+          originalInputs.at(1)->type()->cast<TensorType>();
+      if (!firstInp_tensor || !secondInp_tensor ||
           !(firstInp_tensor->scalarType().has_value())) {
         continue;
       }
       auto newInputNode = it->owningGraph()->create(aten::type_as, 1);
       newInputNode->insertBefore(*it);
-      newInputNode->addInput(orignalInputs.at(1));
-      newInputNode->addInput(orignalInputs.at(0));
+      newInputNode->addInput(originalInputs.at(1));
+      newInputNode->addInput(originalInputs.at(0));
       it->replaceInput(1, newInputNode->outputs().at(0));
     }
   }
@@ -131,5 +143,4 @@ void RemoveInplaceOps(const std::shared_ptr<Graph>& graph) {
   ImplicitCastForBinaryInplaceOps(graph->block());
   RemoveInplaceOps(graph->block());
 }
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
