@@ -1,12 +1,9 @@
-#include <torch/csrc/jit/passes/dead_code_elimination.h>
+#include <torch/csrc/jit/passes/remove_redundant_profiles.h>
 
 #include <torch/csrc/jit/ir/alias_analysis.h>
-#include <torch/csrc/jit/ir/ir_views.h>
 #include <torch/csrc/jit/jit_log.h>
-#include <torch/csrc/utils/memory.h>
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 
 void RemoveRedundantProfiles(Block* block, AliasDb& db) {
   for (auto it = block->nodes().end()->reverseIterator();
@@ -18,8 +15,8 @@ void RemoveRedundantProfiles(Block* block, AliasDb& db) {
       RemoveRedundantProfiles(b, db);
     }
 
-    // we only check prim::profile and not prim::profile_optional bc profile
-    // is inserted on each use, while profile_optional is inserted on the def
+    // we only check prim::profile and not prim::profile_ivalue bc profile
+    // is inserted on each use, while profile_ivalue is inserted on the def
     if (n->kind() != prim::profile ||
         n->input()->node()->kind() != prim::profile) {
       continue;
@@ -44,5 +41,4 @@ void RemoveRedundantProfiles(std::shared_ptr<Graph>& graph) {
   RemoveRedundantProfiles(graph->block(), db);
 }
 
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
