@@ -1,10 +1,10 @@
+#include <c10/util/irange.h>
 #include <torch/csrc/jit/runtime/instruction.h>
 #include <cstring>
 #include <iostream>
 
-namespace torch {
-namespace jit {
-std::ostream& operator<<(std::ostream& out, OpCode op) {
+namespace torch::jit {
+static std::ostream& operator<<(std::ostream& out, OpCode op) {
   switch (op) {
 #define OP_STRING(x, _) \
   case x:               \
@@ -26,11 +26,12 @@ char const* toString(OpCode op) {
   return nullptr;
 }
 
-const char* OpInfo(OpCode op) {
+static const char* OpInfo(OpCode op) {
   switch (op) {
 #define OP_INFO(x, info) \
   case x:                \
     return info;
+    // NOLINTNEXTLINE(bugprone-branch-clone)
     FORALL_OPCODES(OP_INFO)
 #undef OP_INFO
   }
@@ -43,18 +44,19 @@ static_assert(
     "Instructions should be 8 bytes");
 std::ostream& operator<<(std::ostream& out, Instruction inst) {
   // TODO: use op info to print out the op in a more user-friendly way
-  int nargs = std::strlen(OpInfo(inst.op));
+  auto nargs = std::strlen(OpInfo(inst.op));
   out << inst.op;
   if (nargs > 0) {
-    out << " " << inst.X;
+    out << ' ' << inst.X;
   }
   if (nargs > 1) {
-    out << " " << inst.N;
+    out << ' ' << inst.N;
   }
   return out;
 }
 
-static constexpr char* strOpCode[] = {
+// NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,modernize-avoid-c-arrays)
+static constexpr const char* strOpCode[] = {
 #define STR_OP(x, _) #x,
     FORALL_OPCODES(STR_OP)
 #undef STR_OP
@@ -62,7 +64,7 @@ static constexpr char* strOpCode[] = {
 
 OpCode parseOpCode(const char* str) {
   const int n = sizeof(strOpCode) / sizeof(strOpCode[0]);
-  for (int i = 0; i < n; ++i) {
+  for (const auto i : c10::irange(n)) {
     if (strcmp(strOpCode[i], str) == 0)
       return (OpCode)i;
   }
@@ -71,12 +73,16 @@ OpCode parseOpCode(const char* str) {
 
 bool isOpSupportedInMobile(OpCode op) {
   // clang-format off
+  // NOLINTNEXTLINE(cppcoreguidelines-avoid-c-arrays,modernize-avoid-c-arrays)
   static constexpr OpCode supported_ops_in_mobile[] {
       OP, OPN, LOAD, MOVE, STOREN, STORE, DROP, DROPR, LOADC, JF, JMP, LOOP,
       RET, GET_ATTR, SET_ATTR, LIST_CONSTRUCT, TUPLE_CONSTRUCT, WARN,
       INTERFACE_CALL, LIST_UNPACK, TUPLE_SLICE, DICT_CONSTRUCT,
-      NAMED_TUPLE_CONSTRUCT, CREATE_OBJECT
-  };
+      NAMED_TUPLE_CONSTRUCT, CREATE_OBJECT, ISINSTANCE, CALL,
+      RAISE_EXCEPTION, UNCHECKED_CAST, __IS__, UN_INITIALIZED,
+      __ISNOT__, FORMAT, DEVICE, DICT_INDEX,
+      DTYPE, TUPLE_INDEX, DIM, __NOT__,
+      TO_LIST, NUM_TO_TENSOR, IS_CUDA};
   // clang-format on
 
   for (auto sop : supported_ops_in_mobile) {
@@ -86,5 +92,4 @@ bool isOpSupportedInMobile(OpCode op) {
   return false;
 }
 
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
