@@ -1,24 +1,23 @@
 #include <torch/csrc/jit/runtime/profiling_graph_executor_impl.h>
 
+#include <c10/util/irange.h>
 #include <torch/csrc/jit/jit_log.h>
-#include <torch/csrc/jit/passes/bailout_graph.h>
+#include <torch/csrc/jit/passes/add_if_then_else.h>
 #include <torch/csrc/jit/passes/batch_mm.h>
 #include <torch/csrc/jit/passes/canonicalize_graph_fuser_ops.h>
+#include <torch/csrc/jit/passes/check_strict_fusion.h>
 #include <torch/csrc/jit/passes/clear_profiling.h>
 #include <torch/csrc/jit/passes/clear_undefinedness.h>
 #include <torch/csrc/jit/passes/common_subexpression_elimination.h>
 #include <torch/csrc/jit/passes/constant_pooling.h>
 #include <torch/csrc/jit/passes/constant_propagation.h>
 #include <torch/csrc/jit/passes/create_autodiff_subgraphs.h>
-#include <torch/csrc/jit/passes/cuda_graph_fuser.h>
 #include <torch/csrc/jit/passes/dead_code_elimination.h>
 #include <torch/csrc/jit/passes/decompose_ops.h>
 #include <torch/csrc/jit/passes/graph_fuser.h>
-#include <torch/csrc/jit/passes/guard_elimination.h>
 #include <torch/csrc/jit/passes/inline_autodiff_subgraphs.h>
 #include <torch/csrc/jit/passes/inliner.h>
 #include <torch/csrc/jit/passes/inplace_check.h>
-#include <torch/csrc/jit/passes/insert_guards.h>
 #include <torch/csrc/jit/passes/loop_unrolling.h>
 #include <torch/csrc/jit/passes/lower_grad_of.h>
 #include <torch/csrc/jit/passes/lower_tuples.h>
@@ -32,16 +31,40 @@
 #include <torch/csrc/jit/passes/tensorexpr_fuser.h>
 #include <torch/csrc/jit/passes/update_differentiable_graph_requires_grad.h>
 #include <torch/csrc/jit/passes/utils/subgraph_utils.h>
+#include <chrono>
+#include <mutex>
+#include <optional>
 
+// clang-format off
 C10_DEFINE_bool(
     torch_jit_enable_new_executor,
     true,
-    "If this flag is set to false TorchScript will be using the legacy/original executor");
+    "If this flag is set to false TorchScript will be using the legacy/original executor")
 
 C10_DEFINE_bool(
     torch_jit_disable_warning_prints,
     false,
-    "Disables warning.warn prints in TorchScript graph");
+    "Disables warning.warn prints in TorchScript graph")
+
+C10_DEFINE_bool(
+    torch_jit_static_then_dynamic,
+    false,
+    "fuse on two static compilations then 10 dynamic")
+
+C10_DEFINE_bool(
+    torch_jit_always_dynamic,
+    false,
+    "fuse on 12 dynamic compilations")
+
+C10_DEFINE_bool(
+    torch_jit_release_profiling_graph_after_optimization,
+    false,
+    "After getOptimizedPlanFor release the optimization record for reduction of memory in inference. This is aggressive memory saving, and please be cautious!")
+
+C10_DEFINE_int32(
+    torch_jit_release_profiling_graph_delay_in_seconds,
+    60,
+    "How long to wait before releasing the profiling graph after optimizaiton is done. Only used if torch_jit_release_profiling_graph_after_optimization is set to true.")
 
 constexpr size_t kDefaultNumProfiledRuns = 1;
 constexpr size_t kDefaultBailoutDepth = 20;
@@ -49,14 +72,22 @@ constexpr size_t kDefaultBailoutDepth = 20;
 C10_DEFINE_int64(
     torch_jit_num_profiled_runs,
     kDefaultNumProfiledRuns,
-    "Number of profiling runs");
+    "Number of profiling runs")
 C10_DEFINE_int64(
     torch_jit_bailout_depth,
     kDefaultBailoutDepth,
-    "Number of re-specializations");
+    "Number of re-specializations")
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
+
+namespace {
+int32_t getNowInSecs() {
+  auto currentTimePoint = std::chrono::system_clock::now();
+  auto durationSinceEpoch = std::chrono::duration_cast<std::chrono::seconds>(
+      currentTimePoint.time_since_epoch());
+  return static_cast<int32_t>(durationSinceEpoch.count());
+}
+} // namespace
 
 #if defined(C10_MOBILE)
 static std::atomic<bool> executor_mode{true};
@@ -66,8 +97,46 @@ static std::atomic<bool> executor_mode{true};
 static std::atomic<bool> profiling_mode{true};
 #endif
 
+static std::mutex fusion_strategy_lock;
+
+static FusionStrategy getInitialStrategy() {
+  if (FLAGS_torch_jit_always_dynamic) {
+    return {{FusionBehavior::DYNAMIC, 12}};
+  }
+  FusionStrategy mixed = {
+      {FusionBehavior::STATIC, 2}, {FusionBehavior::DYNAMIC, 10}};
+  if (FLAGS_torch_jit_static_then_dynamic) {
+    return mixed;
+  }
+// TODO remove ifdef
+#ifdef FBCODE_CAFFE2
+  return {{FusionBehavior::STATIC, 20}};
+#endif
+  return mixed;
+}
+
+// defer initial value so that we can load in gflags
+static std::optional<FusionStrategy> fusion_strategy = std::nullopt;
+
+FusionStrategy getFusionStrategy() {
+  std::lock_guard<std::mutex> guard(fusion_strategy_lock);
+  if (fusion_strategy == std::nullopt) {
+    fusion_strategy = getInitialStrategy();
+  }
+  return *fusion_strategy;
+}
+
+FusionStrategy setFusionStrategy(FusionStrategy& strategy) {
+  std::lock_guard<std::mutex> guard(fusion_strategy_lock);
+  if (fusion_strategy == std::nullopt) {
+    fusion_strategy = getInitialStrategy();
+  }
+  FusionStrategy old_strategy = *fusion_strategy;
+  fusion_strategy = strategy;
+  return old_strategy;
+}
+
 static std::atomic<size_t> num_profiled_runs{kDefaultNumProfiledRuns};
-static std::atomic<size_t> bailout_depth{kDefaultBailoutDepth};
 
 std::atomic<bool>& getProfilingMode() {
   return profiling_mode;
@@ -86,13 +155,13 @@ std::atomic<size_t>& getNumProfiledRuns() {
   return num_profiled_runs;
 }
 
-std::atomic<size_t>& getBailoutDepth() {
+size_t getBailoutDepth() {
   // Initialize bailout_depth from command-line flag.
-  static const size_t init = []() {
-    return bailout_depth = FLAGS_torch_jit_bailout_depth;
-  }();
-  (void)init; // Silence clang-tidy.
-  return bailout_depth;
+  size_t depth = 0;
+  for (const auto& pair : getFusionStrategy()) {
+    depth += pair.second;
+  }
+  return depth;
 }
 
 static bool needsGradientInProfilingMode(Block* b) {
@@ -119,10 +188,83 @@ static bool needsGradientInProfilingMode(Block* b) {
   return false;
 }
 
-bool guardDifferentiableGraph(Node* dnode) {
+// `prim::RequiresGradCheck` guarantees that requires_grad properties
+// of input tensors will match the profiled, otherwise a fallback path
+// will be triggered. This allow us to prune off gradients in backward
+// graph for inputs that don't need gradients. We transfer requires_grad
+// properties from inputs to the `prim::DifferentiableGraph` onto inputs to the
+// differentiable graph. Autodiff will inspect these properties and prune
+// off gradients that aren't required
+// `requires_grad` properties from `dnode->outputs()` will also be transferred
+[[maybe_unused]] static void setRequiresGradOnDiffGraph(Node* dnode) {
+  auto gi = dnode->g(attr::Subgraph)->inputs();
+  for (size_t i = 0; i < dnode->inputs().size(); i++) {
+    if (auto ty = dnode->input(i)->type()->cast<TensorType>()) {
+      auto gi_ty = gi[i]->type()->expect<TensorType>();
+      gi[i]->setType(gi_ty->withRequiresGrad(ty->requires_grad()));
+      GRAPH_DEBUG(
+          "Setting ",
+          *gi_ty->withRequiresGrad(ty->requires_grad()),
+          " on ",
+          gi[i],
+          " ",
+          gi[i]->debugName());
+    }
+  }
+
+  // We also need to put requires_grad on outputs within subgraph, so autodiff
+  // can  set df_input_vjps and DifferentiableGraphOp can set `requires_grad=`
+  // properly
+  auto go = dnode->g(attr::Subgraph)->outputs();
+  auto set_requires_grad = [](const TensorTypePtr& t, Value* val) -> bool {
+    if (t && t->requiresGrad().has_value()) {
+      GRAPH_DEBUG("setting type ", *t);
+      val->setType(t);
+      return true;
+    }
+    return false;
+  };
+
+  for (const auto i : c10::irange(go.size())) {
+    auto ty = go[i]->type()->cast<TensorType>();
+    if (ty) {
+      auto n = go[i]->node();
+      auto dno = dnode->outputs().at(i);
+      for (auto dno_use : dno->uses()) {
+        GRAPH_DEBUG("found user of ", i, " as ", *dno_use.user);
+        if (n->kind() == prim::profile) {
+          if (set_requires_grad(
+                  n->ty(attr::profiled_type)->expect<TensorType>(), go[i])) {
+            break;
+          }
+        } else if (dno_use.user->kind() == prim::profile) {
+          if (set_requires_grad(
+                  dno_use.user->ty(attr::profiled_type)->expect<TensorType>(),
+                  go[i])) {
+            break;
+          }
+        } else if (dno_use.user->kind() == prim::DifferentiableGraph) {
+          Value* o =
+              dno_use.user->g(attr::Subgraph)->inputs().at(dno_use.offset);
+          // Is it safe to not check other uses, because we are inside a
+          // DifferentiableGraph?
+          auto nn = o->uses().at(0).user;
+          if (nn->kind() == prim::profile) {
+            if (set_requires_grad(
+                    nn->ty(attr::profiled_type)->expect<TensorType>(), go[i])) {
+              break;
+            }
+          }
+        }
+      }
+    }
+  }
+}
+
+static bool guardDifferentiableGraph(Node* dnode) {
   auto gi = dnode->g(attr::Subgraph)->inputs();
   bool all_inputs_seen = true;
-  for (size_t i = 0; i < gi.size(); i++) {
+  for (const auto i : c10::irange(gi.size())) {
     auto ty = gi[i]->type()->cast<TensorType>();
     if (ty) {
       auto n = gi[i]->uses().at(0).user;
@@ -147,6 +289,13 @@ bool guardDifferentiableGraph(Node* dnode) {
         }
       }
 
+      // Propagate the requires_grad property to inputs
+      // A RequiresGrad check gets added (insertTypeGuard, below)
+      // so requires_grad is guaranteed to match for the inputs;
+      // but other properties are not guaranteed to match
+      auto requires_grad = dni->type()->expectRef<TensorType>().requiresGrad();
+      gi[i]->setType(ty->withRequiresGrad(requires_grad));
+
       // we check if the optional is defined
       all_inputs_seen &= (dni->type()->cast<TensorType>() != TensorType::get());
     }
@@ -169,15 +318,16 @@ bool guardDifferentiableGraph(Node* dnode) {
     // we inline the differentiable graph as a fallback
     // ideally we would set this up for re-profiling
     UpdateDifferentiableGraphRequiresGrad(
-        dnode->g(attr::Subgraph), c10::nullopt);
+        dnode->g(attr::Subgraph), std::nullopt);
     SubgraphUtils::unmergeSubgraph(dnode);
     return false;
   }
 }
 
 void runNooptPassPipeline(std::shared_ptr<Graph>& graph) {
-  GRAPH_DEBUG(
-      "Before LowerGradOf (beginning of runNooptPassPipeline)\n", *graph);
+  GRAPH_DEBUG("Before Inliner (beginning of runNooptPassPipeline)\n", *graph);
+  Inline(*graph);
+  GRAPH_DEBUG("After Inline, Before NoGrad\n", *graph);
   LowerGradOf(*graph);
   GRAPH_DEBUG("After LowerGradOf, before RemoveExpands\n", *graph);
   RemoveExpands(graph);
@@ -189,27 +339,13 @@ void runNooptPassPipeline(std::shared_ptr<Graph>& graph) {
       "After EliminateDeadCode (end of runNooptPassPipeline)\n", *graph);
 }
 
-void runPreAutodiffPassPipeline(std::shared_ptr<Graph>& graph) {
+static void runPreAutodiffPassPipeline(std::shared_ptr<Graph>& graph) {
   GRAPH_DEBUG(
       "Before InsertGuards (beginning of runPreAutodiffPassPipeline)\n",
       *graph);
 
-  if (tensorExprFuserEnabled() || RegisterCudaFuseGraph::isRegistered()) {
-    // With TE fuser or nvfuser, we don't generate bailouts
-    LowerGradOf(*graph);
-    GRAPH_DEBUG("After LowerGradOf, before specializeAutogradZero\n", *graph);
-  } else {
-    InsertGuards(graph);
-    GRAPH_DEBUG("After InsertGuards, before LowerGradOf\n", *graph);
-    LowerGradOf(*graph);
-    GRAPH_DEBUG("After LowerGradOf, before EliminateRedundantGuards\n", *graph);
-    EliminateRedundantGuards(graph);
-    GRAPH_DEBUG(
-        "After EliminateRedundantGuards, before InsertBailOuts\n", *graph);
-    InsertBailOuts(graph);
-    GRAPH_DEBUG(
-        "After InsertBailOuts, before specializeAutogradZero\n", *graph);
-  }
+  LowerGradOf(*graph);
+  GRAPH_DEBUG("After LowerGradOf, before specializeAutogradZero\n", *graph);
 
   specializeAutogradZero(graph);
   GRAPH_DEBUG("After specializeAutogradZero\n", *graph);
@@ -259,112 +395,29 @@ void runPreAutodiffPassPipeline(std::shared_ptr<Graph>& graph) {
     EliminateCommonSubexpression(graph);
     GRAPH_DEBUG(
         "After EliminateCommonSubexpression, before CheckInplace\n", *graph);
-
     CheckInplace(graph);
   }
   GRAPH_DEBUG(
       "After CheckInplace (end of runPreAutodiffPassPipeline)\n", *graph);
 }
 
-void runDiffGraphPasses(std::shared_ptr<Graph>& graph) {
-  GRAPH_DEBUG(
-      "Before EliminateDeadCode (beginning of runDiffGraphPasses)\n", *graph);
-  // runOptimization:
-  {
-    // Basic graph preprocessing to eliminate noise.
-    EliminateDeadCode(graph);
-    GRAPH_DEBUG(
-        "After EliminateDeadCode, before EliminateCommonSubexpression\n",
-        *graph);
-    EliminateCommonSubexpression(graph);
-    GRAPH_DEBUG(
-        "After EliminateCommonSubexpression, before PeepholeOptimize\n",
-        *graph);
-
-    PeepholeOptimize(graph);
-    GRAPH_DEBUG("After PeepholeOptimize, before ConstantPropagation\n", *graph);
-    ConstantPropagation(graph);
-    GRAPH_DEBUG("After ConstantPropagation, before ConstantPooling\n", *graph);
-    ConstantPooling(graph);
-    GRAPH_DEBUG("After ConstantPooling, before UnrollLoops\n", *graph);
-
-    UnrollLoops(graph);
-    GRAPH_DEBUG("After UnrollLoops, before RemoveListMutation\n", *graph);
-    // run again with unrolled loops
-    RemoveListMutation(graph);
-    GRAPH_DEBUG("After RemoveListMutation, before PeepholeOptimize\n", *graph);
-    PeepholeOptimize(graph);
-    GRAPH_DEBUG("After PeepholeOptimize, before ConstantPropagation\n", *graph);
-    ConstantPropagation(graph);
-    GRAPH_DEBUG(
-        "After ConstantPropagation, before EliminateCommonSubexpression\n",
-        *graph);
-
-    EliminateCommonSubexpression(graph);
-    GRAPH_DEBUG(
-        "After EliminateCommonSubexpression, before CheckInplace\n", *graph);
-
-    CheckInplace(graph);
-  }
-  GRAPH_DEBUG("After CheckInplace, before customPrePasses\n", *graph);
-
-  // runNondiffOptimization
-  {
-    // Run custom passes that different backends can register.
-    for (const auto& passPair : getCustomPrePasses()) {
-      passPair.first(graph);
-    }
-    GRAPH_DEBUG("After customPrePasses, before LowerSimpleTuples\n", *graph);
-
-    // TupleConstruct / TupleUnpack pairs can still be present at this point
-    // and must be removed for fusion.
-    LowerSimpleTuples(graph);
-    GRAPH_DEBUG("After LowerSimpleTuples\n", *graph);
-
-    if (tensorExprFuserEnabled()) {
-      // Remove prim::profile nodes and embed the profile info directly in the
-      // IR in value types. We're doing such transformation as optimizations
-      // that try to merge/fuse nodes in the graph (e.g. BatchMM and GraphFuser)
-      // work worse in the presence of intermittent prim::profile nodes.
-      // Optimizations relying on the type info are also responsible for
-      // inserting proper type checks. Once we're done with these optimizations
-      // we will wipe the tensor type information from the IR, so that it's not
-      // accidentally used by any other pass.
-      RemoveProfileNodesAndSpecializeTypes(graph);
-      GRAPH_DEBUG(
-          "After RemoveProfileNodesAndSpecializeTypes, before BatchMM\n",
-          *graph);
-      // Rewrite subgraphs with many MMs into expressions that batch them.
-      BatchMM(graph);
-      GRAPH_DEBUG("After BatchMM, before Fusion\n", *graph);
-
-      FuseTensorExprs(graph, getFusionGroupInlining() ? 2 : 1);
-      GRAPH_DEBUG(
-          "After Fusion, before RemoveTensorTypeSpecializations\n", *graph);
-
-      // Wipe tensor type info from the IR
-      RemoveTensorTypeSpecializations(graph);
-      GRAPH_DEBUG(
-          "After RemoveTensorTypeSpecializations, before customPostPasses\n",
-          *graph);
-    } else {
-      // Rewrite subgraphs with many MMs into expressions that batch them.
-      BatchMM(graph);
-      GRAPH_DEBUG("After BatchMM, before Fusion\n", *graph);
-
-      FuseGraph(graph, true);
-      GRAPH_DEBUG("After Fusion, before customPostPasses\n", *graph);
-    }
-
-    // Run custom post-fusion passes
-    for (const auto& passPair : getCustomPostPasses()) {
-      passPair.first(graph);
+FusionBehavior ProfilingGraphExecutorImpl::getCurrentBehavior(
+    size_t remaining_depth) {
+  size_t curr_depth = 0;
+  for (int i = static_cast<int>(fusion_strategy_.size()) - 1; i >= 0; i--) {
+    curr_depth += fusion_strategy_[i].second;
+    if (remaining_depth <= curr_depth) {
+      return fusion_strategy_[i].first;
     }
   }
-  GRAPH_DEBUG("After customPostPasses (end of runDiffGraphPasses)\n", *graph);
+  // should never get here
+  TORCH_WARN("Strategy changed mid-invocation, NYI");
+  return FusionBehavior::STATIC;
 }
 
-void runNoGradOptimizations(std::shared_ptr<Graph>& graph) {
+void ProfilingGraphExecutorImpl::runNoGradOptimizations(
+    std::shared_ptr<Graph>& graph,
+    size_t remaining_bailout_depth) {
   GRAPH_DEBUG(
       "After customPostPasses (beginning of runNoGradOptimizations)\n", *graph);
   // runNondiffOptimization
@@ -396,16 +449,11 @@ void runNoGradOptimizations(std::shared_ptr<Graph>& graph) {
       // Rewrite subgraphs with many MMs into expressions that batch them.
       BatchMM(graph);
       GRAPH_DEBUG("After BatchMM, before Fusion\n", *graph);
-
-      FuseTensorExprs(graph, getFusionGroupInlining() ? 2 : 1);
-      GRAPH_DEBUG(
-          "After Fusion, before RemoveTensorTypeSpecializations\n", *graph);
-
-      // Wipe tensor type info from the IR
-      RemoveTensorTypeSpecializations(graph);
-      GRAPH_DEBUG(
-          "After RemoveTensorTypeSpecializations, before customPostPasses\n",
-          *graph);
+      auto min_size = getFusionGroupInlining() ? 2 : 1;
+      bool dyn_shapes = getCurrentBehavior(remaining_bailout_depth) ==
+          FusionBehavior::DYNAMIC;
+      FuseTensorExprs(graph, min_size, /* composed op*/ false, dyn_shapes);
+      GRAPH_DEBUG("After Fusion, before customPostPasses\n", *graph);
     } else {
       // Rewrite subgraphs with many MMs into expressions that batch them.
       BatchMM(graph);
@@ -419,13 +467,18 @@ void runNoGradOptimizations(std::shared_ptr<Graph>& graph) {
     for (const auto& passPair : getCustomPostPasses()) {
       passPair.first(graph);
     }
+    GRAPH_DEBUG(
+        "After customPostPasses, before RemoveTensorTypeSpecializations \n",
+        *graph);
+    RemoveTensorTypeSpecializations(graph);
+    GRAPH_DEBUG("After RemoveTensorTypeSpecializations\n", *graph);
   }
-  GRAPH_DEBUG(
-      "After customPostPasses (end of runNoGradOptimizations)\n", *graph);
+  GRAPH_DEBUG("End of runNoGradOptimizations\n");
 }
 
 void ProfilingGraphExecutorImpl::runProfilingOptimizations(
-    std::shared_ptr<Graph>& copy) {
+    std::shared_ptr<Graph>& copy,
+    size_t remaining_bailout_depth) {
   GRAPH_DEBUG("Before runProfilingOptimizations:\n", *copy);
   if (!getGraphExecutorOptimize()) {
     runNooptPassPipeline(copy);
@@ -452,13 +505,14 @@ void ProfilingGraphExecutorImpl::runProfilingOptimizations(
       GRAPH_DEBUG("After guardDifferentiableGraph:\n", *copy);
       auto diff_graph = std::move(dnode->g(attr::Subgraph));
       Gradient gradient = differentiate(diff_graph);
+      RemoveTensorTypeSpecializations(gradient.f);
+      ProfilingRecord::removeProfilingNodes(gradient.f->block());
       GRAPH_DEBUG("Forward graph:\n", *(gradient.f));
       GRAPH_DEBUG("Backward graph:\n", *(gradient.df));
       // just like inside autograd.Functions, the forward of a differentiable
       // graph is essentially in a torch.no_grad context.
       UpdateDifferentiableGraphRequiresGrad(gradient.f, false);
       GRAPH_DEBUG("After UpdateDifferentiableGraphRequiresGrad ", *gradient.f);
-      runDiffGraphPasses(gradient.f);
       // replaces fallback graphs inserted by TE Fuser
       replaceFallbackGraphWithFallbackFunction(gradient.f->block());
       packGradient(gradient, dnode);
@@ -466,13 +520,13 @@ void ProfilingGraphExecutorImpl::runProfilingOptimizations(
     }
     InlineAutodiffSubgraphs(
         copy,
-        getAutodiffSubgraphInlining() ? autodiffSubgraphInlineThreshold : 1);
+        getAutodiffSubgraphInlining() ? autodiffSubgraphNodeThreshold : 1);
     replaceFallbackGraphWithFallbackFunction(copy->block());
-    RemoveProfilingNodes(copy);
+    ProfilingRecord::removeProfilingNodes(copy->block());
     GRAPH_DEBUG(
         "After InlineAutodiffSubgraphs and Removing Profiling Nodes\n", *copy);
   } else {
-    runNoGradOptimizations(copy);
+    runNoGradOptimizations(copy, remaining_bailout_depth);
   }
   EliminateDeadCode(copy);
   GRAPH_DEBUG("After runProfilingOptimizations:\n", *copy);
@@ -542,15 +596,27 @@ void ProfilingGraphExecutorImpl::runProfilingInsensitiveOptimizations(
 ProfilingGraphExecutorImpl::ProfilingGraphExecutorImpl(
     const std::shared_ptr<Graph>& graph,
     std::string function_name)
-    : GraphExecutorImplBase(graph, std::move(function_name)) {}
+    : GraphExecutorImplBase(graph, std::move(function_name)) {
+  fusion_strategy_ = getFusionStrategy();
+}
+
+size_t ProfilingGraphExecutorImpl::getInstantiatedBailoutDepth() {
+  // Initialize bailout_depth from command-line flag.
+  size_t depth = 0;
+  for (const auto& pair : fusion_strategy_) {
+    depth += pair.second;
+  }
+  return depth;
+}
 
 const ExecutionPlan& ProfilingGraphExecutorImpl::getOptimizedPlanFor(
     Stack& stack,
-    size_t remaining_bailout_depth) {
+    std::optional<size_t> remaining_bailout_depth) {
   GRAPH_DEBUG("Running ProfilingGraphExecutorImpl ", this);
 
+  // TODO: instantiate simple executor when getProfilingMode() is false
   // no opt mode
-  if (!getGraphExecutorOptimize()) {
+  if (!getGraphExecutorOptimize() || !getProfilingMode()) {
     if (!fallback_plan_) {
       auto copy = graph->copy();
       GRAPH_DEBUG(
@@ -570,7 +636,11 @@ const ExecutionPlan& ProfilingGraphExecutorImpl::getOptimizedPlanFor(
   // getPlanFor(remaining_bailout_depth) is corrected and persisted by the Code
   // object in interpreter.
   if (!remaining_bailout_depth_.has_value() || !tensorExprFuserEnabled()) {
-    remaining_bailout_depth_ = remaining_bailout_depth;
+    if (remaining_bailout_depth.has_value()) {
+      remaining_bailout_depth_ = *remaining_bailout_depth;
+    } else {
+      remaining_bailout_depth_ = getInstantiatedBailoutDepth();
+    }
   }
 
   // simple executor
@@ -579,14 +649,17 @@ const ExecutionPlan& ProfilingGraphExecutorImpl::getOptimizedPlanFor(
     runProfilingInsensitiveOptimizations(copy);
     GRAPH_DUMP("Optimized SimpleExecutor Graph: ", copy);
     optimized_plan_ = ExecutionPlan(copy, function_name_);
+    time_optimized_plan_created_ = getNowInSecs();
     return *optimized_plan_;
   }
 
+  bool profiling_record_created_in_this_call = false;
   // if a profiling graph hasn't been created yet
   if (!pr_) {
     auto copy = graph->copy();
     runProfilingInsensitiveOptimizations(copy);
     pr_ = ProfilingRecord::instrumentGraph(copy);
+    profiling_record_created_in_this_call = true;
     // `InsertProfileNodesForSpecializeAutogradZero` profiles a definition vs a
     // use and it doesn't expect any profile nodes between a graph input and its
     // consumer, `aten::_grad_sum_to_size`. This means we need to run it first,
@@ -605,27 +678,43 @@ const ExecutionPlan& ProfilingGraphExecutorImpl::getOptimizedPlanFor(
 
   auto copy = pr_->graph()->copy();
   ProfilingRecord::removeProfileCounter(copy->block());
-  runProfilingOptimizations(copy);
+  runProfilingOptimizations(copy, *remaining_bailout_depth_);
   // replaces a fallback graph inserted by
   // specialize_autogradzero if one exists
   replaceFallbackGraphWithFallbackFunction(copy->block());
+  runFinalOptimizations(copy);
+  CheckStrictFusion(copy);
   GRAPH_DUMP("Optimized Graph: ", copy);
-  optimized_plan_ =
-      ExecutionPlan(copy, function_name_, *remaining_bailout_depth_);
+  optimized_plan_ = ExecutionPlan(copy, function_name_);
+  time_optimized_plan_created_ = getNowInSecs();
+  // If the profiled graph was created in this call, then we can release it
+  // right.
+  if (FLAGS_torch_jit_release_profiling_graph_after_optimization &&
+      profiling_record_created_in_this_call) {
+    clearTheGraphCompilationIntermediateGraphs();
+  }
   return *optimized_plan_;
 }
 
 const ExecutionPlan& ProfilingGraphExecutorImpl::getPlanFor(
     Stack& stack,
-    size_t remaining_bailout_depth) {
+    std::optional<size_t> remaining_bailout_depth) {
   std::lock_guard<std::mutex> lock(compile_mutex);
 
   // IMPORTANT: This is a hot path of calling a torchscript function. Try not to
   // add any code above this.
   if (optimized_plan_) {
+    if (FLAGS_torch_jit_release_profiling_graph_after_optimization &&
+        !is_graph_extra_memory_released_) {
+      int32_t now = getNowInSecs();
+      if ((now - time_optimized_plan_created_) >
+          FLAGS_torch_jit_release_profiling_graph_delay_in_seconds) {
+        clearTheGraphCompilationIntermediateGraphs();
+      }
+    }
     return *optimized_plan_;
   }
-
+  // if depth is not set, use
   return getOptimizedPlanFor(stack, remaining_bailout_depth);
 }
 
@@ -637,9 +726,9 @@ GraphExecutorState ProfilingGraphExecutorImpl::getDebugState() {
   return state;
 }
 
-Node* insertFallbackFunctionCall(
+static Node* insertFallbackFunctionCall(
     Graph* graph,
-    Function* func,
+    GraphFunction* func,
     ArrayRef<Value*> inputs) {
   auto tuple_type = func->graph()->return_node()->input(0)->type();
   Value* fn_constant = graph->insertNode(graph->create(prim::Constant))
@@ -658,7 +747,7 @@ Node* insertFallbackFunctionCall(
   return fun_unpack_tuple;
 }
 
-Function* createFallbackPathFunction(
+static GraphFunction* createFallbackPathFunction(
     Block* b,
     const std::string& function_name) {
   auto value_map = [](Value* v) { return v; };
@@ -695,7 +784,7 @@ void ProfilingGraphExecutorImpl::replaceFallbackGraphWithFallbackFunction(
       WithInsertPoint wip{*it};
       auto function_call = insertFallbackFunctionCall(
           b->owningGraph(), fallback_func, it->inputs());
-      for (size_t i = 0; i < function_call->outputs().size(); i++) {
+      for (const auto i : c10::irange(function_call->outputs().size())) {
         it->output(i)->replaceAllUsesWith(function_call->output(i));
       }
       it.destroyCurrent();
@@ -708,5 +797,32 @@ void ProfilingGraphExecutorImpl::replaceFallbackGraphWithFallbackFunction(
   }
 }
 
-} // namespace jit
-} // namespace torch
+void ProfilingGraphExecutorImpl::runFinalOptimizations(
+    std::shared_ptr<Graph>& graph) {
+  AddIfThenElseOp(graph);
+}
+
+void ProfilingGraphExecutorImpl::debugFlushCompilationCache() {
+  std::lock_guard<std::mutex> lock(compile_mutex);
+  pr_.reset();
+  fallback_plan_.reset();
+  profiling_plan_.reset();
+  optimized_plan_.reset();
+  // prevent memory leaks
+  fallback_functions_.clear();
+  remaining_bailout_depth_.reset();
+  // TODO - would be nice to have it initialized in subsequent use
+  fusion_strategy_ = getFusionStrategy();
+  time_optimized_plan_created_ = 0;
+  is_graph_extra_memory_released_ = false;
+}
+
+void ProfilingGraphExecutorImpl::clearTheGraphCompilationIntermediateGraphs() {
+  is_graph_extra_memory_released_ = true;
+  profiling_plan_.reset();
+  fallback_plan_.reset();
+  graph.reset();
+  pr_.reset();
+}
+
+} // namespace torch::jit
