@@ -4,7 +4,7 @@
 #include <ATen/core/ivalue.h>
 #include <ATen/core/jit_type.h>
 #include <ATen/core/stack.h>
-#include <torch/csrc/WindowsTorchApiMacro.h>
+#include <torch/csrc/Export.h>
 #include <torch/csrc/jit/ir/ir.h>
 
 #include <list>
@@ -76,13 +76,13 @@
 //         locations that have `ShapeSymbol S1` in run #1`getSymbolInSet` will
 //         return the same symbol.
 
-namespace torch {
-namespace jit {
+namespace torch::jit {
 
 using ::c10::TensorTypePtr;
 using Dimension = int64_t;
 
-TORCH_API void RegisterProfilingNode(const std::function<bool(const Node*)>&);
+TORCH_API void RegisterProfilingNode(
+    const std::function<bool(const Node*)>& /*func*/);
 
 struct ProfilingRecord;
 
@@ -182,38 +182,25 @@ struct ProfilingRecord {
   TORCH_API static void removeProfileCounter(Block* b);
 
   std::shared_ptr<Graph> profiled_graph_;
-  std::mutex mutex_;
+  mutable std::mutex mutex_;
   size_t profiling_count_;
-  // the key is a frame id
-  // the value is a mapping from a Value in a graph
-  // to a profiled TensorType
-  std::map<int64_t, std::map<Value*, TensorTypePtr>> profiled_types_per_frame_;
 
-  // A thin wrapper around `partitionSetByDimension` to ensure
-  // `new_sizes` and `sym_shapes` are of the same rank
+  bool ready() const;
 
-  c10::SymbolicShape mergeSymbolicShapes(
-      const c10::SymbolicShape& new_sizes,
-      const c10::SymbolicShape& sym_shapes,
-      SetPartitioningHelper& partition_helper);
-
-  bool ready() const {
-    return profiling_count_ == 0;
-  }
   std::shared_ptr<Graph> graph() const {
     return profiled_graph_;
   }
 
   TORCH_API ProfileIValueOp* createProfileIValueNode(Value* in_val);
+  TORCH_API ProfileIValueOp* createProfileIValueNode(ArrayRef<Value*> inputs);
 
  private:
   ProfileOp* createProfileNode(
       const std::function<void(Stack&)>& fp,
       at::ArrayRef<Value*> inputs);
   void instrumentBlock(Block* block);
-  void insertShapeProfile(Node* n, size_t offset);
+  void insertShapeProfile(Node* n, size_t offset, const TypePtr& input_type);
   ProfilingRecord(std::shared_ptr<Graph> g);
 };
 
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit
