@@ -1,10 +1,12 @@
 #include <torch/csrc/jit/tensorexpr/mem_dependency_checker.h>
-#include <fstream>
 
-namespace torch {
-namespace jit {
-namespace tensorexpr {
-namespace analysis {
+#include <c10/util/irange.h>
+
+#include <fstream>
+#include <iostream>
+#include <utility>
+
+namespace torch::jit::tensorexpr::analysis {
 
 const char* AccessToString(AccessType a) {
   switch (a) {
@@ -30,7 +32,7 @@ const char* AccessToString(AccessType a) {
   return "Unknown";
 }
 
-void getDependencyChain(
+static void getDependencyChain(
     const std::shared_ptr<AccessInfo>& info,
     DependencySet& dependencies) {
   if (!dependencies.insert(info).second) {
@@ -42,7 +44,7 @@ void getDependencyChain(
   }
 }
 
-void getDependentsChain(
+static void getDependentsChain(
     const std::shared_ptr<AccessInfo>& info,
     DependencySet& dependents) {
   if (!dependents.insert(info).second) {
@@ -56,19 +58,15 @@ void getDependentsChain(
 
 // AccessInfo
 
-std::vector<const Expr*> AccessInfo::getIndices() const {
-  std::vector<const Expr*> indices;
+std::vector<ExprPtr> AccessInfo::getIndices() const {
+  std::vector<ExprPtr> indices;
 
   if (expr_) {
-    if (auto* load = dynamic_cast<const Load*>(expr_)) {
+    if (auto load = to<Load>(expr_)) {
       indices = load->indices();
-    } else if (auto* call = dynamic_cast<const FunctionCall*>(expr_)) {
-      indices = call->params();
-    } else if (auto* reduce = dynamic_cast<const ReduceOp*>(expr_)) {
-      indices = reduce->output_args();
     }
   } else {
-    if (auto* store = dynamic_cast<const Store*>(stmt_)) {
+    if (auto store = to<Store>(stmt_)) {
       indices = store->indices();
     }
   }
@@ -77,12 +75,16 @@ std::vector<const Expr*> AccessInfo::getIndices() const {
 
 void AccessInfo::addDependency(const std::shared_ptr<AccessInfo>& write) {
   auto res = dependencies_.emplace(write->id(), write);
-  TORCH_INTERNAL_ASSERT(res.second);
+  TORCH_INTERNAL_ASSERT(
+      res.second,
+      buildErrorMessage("Duplicate entry in mem dep checker in the fuser."));
 }
 
 void AccessInfo::addDependent(const std::shared_ptr<AccessInfo>& read) {
   auto res = dependents_.emplace(read->id(), read);
-  TORCH_INTERNAL_ASSERT(res.second);
+  TORCH_INTERNAL_ASSERT(
+      res.second,
+      buildErrorMessage("Duplicate entry in mem dep checker in the fuser."));
 }
 
 bool AccessInfo::hasDependency(const std::shared_ptr<AccessInfo>& info) const {
@@ -108,7 +110,7 @@ DependencySet AccessInfo::getIndirectDependencies() {
 DependencySet AccessInfo::getDirectDependents() {
   DependencySet res;
   for (auto& depPair : dependents_) {
-    res.insert(depPair.second);
+    res.insert(depPair.second.lock());
   }
   return res;
 }
@@ -149,8 +151,8 @@ bool AccessInfo::isWrite() const {
 }
 
 void AccessInfo::print() const {
-  std::cout << id_ << ". " << AccessToString(type_) << ": " << *var_ << "[";
-  if (bounds_.size() > 0) {
+  std::cout << id_ << ". " << AccessToString(type_) << ": " << *var_ << '[';
+  if (!bounds_.empty()) {
     for (size_t i = 0; i < bounds_.size() - 1; ++i) {
       bounds_[i].print();
       std::cout << ", ";
@@ -159,38 +161,40 @@ void AccessInfo::print() const {
     size_t i = bounds_.size() - 1;
     bounds_[i].print();
   }
-  std::cout << "]";
+  std::cout << ']';
 
   if (!dependencies_.empty()) {
     std::cout << " - depends on: ";
     for (auto& pair : dependencies_) {
-      std::cout << pair.second->id() << " ";
+      std::cout << pair.second->id() << ' ';
     }
   }
 
   if (!dependents_.empty()) {
     std::cout << " - dependents: ";
     for (auto& pair : dependents_) {
-      std::cout << pair.second->id() << " ";
+      std::cout << pair.second.lock()->id() << ' ';
     }
   }
 
-  std::cout << "\n";
+  std::cout << '\n';
 }
 
 void AccessInfo::dumpDOT(std::ostream& os) const {
   if (type_ == AccessType::Input || type_ == AccessType::Output ||
       type_ == AccessType::Alloc) {
-    os << "n" << id_ << " [\n";
-    os << "label = \"" << AccessToString(type_) << "\\n " << *var_ << "[";
-    if (bounds_.size() > 0) {
+    os << 'n' << id_ << " [\n";
+    os << "label = \"" << AccessToString(type_) << "\\n " << *var_ << '[';
+    if (!bounds_.empty()) {
       for (size_t i = 0; i < bounds_.size() - 1; ++i) {
-        os << *IRSimplifier::simplify(new Add(bounds_[i].end, new IntImm(1)))
+        os << *IRSimplifier::simplify(
+                  alloc<Add>(bounds_[i].end, immLike(bounds_[i].end, 1)))
            << ", ";
       }
 
       size_t i = bounds_.size() - 1;
-      os << *IRSimplifier::simplify(new Add(bounds_[i].end, new IntImm(1)));
+      os << *IRSimplifier::simplify(
+          alloc<Add>(bounds_[i].end, immLike(bounds_[i].end, 1)));
       os << "]\"\n ";
     }
     if (isWrite()) {
@@ -199,17 +203,17 @@ void AccessInfo::dumpDOT(std::ostream& os) const {
       os << "\tshape = \"house\"\n";
     }
   } else {
-    os << "n" << id_ << " [\n";
+    os << 'n' << id_ << " [\n";
     os << "label = \"" << AccessToString(type_) << " (#" << id_ << ")\\n";
     os << "buf : " << *var_ << "\\n";
-    os << "bounds : \[";
-    if (bounds_.size() > 0) {
+    os << "bounds : [";
+    if (!bounds_.empty()) {
       for (size_t i = 0; i < bounds_.size() - 1; ++i) {
-        os << "(" << *bounds_[i].start << ", " << *bounds_[i].end << "), ";
+        os << '(' << *bounds_[i].start << ", " << *bounds_[i].end << "), ";
       }
 
       size_t i = bounds_.size() - 1;
-      os << "(" << *bounds_[i].start << ", " << *bounds_[i].end << ")]";
+      os << '(' << *bounds_[i].start << ", " << *bounds_[i].end << ")]";
     }
     os << "\"\n";
     os << "\tshape = \"box\"\n";
@@ -224,8 +228,8 @@ void AccessInfo::dumpDOT(std::ostream& os) const {
   }
   os << "]\n";
   for (auto& pair : dependencies_) {
-    os << "n" << pair.second->id() << " -> "
-       << "n" << id_ << " [color=\"" << edgeColour << "\"]\n";
+    os << 'n' << pair.second->id() << " -> " << 'n' << id_ << " [color=\""
+       << edgeColour << "\"]\n";
   }
 }
 
@@ -256,12 +260,12 @@ MemDependencyChecker::MemDependencyChecker() {
 }
 
 MemDependencyChecker::MemDependencyChecker(
-    const std::unordered_set<const Buf*>& inputs,
-    const std::unordered_set<const Buf*>& outputs) {
-  for (auto* s : inputs) {
+    const std::unordered_set<BufPtr>& inputs,
+    const std::unordered_set<BufPtr>& outputs) {
+  for (const auto& s : inputs) {
     inputs_[s] = nullptr;
   }
-  for (auto* s : outputs) {
+  for (const auto& s : outputs) {
     outputs_[s] = nullptr;
   }
 
@@ -321,15 +325,15 @@ DependencySet MemDependencyChecker::getAllWriteDependencies(
   return writes;
 }
 
-bool MemDependencyChecker::dependsDirectly(const Expr* A, const Stmt* B) {
+bool MemDependencyChecker::dependsDirectly(const ExprPtr& A, const StmtPtr& B) {
   return dependsDirectlyHelper(A, B);
 }
 
-bool MemDependencyChecker::dependsDirectly(const Stmt* A, const Stmt* B) {
+bool MemDependencyChecker::dependsDirectly(const StmtPtr& A, const StmtPtr& B) {
   return dependsDirectlyHelper(A, B);
 }
 
-bool MemDependencyChecker::dependsDirectly(const Buf* O, const Stmt* B) {
+bool MemDependencyChecker::dependsDirectly(const BufPtr& O, const StmtPtr& B) {
   auto outputAccess = output(O);
   auto bWrites = getAllWritesWithin(B);
 
@@ -342,7 +346,7 @@ bool MemDependencyChecker::dependsDirectly(const Buf* O, const Stmt* B) {
   return false;
 }
 
-bool MemDependencyChecker::dependsDirectly(const Stmt* A, const Buf* I) {
+bool MemDependencyChecker::dependsDirectly(const StmtPtr& A, const BufPtr& I) {
   auto aReads = getAllReadsWithin(A);
   auto inputAccess = input(I);
 
@@ -355,7 +359,7 @@ bool MemDependencyChecker::dependsDirectly(const Stmt* A, const Buf* I) {
   return false;
 }
 
-bool MemDependencyChecker::dependsDirectly(const Expr* A, const Buf* I) {
+bool MemDependencyChecker::dependsDirectly(const ExprPtr& A, const BufPtr& I) {
   auto aReads = getAllReadsWithin(A);
   auto inputAccess = input(I);
 
@@ -374,15 +378,21 @@ bool MemDependencyChecker::dependsDirectly(
   return A->hasDependency(B) && B->isWrite();
 }
 
-bool MemDependencyChecker::dependsIndirectly(const Expr* A, const Stmt* B) {
+bool MemDependencyChecker::dependsIndirectly(
+    const ExprPtr& A,
+    const StmtPtr& B) {
   return dependsIndirectlyHelper(A, B);
 }
 
-bool MemDependencyChecker::dependsIndirectly(const Stmt* A, const Stmt* B) {
+bool MemDependencyChecker::dependsIndirectly(
+    const StmtPtr& A,
+    const StmtPtr& B) {
   return dependsIndirectlyHelper(A, B);
 }
 
-bool MemDependencyChecker::dependsIndirectly(const Buf* O, const Stmt* B) {
+bool MemDependencyChecker::dependsIndirectly(
+    const BufPtr& O,
+    const StmtPtr& B) {
   auto outputAccess = output(O);
 
   DependencySet dependencies;
@@ -398,7 +408,9 @@ bool MemDependencyChecker::dependsIndirectly(const Buf* O, const Stmt* B) {
   return false;
 }
 
-bool MemDependencyChecker::dependsIndirectly(const Stmt* A, const Buf* I) {
+bool MemDependencyChecker::dependsIndirectly(
+    const StmtPtr& A,
+    const BufPtr& I) {
   auto aReads = getAllReadsWithin(A);
   auto inputAccess = input(I);
 
@@ -407,7 +419,9 @@ bool MemDependencyChecker::dependsIndirectly(const Stmt* A, const Buf* I) {
   return aDeps.count(inputAccess) != 0;
 }
 
-bool MemDependencyChecker::dependsIndirectly(const Expr* A, const Buf* I) {
+bool MemDependencyChecker::dependsIndirectly(
+    const ExprPtr& A,
+    const BufPtr& I) {
   auto aReads = getAllReadsWithin(A);
   auto inputAccess = input(I);
 
@@ -416,7 +430,7 @@ bool MemDependencyChecker::dependsIndirectly(const Expr* A, const Buf* I) {
   return aDeps.count(inputAccess) != 0;
 }
 
-bool MemDependencyChecker::dependsIndirectly(const Buf* O, const Buf* I) {
+bool MemDependencyChecker::dependsIndirectly(const BufPtr& O, const BufPtr& I) {
   auto outputAccess = output(O);
   auto inputAccess = input(I);
 
@@ -440,7 +454,7 @@ bool MemDependencyChecker::dependsIndirectly(
 }
 
 std::shared_ptr<AccessInfo> MemDependencyChecker::accessFor(
-    const Stmt* A) const {
+    const StmtPtr& A) const {
   auto bound = stmtToAccess_.equal_range(A);
   for (auto it = bound.first; it != bound.second; ++it) {
     if (it->second->expr() == nullptr) {
@@ -451,7 +465,7 @@ std::shared_ptr<AccessInfo> MemDependencyChecker::accessFor(
 }
 
 std::shared_ptr<AccessInfo> MemDependencyChecker::accessFor(
-    const Expr* A) const {
+    const ExprPtr& A) const {
   // TODO exprs can have multiple accesses... we're returning the first but that
   // isn't great. Can't do much here.
   auto bound = exprToAccess_.equal_range(A);
@@ -463,7 +477,7 @@ std::shared_ptr<AccessInfo> MemDependencyChecker::accessFor(
 }
 
 std::unordered_set<std::shared_ptr<AccessInfo>> MemDependencyChecker::
-    accessesWithin(const Stmt* A) const {
+    accessesWithin(const StmtPtr& A) const {
   auto it = scopeToAccesses_.find(A);
   if (it != scopeToAccesses_.end()) {
     return std::unordered_set<std::shared_ptr<AccessInfo>>(
@@ -479,11 +493,11 @@ std::unordered_set<std::shared_ptr<AccessInfo>> MemDependencyChecker::
 }
 
 std::unordered_set<std::shared_ptr<AccessInfo>> MemDependencyChecker::
-    accessesWithin(const Expr* A) const {
+    accessesWithin(const ExprPtr& A) const {
   return {accessFor(A)};
 }
 
-std::shared_ptr<AccessInfo> MemDependencyChecker::input(const Buf* b) const {
+std::shared_ptr<AccessInfo> MemDependencyChecker::input(const BufPtr& b) const {
   auto it = inputs_.find(b);
   if (it == inputs_.end()) {
     return nullptr;
@@ -491,7 +505,8 @@ std::shared_ptr<AccessInfo> MemDependencyChecker::input(const Buf* b) const {
   return it->second;
 }
 
-std::shared_ptr<AccessInfo> MemDependencyChecker::output(const Buf* b) const {
+std::shared_ptr<AccessInfo> MemDependencyChecker::output(
+    const BufPtr& b) const {
   auto it = outputs_.find(b);
   if (it == outputs_.end()) {
     return nullptr;
@@ -501,18 +516,18 @@ std::shared_ptr<AccessInfo> MemDependencyChecker::output(const Buf* b) const {
 
 // Node visitors:
 
-void MemDependencyChecker::visit(const Store* v) {
-  const Stmt* last = lastStmt_;
+void MemDependencyChecker::visit(const StorePtr& v) {
+  StmtPtr last = lastStmt_;
   lastStmt_ = v;
   v->value()->accept(this);
 
-  for (const Expr* ind : v->indices()) {
+  for (const ExprPtr& ind : v->indices()) {
     ind->accept(this);
   }
   lastStmt_ = last;
 
   // Create a new AccessInfo for the store.
-  const Var* var = v->buf()->base_handle();
+  VarPtr var = v->buf()->base_handle();
   auto info = std::make_shared<AccessInfo>(
       nextAccess_++, AccessType::Store, v, var, getIndicesBounds(v->indices()));
 
@@ -533,19 +548,19 @@ void MemDependencyChecker::visit(const Store* v) {
   currentScope_->accesses_.push_back(info);
 }
 
-void MemDependencyChecker::visit(const Load* v) {
+void MemDependencyChecker::visit(const LoadPtr& v) {
   // Create a temporary scope to hold any loads that occur within the indices of
   // this load.
   auto indicesScope =
       std::make_shared<Scope>(currentScope_->block, currentScope_);
   currentScope_ = indicesScope;
 
-  for (const Expr* ind : v->indices()) {
+  for (const ExprPtr& ind : v->indices()) {
     ind->accept(this);
   }
 
   // Create a new AccessInfo for the load.
-  const Var* var = v->buf()->base_handle();
+  VarPtr var = v->buf()->base_handle();
   auto load = std::make_shared<AccessInfo>(
       nextAccess_++,
       AccessType::Load,
@@ -578,81 +593,42 @@ void MemDependencyChecker::visit(const Load* v) {
   currentScope_->accesses_.push_back(load);
 }
 
-void MemDependencyChecker::visit(const FunctionCall* v) {
-  // This is essentially the same as Load.
-  auto paramScope =
-      std::make_shared<Scope>(currentScope_->block, currentScope_);
-  currentScope_ = paramScope;
-
-  for (const Expr* param : v->params()) {
-    param->accept(this);
-  }
-
-  const Var* var = v->tensor()->buf()->base_handle();
-  auto call = std::make_shared<AccessInfo>(
-      nextAccess_++,
-      AccessType::Call,
-      v,
-      lastStmt_,
-      var,
-      getIndicesBounds(v->params()));
-
-  // If there were loads in the parameters, this call depends on them, also
-  // merge.
-  if (!paramScope->accesses_.empty()) {
-    for (auto& access : paramScope->accesses_) {
-      call->addDependency(access);
-      access->addDependent(call);
-    }
-    mergeScope(paramScope, paramScope->parent, false);
-  }
-
-  currentScope_ = paramScope->parent;
-
-  stmtToAccess_.emplace(lastStmt_, call);
-  exprToAccess_.emplace(v, call);
-
-  // Intentionally using operator[], we want it to be created if it does not
-  // exist.
-  auto& writeHistory = currentScope_->openWrites_[var];
-  updateWriteHistory(writeHistory, call, call->id());
-  currentScope_->accesses_.push_back(call);
-}
-
 // This check determines if two accesses within a loop are "safe" from loop-self
 // dependence. This function does not consider overlap in bound range, but
 // rather the stride of the bound relative to the loop variable. This is the
 // section of the code which considers iteration order, if allowed.
-bool executionSafetyCheck(
+static bool executionSafetyCheck(
     const std::shared_ptr<AccessInfo>& info,
     const std::shared_ptr<AccessInfo>& other,
-    const std::vector<const Expr*>& aStrides,
-    const std::vector<const Expr*>& oStrides,
+    const std::vector<ExprPtr>& aStrides,
+    const std::vector<ExprPtr>& oStrides,
     bool parallelized) {
   if (aStrides.empty() || oStrides.empty()) {
     return false;
   }
-  TORCH_INTERNAL_ASSERT(info->bounds().size() == other->bounds().size());
+  TORCH_INTERNAL_ASSERT(
+      info->bounds().size() == other->bounds().size(),
+      buildErrorMessage(
+          "Dimension mismatch for two accesses in mem dep checker in the fuser."));
   for (size_t b = 0; b < info->bounds().size(); ++b) {
-    const Expr* aIndexStride = aStrides[b];
-    const Expr* oIndexStride = oStrides[b];
+    ExprPtr aIndexStride = aStrides[b];
+    ExprPtr oIndexStride = oStrides[b];
     // can't be safe on this index if we can't determine stride.
     if (!aIndexStride->isConstant() || !oIndexStride->isConstant()) {
       continue;
     }
 
-    const Expr* minStride =
-        IRSimplifier::simplify(new Min(aIndexStride, oIndexStride, true));
-    const Expr* maxStride =
-        IRSimplifier::simplify(new Max(aIndexStride, oIndexStride, true));
+    ExprPtr minStride =
+        IRSimplifier::simplify(alloc<Min>(aIndexStride, oIndexStride, true));
+    ExprPtr maxStride =
+        IRSimplifier::simplify(alloc<Max>(aIndexStride, oIndexStride, true));
 
     // If the first access has no stride don't apply safety).
     if (immediateEquals(minStride, 0)) {
       continue;
     }
 
-    const Expr* modCheck =
-        IRSimplifier::simplify(new Mod(maxStride, minStride));
+    ExprPtr modCheck = IRSimplifier::simplify(alloc<Mod>(maxStride, minStride));
 
     // if the strides can't have easily inferable distinct offsets, they're not
     // safe.
@@ -666,33 +642,34 @@ bool executionSafetyCheck(
     // axis is the same sign as the common stride, then they will not
     // overlap.
 
-    const Expr* startDiff = IRSimplifier::simplify(
-        new Sub(info->bounds()[b].start, other->bounds()[b].start));
+    ExprPtr startDiff = IRSimplifier::simplify(
+        alloc<Sub>(info->bounds()[b].start, other->bounds()[b].start));
 
     bool diffNegative = immediateIsNegative(startDiff);
     bool strideNegative = immediateIsNegative(minStride);
 
     // Invert the startDiff so mod works.
     if (diffNegative != strideNegative) {
-      startDiff = IRSimplifier::simplify(new Sub(new IntImm(0), startDiff));
+      startDiff =
+          IRSimplifier::simplify(alloc<Sub>(immLike(startDiff, 0), startDiff));
     }
 
     // If both accesses have the same stride, and the difference in start
     // element is smaller than this stride then the entire range is distinct.
     if (exprEquals(minStride, maxStride)) {
-      const Expr* check1 =
-          IRSimplifier::simplify(new CompareSelect(startDiff, minStride, kLT));
+      ExprPtr check1 = IRSimplifier::simplify(
+          alloc<CompareSelect>(startDiff, minStride, kLT));
       if (check1->isConstant() && immediateEquals(check1, 1)) {
         return true;
       }
     }
 
-    startDiff = IRSimplifier::simplify(new Mod(startDiff, minStride));
+    startDiff = IRSimplifier::simplify(alloc<Mod>(startDiff, minStride));
 
     CompareSelectOperation op = strideNegative ? kLT : kGT;
 
-    const Expr* check =
-        IRSimplifier::simplify(new CompareSelect(startDiff, new IntImm(0), op));
+    ExprPtr check = IRSimplifier::simplify(
+        alloc<CompareSelect>(startDiff, immLike(startDiff, 0), op));
 
     // If the start difference modulo the minimum stride is offset from that
     // stride, then the ranges have distinct strides.
@@ -712,10 +689,10 @@ bool executionSafetyCheck(
   return false;
 }
 
-void MemDependencyChecker::visit(const For* v) {
-  const Var* var = v->var();
+void MemDependencyChecker::visit(const ForPtr& v) {
+  VarPtr var = v->var();
 
-  const Stmt* last = lastStmt_;
+  StmtPtr last = lastStmt_;
   lastStmt_ = v;
 
   v->var()->accept(this);
@@ -724,7 +701,7 @@ void MemDependencyChecker::visit(const For* v) {
   // They exist in the enclosing scope, but accesses within the loop body may
   // depend on them via usage of the loop variable.
   // The way we handle this is to create a new scope so we have an easily
-  // accessible list of the acceses within the extents.
+  // accessible list of the accesses within the extents.
   auto extentsScope =
       std::make_shared<Scope>(currentScope_->block, currentScope_);
   currentScope_ = extentsScope;
@@ -758,49 +735,49 @@ void MemDependencyChecker::visit(const For* v) {
   // access, which we do via substituting the loop var with (var+1) into the
   // indices expr.
 
-  std::vector<std::vector<const Expr*>> loopStrides;
+  std::vector<std::vector<ExprPtr>> loopStrides;
   loopStrides.resize(currentScope_->accesses_.size());
 
   for (size_t a = 0; a < currentScope_->accesses_.size(); ++a) {
     auto& info = currentScope_->accesses_[a];
 
-    std::vector<const Expr*> indices = info->getIndices();
+    std::vector<ExprPtr> indices = info->getIndices();
 
-    std::vector<const Expr*>& loopIndicesStride = loopStrides[a];
+    std::vector<ExprPtr>& loopIndicesStride = loopStrides[a];
     loopIndicesStride.resize(indices.size());
 
     // index expr must depend on the loop var in some way to have a stride.
-    for (size_t i = 0; i < indices.size(); i++) {
+    for (const auto i : c10::irange(indices.size())) {
       VarFinder vf;
       if (vf.find(indices[i]).count(var) == 0) {
-        loopIndicesStride[i] = new IntImm(0);
+        loopIndicesStride[i] = immLike(indices[i], 0);
       } else {
         // If we've previously swapped the start and end of this bound, we
         // should apply the substitution to the reverse of the bounds.
         if (info->bounds()[i].swapped) {
           info->bounds()[i].end = IRSimplifier::simplify(
-              Substitute(info->bounds()[i].end, {{var, v->start()}}));
-          info->bounds()[i].start = IRSimplifier::simplify(Substitute(
+              SubstituteInClone(info->bounds()[i].end, {{var, v->start()}}));
+          info->bounds()[i].start = IRSimplifier::simplify(SubstituteInClone(
               info->bounds()[i].start,
-              {{var, new Sub(v->stop(), new IntImm(1))}}));
+              {{var, alloc<Sub>(v->stop(), immLike(v->stop(), 1))}}));
 
         } else {
           info->bounds()[i].start = IRSimplifier::simplify(
-              Substitute(info->bounds()[i].start, {{var, v->start()}}));
-          info->bounds()[i].end = IRSimplifier::simplify(Substitute(
+              SubstituteInClone(info->bounds()[i].start, {{var, v->start()}}));
+          info->bounds()[i].end = IRSimplifier::simplify(SubstituteInClone(
               info->bounds()[i].end,
-              {{var, new Sub(v->stop(), new IntImm(1))}}));
+              {{var, alloc<Sub>(v->stop(), immLike(v->stop(), 1))}}));
         }
 
-        const Expr* zeroStep = indices[i];
-        const Expr* oneStep =
-            Substitute(indices[i], {{var, new Add(var, new IntImm(1))}});
+        ExprPtr zeroStep = indices[i];
+        ExprPtr oneStep = SubstituteInClone(
+            indices[i], {{var, alloc<Add>(var, immLike(var, 1))}});
         loopIndicesStride[i] =
-            IRSimplifier::simplify(new Sub(oneStep, zeroStep));
+            IRSimplifier::simplify(alloc<Sub>(oneStep, zeroStep));
 
         // If the start < end then swap the order of the bound.
-        const Expr* diff = IRSimplifier::simplify(
-            new Sub(info->bounds()[i].end, info->bounds()[i].start));
+        ExprPtr diff = IRSimplifier::simplify(
+            alloc<Sub>(info->bounds()[i].end, info->bounds()[i].start));
         if (diff->isConstant() && immediateIsNegative(diff)) {
           info->bounds()[i].swap();
         }
@@ -825,13 +802,13 @@ void MemDependencyChecker::visit(const For* v) {
       // does nothing.
       for (auto& bound : bounds) {
         bound.start = IRSimplifier::simplify(
-            Substitute(bound.start, {{var, v->start()}}));
-        bound.end = IRSimplifier::simplify(
-            Substitute(bound.end, {{var, new Sub(v->stop(), new IntImm(1))}}));
+            SubstituteInClone(bound.start, {{var, v->start()}}));
+        bound.end = IRSimplifier::simplify(SubstituteInClone(
+            bound.end, {{var, alloc<Sub>(v->stop(), immLike(v->stop(), 1))}}));
 
         // If the start < end then swap the order of the bound.
-        const Expr* diff =
-            IRSimplifier::simplify(new Sub(bound.end, bound.start));
+        ExprPtr diff =
+            IRSimplifier::simplify(alloc<Sub>(bound.end, bound.start));
         if (diff->isConstant() && immediateIsNegative(diff)) {
           bound.swap();
         }
@@ -844,7 +821,7 @@ void MemDependencyChecker::visit(const For* v) {
       v->loop_options().is_gpu_thread_index();
 
   // Store buffers allocated at this scope.
-  std::unordered_set<const Var*> local_intermediates;
+  std::unordered_set<VarPtr> local_intermediates;
 
   // Scanning from the top of the loop, we look for accesses which may depend
   // on a previous or parallel loop iteration.
@@ -901,7 +878,7 @@ void MemDependencyChecker::visit(const For* v) {
       std::vector<IndexBounds> newBoundSlices;
       for (auto& b : openBounds) {
         OverlapKind overlap = overlaps(b, other->bounds());
-        if (overlap == NoOverlap) {
+        if (overlap == OverlapKind::NoOverlap) {
           newBoundSlices.push_back(b);
           continue;
         }
@@ -910,7 +887,7 @@ void MemDependencyChecker::visit(const For* v) {
         info->addDependency(other);
         other->addDependent(info);
 
-        if (overlap == Contains) {
+        if (overlap == OverlapKind::Contains) {
           continue;
         }
 
@@ -947,8 +924,8 @@ void MemDependencyChecker::visit(const For* v) {
   currentScope_ = currentScope_->parent;
 }
 
-void MemDependencyChecker::visit(const Cond* v) {
-  const Stmt* last = lastStmt_;
+void MemDependencyChecker::visit(const CondPtr& v) {
+  StmtPtr last = lastStmt_;
   lastStmt_ = v;
 
   auto enclosingScope =
@@ -957,8 +934,8 @@ void MemDependencyChecker::visit(const Cond* v) {
   // condition is in enclosing scope.
   v->condition()->accept(this);
 
-  Block* true_stmt = v->true_stmt();
-  Block* false_stmt = v->false_stmt();
+  BlockPtr true_stmt = v->true_stmt();
+  BlockPtr false_stmt = v->false_stmt();
 
   // Create scopes so the Block visitor doesn't create and merge a new scope.
   auto trueScope = std::make_shared<Scope>(true_stmt, enclosingScope);
@@ -978,7 +955,7 @@ void MemDependencyChecker::visit(const Cond* v) {
   // present in both the true and false branches then we can close overlapping
   // accesses in the enclosing scope. Without that analysis future accesses
   // may be dependent on a write of a common range in all three of the
-  // enclosing, true and false scope. This is a false positve so not too bad
+  // enclosing, true and false scope. This is a false positive so not too bad
   // in the short term, I think.
 
   // Merge both true and false branches into the parent, but don't close any
@@ -996,12 +973,12 @@ void MemDependencyChecker::visit(const Cond* v) {
   lastStmt_ = last;
 }
 
-void MemDependencyChecker::visit(const IfThenElse* v) {
+void MemDependencyChecker::visit(const IfThenElsePtr& v) {
   // condition is in enclosing scope.
   v->condition()->accept(this);
 
-  const Expr* true_value = v->true_value();
-  const Expr* false_value = v->false_value();
+  ExprPtr true_value = v->true_value();
+  ExprPtr false_value = v->false_value();
 
   auto enclosingScope = currentScope_;
 
@@ -1032,13 +1009,13 @@ void MemDependencyChecker::visit(const IfThenElse* v) {
   currentScope_ = enclosingScope;
 }
 
-void MemDependencyChecker::visit(const CompareSelect* v) {
+void MemDependencyChecker::visit(const CompareSelectPtr& v) {
   // condition is in enclosing scope.
   v->lhs()->accept(this);
   v->rhs()->accept(this);
 
-  const Expr* true_value = v->ret_val1();
-  const Expr* false_value = v->ret_val2();
+  ExprPtr true_value = v->ret_val1();
+  ExprPtr false_value = v->ret_val2();
 
   auto enclosingScope = currentScope_;
 
@@ -1071,15 +1048,15 @@ void MemDependencyChecker::visit(const CompareSelect* v) {
 
 // Inserts accesses for a map of buffers (ie. for inputs and outputs).
 void MemDependencyChecker::insertBuffers(
-    std::unordered_map<const Buf*, std::shared_ptr<AccessInfo>>& bufs,
+    std::unordered_map<BufPtr, std::shared_ptr<AccessInfo>>& bufs,
     AccessType type) {
   for (auto& pair : bufs) {
-    const Buf* b = pair.first;
-    const Var* var = b->base_handle();
+    const BufPtr& b = pair.first;
+    VarPtr var = b->base_handle();
     IndexBounds bounds;
-    for (auto* d : b->dims()) {
-      bounds.push_back(
-          {new IntImm(0), IRSimplifier::simplify(new Sub(d, new IntImm(1)))});
+    for (const auto& d : b->dims()) {
+      bounds.emplace_back(
+          immLike(d, 0), IRSimplifier::simplify(alloc<Sub>(d, immLike(d, 1))));
     }
     auto info =
         std::make_shared<AccessInfo>(nextAccess_++, type, nullptr, var, bounds);
@@ -1092,7 +1069,7 @@ void MemDependencyChecker::insertBuffers(
   }
 }
 
-void MemDependencyChecker::visit(const Block* v) {
+void MemDependencyChecker::visit(const BlockPtr& v) {
   auto prev_scope = currentScope_;
 
   // handle kernel inputs.
@@ -1101,14 +1078,14 @@ void MemDependencyChecker::visit(const Block* v) {
   }
 
   if (currentScope_->block != v) {
-    currentScope_ = std::make_shared<Scope>((Block*)v, prev_scope);
+    currentScope_ = std::make_shared<Scope>((BlockPtr)v, prev_scope);
   }
 
-  for (auto* s : *v) {
+  for (const auto& s : *v) {
     s->accept(this);
   }
 
-  for (auto* v : currentScope_->localVars) {
+  for (const auto& v : currentScope_->localVars) {
     knownVarBounds_.erase(v);
   }
   for (auto& pair : currentScope_->shadowedVarBounds) {
@@ -1128,15 +1105,15 @@ void MemDependencyChecker::visit(const Block* v) {
   }
 }
 
-void MemDependencyChecker::visit(const Let* v) {
-  const Stmt* last = lastStmt_;
+void MemDependencyChecker::visit(const LetPtr& v) {
+  StmtPtr last = lastStmt_;
   lastStmt_ = v;
 
   IRVisitor::visit(v);
 
   lastStmt_ = last;
 
-  const Var* var = v->var();
+  VarPtr var = v->var();
   if (knownVarBounds_.count(var) != 0) {
     currentScope_->shadowedVarBounds[var] = knownVarBounds_[var];
   }
@@ -1147,22 +1124,30 @@ void MemDependencyChecker::visit(const Let* v) {
 
 // Don't support AtomicAdd yet, it's a bit more complex since it's both a read
 // and a write. It's only inserted during Cuda codegen so this should be okay.
-void MemDependencyChecker::visit(const AtomicAdd* v) {
+void MemDependencyChecker::visit(const AtomicAddPtr& v) {
   throw std::runtime_error("MemDependencyChecker AtomicAdd unimplemented");
 }
 
-void MemDependencyChecker::visit(const Allocate* v) {
-  const Stmt* last = lastStmt_;
+void MemDependencyChecker::visit(const AllocatePtr& v) {
+  StmtPtr last = lastStmt_;
   lastStmt_ = v;
 
   IRVisitor::visit(v);
 
-  const Var* var = v->buffer_var();
+  VarPtr var = v->buffer_var();
   IndexBounds bounds;
-  for (auto* d : v->dims()) {
-    bounds.push_back(
-        {new IntImm(0), IRSimplifier::simplify(new Sub(d, new IntImm(1)))});
-  }
+  // TODO: remove the "buf_flat_size" process below and extend the buf bound
+  // check to support N-d indices access and 1-d index access.
+  // "Allocate" stmt is based on "Buf" which supports N-d indices access and 1-d
+  // index access. Currently the write bound check in memory analysis cannot
+  // identify 1-d index access for N-d bufs. Thus we flatten N-d bufs here to
+  // avoid failing the bound check. But this is not the correct approach and
+  // should be fixed.
+  ExprPtr flat_size = buf_flat_size(v->buf());
+  flat_size =
+      IRSimplifier::simplify(alloc<Sub>(flat_size, immLike(flat_size, 1)));
+  bounds.emplace_back(immLike(flat_size, 0), flat_size);
+
   auto info = std::make_shared<AccessInfo>(
       nextAccess_++, AccessType::Alloc, nullptr, var, bounds);
 
@@ -1175,15 +1160,19 @@ void MemDependencyChecker::visit(const Allocate* v) {
   lastStmt_ = last;
 }
 
-void MemDependencyChecker::visit(const Free* v) {
-  const Stmt* last = lastStmt_;
+void MemDependencyChecker::visit(const FreePtr& v) {
+  StmtPtr last = lastStmt_;
   lastStmt_ = v;
 
   IRVisitor::visit(v);
 
-  const Var* var = v->buffer_var();
+  VarPtr var = v->buffer_var();
   auto it = intermediates_.find(var);
-  TORCH_INTERNAL_ASSERT(it != intermediates_.end());
+  TORCH_INTERNAL_ASSERT(
+      it != intermediates_.end(),
+      buildErrorMessage(
+          "Expected to find '" + var->name_hint() +
+          "' in intermediate vars in mem dep checker in the fuser."));
 
   IndexBounds bounds = it->second->bounds();
   auto info = std::make_shared<AccessInfo>(
@@ -1214,7 +1203,7 @@ void MemDependencyChecker::updateWriteHistory(
 
     OverlapKind overlap = overlaps(indexBounds, info->bounds());
 
-    if (overlap == NoOverlap) {
+    if (overlap == OverlapKind::NoOverlap) {
       ++it;
       continue;
     }
@@ -1233,12 +1222,12 @@ void MemDependencyChecker::updateWriteHistory(
       continue;
     }
 
-    if (overlap == ContainedOrEqual) {
+    if (overlap == OverlapKind::ContainedOrEqual) {
       // Total overlap is easy - the new access totally replaces the old.
       it = writeHistory.erase(it);
     } else {
       // The new write partially overlaps a previous write. We want to keep
-      // both, but only track the unconvered part of the earlier write.
+      // both, but only track the uncovered part of the earlier write.
 
       // Determine the slices of the earlier bound not covered by info.
       auto newBounds =
@@ -1249,14 +1238,15 @@ void MemDependencyChecker::updateWriteHistory(
 
       // Add all new slices.
       for (auto& b : newBounds) {
-        it = writeHistory.insert(it, std::make_pair(b, other));
+        writeHistory.insert(it, std::make_pair(b, other));
       }
-      it++;
+      // No need to increment the iterator since it has been updated after
+      // `erase` above.
     }
   }
 
   if (insert && isWrite) {
-    writeHistory.emplace_back(std::make_pair(info->bounds(), info));
+    writeHistory.emplace_back(info->bounds(), info);
   }
 }
 
@@ -1281,7 +1271,7 @@ void MemDependencyChecker::mergeScope(
 
   // Copy open writes up.
   for (auto& pair : child->openWrites_) {
-    const Var* var = pair.first;
+    VarPtr var = pair.first;
 
     // Intentionally using operator[], we want it to be created if it does not
     // exist.
@@ -1304,7 +1294,7 @@ class VarBoundBinder : public IRVisitor {
  public:
   VarBoundBinder(const VarBoundMap& vars) : vars_(vars) {}
 
-  Bound getBounds(const Expr* e) {
+  Bound getBounds(const ExprPtr& e) {
     min_ = e;
     max_ = e;
     e->accept(this);
@@ -1314,33 +1304,30 @@ class VarBoundBinder : public IRVisitor {
   }
 
  private:
-  void visit(const Var* v) override {
+  void visit(const VarPtr& v) override {
     auto it = vars_.find(v);
     if (it == vars_.end()) {
       return;
     }
 
-    min_ = Substitute(min_, {{v, it->second.start}});
-    max_ = Substitute(max_, {{v, it->second.end}});
+    min_ = SubstituteInClone(min_, {{v, it->second.start}});
+    max_ = SubstituteInClone(max_, {{v, it->second.end}});
   }
 
-  const Expr* min_{nullptr};
-  const Expr* max_{nullptr};
+  ExprPtr min_{nullptr};
+  ExprPtr max_{nullptr};
   const VarBoundMap& vars_;
 };
 
 std::vector<Bound> MemDependencyChecker::getIndicesBounds(
-    const std::vector<const Expr*>& indices) {
+    const std::vector<ExprPtr>& indices) {
   std::vector<Bound> bounds;
   bounds.reserve(indices.size());
   VarBoundBinder binder(knownVarBounds_);
-  for (auto* s : indices) {
+  for (const auto& s : indices) {
     bounds.push_back(binder.getBounds(s));
   }
   return bounds;
 }
 
-} // namespace analysis
-} // namespace tensorexpr
-} // namespace jit
-} // namespace torch
+} // namespace torch::jit::tensorexpr::analysis
