@@ -1,6 +1,4 @@
-#include <pybind11/functional.h>
 #include <pybind11/operators.h>
-#include <pybind11/stl.h>
 #include <torch/csrc/jit/python/pybind_utils.h>
 #include <torch/csrc/jit/tensorexpr/codegen.h>
 #include <torch/csrc/utils/pybind.h>
@@ -11,12 +9,15 @@
 #include <torch/csrc/jit/tensorexpr/ir_printer.h>
 #include <torch/csrc/jit/tensorexpr/ir_simplifier.h>
 #include <torch/csrc/jit/tensorexpr/kernel.h>
-#include <torch/csrc/jit/tensorexpr/llvm_codegen.h>
 #include <torch/csrc/jit/tensorexpr/loopnest.h>
 #include <torch/csrc/jit/tensorexpr/lowerings.h>
 #include <torch/csrc/jit/tensorexpr/reduction.h>
 #include <torch/csrc/jit/tensorexpr/tensorexpr_init.h>
 
+#ifdef TORCH_ENABLE_LLVM
+#include <torch/csrc/jit/tensorexpr/llvm_codegen.h>
+#endif
+
 #include <utility>
 
 template <>
