@@ -13,9 +13,9 @@ struct NoOpPythonTracer : public PythonTracerBase {
   void restart() override {}
   void register_gc_callback() override {}
   std::vector<std::shared_ptr<Result>> getEvents(
-      std::function<c10::time_t(c10::approx_time_t)>,
-      std::vector<CompressedEvent>&,
-      c10::time_t) override {
+      std::function<c10::time_t(c10::approx_time_t)> /*time_converter*/,
+      std::vector<CompressedEvent>& /*enters*/,
+      c10::time_t /*end_time_ns*/) override {
     return {};
   }
 };
@@ -25,7 +25,7 @@ struct NoOpMemoryPythonTracer : public PythonMemoryTracerBase {
   ~NoOpMemoryPythonTracer() override = default;
   void start() override {}
   void stop() override {}
-  void export_memory_history(const std::string&) override {}
+  void export_memory_history(const std::string& /*path*/) override {}
 };
 
 } // namespace
