@@ -5,7 +5,7 @@ torchrun --standalone --nnodes=1 --nproc-per-node=4 comm_mode_features_example.p
 
 import argparse
 import os
-from typing import Callable, Union
+from typing import TYPE_CHECKING
 
 import torch
 import torch.nn as nn
@@ -26,6 +26,10 @@ from torch.testing._internal.distributed._tensor.common_dtensor import (
 from torch.utils.checkpoint import checkpoint
 
 
+if TYPE_CHECKING:
+    from collections.abc import Callable
+
+
 def get_device_type() -> str:
     device_type = "cpu"
     if torch.accelerator.device_count() >= 4:
@@ -51,7 +55,7 @@ class CommDebugModeExample:
         self.device_type = get_device_type()
 
     def _MLP_model_setup(
-        self, model_type: type, parallelize_plan: Union[None, dict] = None
+        self, model_type: type, parallelize_plan: None | dict = None
     ) -> tuple[nn.Module, torch.Tensor]:
         """
         Creates MLP or MLPStacked model for examples
