@@ -54,8 +54,8 @@ std::string extractToTemporaryFolder(
             << " from archive path: " << path << " size: " << dataSize;
 
     File extracted(extractedFilename, O_CREAT | O_WRONLY, 0640);
-    const auto bytesWritten = writeFull(
-        extracted.fd(), const_cast<void*>(dataPointer.get()), dataSize);
+    const auto bytesWritten =
+        writeFull(extracted.fd(), dataPointer.get(), dataSize);
     TORCH_CHECK(
         bytesWritten != -1,
         "failure copying from archive path ",
