@@ -3,7 +3,6 @@
 #include <fmt/ostream.h>
 #include <fmt/ranges.h>
 #include <limits>
-#include <queue>
 
 #include <c10/util/Enumerate.h>
 #include <c10/util/FbcodeMaps.h>
@@ -683,6 +682,36 @@ void Graph::applyDevicePlacement(const Placement& placement) {
   }
 }
 
+void Graph::overrideWeightsDevice(
+    const std::unordered_map<std::string, std::optional<c10::Device>>&
+        submodNameToDevice) {
+  for (auto& [weightName, weightMeta] : weightsMeta_) {
+    for (auto& [name, device] : submodNameToDevice) {
+      if (device.has_value() && weightMeta.device() != device &&
+          c10::starts_with(weightName, name) &&
+          (weightName == name || weightName[name.length()] == '.')) {
+        LOG(INFO) << "Overriding " << weightName << " from "
+                  << weightMeta.device() << " to device " << device.value();
+        weightMeta.setDevice(device.value());
+        break;
+      }
+    }
+  }
+
+  for (auto& [tensorName, tensorMeta] : tensorValuesMeta_) {
+    for (auto& [name, device] : submodNameToDevice) {
+      if (device.has_value() && tensorMeta.device() != device &&
+          c10::starts_with(tensorName, name) &&
+          (tensorName == name || tensorName[name.length()] == '.')) {
+        LOG(INFO) << "Overriding " << tensorName << " from "
+                  << tensorMeta.device() << " to device " << device.value();
+        tensorMeta.setDevice(device.value());
+        break;
+      }
+    }
+  }
+}
+
 Node* Graph::nodeAfter(Node* n) {
   TORCH_CHECK(n->owningGraph() == this);
   if (n == outputNode_) {
@@ -1001,9 +1030,18 @@ std::ostream& operator<<(std::ostream& out, const Constant& constant) {
         } else if constexpr (is_same_v<T, c10::Layout>) {
           out << kLayoutPrefix << arg;
         } else if constexpr (is_same_v<T, c10::Device>) {
-          out << kDevicePrefix << "{" << arg << "}";
+          out << kDevicePrefix << '{' << arg << '}';
         } else if constexpr (is_same_v<T, vector<string>>) {
           out << fmt::format("[{}]", fmt::join(arg, ","));
+        } else if constexpr (is_same_v<T, vector<vector<int64_t>>>) {
+          out << '[';
+          for (const auto& [idx, inner_list] : c10::enumerate(arg)) {
+            if (idx > 0) {
+              out << ", ";
+            }
+            out << fmt::format("{}", fmt::streamed(inner_list));
+          }
+          out << ']';
         } else if constexpr (is_same_v<T, unique_ptr<Graph>>) {
           out << fmt::format("<subgraph>");
           VLOG(0) << "Subgraph pretty print is not implemented";
@@ -1024,16 +1062,16 @@ void printValue(std::ostream& out, const Value* v) {
 }
 
 void printNamedArgument(std::ostream& out, const NamedArgument& nv) {
-  out << nv.name << "=" << *nv.value;
+  out << nv.name << '=' << *nv.value;
 }
 
 void printAttribute(std::ostream& out, const Attribute& nv) {
-  out << nv.name << "=" << nv.value;
+  out << nv.name << '=' << nv.value;
 }
 } // namespace
 
 std::ostream& operator<<(std::ostream& out, const Value& v) {
-  out << "%" << v.name();
+  out << '%' << v.name();
   // If a list, distinguish it by adding a []
   // Looks like %my_list[]
   if (v.type() == Type::Kind::TensorList) {
@@ -1055,14 +1093,14 @@ std::ostream& operator<<(std::ostream& out, const Node& node) {
     printList(out, false, node.inputs(), [](std::ostream& out, const auto& nv) {
       out << *nv.value;
     });
-    out << ")";
+    out << ')';
     return out;
   }
 
   printList(out, false, node.outputs_, printValue);
 
   out << " = ";
-  out << node.target_ << "(";
+  out << node.target_ << '(';
   printList(out, false, node.inputs_, printNamedArgument);
   if (!node.inputs_.empty() && !node.attributes_.empty()) {
     // Emit a connective ',' between inputs and attributes.
@@ -1070,13 +1108,13 @@ std::ostream& operator<<(std::ostream& out, const Node& node) {
   }
 
   printList(out, false, node.attributes_, printAttribute);
-  out << ")";
+  out << ')';
   return out;
 }
 
 std::ostream& operator<<(std::ostream& out, const Graph& graph) {
   for (const auto& node : graph.nodes_) {
-    out << node << "\n";
+    out << node << '\n';
   }
   return out;
 }
